Package: PNADCperiods
Title: Identify Reference Periods in Brazil's PNADC Survey Data
Version: 0.1.2
Date: 2026-04-27
Authors@R: c(
    person("Rogerio", "Barbosa", email = "rogerio.barbosa@iesp.uerj.br", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6796-4547",
                       contribution = "R package, dashboard, and website")),
    person("Marcos", "Hecksher", email = "mdhecksher@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0003-2992-1252",
                       contribution = "Mensalization methodology"))
    )
Description: Identifies reference periods (months, fortnights, and weeks) in
    Brazil's quarterly PNADC (Pesquisa Nacional por Amostra de Domicilios
    Continua) survey data and computes calibrated weights for sub-quarterly
    analysis. The core algorithm uses IBGE (Instituto Brasileiro de Geografia
    e Estatistica) 'Parada Tecnica' (technical break) rules combined with
    respondent birthdates to determine which temporal period each survey
    observation refers to. Period identification follows a nested hierarchy
    enforced by construction: fortnights require months, weeks require
    fortnights. Achieves approximately 97% monthly determination rate with
    the full series (2012-2025). Strict fortnight and week rates are
    approximately 9% and 3% respectively, as they cannot leverage
    cross-quarter panel aggregation. Experimental strategies (probabilistic
    assignment and UPA (Primary Sampling Unit) aggregation) further improve
    these determination rates. The package provides adaptive hierarchical
    weight calibration (4/2/1 cell levels for month/fortnight/week) with
    period-specific smoothing to produce survey weights calibrated to
    SIDRA (Sistema IBGE de Recuperacao Automatica) population totals. Also
    includes a SIDRA mensalization module that converts 86+ official rolling
    quarter series from the IBGE SIDRA API (Application Programming
    Interface) into exact monthly estimates, without requiring access to
    microdata. Hecksher (2020)
    <https://repositorio.ipea.gov.br/handle/11058/9859>.
License: MIT + file LICENSE
Language: en-US
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 4.1.0)
Imports: data.table (>= 1.14.0), checkmate (>= 2.0.0), sidrar (>=
        0.2.9), lubridate (>= 1.9.4)
Suggests: dplyr, fst, haven, testthat (>= 3.0.0), knitr, rmarkdown,
        pkgdown, ggplot2, scales
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://antrologos.github.io/PNADCperiods/,
        https://github.com/antrologos/PNADCperiods
BugReports: https://github.com/antrologos/PNADCperiods/issues
NeedsCompilation: no
Packaged: 2026-04-27 23:46:10 UTC; antro
Author: Rogerio Barbosa [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6796-4547>, contribution: R package,
    dashboard, and website),
  Marcos Hecksher [aut] (ORCID: <https://orcid.org/0000-0003-2992-1252>,
    contribution: Mensalization methodology)
Maintainer: Rogerio Barbosa <rogerio.barbosa@iesp.uerj.br>
Repository: CRAN
Date/Publication: 2026-04-28 08:00:02 UTC
Built: R 4.6.0; ; 2026-05-01 21:24:35 UTC; unix

# Generated by roxygen2: do not edit by hand

export(clear_sidra_cache)
export(compute_series_starting_points)
export(compute_starting_points_from_microdata)
export(compute_z_aggregates)
export(fetch_monthly_population)
export(fetch_sidra_rolling_quarters)
export(get_sidra_series_metadata)
export(mensalize_sidra_series)
export(pnadc_apply_periods)
export(pnadc_experimental_periods)
export(pnadc_identify_periods)
export(validate_pnadc)
import(data.table)
importFrom(checkmate,assert_character)
importFrom(checkmate,assert_choice)
importFrom(checkmate,assert_data_frame)
importFrom(checkmate,assert_int)
importFrom(checkmate,assert_logical)
importFrom(checkmate,assert_string)
importFrom(lubridate,ymd)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

# PNADCperiods

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/PNADCperiods)](https://CRAN.R-project.org/package=PNADCperiods)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/grand-total/PNADCperiods)](https://CRAN.R-project.org/package=PNADCperiods)
[![R-CMD-check](https://github.com/antrologos/PNADCperiods/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/antrologos/PNADCperiods/actions/workflows/R-CMD-check.yaml)
[![pkgdown](https://github.com/antrologos/PNADCperiods/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/antrologos/PNADCperiods/actions/workflows/pkgdown.yaml)
[![codecov](https://codecov.io/gh/antrologos/PNADCperiods/branch/master/graph/badge.svg)](https://app.codecov.io/gh/antrologos/PNADCperiods)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

Convert Brazil's quarterly PNADC survey data into sub-quarterly time series (monthly, fortnightly, weekly) and mensalize IBGE SIDRA aggregate series.

## Installation

Install the released version from CRAN:

```r
install.packages("PNADCperiods")
```

Or the development version from GitHub:

```r
# install.packages("remotes")
remotes::install_github("antrologos/PNADCperiods")
```

## Main Features

- **Microdata mensalization**: Identify reference months, fortnights, and weeks in PNADC microdata
- **SIDRA mensalization**: Convert 86+ rolling quarterly IBGE series to exact monthly values
- **Weight calibration**: Hierarchical raking to IBGE population totals
- **~97% monthly determination** with full data stacking; experimental strategies improve fortnight/week rates further

## Interactive Dashboard

Explore 86+ official PNADC series with an interactive dashboard -- no R required:

**<https://antrologos.shinyapps.io/PNADCperiods-dashboard/>**

The dashboard is built on top of this package and lets you visualize mensalized series, compare strategies, and download monthly time-series as CSV.

## Basic Usage

### Microdata Mensalization

```r
library(PNADCperiods)

# Build crosswalk identifying reference periods
crosswalk <- pnadc_identify_periods(pnadc_stacked)

# Apply to data with weight calibration
result <- pnadc_apply_periods(pnadc, crosswalk, weight_var = "V1028", anchor = "quarter")
```

### SIDRA Series Mensalization

```r
# Fetch rolling quarterly data from SIDRA
rolling <- fetch_sidra_rolling_quarters(theme = "labor_market")

# Convert to exact monthly series
monthly <- mensalize_sidra_series(rolling)
```

## Key Functions

### Microdata

| Function | Description |
|----------|-------------|
| `pnadc_identify_periods()` | Build crosswalk: identify months/fortnights/weeks |
| `pnadc_apply_periods()` | Apply crosswalk + calibrate weights |
| `pnadc_experimental_periods()` | Boost rates with probabilistic strategies |
| `validate_pnadc()` | Validate input columns |

### SIDRA Series

| Function | Description |
|----------|-------------|
| `get_sidra_series_metadata()` | List 86+ available PNADC series |
| `fetch_sidra_rolling_quarters()` | Download rolling quarter data from SIDRA |
| `mensalize_sidra_series()` | Convert rolling quarters to exact monthly |
| `fetch_monthly_population()` | Fetch population totals from SIDRA |

## Documentation

- [Getting Started](https://antrologos.github.io/PNADCperiods/articles/getting-started.html)
- [SIDRA Mensalization](https://antrologos.github.io/PNADCperiods/articles/sidra-mensalization.html)
- [How It Works](https://antrologos.github.io/PNADCperiods/articles/how-it-works.html)
- [Applied Examples](https://antrologos.github.io/PNADCperiods/articles/applied-examples.html)
- [Full Reference](https://antrologos.github.io/PNADCperiods/reference/index.html)

## Authors

- **Rogerio J. Barbosa** (Ceres-IESP/UERJ) -- R package, dashboard, and website
- **Marcos Hecksher** (Ipea) -- Mensalization methodology

## Credits

Original PNADC data is collected by the Brazilian Institute of Geography and Statistics (IBGE). The mensalization methodology was developed by Marcos Hecksher ([Ipea](https://www.ipea.gov.br/)) --- see Hecksher (2020, IPEA Nota Tecnica Disoc n. 62 and n. 87; Carta de Conjuntura v. 47). The R package, interactive dashboard, and documentation website were developed by Rogerio J. Barbosa at the [Center for the Study of Wealth and Social Stratification (Ceres - IESP/UERJ)](https://ceres-iesp.uerj.br/).

**Citation:**

> Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version 0.1.1. https://CRAN.R-project.org/package=PNADCperiods

```r
citation("PNADCperiods")
```

## Getting Help

- [GitHub Issues](https://github.com/antrologos/PNADCperiods/issues) for bug reports and questions

## License

MIT

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  eval = FALSE,
  echo = TRUE,
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 10,
  fig.height = 6,
  purl = FALSE
)

#' Starting Points for SIDRA Series Mensalization
#'
#' Pre-computed starting point values (y0) for mensalizing IBGE's rolling
#' quarterly series into exact monthly estimates.
#'
#' @name pnadc_series_starting_points
#' @docType data
#' @usage data(pnadc_series_starting_points)
#'
#' @format A data.table with 159 rows and 3 columns:
#' \describe{
#'   \item{series_name}{Character. Name of the SIDRA series (53 series)}
#'   \item{mesnotrim}{Integer. Month position in quarter (1, 2, or 3)}
#'   \item{y0}{Numeric. Starting point value for this series and position}
#' }
#'
#' @details
#' These starting points were computed from PNADC microdata using the full
#' R package pipeline, ensuring consistency with
#' \code{\link{compute_starting_points_from_microdata}}:
#' \enumerate{
#'   \item Weight calibration via \code{\link{pnadc_apply_periods}}: all months
#'     scaled to SIDRA monthly population totals
#'   \item z_ aggregates computed via \code{\link{compute_z_aggregates}} using
#'     calibrated \code{weight_monthly}
#'   \item Starting points computed via \code{\link{compute_series_starting_points}}
#'     with CNPJ-aware calibration periods
#' }
#'
#' The calibration period (2013-2019) was chosen because:
#' \itemize{
#'   \item It includes stable pre-pandemic data
#'   \item IBGE methodology was consistent during this period
#'   \item Sufficient observations for reliable estimates
#' }
#'
#' CNPJ series (empregadorcomcnpj, empregadorsemcnpj, contapropriacomcnpj,
#' contapropriasemcnpj) use calibration period 2016-2019 with cumulative sum
#' starting from October 2015 due to V4019 variable availability.
#'
#' @section Methodology Consistency:
#' The bundled starting points are generated using the same pipeline as
#' \code{\link{compute_starting_points_from_microdata}}, ensuring that users
#' who compute custom starting points will get consistent results.
#'
#' @section When to Use Custom Starting Points:
#' The bundled starting points are suitable for most users. Consider computing
#' custom starting points with \code{\link{compute_starting_points_from_microdata}} if:
#' \itemize{
#'   \item IBGE makes major methodological changes to the PNADC
#'   \item You need series not included in the bundled data
#'   \item You want to use a different calibration period
#'   \item You are working with updated or different microdata
#' }
#'
#' @source Computed using \code{data-raw/regenerate_starting_points_from_microdata.R}
#'
#' @seealso
#' \code{\link{mensalize_sidra_series}} which uses this data by default
#' \code{\link{compute_series_starting_points}} for custom calibration
#'
#' @examples
#' data(pnadc_series_starting_points)
#' head(pnadc_series_starting_points)
#' unique(pnadc_series_starting_points$series_name)
#'
"pnadc_series_starting_points"

#' PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data
#'
#' The PNADCperiods package provides tools to identify the exact reference
#' period (month, fortnight, or week) of Brazil's quarterly official household
#' survey, PNADC (Pesquisa Nacional por Amostra de Domicilios Continua - IBGE),
#' allowing for analyzing the survey data at sub-quarterly temporal granularity.
#'
#' The package offers four main capabilities:
#' \enumerate{
#'   \item \strong{Reference period identification}: Determines which temporal
#'     period (month, fortnight, or week) within each quarter each survey
#'     observation refers to, using IBGE's "Parada Técnica" rules and
#'     respondent birthdates. The identification is \strong{nested by construction}:
#'     fortnights require months, and weeks require fortnights.
#'   \item \strong{Period-specific weight calibration}: Adjusts survey weights
#'     for sub-quarterly estimates using adaptive hierarchical rake weighting
#'     (4/2/1 cell levels for month/fortnight/week respectively)
#'   \item \strong{Experimental strategies}: Probabilistic assignment and UPA
#'     aggregation to boost fortnight/week determination rates for sensitivity
#'     analysis
#'   \item \strong{SIDRA series mensalization}: Converts IBGE's rolling quarterly
#'     (trimestre móvel) aggregate series to exact monthly estimates
#' }
#'
#' \strong{Determination rates (strict, full series 2012-2025):}
#' \itemize{
#'   \item Monthly: ~97\%, ~98\% with annual anchor
#'   \item Fortnight: ~9\%
#'   \item Week: ~3\%
#' }
#'
#' With smaller datasets, rates may differ (e.g., 8 quarters: ~94\% monthly).
#' Experimental strategies (probabilistic + UPA aggregation) further improve
#' these determination rates.
#'
#' The package is highly optimized for large datasets (~450,000 rows/sec).
#' Uses pre-computed lookup tables for 20x faster date creation.
#'
#' \strong{Note:} Strict fortnight and week determination rates are inherently low
#' because they cannot leverage cross-quarter aggregation like months can. Only
#' birthday constraints within a single quarter are available to narrow the
#' interview window.
#'
#' The main functions are:
#' \itemize{
#'   \item \code{\link{pnadc_identify_periods}}: Builds a crosswalk containing
#'     month, fortnight, and week reference periods with IBGE calendar-based dates
#'   \item \code{\link{pnadc_apply_periods}}: Applies the crosswalk to any
#'     PNADC dataset and optionally calibrates weights
#'   \item \code{\link{pnadc_experimental_periods}}: Applies experimental strategies
#'     (probabilistic, UPA aggregation) for improved fortnight/week determination
#'   \item \code{\link{get_sidra_series_metadata}}: Lists 86+ available PNADC
#'     rolling quarter series from SIDRA
#'   \item \code{\link{fetch_sidra_rolling_quarters}}: Downloads rolling quarterly
#'     data from IBGE SIDRA API
#'   \item \code{\link{mensalize_sidra_series}}: Converts rolling quarters to exact
#'     monthly estimates
#' }
#'
#' @references
#' HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores
#' Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62.
#' Brasilia, DF: IPEA, 2020.
#' \url{https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453}
#'
#' HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo
#' a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
#'
#' HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco,
#' aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
#'
#' IBGE. Manual Basico da Entrevista PNADC (methodology on "Parada Tecnica").
#'
#' @author Rogerio Barbosa \email{rogerio.barbosa@@iesp.uerj.br} (R package, dashboard, and website; Ceres-IESP/UERJ)
#' @author Marcos Hecksher \email{mdhecksher@@gmail.com} (mensalization methodology; Ipea)
#'
#' @import data.table
#' @importFrom lubridate ymd
#' @importFrom checkmate assert_data_frame assert_int assert_logical assert_string assert_character assert_choice
#' @importFrom stats lm predict
#' @importFrom utils data head setTxtProgressBar txtProgressBar
#' @keywords internal
"_PACKAGE"

# Prevent R CMD check notes about data.table syntax
utils::globalVariables(c(
  # PNADC variables
  "Ano", "Trimestre", "UPA", "V1008", "V1014", "V2003",
  "V2008", "V20081", "V20082", "V2009",
  "V1028", "V1032", "UF", "posest", "posest_sxi",
  # Computed variables - reference month identification
  "ref_month", "ref_month_start", "ref_month_end", "ref_month_in_quarter", "ref_month_yyyymm", "ref_month_weeks",
  "birthday", "first_sat_after_birthday",
  "visit_before_birthday", "month1", "month2", "month3",
  "first_sat_m1", "first_sat_m2", "first_sat_m3",
  "alt_sat_m1", "alt_sat_m2", "alt_sat_m3",
  "date_min", "date_max", "month_min_pos", "month_max_pos",
  "alt_date_min", "alt_date_max", "alt_month_min_pos", "alt_month_max_pos",
  "upa_month_min", "upa_month_max",
  "alt_upa_month_min", "alt_upa_month_max",
  "upa_month_min_final", "upa_month_max_final",
  "requires_exception", "requires_exc_m1", "requires_exc_m2", "requires_exc_m3",
  "trim_exc_m1", "trim_exc_m2", "trim_exc_m3",
  "requires_exc_fortnight", "trim_exc_fortnight",
  "requires_exc_week", "trim_exc_week",
  # Computed variables - reference fortnight identification
  "ref_fortnight", "ref_fortnight_start", "ref_fortnight_end", "ref_fortnight_in_quarter", "ref_fortnight_yyyyff",
  "fortnight_min_pos", "fortnight_max_pos", "fortnight_min_yyyyff", "fortnight_max_yyyyff",
  "alt_fortnight_min_pos", "alt_fortnight_max_pos",
  "hh_fortnight_min", "hh_fortnight_max",
  "alt_hh_fortnight_min", "alt_hh_fortnight_max",
  "determined_fortnight",
  # Computed variables - reference week identification
  "ref_week", "ref_week_start", "ref_week_end", "ref_week_in_quarter", "ref_week_yyyyww",
  "week_min_yyyyww", "week_max_yyyyww", "week_min_pos", "week_max_pos",
  "alt_week_min_yyyyww", "alt_week_max_yyyyww",
  "hh_week_min", "hh_week_max", "alt_hh_week_min", "alt_hh_week_max",
  # Sequential week values for year boundary comparisons
  "week_min_seq", "week_max_seq", "hh_week_min_seq", "hh_week_max_seq",
  "alt_week_min_seq", "alt_week_max_seq", "alt_hh_week_min_seq", "alt_hh_week_max_seq",
  "trim_has_exception",
  "determined_month", "determined_week",
  # Computed variables - weight calibration (monthly, fortnightly, and weekly)
  "celula1", "celula2", "celula3", "celula4",
  "weight_current", "weight_calibrated", "weight_monthly", "weight_fortnight", "weight_weekly",
  "pop_quarter", "pop_month", "pop_fortnight", "pop_week",
  "n_cells_quarter", "n_cells_month", "n_cells_fortnight", "n_cells_week",
  "m_populacao", "f_populacao", "z_populacao", "w_populacao",
  # Computed variables - SIDRA population fetch
  "Valor", "anomesexato", "anomesfinaltrimmovel", "populacao",
  # Computed variables - smooth aggregates
  "month_pos", "weight_smoothed", "pop_current",
  "row_num", "row_num2", "d_pop", "quarter_yyyyq",
  # Computed variables - monthly to weekly/fortnight targets
  "yyyyww", "yyyymm", "yyyyff", "n_days", "days_in_month", "ref_month_yyyymm",
  # data.table join prefix variables (i.* references right table columns)
  "i.month1", "i.month2", "i.month3",
  "i.first_sat_m1", "i.first_sat_m2", "i.first_sat_m3",
  "i.alt_sat_m1", "i.alt_sat_m2", "i.alt_sat_m3",
  "i.ref_month", "i.ref_month_start", "i.ref_month_end", "i.ref_month_in_quarter", "i.ref_month_yyyymm", "i.ref_month_weeks",
  "i.ref_fortnight", "i.ref_fortnight_start", "i.ref_fortnight_end", "i.ref_fortnight_in_quarter", "i.ref_fortnight_yyyyff",
  "i.ref_week", "i.ref_week_start", "i.ref_week_end", "i.ref_week_in_quarter", "i.ref_week_yyyyww",
  "i.determined_month", "i.determined_fortnight", "i.determined_week",
  "i.m_populacao", "i.f_populacao", "i.w_populacao",
  # data.table special symbols and output column reference
  ".SD", ".N", ".I", "..output_cols", ".",
  # Calibration variables
  "v1032", "v2009", "uf", "..xw_cols",
  "n_months_in_year", "pop_year", "n_years_in_month", "year",
  # Unified calibration variables
  "anchor_year", "target_pop", "i.target_pop",
  "n_cells_anchor", "n_cells_period", "pop_anchor", "pop_period",
  # Optimized calibration internal variables
  ".anchor_key", ".is_determined", "i.pop_orig",
  "ref_week_iso_yyyyww", "ref_week_yyyyww",
  # Smoothing variables
  "period_pos", "i.pos", "cell_pop", "pop_smoothed", "pop_lag", "pop_lead",
  "smooth_factor", "i.smooth_factor", "pop_orig", "pop_new", "final_factor",
  "i.final_factor",
  # Month-conditional refinement variables
  "month_num_temp", "last_day_temp", "month_start_yyyyww", "month_end_yyyyww",
  # Nested fortnight/week identification variables (Phase 2 & 3)
  "fortnight_lower", "fortnight_upper",
  "fortnight_month", "fortnight_half", "fortnight_month_num",
  "fortnight_start_day", "fortnight_end_day",
  "fortnight_start_yyyyww", "fortnight_end_yyyyww",
  # Experimental period identification variables (restructured nested strategies)
  "ref_month_exp", "ref_month_exp_confidence",
  "ref_fortnight_exp", "ref_fortnight_exp_confidence",
  "ref_week_exp", "ref_week_exp_confidence",
  "hh_date_min", "hh_date_max", "fortnight_range", "date_midpoint",
  "midpoint_day", "boundary_distance",
  "ref_fortnight_likely", "ref_fortnight_confidence",
  "ref_week_likely", "ref_week_confidence",
  "ref_fortnight_upa", "ref_week_upa",
  "consensus_fortnight", "consensus_week", "n_unique", "n_households",
  "i.hh_date_min", "i.hh_date_max", "i.hh_fortnight_min", "i.hh_fortnight_max",
  "i.fortnight_range", "i.consensus_fortnight", "i.consensus_week",
  "needs_reagg",
  # Probabilistic strategy variables
  "hh_week_min_yyyyww", "hh_week_max_yyyyww",
  "hh_week_min_seq", "hh_week_max_seq", "week_range",
  "midpoint_week_yyyyww", "week_boundary_distance",
  "upa_date_min", "upa_date_max",
  "upa_month_min_pos", "upa_month_max_pos", "month_range",
  "month1_start", "month2_start", "month3_start",
  "boundary_date", "days_before_boundary", "days_after_boundary", "total_days",
  "month_identified", "effective_month",
  "hh_date_midpoint", "boundary_day_15",
  "days_in_first_fortnight", "days_in_second_fortnight", "total_interval_days",
  "fortnight_identified", "effective_fortnight",
  "fortnight_start_date", "fortnight_end_date",
  "week_date_midpoint", "week_boundary_date",
  "days_in_first_week", "days_in_second_week", "total_week_interval",
  # UPA aggregation strategy variables
  "n_total", "n_strict", "n_narrow", "consensus_month", "prop_strict",
  "has_narrow_month_range", "has_narrow_fortnight_range", "has_narrow_week_range",
  "prop_narrow", "likely_month", "likely_fortnight", "likely_week",
  "days_before", "days_after", "confidence",
  "upa_fortnight_min", "upa_fortnight_max", "upa_fortnight_range",
  "upa_week_min", "upa_week_max", "upa_week_range",
  "upa_week_min_seq", "upa_week_max_seq",
  "week_min", "week_max", "week_min_seq", "week_max_seq",
  "midpoint_week_seq",
  # data.table join prefix variables for experimental strategies
  "i.consensus_month", "i.likely_month", "i.likely_fortnight", "i.likely_week",
  "i.confidence", "i.month_identified", "i.effective_month",
  "i.fortnight_identified", "i.effective_fortnight",
  "i.upa_date_min", "i.upa_date_max", "i.upa_month_min_pos", "i.upa_month_max_pos",
  "i.month_range", "upa_month_min", "upa_month_max", "upa_month_range",
  # Combine crosswalks and derived columns
  "probabilistic_assignment",
  "i.ref_month_exp", "i.ref_month_exp_confidence",
  "i.ref_fortnight_exp", "i.ref_fortnight_exp_confidence",
  "i.ref_week_exp", "i.ref_week_exp_confidence",
  # IBGE calendar variables (quarter/month/fortnight/week bounds)
  "ibge_month_start_int", "ibge_first_sat_day", "i.ibge_month_start_int", "i.ibge_n_weeks",
  "quarter_end", "alt_quarter_end",
  "temp_month", "temp_month_in_q", "temp_fortnight_in_month",
  "ref_fortnight_weeks",
  "alt_week_min_pos", "alt_week_max_pos",
  "hh_week_min_pos", "hh_week_max_pos",
  "fortnight_boundary",
  # UPA aggregation week nesting validation variables
  "effective_fortnight_for_week", "week_lower_bound", "week_upper_bound",
  # Exception detection variables (experimental strategy)
  "alt_sat_m1", "alt_sat_m2", "alt_sat_m3",
  "alt_date_min", "alt_date_max", "alt_month_min_pos", "alt_month_max_pos",
  "upa_month_min_final", "upa_month_max_final",
  "month_min_pos_final", "month_max_pos_final",
  "midpoint_in_tech_stop", "tech_stop_type_month", "adjusted_midpoint",
  "month1_min_days", "month2_min_days", "month3_min_days",
  # Variables from pnadc_identify_periods and identify_reference_week
  "date_min_sat", "date_max_sat", "alt_date_min_sat", "alt_date_max_sat",
  "fortnight_min_in_month", "fortnight_max_in_month", "fortnight1_end",
  "alt_fortnight_min_in_month", "alt_fortnight_max_in_month",
  "week_month_in_quarter", "week_month_num", "week_fortnight_in_month",
  "fortnight_first_week_in_month", "fortnight_last_week_in_month",
  "fortnight_first_week_in_quarter", "fortnight_last_week_in_quarter",
  "alt_hh_week_min_pos", "alt_hh_week_max_pos",
  "in_tech_stop_range", "in_technical_stop_range", "tech_stop_type",
  "ref_week_temp", "hh_consensus_week", "fallback_week",
  "week_month_in_qtr", "week_in_month", "ibge_month_start_date",
  "i.trim_exc_m1", "i.trim_exc_m2", "i.trim_exc_m3",
  # Variables from pnadc_identify_periods week refinement
  "temp_month_num",
  # ============================================================================
  # SIDRA Series Mensalization Variables
  # ============================================================================
  # Metadata columns (hierarchical structure)
  "series_name", "api_path", "table_id", "variable_id",
  "classification_id", "classification_value",
  "theme", "theme_category", "subcategory",
  "description_pt", "description_en", "description",
  "unit", "unit_label_pt", "unit_label_en",
  "is_derived", "requires_deflation",
  # Mensalization algorithm variables
  "mesnotrim", "anomesfinaltrimmovel", "anomesexato",
  "d3", "cum", "cum1", "cum2", "cum3",
  "d3m1", "d3m2", "d3m3",
  "e0", "y0", "y",
  # SIDRA API result columns
  "Valor", "Trimestre Movel", "Trimestre.Movel",
  # Deflation variables
  "ipca_index", "deflator",
  # Starting points data
  "mesnotrim_1", "mesnotrim_2", "mesnotrim_3",
  # ============================================================================
  # Mensalization Derived Series Variables
  # ============================================================================
  "m_contribuinteprev", "m_desalentado", "m_domestico", "m_domesticocomcart",
  "m_domesticosemcart", "m_empregado", "m_empregpriv", "m_empregprivcomcart",
  "m_empregprivsemcart", "m_empregpubl", "m_empregpublcomcart",
  "m_empregpublsemcart", "m_estatutmilitar", "m_forcaampliada",
  "m_forcapotencial", "m_niveldesocup_calc", "m_nivelocup_calc",
  "m_perccontribprev_calc", "m_percdesalento_calc", "m_pop14mais",
  "m_popdesocup", "m_popnaforca", "m_popocup", "m_subocuphoras",
  "m_taxacombdesopot_calc", "m_taxacombdesosub_calc", "m_taxacompsubutlz_calc",
  "m_taxadesocup_calc", "m_taxapartic_calc", "m_taxasubocuphoras_calc",
  # ============================================================================
  # Probability/Aggregation Strategy Variables (pnadc-experimental-periods.R)
  # ============================================================================
  "..cols_to_keep",
  ".ppc_f_current", ".ppc_f_factor", ".ppc_f_target", ".ppc_fortnight",
  ".ppc_m_current", ".ppc_m_factor", ".ppc_m_target", ".ppc_month",
  ".ppc_q_current", ".ppc_q_factor", ".ppc_q_target", ".ppc_quarter",
  "days_within_fortnight_1", "days_within_fortnight_2",
  "days_within_month_1_min1_max2", "days_within_month_2_min1_max2",
  "days_within_month_2_min2_max3", "days_within_month_3_min2_max3",
  "days_within_week_1", "days_within_week_2", "days_within_week_3",
  "days_within_week_4",
  "determined_aggreg_fortnight", "determined_aggreg_month",
  "determined_aggreg_week", "determined_probable_fortnight",
  "determined_probable_month", "determined_probable_week",
  "fortnight_1_end", "fortnight_1_start", "fortnight_2_end", "fortnight_2_start",
  "fortnight_max_hh", "fortnight_min_hh", "fortnight_pos", "fortnight_prob_filter",
  "i.alt_date_max", "i.alt_date_min", "i.date_max", "i.date_min",
  "i.f_total_orig", "i.fortnight_1_end", "i.fortnight_1_start",
  "i.fortnight_2_end", "i.fortnight_2_start", "i.is_leap", "i.m_total_orig",
  "i.q_total_orig",
  "i.week_1_start", "i.week_1_end", "i.week_2_start", "i.week_2_end",
  "i.week_3_start", "i.week_3_end", "i.week_4_start", "i.week_4_end",
  "i.week_5_start", "i.week_5_end", "i.week_6_start", "i.week_6_end",
  "i.week_7_start", "i.week_7_end", "i.week_8_start", "i.week_8_end",
  "i.week_9_start", "i.week_9_end", "i.week_10_start", "i.week_10_end",
  "i.week_11_start", "i.week_11_end", "i.week_12_start", "i.week_12_end",
  "ibge_month_start", "is_leap",
  "month_max_upa", "month_min_upa", "month_prob_filter",
  "n_fortnight", "n_month", "n_week",
  "prob_month_min1_max2", "prob_month_min2_max3",
  "prob_ref_fortnight_in_month", "prob_ref_fortnight_in_month_max",
  "prob_ref_fortnight_in_month_min",
  "prob_ref_month_in_quarter", "prob_ref_month_in_quarter_max",
  "prob_ref_month_in_quarter_min",
  "prob_ref_week_in_month", "prob_ref_week_in_month_max",
  "prob_ref_week_in_month_min",
  "prob_week_1_2", "prob_week_3_4",
  "prop_fortnight", "prop_fortnight_1", "prop_fortnight_2",
  "prop_month", "prop_month_1_min1_max2", "prop_month_2_min1_max2",
  "prop_month_2_min2_max3", "prop_month_3_min2_max3",
  "prop_week", "prop_week_1", "prop_week_2", "prop_week_3", "prop_week_4",
  "ref_fortnight_in_month", "ref_fortnight_in_month_aggreg",
  "ref_fortnight_in_month_aggreg_confidence",
  "ref_month_in_quarter_aggreg", "ref_month_in_quarter_aggreg_confidence",
  "ref_month_in_year",
  "ref_week_in_month", "ref_week_in_month_aggreg",
  "ref_week_in_month_aggreg_confidence",
  "week_1_start", "week_1_end", "week_2_start", "week_2_end",
  "week_3_start", "week_3_end", "week_4_start", "week_4_end",
  "week_5_start", "week_8_end", "week_9_start", "week_12_end",
  "week_max_hh", "week_min_hh", "week_pos", "week_prob_filter",
  # ============================================================================
  # Derived Series and Aggregate Variables (compute_z_aggregates / compute_series_starting_points)
  # ============================================================================
  ".deflator_efet", ".deflator_hab", ".ipca_lagged",
  ".ipca100dez1993_lagged", "ipca100dez1993_lagged",
  ".inpc100dez1993_lagged",
  "ipca100dez1993", "inpc100dez1993",
  "comrendtodos", "rendhabnominaltodos", "rendefetnominaltodos",
  "massahabnominaltodos", "massaefetnominaltodos",
  # PNADC microdata variables used in compute_z_aggregates
  "V4019", "VD4001", "VD4002", "VD4003", "VD4004", "VD4004A", "VD4005",
  "VD4009", "VD4010", "VD4012", "VD4016", "VD4017", "VD4019", "VD4020",
  "VD4031", "VD4035",
  # z_ aggregate variables
  "z_comrendtodos",
  "z_contapropriacomcnpj", "z_contapropriasemcnpj",
  "z_contribuinteprev", "z_desalentado", "z_domestico",
  "z_empregado", "z_empregadorcomcnpj", "z_empregadorsemcnpj",
  "z_empregpriv", "z_empregpubl",
  "z_forcaampliada", "z_forcapotencial",
  "z_massaefetnominaltodos", "z_massaefetrealtodos",
  "z_massahabnominaltodos", "z_massahabrealtodos",
  "z_pop14mais", "z_popdesocup", "z_popforadaforca",
  "z_popnaforca", "z_popocup", "z_subocuphoras",
  # z_ income/hours disaggregated variables (by position/sector)
  "z_income_hab_empregado", "z_income_hab_empregpriv",
  "z_income_hab_empregprivcomcart", "z_income_hab_empregprivsemcart",
  "z_income_hab_domestico", "z_income_hab_domesticocomcart",
  "z_income_hab_domesticosemcart",
  "z_income_hab_empregpubl", "z_income_hab_empregpublcomcart",
  "z_income_hab_empregpublsemcart", "z_income_hab_estatutmilitar",
  "z_income_hab_empregador", "z_income_hab_contapropria",
  "z_income_hab_empregadorcomcnpj", "z_income_hab_empregadorsemcnpj",
  "z_income_hab_contapropriacomcnpj", "z_income_hab_contapropriasemcnpj",
  "z_income_hab_agropecuaria", "z_income_hab_industria",
  "z_income_hab_construcao", "z_income_hab_comercio",
  "z_income_hab_transporte", "z_income_hab_alojaliment",
  "z_income_hab_infcomfinimobadm", "z_income_hab_adminpublica",
  "z_income_hab_outroservico", "z_income_hab_servicodomestico",
  "z_hours_efet_empregado", "z_hours_efet_empregpriv",
  "z_hours_efet_empregprivcomcart", "z_hours_efet_empregprivsemcart",
  "z_hours_efet_domestico", "z_hours_efet_domesticocomcart",
  "z_hours_efet_domesticosemcart",
  "z_hours_efet_empregpubl", "z_hours_efet_empregpublcomcart",
  "z_hours_efet_empregpublsemcart", "z_hours_efet_estatutmilitar",
  "z_hours_efet_empregador", "z_hours_efet_contapropria",
  "z_hours_efet_empregadorcomcnpj", "z_hours_efet_empregadorsemcnpj",
  "z_hours_efet_contapropriacomcnpj", "z_hours_efet_contapropriasemcnpj",
  "z_hours_efet_agropecuaria", "z_hours_efet_industria",
  "z_hours_efet_construcao", "z_hours_efet_comercio",
  "z_hours_efet_transporte", "z_hours_efet_alojaliment",
  "z_hours_efet_infcomfinimobadm", "z_hours_efet_adminpublica",
  "z_hours_efet_outroservico", "z_hours_efet_servicodomestico"
))

#' Fetch SIDRA Rolling Quarter Series
#'
#' Functions to download PNADC labor market indicators from IBGE's SIDRA API.
#' These series are published as rolling quarterly averages (trimestre movel).
#'
#' @name fetch-sidra-series
#' @keywords internal
NULL

# ============================================================================
# UNIFIED CACHING INFRASTRUCTURE FOR SIDRA DATA
# ============================================================================
#
# All SIDRA-fetched data (rolling quarters, population) is cached in a single
# environment with namespaced keys:
#   - rolling_quarters_data, rolling_quarters_time
#   - population_data, population_time
#
# This simplifies cache management and ensures consistent behavior.
# ============================================================================

# Unified package-level cache environment for ALL SIDRA data
.sidra_cache <- new.env(parent = emptyenv())

#' Clear All SIDRA Caches
#'
#' Clears all cached SIDRA data (both rolling quarter series and population).
#' Use this if you need to force a fresh download, for example after IBGE
#' updates their data.
#'
#' @return Invisibly returns TRUE if any cache was cleared, FALSE if all empty.
#'
#' @examples
#' clear_sidra_cache()
#'
#' @export
clear_sidra_cache <- function() {
  cache_names <- ls(envir = .sidra_cache)
  had_cache <- length(cache_names) > 0

  if (had_cache) {
    rm(list = cache_names, envir = .sidra_cache)
  }

  invisible(had_cache)
}

#' Check if SIDRA Cache Entry is Valid
#'
#' @param cache_name Name of the cached object (e.g., "rolling_quarters", "population")
#' @param max_age_hours Maximum cache age in hours. NULL for no expiration (default).
#' @return Logical. TRUE if cache exists and is not expired, FALSE otherwise.
#' @keywords internal
#' @noRd
.is_cache_valid <- function(cache_name, max_age_hours = NULL) {
  data_name <- paste0(cache_name, "_data")
  time_name <- paste0(cache_name, "_time")

  if (!exists(data_name, envir = .sidra_cache) ||
      !exists(time_name, envir = .sidra_cache)) {
    return(FALSE)
  }

  # Check expiration if max_age_hours is specified

if (!is.null(max_age_hours)) {
    cache_time <- get(time_name, envir = .sidra_cache)
    age_hours <- as.numeric(difftime(Sys.time(), cache_time, units = "hours"))
    if (age_hours >= max_age_hours) {
      return(FALSE)
    }
  }

  TRUE
}

#' Get Cache Entry
#'
#' @param cache_name Name of the cached object
#' @return The cached data, or NULL if not found.
#' @keywords internal
#' @noRd
.get_cache <- function(cache_name) {
  data_name <- paste0(cache_name, "_data")
  if (exists(data_name, envir = .sidra_cache)) {
    data.table::copy(get(data_name, envir = .sidra_cache))
  } else {
    NULL
  }
}

#' Set Cache Entry
#'
#' @param cache_name Name of the cached object
#' @param data Data to cache
#' @keywords internal
#' @noRd
.set_cache <- function(cache_name, data) {
  data_name <- paste0(cache_name, "_data")
  time_name <- paste0(cache_name, "_time")
  assign(data_name, data.table::copy(data), envir = .sidra_cache)
  assign(time_name, Sys.time(), envir = .sidra_cache)
}

#' Get Cache Timestamp
#'
#' @param cache_name Name of the cached object
#' @return POSIXct timestamp or NULL if not cached.
#' @keywords internal
#' @noRd
.get_cache_time <- function(cache_name) {
  time_name <- paste0(cache_name, "_time")
  if (exists(time_name, envir = .sidra_cache)) {
    get(time_name, envir = .sidra_cache)
  } else {
    NULL
  }
}


#' Fetch Rolling Quarter Series from SIDRA
#'
#' Downloads PNADC labor market indicators from IBGE's SIDRA API. These series
#' are published as rolling quarterly averages (trimestre movel), with 12
#' observations per year.
#'
#' @param series Character vector of series names to fetch, or "all" (default)
#'   for all available series. Use \code{get_sidra_series_metadata()$series_name}
#'   to see available names.
#' @param theme Character vector of themes to filter by. Valid options:
#'   "labor_market", "earnings", "demographics", "social_protection", "prices".
#'   Use NULL for no filter.
#' @param theme_category Character vector of theme categories to filter by.
#'   Use NULL for no filter.
#' @param subcategory Character vector of subcategories to filter by.
#'   Use NULL for no filter.
#' @param exclude_derived Logical. If TRUE, exclude series marked as derived
#'   (is_derived = TRUE in metadata). Default FALSE for backward compatibility.
#'   Derived series (rates) are computed from other series during mensalization,
#'   so excluding them saves API calls when fetching for mensalization.
#' @param use_cache Logical. Use cached data if available? Default FALSE.
#'   When TRUE, shows the date when data was cached (may be outdated).
#'   Use \code{\link{clear_sidra_cache}} to force fresh download.
#' @param verbose Logical. Print progress messages? Default TRUE.
#' @param retry_failed Logical. Retry failed series downloads? Default TRUE.
#' @param max_retries Integer. Maximum retry attempts per series. Default 3.
#'
#' @return A data.table with columns:
#'   \describe{
#'     \item{anomesfinaltrimmovel}{Integer. YYYYMM of rolling quarter end month}
#'     \item{mesnotrim}{Integer. Month position in quarter (1, 2, or 3)}
#'     \item{<series_name>}{Numeric. One column per requested series}
#'   }
#'
#' @details
#' Rolling quarters are labeled by their ending month:
#' \itemize{
#'   \item 201201 = Nov 2011 - Jan 2012 (mesnotrim = 1)
#'   \item 201202 = Dec 2011 - Feb 2012 (mesnotrim = 2)
#'   \item 201203 = Jan - Mar 2012 (mesnotrim = 3)
#'   \item 201204 = Feb - Apr 2012 (mesnotrim = 1)
#'   \item etc.
#' }
#'
#' The \code{mesnotrim} column indicates the month's position within its rolling
#' quarter, which is essential for the mensalization algorithm.
#'
#' @section Rate Limiting:
#' SIDRA API may have rate limits. The function includes automatic retry logic
#' with exponential backoff for failed requests.
#'
#' @section Internet Resource Behaviour:
#' Per CRAN policy, this function fails gracefully when the SIDRA API is
#' unreachable: it emits an informative \code{message()} (no warning, no
#' error) and returns \code{NULL} invisibly when no series could be
#' fetched. Partial failures (some series succeeded) are also reported
#' via \code{message()}, and the result includes only the successful
#' series.
#'
#' @examples
#' \donttest{
#' rq <- fetch_sidra_rolling_quarters(
#'   series = c("taxadesocup", "popocup", "popdesocup")
#' )
#' head(rq)
#' }
#'
#' \donttest{
#' rq_labor <- fetch_sidra_rolling_quarters(theme = "labor_market")
#' }
#'
#' @seealso
#' \code{\link{get_sidra_series_metadata}} for available series names and metadata
#' \code{\link{mensalize_sidra_series}} to convert to exact months
#'
#' @export
fetch_sidra_rolling_quarters <- function(series = "all",
                                          theme = NULL,
                                          theme_category = NULL,
                                          subcategory = NULL,
                                          exclude_derived = FALSE,
                                          use_cache = FALSE,
                                          verbose = TRUE,
                                          retry_failed = TRUE,
                                          max_retries = 3) {

  # Get metadata for requested series
  meta <- get_sidra_series_metadata(
    series = series,
    theme = theme,
    theme_category = theme_category,
    subcategory = subcategory
  )

  # Optionally exclude derived series (rates computed from other series)
  if (exclude_derived && nrow(meta) > 0) {
    meta <- meta[is_derived == FALSE]
  }

  if (nrow(meta) == 0) {
    stop("No series found matching the specified criteria")
  }

  series_names <- meta$series_name

  # Check cache for complete dataset
  if (use_cache && .is_cache_valid("rolling_quarters")) {
    cached_data <- .get_cache("rolling_quarters")

    # Check if cached data contains all requested series
    cached_series <- setdiff(names(cached_data),
                              c("anomesfinaltrimmovel", "mesnotrim"))
    if (all(series_names %in% cached_series)) {
      # Get cache timestamp
      cache_time <- .get_cache_time("rolling_quarters")
      cache_date <- format(cache_time, "%Y-%m-%d %H:%M")

      if (verbose) {
        message("Using cached data from ", cache_date)
        message("  (Data may be outdated. Use use_cache=FALSE or clear_sidra_cache() to refresh)")
      }

      # Return only requested columns
      cols_to_keep <- c("anomesfinaltrimmovel", "mesnotrim", series_names)
      result <- data.table::copy(cached_data[, ..cols_to_keep])

      if (verbose) {
        message("  ", length(series_names), " series, ",
                nrow(result), " rolling quarters (",
                min(result$anomesfinaltrimmovel), " to ",
                max(result$anomesfinaltrimmovel), ")")
      }
      return(result)
    }
  }

  # Check for sidrar package
  if (!requireNamespace("sidrar", quietly = TRUE)) {
    stop(
      "Package 'sidrar' is required for fetching series from SIDRA.\n",
      "Install with: install.packages('sidrar')",
      call. = FALSE
    )
  }

  if (verbose) {
    message("Fetching ", nrow(meta), " series from SIDRA API...")
    message("This may take a few minutes on first run.")
  }

  # Initialize result with time index
  result <- NULL
  failed_series <- character(0)

  # Progress tracking
  n_series <- nrow(meta)

  for (i in seq_len(n_series)) {
    series_name <- meta$series_name[i]
    api_path <- meta$api_path[i]

    if (verbose) {
      pct <- round(100 * i / n_series)
      message(sprintf("  [%3d%%] Fetching %s...", pct, series_name))
    }

    # Fetch with retry logic.
    # CRAN policy: emit only message() (no warning/error) when the SIDRA
    # API is unreachable, so the function fails gracefully.
    series_data <- NULL
    for (attempt in seq_len(max_retries)) {
      series_data <- tryCatch({
        suppressMessages(sidrar::get_sidra(api = api_path))
      }, error = function(e) {
        if (attempt < max_retries && retry_failed) {
          # Exponential backoff
          Sys.sleep(2^attempt)
          NULL
        } else {
          message("Failed to fetch ", series_name, ": ", conditionMessage(e))
          NULL
        }
      })

      if (!is.null(series_data)) break
    }

    if (is.null(series_data)) {
      failed_series <- c(failed_series, series_name)
      next
    }

    # Process the response
    dt <- .process_sidra_response(series_data, series_name)

    if (is.null(dt)) {
      failed_series <- c(failed_series, series_name)
      next
    }

    # Merge with result
    if (is.null(result)) {
      result <- dt
    } else {
      result <- merge(result, dt, by = "anomesfinaltrimmovel", all = TRUE)
    }
  }

  # CRAN policy: when no series could be fetched, fail gracefully with an
  # informative message and return NULL invisibly (no warning/error).
  if (is.null(result)) {
    message(
      "fetch_sidra_rolling_quarters: no series could be fetched from the ",
      "SIDRA API. Check internet connection or try again later."
    )
    return(invisible(NULL))
  }

  # Add mesnotrim column (month position in quarter)
  result[, mesnotrim := .get_mesnotrim(anomesfinaltrimmovel %% 100L)]

  # Reorder columns
  data.table::setcolorder(result, c("anomesfinaltrimmovel", "mesnotrim"))

  # Sort by time
  data.table::setorder(result, anomesfinaltrimmovel)

  # Report partial failures via message() (not warning) per CRAN policy.
  if (length(failed_series) > 0) {
    message("Failed to fetch ", length(failed_series), " series: ",
            paste(failed_series, collapse = ", "))
  }

  # Always cache the result after fetching (use_cache only controls reading)
  .set_cache("rolling_quarters", result)

  if (verbose) {
    n_success <- length(series_names) - length(failed_series)
    message("Successfully fetched ", n_success, "/", length(series_names),
            " series")
    message("  Rolling quarters: ", min(result$anomesfinaltrimmovel),
            " to ", max(result$anomesfinaltrimmovel),
            " (", nrow(result), " observations)")
  }

  result
}


#' Process SIDRA API Response
#'
#' Internal function to extract the series values from a SIDRA API response.
#'
#' @param raw Raw data.frame from sidrar::get_sidra()
#' @param series_name Name to give the value column
#' @return data.table with anomesfinaltrimmovel and series value columns
#' @keywords internal
#' @noRd
.process_sidra_response <- function(raw, series_name) {

  if (is.null(raw) || nrow(raw) == 0) return(NULL)

  dt <- data.table::as.data.table(raw)

  # Find the period code column (handles various SIDRA naming patterns)
  # For PNADC tables, this is typically "Trimestre Movel (Codigo)"
  code_col <- NULL

  # Try various patterns
  patterns <- c(
    "Trimestre.*M.*vel.*C.*digo",
    "trimestre.*m.*vel.*c.*digo",
    "M.*vel.*C.*digo",
    "Trimestre.*C.*digo"
  )

  for (pattern in patterns) {
    matches <- grep(pattern, names(dt), value = TRUE, ignore.case = TRUE)
    if (length(matches) > 0) {
      code_col <- matches[1]
      break
    }
  }

  # For price indices, the period might be different
  if (is.null(code_col)) {
    # Try "Mes (Codigo)" for monthly series like IPCA
    matches <- grep("M.*s.*C.*digo|Month.*Code", names(dt),
                    value = TRUE, ignore.case = TRUE)
    if (length(matches) > 0) {
      code_col <- matches[1]
    }
  }

  if (is.null(code_col)) {
    warning("Could not find period code column for series: ", series_name)
    return(NULL)
  }

  # Extract values
  result <- dt[, .(
    anomesfinaltrimmovel = as.integer(get(code_col)),
    value = as.numeric(Valor)
  )]

  # Remove NA codes
  result <- result[!is.na(anomesfinaltrimmovel)]

  # Rename value column to series name
  data.table::setnames(result, "value", series_name)

  result
}

# Tests for fetch-sidra-series.R
# Covers: SIDRA cache infrastructure, .process_sidra_response(), metadata filtering,
# and integration tests for fetch_sidra_rolling_quarters()

# =============================================================================
# CACHE INFRASTRUCTURE TESTS (all offline)
# =============================================================================

test_that(".set_cache stores data and timestamp", {
  # 1. Setup: Clear cache to ensure clean state
  PNADCperiods:::clear_sidra_cache()

  # 2. Execute: Store a data.table in cache
  test_dt <- data.table::data.table(
    anomesfinaltrimmovel = c(202301L, 202302L, 202303L),
    popocup = c(100.5, 101.2, 102.0)
  )
  PNADCperiods:::.set_cache("test_series", test_dt)

  # 3. Verify: Data and timestamp exist in cache environment
  cache_env <- PNADCperiods:::.sidra_cache
  expect_true(exists("test_series_data", envir = cache_env))
  expect_true(exists("test_series_time", envir = cache_env))

  # Stored data should match original
  stored <- get("test_series_data", envir = cache_env)
  expect_equal(nrow(stored), 3)
  expect_equal(stored$popocup, c(100.5, 101.2, 102.0))

  # Timestamp should be recent (within last 5 seconds)
  stored_time <- get("test_series_time", envir = cache_env)
  expect_s3_class(stored_time, "POSIXct")
  expect_true(difftime(Sys.time(), stored_time, units = "secs") < 5)

  # 4. Cleanup
  PNADCperiods:::clear_sidra_cache()
})


test_that(".get_cache returns a copy, not a reference, of cached data", {
  # 1. Setup: Clear cache and store data
  PNADCperiods:::clear_sidra_cache()

  original_dt <- data.table::data.table(
    anomesfinaltrimmovel = c(202301L, 202302L),
    valor = c(50.0, 60.0)
  )
  PNADCperiods:::.set_cache("copy_test", original_dt)

  # 2. Execute: Retrieve the cached data
  retrieved <- PNADCperiods:::.get_cache("copy_test")

  # 3. Verify: Modify the retrieved copy and check original is unchanged
  expect_s3_class(retrieved, "data.table")
  retrieved[, valor := valor * 100]

  # Get a fresh copy from cache to verify original is untouched
  fresh_copy <- PNADCperiods:::.get_cache("copy_test")
  expect_equal(fresh_copy$valor, c(50.0, 60.0))

  # Also verify the internal env data is unchanged
  cache_env <- PNADCperiods:::.sidra_cache
  internal <- get("copy_test_data", envir = cache_env)
  expect_equal(internal$valor, c(50.0, 60.0))

  # 4. Cleanup
  PNADCperiods:::clear_sidra_cache()
})


test_that(".get_cache returns NULL when cache is empty", {
  # 1. Setup: Clear cache
  PNADCperiods:::clear_sidra_cache()

  # 2. Execute: Try to retrieve non-existent cache entry
  result <- PNADCperiods:::.get_cache("nonexistent_cache_key")

  # 3. Verify: Should return NULL

  expect_null(result)
})


test_that(".is_cache_valid returns TRUE for fresh cache", {
  # 1. Setup: Clear cache and store fresh data
  PNADCperiods:::clear_sidra_cache()

  test_dt <- data.table::data.table(x = 1:3)
  PNADCperiods:::.set_cache("validity_test", test_dt)

  # 2. Execute & Verify: Fresh cache with no expiration should be valid
  expect_true(PNADCperiods:::.is_cache_valid("validity_test"))

  # Also valid with generous max_age_hours
  expect_true(PNADCperiods:::.is_cache_valid("validity_test", max_age_hours = 24))

  # 3. Cleanup
  PNADCperiods:::clear_sidra_cache()
})


test_that(".is_cache_valid returns FALSE for empty cache", {
  # 1. Setup: Clear cache
  PNADCperiods:::clear_sidra_cache()

  # 2. Execute & Verify: Non-existent entry should not be valid
  expect_false(PNADCperiods:::.is_cache_valid("does_not_exist"))

  # Also FALSE with explicit max_age_hours
  expect_false(PNADCperiods:::.is_cache_valid("does_not_exist", max_age_hours = 24))
})


test_that(".is_cache_valid respects max_age_hours for expired cache", {
  # 1. Setup: Clear cache and store data with a backdated timestamp
  PNADCperiods:::clear_sidra_cache()

  test_dt <- data.table::data.table(x = 1:3)
  PNADCperiods:::.set_cache("expiry_test", test_dt)

  # Manually backdate the timestamp by 3 hours
  cache_env <- PNADCperiods:::.sidra_cache
  assign("expiry_test_time", Sys.time() - 3 * 3600, envir = cache_env)

  # 2. Execute & Verify: Should be invalid if max_age is 1 hour
  expect_false(PNADCperiods:::.is_cache_valid("expiry_test", max_age_hours = 1))

  # But should still be valid if max_age is 24 hours
  expect_true(PNADCperiods:::.is_cache_valid("expiry_test", max_age_hours = 24))

  # And valid with NULL max_age (no expiration)
  expect_true(PNADCperiods:::.is_cache_valid("expiry_test", max_age_hours = NULL))

  # 3. Cleanup
  PNADCperiods:::clear_sidra_cache()
})


test_that(".get_cache_time returns POSIXct timestamp after .set_cache", {
  # 1. Setup: Clear cache and store data
  PNADCperiods:::clear_sidra_cache()

  before <- Sys.time()
  test_dt <- data.table::data.table(x = 1:3)
  PNADCperiods:::.set_cache("time_test", test_dt)
  after <- Sys.time()

  # 2. Execute: Retrieve the cache time
  cache_time <- PNADCperiods:::.get_cache_time("time_test")

  # 3. Verify: Should be a POSIXct between before and after
  expect_s3_class(cache_time, "POSIXct")
  expect_true(cache_time >= before)
  expect_true(cache_time <= after)

  # 4. Cleanup
  PNADCperiods:::clear_sidra_cache()
})


test_that(".get_cache_time returns NULL when not cached", {
  # 1. Setup: Clear cache
  PNADCperiods:::clear_sidra_cache()

  # 2. Execute & Verify: No timestamp for non-existent entry
  result <- PNADCperiods:::.get_cache_time("never_cached")
  expect_null(result)
})


test_that("clear_sidra_cache returns TRUE when cache had data", {
  # 1. Setup: Store something in the cache
  PNADCperiods:::clear_sidra_cache()

  test_dt <- data.table::data.table(x = 1:3)
  PNADCperiods:::.set_cache("clearable", test_dt)

  # 2. Execute: Clear the cache
  result <- PNADCperiods:::clear_sidra_cache()

  # 3. Verify: Should return TRUE (had data to clear)
  expect_true(result)

  # And cache should actually be empty now
  cache_env <- PNADCperiods:::.sidra_cache
  expect_equal(length(ls(envir = cache_env)), 0)
})


test_that("clear_sidra_cache returns FALSE when already empty", {
  # 1. Setup: Ensure cache is empty
  PNADCperiods:::clear_sidra_cache()

  # 2. Execute: Clear again
  result <- PNADCperiods:::clear_sidra_cache()

  # 3. Verify: Should return FALSE (nothing to clear)
  expect_false(result)
})


# =============================================================================
# .process_sidra_response() TESTS (all offline, mock data.frames)
# =============================================================================

test_that(".process_sidra_response extracts values from standard response", {
  # 1. Setup: Create a mock SIDRA API response with typical column names
  mock_response <- data.frame(
    `Trimestre Movel (Codigo)` = c("202301", "202302", "202303"),
    `Trimestre Movel` = c("Jan-Fev-Mar 2023", "Fev-Mar-Abr 2023", "Mar-Abr-Mai 2023"),
    `Valor` = c("100.5", "101.2", "102.0"),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # 2. Execute: Process the response
  result <- PNADCperiods:::.process_sidra_response(mock_response, "test_series")

  # 3. Verify: Should produce a data.table with correct structure
  expect_s3_class(result, "data.table")
  expect_true("anomesfinaltrimmovel" %in% names(result))
  expect_true("test_series" %in% names(result))
  expect_equal(nrow(result), 3)

  # Values should be correctly extracted
  expect_equal(result$anomesfinaltrimmovel, c(202301L, 202302L, 202303L))
  expect_equal(result$test_series, c(100.5, 101.2, 102.0))
})


test_that(".process_sidra_response returns NULL with warning for missing code column", {
  # 1. Setup: Create response without the period code column
  mock_response <- data.frame(
    `Some Other Column` = c("A", "B", "C"),
    `Valor` = c("100.5", "101.2", "102.0"),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # 2. Execute & Verify: Should return NULL and produce a warning
  expect_warning(
    result <- PNADCperiods:::.process_sidra_response(mock_response, "bad_series"),
    "Could not find period code column"
  )
  expect_null(result)
})


test_that(".process_sidra_response returns NULL for empty response", {
  # 1. Setup: Create an empty data.frame
  mock_empty <- data.frame(
    `Trimestre Movel (Codigo)` = character(0),
    `Valor` = character(0),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # 2. Execute: Process empty response
  result <- PNADCperiods:::.process_sidra_response(mock_empty, "empty_series")

  # 3. Verify: Should return NULL (0 rows)
  expect_null(result)
})


test_that(".process_sidra_response returns NULL for NULL input", {
  # 1. Execute & Verify: NULL input should return NULL
  result <- PNADCperiods:::.process_sidra_response(NULL, "null_series")
  expect_null(result)
})


test_that(".process_sidra_response handles NA codes by removing them", {
  # 1. Setup: Response with some NA period codes
  mock_response <- data.frame(
    `Trimestre Movel (Codigo)` = c("202301", NA, "202303"),
    `Valor` = c("100.5", "101.2", "102.0"),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # 2. Execute: Process the response
  result <- PNADCperiods:::.process_sidra_response(mock_response, "na_test")

  # 3. Verify: Should have 2 rows (NA code row removed)
  expect_s3_class(result, "data.table")
  expect_equal(nrow(result), 2)
  expect_equal(result$anomesfinaltrimmovel, c(202301L, 202303L))
})


# =============================================================================
# METADATA FILTERING TESTS (offline)
# =============================================================================

test_that("theme parameter filters series correctly", {
  # 1. Execute: Get labor_market series
  labor_meta <- get_sidra_series_metadata(theme = "labor_market")

  # 2. Verify: All returned series should have labor_market theme
  expect_true(nrow(labor_meta) > 0)
  expect_true(all(labor_meta$theme == "labor_market"))

  # Cross-check: earnings theme should be different
  earnings_meta <- get_sidra_series_metadata(theme = "earnings")
  expect_true(nrow(earnings_meta) > 0)
  expect_true(all(earnings_meta$theme == "earnings"))

  # No overlap in series names between themes
  expect_length(intersect(labor_meta$series_name, earnings_meta$series_name), 0)
})


test_that("exclude_derived logic correctly filters derived series from metadata", {
  # 1. Execute: Get all metadata and identify derived series
  all_meta <- get_sidra_series_metadata()
  derived_names <- all_meta[is_derived == TRUE, series_name]
  non_derived_names <- all_meta[is_derived == FALSE, series_name]

  # 2. Verify: There should be both derived and non-derived series
  expect_true(length(derived_names) > 0)
  expect_true(length(non_derived_names) > 0)

  # Verify that filtering works as expected for fetch_sidra_rolling_quarters
  # (we test the metadata filtering logic, not the full API call)
  filtered_meta <- all_meta[is_derived == FALSE]
  expect_equal(nrow(filtered_meta), length(non_derived_names))

  # None of the derived series should remain
  expect_false(any(filtered_meta$series_name %in% derived_names))

  # Known derived series: taxadesocup, taxapartic, etc. should be marked
  expect_true("taxadesocup" %in% derived_names)
  expect_true("taxapartic" %in% derived_names)

  # Known non-derived series: popocup, popdesocup should not be marked
  expect_true("popocup" %in% non_derived_names)
  expect_true("popdesocup" %in% non_derived_names)
})


test_that("get_sidra_series_metadata returns expected columns and types", {
  # 1. Execute: Get full metadata
  meta <- get_sidra_series_metadata()

  # 2. Verify: Required columns exist
  expected_cols <- c("series_name", "api_path", "table_id", "variable_id",
                     "theme", "theme_category", "description",
                     "unit", "is_derived", "requires_deflation")
  for (col in expected_cols) {
    expect_true(col %in% names(meta), info = paste("Missing column:", col))
  }

  # Types should be correct
  expect_type(meta$series_name, "character")
  expect_type(meta$api_path, "character")
  expect_type(meta$table_id, "integer")
  expect_type(meta$is_derived, "logical")

  # Should have a reasonable number of series (86+ as documented)
  expect_true(nrow(meta) >= 80)

  # No duplicate series names
  expect_equal(length(unique(meta$series_name)), nrow(meta))
})


# =============================================================================
# .get_mesnotrim() HELPER TESTS
# =============================================================================

test_that(".get_mesnotrim returns correct month position in quarter", {
  # Position 1: Jan, Apr, Jul, Oct
  expect_equal(PNADCperiods:::.get_mesnotrim(1), 1)
  expect_equal(PNADCperiods:::.get_mesnotrim(4), 1)
  expect_equal(PNADCperiods:::.get_mesnotrim(7), 1)
  expect_equal(PNADCperiods:::.get_mesnotrim(10), 1)

  # Position 2: Feb, May, Aug, Nov
  expect_equal(PNADCperiods:::.get_mesnotrim(2), 2)
  expect_equal(PNADCperiods:::.get_mesnotrim(5), 2)
  expect_equal(PNADCperiods:::.get_mesnotrim(8), 2)
  expect_equal(PNADCperiods:::.get_mesnotrim(11), 2)

  # Position 3: Mar, Jun, Sep, Dec
  expect_equal(PNADCperiods:::.get_mesnotrim(3), 3)
  expect_equal(PNADCperiods:::.get_mesnotrim(6), 3)
  expect_equal(PNADCperiods:::.get_mesnotrim(9), 3)
  expect_equal(PNADCperiods:::.get_mesnotrim(12), 3)
})


# =============================================================================
# CRAN POLICY: Internet resources must fail gracefully
# (no warning, no error). Mocked test runs offline.
# =============================================================================

test_that("fetch_sidra_rolling_quarters fails gracefully when SIDRA is unreachable", {
  skip_if_not_installed("sidrar")
  skip_if_not_installed("testthat", "3.0.0")

  # Simulate API down: any call to sidrar::get_sidra throws.
  testthat::local_mocked_bindings(
    get_sidra = function(...) stop("simulated network error"),
    .package = "sidrar"
  )

  # max_retries = 1, retry_failed = FALSE so we hit the message branch fast.
  expect_message(
    result <- fetch_sidra_rolling_quarters(
      series = "taxadesocup",
      use_cache = FALSE,
      verbose = FALSE,
      max_retries = 1L,
      retry_failed = FALSE
    ),
    "no series could be fetched"
  )
  expect_null(result)
})


# =============================================================================
# INTEGRATION TEST (requires internet + sidrar)
# =============================================================================

test_that("fetch_sidra_rolling_quarters fetches real data from API", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  # 1. Setup: Clear cache to ensure fresh fetch
  clear_sidra_cache()

  # 2. Execute: Fetch a single small series to minimize API load
  result <- tryCatch(
    fetch_sidra_rolling_quarters(
      series = "taxadesocup",
      use_cache = FALSE,
      verbose = FALSE,
      max_retries = 2
    ),
    error = function(e) {
      skip(paste("SIDRA API unavailable:", conditionMessage(e)))
    }
  )

  # 3. Verify: Should be a data.table with expected structure
  expect_s3_class(result, "data.table")
  expect_true("anomesfinaltrimmovel" %in% names(result))
  expect_true("mesnotrim" %in% names(result))
  expect_true("taxadesocup" %in% names(result))

  # Should have many observations (PNADC starts 2012)
  expect_true(nrow(result) > 100)

  # anomesfinaltrimmovel should be valid YYYYMM integers
  expect_true(all(result$anomesfinaltrimmovel >= 201201L))
  expect_true(all(result$anomesfinaltrimmovel %% 100L >= 1))
  expect_true(all(result$anomesfinaltrimmovel %% 100L <= 12))

  # mesnotrim should be 1, 2, or 3
  expect_true(all(result$mesnotrim %in% c(1L, 2L, 3L)))

  # taxadesocup should be a reasonable unemployment rate (0-50%)
  valid_values <- result$taxadesocup[!is.na(result$taxadesocup)]
  expect_true(length(valid_values) > 0)
  expect_true(all(valid_values > 0 & valid_values < 50))

  # Data should be sorted by time
  expect_equal(result$anomesfinaltrimmovel,
               sort(result$anomesfinaltrimmovel))

  # 4. Cleanup
  clear_sidra_cache()
})

#' Apply Reference Period Crosswalk to PNADC Data
#'
#' Merges a reference period crosswalk with PNADC microdata and optionally
#' calibrates survey weights for sub-quarterly analysis.
#'
#' @description
#' This function takes a crosswalk from \code{\link{pnadc_identify_periods}} and
#' applies it to any PNADC dataset (quarterly or annual). It can optionally
#' calibrate the survey weights to match external population totals at the
#' chosen temporal granularity (month, fortnight, or week).
#'
#' @param data A data.frame or data.table with PNADC microdata. Must contain
#'   join keys \code{Ano}, \code{Trimestre}, \code{UPA}, \code{V1008}, and
#'   \code{V1014} to merge with the crosswalk.
#' @param crosswalk A data.table crosswalk from \code{\link{pnadc_identify_periods}}.
#' @param weight_var Character. Name of the survey weight column. Must be specified:
#'   \itemize{
#'     \item \code{"V1028"} for quarterly PNADC data
#'     \item \code{"V1032"} for annual PNADC data (visit-specific or annual releases organized by quarters)
#'   }
#' @param anchor Character. How to anchor the weight redistribution. Must be specified:
#'   \itemize{
#'     \item \code{"quarter"} for quarterly data or annual releases organized by quarters (preserves quarterly totals)
#'     \item \code{"year"} for annual visit-specific data (preserves yearly totals)
#'   }
#' @param calibrate Logical. If TRUE (default), calibrate weights to external
#'   population totals. If FALSE, only merge the crosswalk without calibration.
#' @param calibration_unit Character. Temporal unit for weight calibration.
#'   One of \code{"month"} (default), \code{"fortnight"}, or \code{"week"}.
#' @param calibration_min_cell_size Integer. Minimum sample size required in a cell
#'   for it to be used in hierarchical raking. Cells smaller than this threshold
#'   are collapsed to coarser levels. Default: 1 (use all cells).
#' @param target_totals Optional data.table with population targets. If NULL
#'   (default), fetches monthly population from SIDRA and derives targets for
#'   fortnight/week. Each time period (month, fortnight, or week) is calibrated
#'   to the FULL Brazilian population from SIDRA.
#'
#'   If providing custom targets, the population column (\code{m_populacao} for
#'   months, \code{f_populacao} for fortnights, \code{w_populacao} for weeks)
#'   must be in **thousands**. The function multiplies by 1000 internally.
#' @param smooth Logical. If TRUE, smooth calibrated weights to
#'   remove quarterly artifacts. Smoothing is adapted per time period:
#'   monthly (3-period window), fortnight (7-period window), weekly (no smoothing).
#'   Default: FALSE.
#' @param keep_all Logical. If TRUE (default), keep all observations including
#'   those with undetermined reference periods. If FALSE, drop undetermined rows.
#' @param verbose Logical. If TRUE (default), print progress messages.
#'
#' @return A data.table with the input data plus crosswalk columns:
#'   \describe{
#'     \item{ref_month_in_quarter, ref_month_in_year}{Month position (1-3 in quarter, 1-12 in year)}
#'     \item{ref_fortnight_in_month, ref_fortnight_in_quarter}{Fortnight position (1-2 in month, 1-6 in quarter)}
#'     \item{ref_week_in_month, ref_week_in_quarter}{Week position (1-4 in month, 1-12 in quarter)}
#'     \item{ref_month_yyyymm, ref_fortnight_yyyyff, ref_week_yyyyww}{Integer period codes}
#'     \item{determined_month, determined_fortnight, determined_week}{Logical determination flags}
#'     \item{weight_monthly, weight_fortnight, or weight_weekly}{Calibrated weights (if calibrate=TRUE)}
#'   }
#'
#' @details
#' ## Weight Calibration
#'
#' When \code{calibrate = TRUE}, the function performs hierarchical rake weighting:
#' \enumerate{
#'   \item Groups observations by nested demographic/geographic cells
#'   \item Iteratively adjusts weights so sub-period totals match anchor-period totals
#'   \item Calibrates final weights against external population totals (FULL Brazilian population)
#'   \item Optionally smooths weights to remove quarterly artifacts
#' }
#'
#' ## Population Targets
#'
#' All time periods (months, fortnights, and weeks) are calibrated to the FULL
#' Brazilian population from SIDRA. This means:
#' \itemize{
#'   \item Monthly weights sum to the Brazilian population for that month
#'   \item Fortnight weights sum to the Brazilian population for the containing month
#'   \item Weekly weights sum to the Brazilian population for the containing month
#' }
#'
#' ## Hierarchical Raking Levels
#'
#' The number of hierarchical cell levels is automatically adjusted based on the
#' calibration unit to avoid sparse cell issues:
#' \itemize{
#'   \item \code{"month"}: 4 levels (age, region, state, post-stratum) - full hierarchy
#'   \item \code{"fortnight"}: 2 levels (age, region) - simplified for lower sample size
#'   \item \code{"week"}: 1 level (age groups only) - minimal hierarchy for sparse data
#' }
#'
#' ## Anchor Period
#'
#' The \code{anchor} parameter determines how weights are redistributed:
#' \itemize{
#'   \item \code{"quarter"}: Quarterly totals are preserved and redistributed to months/fortnights/weeks
#'   \item \code{"year"}: Yearly totals are preserved and redistributed to months/fortnights/weeks
#' }
#'
#' Use \code{anchor = "quarter"} with quarterly V1028 weights, and
#' \code{anchor = "year"} with annual V1032 weights.
#'
#' @examples
#' \dontrun{
#' crosswalk <- pnadc_identify_periods(pnadc_stacked)
#'
#' result <- pnadc_apply_periods(
#'   pnadc_2023,
#'   crosswalk,
#'   weight_var = "V1028",
#'   anchor = "quarter"
#' )
#'
#' result <- pnadc_apply_periods(
#'   pnadc_annual,
#'   crosswalk,
#'   weight_var = "V1032",
#'   anchor = "year"
#' )
#'
#' result <- pnadc_apply_periods(
#'   pnadc_2023,
#'   crosswalk,
#'   weight_var = "V1028",
#'   anchor = "quarter",
#'   calibration_unit = "week"
#' )
#'
#' result <- pnadc_apply_periods(
#'   pnadc_2023,
#'   crosswalk,
#'   weight_var = "V1028",
#'   anchor = "quarter",
#'   calibrate = FALSE
#' )
#' }
#'
#' @seealso \code{\link{pnadc_identify_periods}} to build the crosswalk
#'
#' @export
pnadc_apply_periods <- function(data,
                                crosswalk,
                                weight_var,
                                anchor,
                                calibrate = TRUE,
                                calibration_unit = c("month", "fortnight", "week"),
                                calibration_min_cell_size = 1,
                                target_totals = NULL,
                                smooth = FALSE,
                                keep_all = TRUE,
                                verbose = TRUE) {

  # ============================================================================
  # INPUT VALIDATION
  # ============================================================================

  # Validate required arguments (no defaults)
  if (missing(weight_var)) {
    stop("'weight_var' must be specified: \"V1028\" for quarterly, \"V1032\" for annual")
  }
  if (missing(anchor)) {
    stop("'anchor' must be specified: \"quarter\" for quarterly data, \"year\" for annual data")
  }
  if (!is.character(weight_var) || length(weight_var) != 1L) {
    stop("'weight_var' must be a single character string")
  }
  if (!anchor %in% c("quarter", "year")) {
    stop("'anchor' must be either \"quarter\" or \"year\"")
  }
  if (!is.logical(calibrate) || length(calibrate) != 1L) {
    stop("'calibrate' must be TRUE or FALSE")
  }
  calibration_unit <- match.arg(calibration_unit)
  if (!is.logical(smooth) || length(smooth) != 1L) {
    stop("'smooth' must be TRUE or FALSE")
  }
  if (!is.logical(keep_all) || length(keep_all) != 1L) {
    stop("'keep_all' must be TRUE or FALSE")
  }
  if (!is.logical(verbose) || length(verbose) != 1L) {
    stop("'verbose' must be TRUE or FALSE")
  }

  # Cache column names for repeated checks
  dt_names <- names(data)
  xw_names <- names(crosswalk)

  join_keys <- c("Ano", "Trimestre", "UPA", "V1008", "V1014")

  # Check join keys exist
  # Crosswalk must have ALL join keys (it's generated by pnadc_identify_periods which always includes them)
  missing_in_xw <- setdiff(join_keys, xw_names)
  if (length(missing_in_xw) > 0) {
    stop(sprintf("Crosswalk missing required columns: %s. ",
                 paste(missing_in_xw, collapse = ", ")),
         "The crosswalk should be created by pnadc_identify_periods() which always includes all keys.")
  }

  # Data must also have all join keys for proper merging
  missing_in_data <- setdiff(join_keys, dt_names)
  if (length(missing_in_data) > 0) {
    stop(sprintf("Data missing required join key columns: %s. ",
                 paste(missing_in_data, collapse = ", ")),
         "Required: Ano, Trimestre, UPA, V1008, V1014")
  }

  # Check weight variable exists
  if (!weight_var %in% dt_names) {
    stop(sprintf("Weight variable '%s' not found in data", weight_var))
  }

  # Convert to data.table
  dt <- ensure_data_table(data, copy = FALSE)
  xw <- ensure_data_table(crosswalk, copy = FALSE)  # No copy needed - only read

  # Freeing memory
  gc()

  # Ensure consistent types for join keys
  for (col in join_keys) {
    if (col %in% dt_names && col %in% xw_names) {

      # Coerce both to integer for consistent joins
      if (!is.integer(dt[[col]])) {
        data.table::set(dt, j = col, value = as.integer(dt[[col]]))
      }

      # Coerce crosswalk if needed
      if (!is.integer(xw[[col]])) {
        data.table::set(xw, j = col, value = as.integer(xw[[col]]))
      }
    }
  }

  # Freeing memory
  gc()



  # ============================================================================
  # STEP 1: Merge crosswalk with data
  # ============================================================================

  if (verbose) cat("Applying reference period crosswalk...\n")

  # Select crosswalk columns to merge
  xw_cols <- intersect(
    c("Ano",
      "Trimestre",
      "UPA",
      "V1008",
      "V1014",
      "ref_month_in_quarter",
      "ref_month_in_year",
      "ref_fortnight_in_month",
      "ref_fortnight_in_quarter",
      "ref_week_in_month",
      "ref_week_in_quarter",
      "ref_month_yyyymm",
      "ref_fortnight_yyyyff",
      "ref_week_yyyyww",
      "determined_month",
      "determined_fortnight",
      "determined_week"),
    xw_names
  )

  # Removing unecessary repetitions
  xw_small = unique(xw[, ..xw_cols])

  # Freeing memory
  rm(xw);gc()

  # Set keys before merge
  data.table::setkeyv(dt, join_keys)
  data.table::setkeyv(xw_small, join_keys)

  # Merge (keyed merge is faster)
  dt <- merge(dt, xw_small, by = join_keys, all.x = TRUE)

  n_matched <- sum(dt$determined_month == TRUE)
  if (verbose) {
    cat(sprintf("  Matched %s of %s observations (%.1f%%)\n",
                format(n_matched, big.mark = ","),
                format(nrow(dt), big.mark = ","),
                n_matched / nrow(dt) * 100))
  }

  # ============================================================================
  # STEP 2: Calibrate weights (if requested)
  # ============================================================================

  if (calibrate) {
    if (verbose) cat(sprintf("  Calibrating %s weights (anchor: %s)...\n",
                             calibration_unit, anchor))

    # Check for calibration columns and warn if missing
    dt_names <- names(dt)  # Update after merge
    calib_cols <- c("V2009", "UF", "posest", "posest_sxi")
    missing_calib <- setdiff(calib_cols, dt_names)
    if (length(missing_calib) > 0 && verbose) {
      warning("Missing calibration columns: ", paste(missing_calib, collapse = ", "), ". ",
              "Using simplified cell hierarchy for weight calibration. ",
              "For best results, ensure data includes V2009 (age), UF, posest, and posest_sxi.",
              call. = FALSE)
    }

    # Determine ref_var based on calibration_unit
    ref_var <- switch(calibration_unit,
                      month     = "ref_month_yyyymm",
                      fortnight = "ref_fortnight_yyyyff",
                      week      = "ref_week_yyyyww")

    determined_var <- switch(calibration_unit,
                             month = "determined_month",
                             fortnight = "determined_fortnight",
                             week = "determined_week")

    weight_out_var <- switch(calibration_unit,
                             month = "weight_monthly",
                             fortnight = "weight_fortnight",
                             week = "weight_weekly")

    # Get or fetch population targets.
    # If SIDRA is unreachable, fetch_monthly_population() returns NULL
    # invisibly (per CRAN policy on Internet resources). In that case we
    # cannot proceed with calibrate = TRUE; surface an informative message
    # and return the un-calibrated data.table instead of erroring.
    if (is.null(target_totals)) {
      if (verbose) cat("    Fetching population targets from SIDRA...\n")
      monthly_pop <- fetch_monthly_population(verbose = FALSE)

      if (is.null(monthly_pop)) {
        message(
          "pnadc_apply_periods: SIDRA population targets are unavailable. ",
          "Returning data with crosswalk applied but without calibrated ",
          "weights. Pass `target_totals` explicitly or retry when the API ",
          "is reachable."
        )
        if (verbose) cat("Done.\n")
        return(dt)
      }

      target_totals <- switch(calibration_unit,
                              month = monthly_pop,
                              fortnight = derive_fortnight_population(monthly_pop),
                              week      = derive_weekly_population(monthly_pop))
    }

    # Run unified calibration
    dt <- calibrate_weights_internal(
      dt,
      weight_var = weight_var,
      ref_var = ref_var,
      anchor = anchor,
      target_totals = target_totals,
      smooth = smooth,
      keep_all = keep_all,
      min_cell_size = calibration_min_cell_size,
      verbose = verbose
    )

    # Rename output weight column
    if ("weight_calibrated" %in% names(dt)) {
      data.table::setnames(dt, "weight_calibrated", weight_out_var)
    }

    if (verbose) {
      n_calibrated <- sum(!is.na(dt[[weight_out_var]]))
      cat(sprintf("    Calibrated %s observations\n",
                  format(n_calibrated, big.mark = ",")))
    }
  }

  # ============================================================================
  # STEP 3: Filter if not keeping all
  # ============================================================================

  if (!keep_all) {
    determined_var <- switch(calibration_unit,
                             month = "determined_month",
                             fortnight = "determined_fortnight",
                             week = "determined_week")
    if (determined_var %in% names(dt)) {

      keep_rows <- dt[[determined_var]] == TRUE
      dt <- dt[keep_rows]
    }
  }

  # Freeing memory
  gc()

  if (verbose) cat("Done.\n")

  dt
}


#' Internal Unified Calibration Function
#'
#' Performs hierarchical rake weighting for any anchor/unit combination.
#' The number of hierarchical cell levels is automatically adjusted based on
#' the time period granularity to avoid sparse cell issues.
#'
#' @param dt data.table with PNADC data and reference period columns
#' @param weight_var Name of input weight column
#' @param ref_var Name of reference period column (ref_month_yyyymm, etc.)
#' @param anchor "quarter" or "year"
#' @param target_totals Population targets data.table
#' @param n_cells Integer (1-4) or NULL. Number of hierarchical cell levels.
#'   If NULL (default), auto-selects based on ref_var:
#'   - month: 4 levels (full hierarchy)
#'   - fortnight: 2 levels (age + region)
#'   - week: 1 level (age only)
#' @param min_cell_size Minimum observations per cell. Levels with smaller
#'   cells are skipped. Default 1.
#' @param smooth Apply smoothing?
#' @param keep_all Keep undetermined observations?
#' @param verbose Print progress?
#' @return data.table with weight_calibrated column
#' @keywords internal
#' @noRd

#dt
#weight_var = weight_var
#ref_var = ref_var
#anchor = anchor
#target_totals = target_totals
#n_cells = NULL
#min_cell_size = 10L
#smooth = smooth
#keep_all = keep_all
#verbose = verbose

calibrate_weights_internal <- function(dt,
                                       weight_var,
                                       ref_var,
                                       anchor,
                                       target_totals,
                                       n_cells = NULL,
                                       min_cell_size = 1L,
                                       smooth = FALSE,
                                       keep_all = TRUE,
                                       verbose = FALSE) {

  # Pre-extract ref column for direct access
  ref_col <- dt[[ref_var]]

  # Determine anchor grouping variable
  anchor_vars <- if (anchor == "quarter") {
    c("Ano", "Trimestre")
  } else {
    # For annual anchor, extract year using integer arithmetic
    # ref_var is YYYYMM/YYYYFF/YYYYWW format, so %/% 100 gives YYYY
    data.table::set(dt, j = "anchor_year", value = ref_col %/% 100L)
    "anchor_year"
  }

  # Create integer anchor key for efficient uniqueN

  if (length(anchor_vars) == 2L) {
    # Quarterly: create composite key from Ano and Trimestre
    data.table::set(dt, j = ".anchor_key", value = dt[["Ano"]] * 10L + dt[["Trimestre"]])
  } else {
    # Annual: use anchor_year directly
    data.table::set(dt, j = ".anchor_key", value = dt[["anchor_year"]])
  }

  # Mark determined observations
  is_determined <- !is.na(ref_col)
  data.table::set(dt, j = ".is_determined", value = is_determined)

  n_determined <- sum(is_determined)
  if (n_determined == 0L) {
    if (keep_all) {
      dt[, weight_calibrated := NA_real_]
      dt[, c(".is_determined", ".anchor_key") := NULL]
      if ("anchor_year" %in% names(dt)) dt[, anchor_year := NULL]
      return(dt)
    }
    warning("No observations with determined reference period")
    dt[, c(".is_determined", ".anchor_key") := NULL]
    if ("anchor_year" %in% names(dt)) dt[, anchor_year := NULL]
    return(dt[0])  # Return empty dt
  }

  # Auto-select number of cell levels based on time period granularity
  if (is.null(n_cells)) {
    #n_cells <- if (grepl("week", ref_var, ignore.case = TRUE)) {
    #  1L  # Weekly: use age groups only (celula1)
    #} else if (grepl("fortnight", ref_var, ignore.case = TRUE)) {
    #  2L  # Fortnight: use age + region (celula1, celula2)
    #} else {
    #  4L  # Monthly: use full hierarchy (celula1-4)
    #}

    n_cells <- 4

    if (verbose) {
      #cat(sprintf("    Auto-selected %d cell level(s) for %s calibration\n",
      #            n_cells, ref_var))
    }
  } else {
    if (!n_cells %in% 1L:4L) {
      stop("'n_cells' must be an integer between 1 and 4")
    }
  }

  # Initialize working weight using direct column copy
  data.table::set(dt, j = "weight_current", value = dt[[weight_var]])

  # Step 1: Create calibration cells (only create needed levels)
  dt <- create_calibration_cells_unified(dt, n_cells = n_cells)

  # Pre-extract weight column for reweighting loop
  weight_vec <- dt[[weight_var]]

  # Step 2: Iterative hierarchical reweighting with cell size checking
  levels_applied <- 0L
  for (level in seq_len(n_cells)) {
    cell_var <- paste0("celula", level)

    # Check minimum cell size before applying this level
    # Only check determined observations
    cell_sizes <- dt[.is_determined == TRUE, .N, by = c(cell_var, ref_var)]
    min_size <- min(cell_sizes$N)

    if (min_size < min_cell_size) {
      if (verbose) {
        cat(sprintf("    Skipping %s: min cell size %d < threshold %d\n",
                    cell_var, min_size, min_cell_size))
      }
      break
    }

    # Inline reweighting with pre-extracted columns
    dt <- reweight_at_cell_level(dt          = dt,
                                                          cell_var    = cell_var,
                                                          anchor_vars = anchor_vars,
                                                          ref_var     = ref_var,
                                                          weight_vec  = weight_vec)
    levels_applied <- levels_applied + 1L
  }

  if (verbose && levels_applied > 0) {
    cat(sprintf("    Applied %d hierarchical cell level(s)\n", levels_applied))
  }

  # Step 3: Final calibration to external totals (only for determined obs)
  dt <- calibrate_to_external_totals(dt, target_totals, ref_var)

  # Step 4: Smooth weights (if requested and appropriate for the time period)
  if (smooth) {
    dt <- smooth_calibrated_weights(dt, ref_var)
  }

  # Step 5: Parent-period constraint (no-op: all periods calibrated to SIDRA)
  dt <- apply_parent_period_constraint(dt, weight_var, ref_var, verbose)

  # Rename final weight
  data.table::setnames(dt, "weight_current", "weight_calibrated")

  # Set NA for undetermined observations
  dt[.is_determined == FALSE, weight_calibrated := NA_real_]

  # Clean up temporary columns
  cell_cols <- paste0("celula", seq_len(n_cells))
  temp_cols <- c("anchor_year", ".anchor_key", ".is_determined",
                 cell_cols,
                 "pop_current", "target_pop")
  existing_temp <- temp_cols[temp_cols %in% names(dt)]
  if (length(existing_temp) > 0) {
    dt[, (existing_temp) := NULL]
  }

  dt
}


#' Create Calibration Cells
#'
#' @param dt data.table with PNADC data
#' @param n_cells Integer (1-4). Number of cell levels to create. Default 4.
#' @keywords internal
#' @noRd
create_calibration_cells_unified <- function(dt, n_cells = 4L) {

  # Cache column names
  dt_names <- names(dt)

  # Convert age column to integer if needed
  age_col <- if ("V2009" %in% dt_names) "V2009" else if ("v2009" %in% dt_names) "v2009" else NULL

  if (!is.null(age_col) && !is.integer(dt[[age_col]])) {
    data.table::set(dt, j = age_col, value = as.integer(dt[[age_col]]))
  }

  # Only convert other columns if we'll use them
  if (n_cells >= 2L && "posest_sxi" %in% dt_names && !is.integer(dt[["posest_sxi"]])) {
    data.table::set(dt, j = "posest_sxi", value = as.integer(dt[["posest_sxi"]]))
  }
  if (n_cells >= 4L && "posest" %in% dt_names && !is.integer(dt[["posest"]])) {
    data.table::set(dt, j = "posest", value = as.integer(dt[["posest"]]))
  }
  if (n_cells >= 3L) {
    uf_col <- if ("UF" %in% dt_names) "UF" else if ("uf" %in% dt_names) "uf" else NULL
    if (!is.null(uf_col) && !is.integer(dt[[uf_col]])) {
      data.table::set(dt, j = uf_col, value = as.integer(dt[[uf_col]]))
    }
  }

  # Handle case where age column is missing
  if (is.null(age_col)) {
    warning("Age column (V2009/v2009) not found. Using simple calibration.")
    for (i in seq_len(n_cells)) {
      data.table::set(dt, j = paste0("celula", i), value = 1L)
    }
    return(dt)
  }

  # Pre-extract age vector for fcase
  age_vec <- dt[[age_col]]

  # Celula 1: Age groups (always needed)
  data.table::set(dt, j = "celula1", value = data.table::fcase(
    age_vec <= 13L, 0L,
    age_vec <= 29L, 1L,
    age_vec <= 59L, 2L,
    default = 3L
  ))

  # Only create higher-level cells if needed
  if (n_cells >= 2L) {
    # Celula 2: Post-stratum group + age
    if ("posest_sxi" %in% dt_names) {
      # Use direct column access
      celula1_vec <- dt[["celula1"]]
      posest_sxi_vec <- dt[["posest_sxi"]]
      data.table::set(dt, j = "celula2", value = posest_sxi_vec %/% 100L + 10L * celula1_vec)
    } else {
      data.table::set(dt, j = "celula2", value = dt[["celula1"]])
    }
  }

  if (n_cells >= 3L) {
    # Celula 3: State + celula2
    uf_col <- if ("UF" %in% dt_names) "UF" else if ("uf" %in% dt_names) "uf" else NULL
    if (!is.null(uf_col)) {
      uf_vec <- dt[[uf_col]]
      celula2_vec <- dt[["celula2"]]
      data.table::set(dt, j = "celula3", value = uf_vec + 100L * celula2_vec)
    } else {
      data.table::set(dt, j = "celula3", value = dt[["celula2"]])
    }
  }

  if (n_cells >= 4L) {
    # Celula 4: Post-stratum + celula2
    if ("posest" %in% dt_names) {
      posest_vec <- dt[["posest"]]
      celula2_vec <- dt[["celula2"]]
      data.table::set(dt, j = "celula4", value = posest_vec + 1000L * celula2_vec)
    } else {
      data.table::set(dt, j = "celula4", value = dt[["celula3"]])
    }
  }

  dt
}


#' Reweight at Cell Level
#'
#' @keywords internal
#' @noRd
reweight_at_cell_level <- function(dt, cell_var, anchor_vars, ref_var, weight_vec) {

  # Only process determined observations
  # Use direct column access throughout

  # Anchor-level aggregations: sum of original weights per cell-anchor
  # Use pre-extracted weight_vec instead of get(weight_var)
  dt[.is_determined == TRUE, pop_anchor := sum(weight_vec[.I], na.rm = TRUE),
     by = c(cell_var, anchor_vars)]

  # Count unique ref periods per cell-anchor
  dt[.is_determined == TRUE, n_cells_anchor := data.table::uniqueN(.SD[[1]]),
     by = c(cell_var, anchor_vars), .SDcols = ref_var]

  # Period-level aggregations: sum of current weights
  dt[.is_determined == TRUE, pop_period := sum(weight_current, na.rm = TRUE),
     by = c(cell_var, ref_var)]

  # Count unique anchors using pre-computed .anchor_key
  # This replaces uniqueN(do.call(paste, ...)) which was extremely slow
  dt[.is_determined == TRUE, n_cells_period := data.table::uniqueN(.anchor_key),
     by = c(cell_var, ref_var)]

  # Apply reweighting ratio (only for determined observations)
  dt[.is_determined == TRUE & n_cells_anchor <= n_cells_period & pop_period > 0,
     weight_current := weight_current * (pop_anchor / pop_period)]

  # Clean up temporary columns
  dt[, c("pop_anchor", "pop_period", "n_cells_anchor", "n_cells_period") := NULL]

  dt
}


#' Calibrate to External Population Totals
#'
#' Scales weights so that the sum of calibrated weights for each period matches
#' the corresponding SIDRA monthly population total. All months, fortnights,
#' and weeks are treated uniformly -- each period is scaled to its SIDRA target.
#'
#' @param dt data.table with weight_current and .is_determined columns
#' @param target_totals data.table with population targets (in thousands)
#' @param ref_var Name of reference period variable (e.g., "ref_month_yyyymm")
#' @keywords internal
#' @noRd
calibrate_to_external_totals <- function(dt, target_totals, ref_var) {

  # Don't copy target_totals, work with it directly
  tt <- ensure_data_table(target_totals, copy = FALSE)
  tt_names <- names(tt)

  # Find the matching column in targets
  pop_col <- intersect(c("m_populacao", "f_populacao", "w_populacao", "population", "pop"),
                       tt_names)[1]
  if (is.na(pop_col)) {
    stop("Target totals must have a population column (m_populacao, f_populacao, w_populacao, population, or pop)")
  }

  # Find the ref column in targets
  ref_col_map <- list(
    "ref_month_yyyymm"     = c("ref_month_yyyymm", "anomesexato", "yyyymm"),
    "ref_fortnight_yyyyff" = c("ref_fortnight_yyyyff", "yyyyff"),
    "ref_week_yyyyww"      = c("ref_week_yyyyww", "ref_week_iso_yyyyww", "yyyyww")
  )

  if (!ref_var %in% names(ref_col_map)) {
    stop(sprintf("Unknown reference variable: %s. Expected one of: %s",
                 ref_var, paste(names(ref_col_map), collapse = ", ")))
  }

  tt_ref_col <- NULL
  for (candidate in ref_col_map[[ref_var]]) {
    if (candidate %in% tt_names) {
      tt_ref_col <- candidate
      break
    }
  }
  if (is.null(tt_ref_col)) {
    stop(sprintf("Target totals must have a column matching %s. Expected one of: %s",
                 ref_var, paste(ref_col_map[[ref_var]], collapse = ", ")))
  }

  # Create minimal lookup table (only copy if we need to rename)
  if (tt_ref_col != ref_var || pop_col != "target_pop") {
    tt <- data.table::copy(tt[, c(tt_ref_col, pop_col), with = FALSE])
    if (tt_ref_col != ref_var) {
      data.table::setnames(tt, tt_ref_col, ref_var)
    }
    data.table::setnames(tt, pop_col, "target_pop")
  } else {
    tt <- tt[, c(ref_var, pop_col), with = FALSE]
    data.table::setnames(tt, pop_col, "target_pop")
  }

  # Calculate current totals (only for determined observations)
  dt[.is_determined == TRUE, pop_current := sum(weight_current, na.rm = TRUE), by = ref_var]

  # Join targets
  dt[tt, on = ref_var, target_pop := i.target_pop]

  # All months, fortnights, and weeks: scale to SIDRA population targets
  # target_pop is in thousands, so multiply by 1000 to get actual population
  dt[.is_determined == TRUE & !is.na(target_pop) & pop_current > 0,
     weight_current := weight_current * (target_pop * 1000 / pop_current)]

  # Clean up
  cols_to_remove <- intersect(c("pop_current", "target_pop"), names(dt))
  if (length(cols_to_remove) > 0) {
    dt[, (cols_to_remove) := NULL]
  }

  dt
}


#' Smooth Calibrated Weights
#'
#' @param dt data.table with calibrated weights
#' @param ref_var Name of reference period variable (e.g., "ref_month_yyyymm")
#' @return data.table with smoothed weights in weight_current column
#' @keywords internal
#' @noRd
smooth_calibrated_weights <- function(dt, ref_var) {

  # Skip smoothing for weekly data
  if (grepl("week", ref_var, ignore.case = TRUE)) {
    return(dt)
  }

  # Determine smoothing parameters
  if (grepl("fortnight", ref_var, ignore.case = TRUE)) {
    window_size <- 7L
    cell_var <- "celula2"
  } else {
    window_size <- 3L
    cell_var <- "celula4"
  }

  # Check if the required cell variable exists
  dt_names <- names(dt)
  if (!cell_var %in% dt_names) {
    cell_var <- if ("celula2" %in% dt_names) "celula2" else
      if ("celula1" %in% dt_names) "celula1" else NULL
    if (is.null(cell_var)) {
      return(dt)
    }
  }

  # Get unique periods from determined observations only
  ref_vec <- dt[[ref_var]]
  determined_mask <- dt[[".is_determined"]]
  periods <- sort(unique(ref_vec[determined_mask]))
  n_periods <- length(periods)

  if (n_periods < 2L * window_size + 1L) {
    return(dt)
  }

  # Single aggregation pass on determined data only
  # Aggregate by cell and ref_var to get cell-period populations
  cell_period_agg <- dt[.is_determined == TRUE,
                        .(cell_pop = sum(weight_current, na.rm = TRUE)),
                        keyby = c(cell_var, ref_var)]

  # Derive original period totals from the smaller aggregated table
  original_pop <- cell_period_agg[, .(pop_orig = sum(cell_pop)), keyby = ref_var]

  # Add period_pos using match()
  cell_period_agg[, period_pos := match(cell_period_agg[[ref_var]], periods)]

  # Rekey for frollmean
  data.table::setkeyv(cell_period_agg, c(cell_var, "period_pos"))

  # Compute centered rolling mean per cell
  cell_period_agg[, pop_smoothed := data.table::frollmean(
    cell_pop,
    n = window_size,
    align = "center",
    na.rm = TRUE
  ), by = c(cell_var)]

  # Handle boundary NAs
  cell_period_agg[is.na(pop_smoothed), pop_smoothed := cell_pop]

  # Calculate smoothing factor
  cell_period_agg[, smooth_factor := data.table::fifelse(
    cell_pop > 0 & pop_smoothed > 0,
    pop_smoothed / cell_pop,
    1.0
  )]

  # Join smooth_factor using (cell_var, ref_var)
  # Create minimal join table
  smooth_join <- cell_period_agg[, c(cell_var, ref_var, "smooth_factor"), with = FALSE]
  data.table::setkeyv(smooth_join, c(cell_var, ref_var))

  # Join and apply smoothing factor (only for determined)
  dt[smooth_join, on = c(cell_var, ref_var), smooth_factor := i.smooth_factor]
  dt[.is_determined == TRUE & !is.na(smooth_factor) & !is.na(weight_current),
     weight_current := weight_current * smooth_factor]

  # Recalibrate to preserve original period totals
  dt[.is_determined == TRUE, pop_new := sum(weight_current, na.rm = TRUE), by = ref_var]

  # Join original totals
  dt[original_pop, on = ref_var, pop_orig := i.pop_orig]

  # Apply final adjustment
  dt[.is_determined == TRUE & pop_new > 0 & !is.na(pop_orig) & !is.na(weight_current),
     weight_current := weight_current * (pop_orig / pop_new)]

  # Clean up temporary columns
  temp_cols <- c("smooth_factor", "pop_new", "pop_orig")
  existing_temp <- temp_cols[temp_cols %in% names(dt)]
  if (length(existing_temp) > 0) {
    dt[, (existing_temp) := NULL]
  }

  dt
}


#' Apply Parent-Period Constraint
#'
#' Previously ensured child-period weights aggregated to parent-period totals.
#' Now skipped for all period types because each period (month, fortnight, week)
#' is independently calibrated to the FULL SIDRA population. Each period
#' represents the full Brazilian population, not a subdivision of a parent period.
#'
#' @param dt data.table with calibrated weights in weight_current column
#' @param weight_var Original weight variable name (e.g., "V1028")
#' @param ref_var Reference period variable (e.g., "ref_month_yyyymm")
#' @param verbose Print progress messages?
#' @return data.table unchanged (constraint is no longer applied)
#' @keywords internal
#' @noRd
apply_parent_period_constraint <- function(dt, weight_var, ref_var, verbose = FALSE) {

  # ==========================================================================
  # All periods skip the parent-period SUM constraint
  # ==========================================================================
  # Each period (month, fortnight, week) represents the FULL Brazilian
  # population, calibrated to SIDRA monthly population totals.
  # They are NOT subdivisions of a parent period.
  #
  # The old constraint forced child-period weight sums to match the
  # parent-period V1028 sum, which crushed externally-calibrated weights.
  #
  # Correct behavior: Each period keeps its external SIDRA calibration.
  # Parent-period consistency is evaluated via WEIGHTED AVERAGES of estimates,
  # not via forcing weight SUMS to match.

  if (verbose) cat("    Skipping parent-period constraint (all periods calibrated to SIDRA population)\n")
  dt
}


#' Derive Fortnight Population from Monthly
#'
#' Each fortnight within a month receives the FULL month's population as its
#' calibration target. This ensures that weights for each fortnight sum to the
#' actual Brazilian population, consistent with the monthly calibration approach.
#'
#' @keywords internal
#' @noRd
derive_fortnight_population <- function(monthly_pop) {

  # Don't copy if not needed
  mt <- ensure_data_table(monthly_pop, copy = FALSE)

  # Standardize column name
  if ("anomesexato" %in% names(mt) && !"ref_month_yyyymm" %in% names(mt)) {
    mt <- data.table::copy(mt)  # Copy only if we modify
    mt[, ref_month_yyyymm := anomesexato]
  }

  # Each fortnight gets the FULL month's population (not divided)
  fortnights <- mt[, .(
    ref_fortnight_yyyyff = c(
      ref_month_yyyymm %/% 100 * 100 + ((ref_month_yyyymm %% 100) - 1) * 2 + 1,
      ref_month_yyyymm %/% 100 * 100 + ((ref_month_yyyymm %% 100) - 1) * 2 + 2
    ),
    f_populacao = m_populacao
  ), by = ref_month_yyyymm]

  fortnights[, ref_month_yyyymm := NULL]

  fortnights
}


#' Derive Weekly Population from Monthly (IBGE Calendar)
#'
#' Each IBGE week receives the FULL month's population as its calibration target.
#' In the IBGE calendar, each week belongs to exactly one IBGE month (the month
#' where the week's Saturday falls with >= 4 days), so there is no ambiguity.
#' This ensures that weights for each week sum to the actual Brazilian population.
#'
#' Uses integer arithmetic for performance - avoids creating Date objects.
#'
#' @keywords internal
#' @noRd
derive_weekly_population <- function(monthly_pop) {

  # Don't copy if not needed
  mt <- ensure_data_table(monthly_pop, copy = FALSE)

  # Standardize column name
  if ("anomesexato" %in% names(mt) && !"ref_month_yyyymm" %in% names(mt)) {
    mt <- data.table::copy(mt)  # Copy only if we modify
    mt[, ref_month_yyyymm := anomesexato]
  }

  # Vectorized approach instead of lapply + rbindlist
  # Each IBGE month has exactly 4 reference weeks
  n_months <- nrow(mt)

  # Pre-allocate result
  yyyymm_vec <- mt$ref_month_yyyymm
  pop_vec <- mt$m_populacao
  year_vec <- yyyymm_vec %/% 100L
  month_vec <- yyyymm_vec %% 100L

  # Week positions within the year (1-48 for each month's 4 weeks)
  week_start <- (month_vec - 1L) * 4L + 1L

  # Expand to all weeks (4 per month)
  result <- data.table::data.table(
    ref_week_yyyyww = c(
      year_vec * 100L + week_start,
      year_vec * 100L + week_start + 1L,
      year_vec * 100L + week_start + 2L,
      year_vec * 100L + week_start + 3L
    ),
    w_populacao = rep(pop_vec, 4L)
  )

  # Sort by week
  data.table::setorder(result, ref_week_yyyyww)

  result
}

# Tests for weight calibration correctness
# Core invariant: each sub-monthly period (fortnight, week) is calibrated to the
# MONTHLY population total from SIDRA. So sum(weight_fortnight) per fortnight =
# sum(weight_monthly) per month = SIDRA population. Same for weeks.

# =============================================================================
# WEIGHT SUM PRESERVATION TESTS
# =============================================================================

test_that("all months are calibrated to SIDRA target population", {
  # 1. Setup: Create realistic data with multiple quarters
  set.seed(123)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Create custom target_totals so test is offline and deterministic.
  # Use the quarterly V1028 sum / 1000 as a population target so that
  # target_pop * 1000 gives a value the calibration can converge to.
  qtr_wsum <- data[, .(qtr_wsum = sum(V1028, na.rm = TRUE)), by = .(Ano, Trimestre)]
  targets <- qtr_wsum[, {
    months <- (Trimestre - 1L) * 3L + 1:3
    data.table::data.table(
      ref_month_yyyymm = Ano * 100L + months,
      m_populacao = qtr_wsum / 1000
    )
  }, by = .(Ano, Trimestre)][, .(ref_month_yyyymm, m_populacao)]

  # 2. Execute: Identify periods and apply calibration with custom targets
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = targets,
    verbose = FALSE
  )

  # 3. Verify: Weights are valid for determined observations
  determined <- result[determined_month == TRUE]

  # All determined obs have positive weights
  expect_true(all(determined$weight_monthly > 0),
              label = "All determined observations have positive weights")

  # No NA weights for determined obs
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA weights for determined observations")

  # 4. Verify: ALL months (1, 2, 3) are calibrated to the same SIDRA target
  # Each month's weight sum should equal target_pop * 1000
  month_sums <- determined[, .(w_sum = sum(weight_monthly)),
                           by = .(Ano, ref_month_in_year)]
  month_sums[, ref_month_yyyymm := Ano * 100L + ref_month_in_year]
  merged <- merge(month_sums, targets, by = "ref_month_yyyymm")
  expect_equal(merged$w_sum, merged$m_populacao * 1000, tolerance = 0.01,
               label = "All months calibrated to SIDRA target population")
})


test_that("each fortnight's weights sum to the monthly population total", {
  # 1. Setup: Create data and identify periods
  set.seed(124)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Create custom target_totals scaled to the test data's weight sums.
  # With simulated data, quarterly V1028 sum differs from real SIDRA population.
  # Setting m_populacao = quarterly_sum / 1000 gives a consistent target value.
  qtr_wsum <- data[, .(qtr_wsum = sum(V1028, na.rm = TRUE)), by = .(Ano, Trimestre)]
  targets <- qtr_wsum[, {
    months <- (Trimestre - 1L) * 3L + 1:3
    data.table::data.table(
      ref_month_yyyymm = Ano * 100L + months,
      m_populacao = qtr_wsum / 1000
    )
  }, by = .(Ano, Trimestre)][, .(ref_month_yyyymm, m_populacao)]

  # 2. Execute: Apply fortnight calibration with custom targets
  fortnight_targets <- PNADCperiods:::derive_fortnight_population(targets)
  result_fortnight <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "fortnight",
    target_totals = fortnight_targets,
    verbose = FALSE
  )

  # 3. Execute: Apply month calibration with same base targets
  result_month <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = targets,
    verbose = FALSE
  )

  # 4. Each INDIVIDUAL fortnight's weights should sum to the monthly population total
  # This is the key calibration invariant: fortnights and weeks are calibrated
  # to the SIDRA monthly population target, so each sub-period = monthly total
  t_fortnight <- result_fortnight[!is.na(ref_fortnight_in_month), .(
    N_fortnight = sum(weight_fortnight, na.rm = TRUE)
  ), by = .(Ano, ref_month_in_year, ref_fortnight_in_month)][
    order(Ano, ref_month_in_year, ref_fortnight_in_month)]

  t_month <- result_month[!is.na(ref_month_in_quarter), .(
    N_month = sum(weight_monthly, na.rm = TRUE)
  ), by = .(Ano, ref_month_in_year)][
    order(Ano, ref_month_in_year)]

  t <- merge(t_fortnight, t_month, by = c("Ano", "ref_month_in_year"))

  if (nrow(t) > 0) {
    expect_equal(t$N_fortnight, t$N_month, tolerance = 0.01,
                 label = "Each fortnight's weights must sum to monthly population total")
  }
})


test_that("each week's weights sum to the monthly population total", {
  # 1. Setup: Create data and identify periods
  set.seed(125)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Custom targets scaled to test data (same approach as fortnight test)
  qtr_wsum <- data[, .(qtr_wsum = sum(V1028, na.rm = TRUE)),
                  by = .(Ano, Trimestre)]
  targets <- qtr_wsum[, {
    months <- (Trimestre - 1L) * 3L + 1:3
    data.table::data.table(
      ref_month_yyyymm = Ano * 100L + months,
      m_populacao = qtr_wsum / 1000
    )
  }, by = .(Ano, Trimestre)][, .(ref_month_yyyymm, m_populacao)]

  # 2. Execute: Apply week calibration with custom targets
  week_targets <- PNADCperiods:::derive_weekly_population(targets)
  result_week <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "week",
    target_totals = week_targets,
    verbose = FALSE
  )

  # 3. Execute: Apply month calibration with same base targets
  result_month <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = targets,
    verbose = FALSE
  )

  # 4. Each INDIVIDUAL week's weights should sum to monthly total
  t_week <- result_week[!is.na(ref_week_in_month), .(
    N_week = sum(weight_weekly, na.rm = TRUE)
  ), by = .(Ano, ref_month_in_year, ref_week_in_month)][
    order(Ano, ref_month_in_year, ref_week_in_month)]

  t_month <- result_month[!is.na(ref_month_in_quarter), .(
    N_month = sum(weight_monthly, na.rm = TRUE)
  ), by = .(Ano, ref_month_in_year)][
    order(Ano, ref_month_in_year)]

  t <- merge(t_week, t_month, by = c("Ano", "ref_month_in_year"))

  if (nrow(t) > 0) {
    expect_equal(t$N_week, t$N_month, tolerance = 0.01,
                 label = "Each week's weights must sum to monthly total")
  }
})


test_that("no negative weights are produced in any calibration unit", {
  # 1. Setup: Create data
  set.seed(126)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Mock targets derived from V1028 sums (offline; no SIDRA fetch).
  mock_monthly <- create_mock_pop_targets(data)

  # 2. Execute: Test all calibration units
  for (unit in c("month", "fortnight", "week")) {
    unit_targets <- switch(unit,
      "month"     = mock_monthly,
      "fortnight" = PNADCperiods:::derive_fortnight_population(mock_monthly),
      "week"      = PNADCperiods:::derive_weekly_population(mock_monthly)
    )
    result <- pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = TRUE,
      calibration_unit = unit,
      target_totals = unit_targets,
      verbose = FALSE
    )

    # 3. Verify: No negative weights allowed
    weight_col <- paste0("weight_", ifelse(unit == "fortnight", "fortnight",
                                           ifelse(unit == "week", "weekly", "monthly")))

    negative_weights <- result[!is.na(get(weight_col)) & get(weight_col) < 0]

    # 4. Context: Negative weights are methodologically invalid
    expect_equal(nrow(negative_weights), 0,
                 label = paste("No negative weights allowed in", unit, "calibration"))
  }
})


test_that("anchor='year' produces valid calibrated weights", {
  # 1. Setup: Create multi-year data
  set.seed(127)
  data <- create_realistic_pnadc(n_quarters = 8, n_upas = 20, start_year = 2022)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Apply with year anchor (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "year",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    verbose = FALSE
  )

  # 3. Verify: Weights are valid
  determined <- result[determined_month == TRUE]
  expect_true(all(determined$weight_monthly > 0),
              label = "All weights are positive")
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA weights for determined observations")

  # 4. Context: Year anchor calibrates at year level
  # Each month is still scaled to external population targets
})


# =============================================================================
# INDETERMINATE OBSERVATIONS
# =============================================================================

test_that("indeterminate observations have NA weights", {
  # 1. Setup: Create data with some indeterminate cases
  set.seed(128)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 15)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Apply calibration with keep_all=TRUE (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    keep_all = TRUE,
    verbose = FALSE
  )

  # 3. Verify: Indeterminate observations must have NA weights
  indeterminate <- result[is.na(determined_month) | determined_month == FALSE]

  if (nrow(indeterminate) > 0) {
    expect_true(all(is.na(indeterminate$weight_monthly)),
                label = "Indeterminate observations must have NA monthly weights")
  }

  # 4. Context: Only determined observations can have calibrated weights
})


test_that("keep_all=FALSE excludes indeterminate observations", {
  # 1. Setup: Create data
  set.seed(129)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 15)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Apply with keep_all=FALSE (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    keep_all = FALSE,
    verbose = FALSE
  )

  # 3. Verify: All returned observations must be determined
  expect_true(all(result$determined_month == TRUE, na.rm = TRUE),
              label = "keep_all=FALSE returns only determined observations")

  # 4. Verify: No NA weights in result
  expect_true(all(!is.na(result$weight_monthly)),
              label = "No NA weights when keep_all=FALSE")
})


# =============================================================================
# EDGE CASES
# =============================================================================

test_that("calibration works with single quarter data", {
  # 1. Setup: Minimal single-quarter data
  set.seed(130)
  data <- create_realistic_pnadc(n_quarters = 1, n_upas = 10)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Apply calibration (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    verbose = FALSE
  )

  # 3. Verify: Basic structure preserved
  expect_true("weight_monthly" %in% names(result))
  expect_equal(nrow(result), nrow(data))

  # 4. Verify: Weights are valid even with sparse data
  if (any(result$determined_month == TRUE, na.rm = TRUE)) {
    determined <- result[determined_month == TRUE]
    expect_true(all(determined$weight_monthly > 0),
                label = "All determined observations have positive weights")
  }
})


test_that("calibration handles empty periods gracefully", {
  # 1. Setup: Create data where some months may be empty
  set.seed(131)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Should not error with sparse data (mock targets, no SIDRA)
  expect_no_error({
    result <- pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = TRUE,
      calibration_unit = "month",
      target_totals = create_mock_pop_targets(data),
      verbose = FALSE
    )
  })

  # 3. Context: Implementation should handle empty cells gracefully
})


# =============================================================================
# SMOOTHING BEHAVIOR
# =============================================================================

test_that("smooth=TRUE modifies weights differently than smooth=FALSE", {
  # 1. Setup: Create data with enough quarters for smoothing to have effect
  # Smoothing uses 3-period window for months, so need at least 6+ quarters
  set.seed(132)
  data <- create_realistic_pnadc(n_quarters = 8, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Mock targets (offline; no SIDRA fetch).
  mock_targets <- create_mock_pop_targets(data)

  # 2. Execute: Apply with and without smoothing
  result_no_smooth <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = mock_targets,
    smooth = FALSE,
    verbose = FALSE
  )

  result_smooth <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = mock_targets,
    smooth = TRUE,
    verbose = FALSE
  )

  # 3. Verify: Both runs should complete successfully
  expect_true(is.data.frame(result_no_smooth))
  expect_true(is.data.frame(result_smooth))
  expect_true("weight_monthly" %in% names(result_no_smooth))
  expect_true("weight_monthly" %in% names(result_smooth))

  # 4. Context: Smoothing parameter is accepted and processed
  # Note: Whether smoothing produces different weights depends on data characteristics
  # (month distribution, determination patterns, etc.). The key is that the parameter
  # is processed without error and produces valid weights.
})


test_that("smoothing produces valid non-negative weights", {
  # 1. Setup: Create data
  set.seed(133)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Apply with smoothing (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    smooth = TRUE,
    verbose = FALSE
  )

  # 3. Verify: Smoothed weights are valid
  determined <- result[determined_month == TRUE]
  expect_true(all(determined$weight_monthly > 0),
              label = "Smoothed weights are positive")
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA values after smoothing")
})

# Integration tests for end-to-end pipelines
# These tests verify the complete workflow: identify → apply → calibrate

# =============================================================================
# BASIC END-TO-END PIPELINE TESTS
# =============================================================================

test_that("full pipeline works end-to-end with monthly calibration", {
  # 1. Setup: Create realistic multi-quarter data
  set.seed(900)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 30)

  # Add calibration columns
  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Full pipeline
  # Step 1: Identify periods
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Step 2: Apply and calibrate (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    smooth = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Basic structure
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), nrow(data))
  expect_true("weight_monthly" %in% names(result))
  expect_true("determined_month" %in% names(result))

  # 4. Verify: Weights are valid (not sum preservation)
  determined <- result[determined_month == TRUE]
  expect_true(all(determined$weight_monthly > 0),
              label = "Pipeline produces positive weights")
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA weights for determined observations")

  # 5. Verify: Determination rates are reasonable
  # With 4 quarters, expect >70% month determination
  det_rate <- result[, mean(determined_month, na.rm = TRUE)]
  expect_true(det_rate > 0.70,
              label = paste0("Month determination rate (", round(det_rate, 3),
                             ") should be >70% with 4 quarters"))
})


test_that("full pipeline works with fortnight calibration", {
  # 1. Setup: Create data
  set.seed(901)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 25)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Full pipeline with fortnight (mock targets, no SIDRA)
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "fortnight",
    target_totals = PNADCperiods:::derive_fortnight_population(
      create_mock_pop_targets(data)
    ),
    smooth = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Structure
  expect_true("weight_fortnight" %in% names(result))
  expect_true("determined_fortnight" %in% names(result))

  # 4. Verify: Each fortnight's weights sum to monthly total
  # Use custom targets scaled to test data's quarterly V1028 sum so calibration converges
  qtr_wsum <- data[, .(qtr_wsum = sum(V1028, na.rm = TRUE)),
                  by = .(Ano, Trimestre)]
  targets <- qtr_wsum[, {
    months <- (Trimestre - 1L) * 3L + 1:3
    data.table::data.table(
      ref_month_yyyymm = Ano * 100L + months,
      m_populacao = qtr_wsum / 1000
    )
  }, by = .(Ano, Trimestre)][, .(ref_month_yyyymm, m_populacao)]

  fn_targets <- PNADCperiods:::derive_fortnight_population(targets)
  result_fn <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "fortnight",
    target_totals = fn_targets,
    smooth = FALSE,
    verbose = FALSE
  )
  result_mo <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = targets,
    smooth = FALSE,
    verbose = FALSE
  )

  t_fortnight <- result_fn[!is.na(ref_fortnight_in_month), .(
    N_fn = sum(weight_fortnight, na.rm = TRUE)
  ), by = .(Ano, ref_month_in_year, ref_fortnight_in_month)]

  t_month <- result_mo[!is.na(ref_month_in_quarter), .(
    N_mo = sum(weight_monthly, na.rm = TRUE)
  ), by = .(Ano, ref_month_in_year)]

  chk <- merge(t_fortnight, t_month,
               by = c("Ano", "ref_month_in_year"))

  if (nrow(chk) > 0) {
    expect_equal(chk$N_fn, chk$N_mo, tolerance = 0.01,
                 label = "Fortnight weights sum to monthly total")
  }
})


test_that("full pipeline with smoothing produces consistent results", {
  # 1. Setup: Create data with enough quarters for smoothing
  set.seed(902)
  data <- create_realistic_pnadc(n_quarters = 6, n_upas = 25)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Pipeline with smoothing (mock targets, no SIDRA)
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    smooth = TRUE,
    verbose = FALSE
  )

  # 3. Verify: Smoothed weights are valid
  determined <- result[determined_month == TRUE]
  expect_true(all(determined$weight_monthly > 0),
              label = "Smoothed weights are positive")
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA values after smoothing")
})


# =============================================================================
# EXPERIMENTAL STRATEGIES INTEGRATION
# =============================================================================

test_that("pipeline with experimental strategies maintains invariants", {
  # 1. Setup: Create data
  set.seed(903)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 25)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Strict identification
  crosswalk_strict <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 3. Execute: Experimental strategies
  crosswalk_exp <- pnadc_experimental_periods(
    crosswalk_strict,
    strategy = "both",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  # 4. Execute: Apply calibration with experimental crosswalk
  #    (mock targets, no SIDRA)
  result <- pnadc_apply_periods(
    data, crosswalk_exp,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    verbose = FALSE
  )

  # 5. Verify: Weights are valid (experimental strategies)
  determined <- result[determined_month == TRUE]
  expect_true(all(determined$weight_monthly > 0),
              label = "Experimental pipeline produces positive weights")
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA weights for determined observations")

  # 6. Verify: Experimental improved determination rate
  strict_rate <- crosswalk_strict[, mean(determined_month, na.rm = TRUE)]
  exp_rate <- crosswalk_exp[, mean(determined_month, na.rm = TRUE)]

  expect_true(exp_rate >= strict_rate,
              label = paste0("Experimental (", round(exp_rate, 3),
                             ") >= strict (", round(strict_rate, 3), ")"))
})


# =============================================================================
# MULTI-YEAR PIPELINE TESTS
# =============================================================================

test_that("pipeline works with multi-year data (year anchor)", {
  # 1. Setup: Create 2-year data (8 quarters)
  set.seed(904)
  data <- create_realistic_pnadc(n_quarters = 8, n_upas = 25, start_year = 2022)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Pipeline with year anchor (mock targets, no SIDRA)
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "year",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(data),
    verbose = FALSE
  )

  # 3. Verify: Weights are valid
  determined <- result[determined_month == TRUE]
  expect_true(all(determined$weight_monthly > 0),
              label = "Multi-year pipeline produces positive weights")
  expect_false(any(is.na(determined$weight_monthly)),
               label = "No NA weights for determined observations")

  # 4. Verify: High determination rate with 8 quarters
  det_rate <- result[, mean(determined_month, na.rm = TRUE)]
  expect_true(det_rate > 0.85,
              label = paste0("8-quarter determination (", round(det_rate, 3),
                             ") should be >85%"))
})


# =============================================================================
# ERROR HANDLING IN PIPELINE
# =============================================================================

test_that("pipeline handles missing weight columns gracefully", {
  # 1. Setup: Create data then remove V1028 to simulate missing weight column
  set.seed(905)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)
  data[, V1028 := NULL]  # Remove weight column to test error handling

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Verify: Should error when requesting calibration without weight columns
  expect_error(
    pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",  # Column doesn't exist anymore
      anchor = "quarter",
      calibrate = TRUE,
      calibration_unit = "month",
      verbose = FALSE
    ),
    "V1028|weight_var|missing",
    label = "Missing weight column should error"
  )
})


test_that("pipeline works without calibration", {
  # 1. Setup: Create data with V1028 column but won't calibrate
  set.seed(906)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)

  # Add V1028 column (required even when not calibrating)
  data[, V1028 := runif(.N, 500, 2000)]

  # 2. Execute: Pipeline without calibration
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  result <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = FALSE,  # No calibration
    verbose = FALSE
  )

  # 3. Verify: Basic structure without calibrated weight columns
  expect_true(is.data.frame(result))
  expect_false("weight_monthly" %in% names(result))
  expect_true("determined_month" %in% names(result))
  expect_true("ref_month_in_quarter" %in% names(result))
  expect_true("V1028" %in% names(result))  # Original weight column preserved
})


# =============================================================================
# KEEP_ALL PARAMETER INTEGRATION
# =============================================================================

test_that("pipeline with keep_all=FALSE filters correctly", {
  # 1. Setup: Create data
  set.seed(907)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 15)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Pipeline with keep_all=TRUE (mock targets, no SIDRA)
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)
  mock_targets <- create_mock_pop_targets(data)

  result_all <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = mock_targets,
    keep_all = TRUE,
    verbose = FALSE
  )

  # 3. Execute: Pipeline with keep_all=FALSE
  result_filtered <- pnadc_apply_periods(
    data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = mock_targets,
    keep_all = FALSE,
    verbose = FALSE
  )

  # 4. Verify: keep_all=FALSE returns fewer rows
  expect_true(nrow(result_filtered) <= nrow(result_all))

  # 5. Verify: All rows in filtered result are determined
  expect_true(all(result_filtered$determined_month == TRUE, na.rm = TRUE))

  # 6. Verify: No NA weights in filtered result
  expect_true(all(!is.na(result_filtered$weight_monthly)))
})


# =============================================================================
# CONSISTENCY CHECKS
# =============================================================================

test_that("pipeline produces consistent results with same data", {
  # 1. Setup: Create data
  set.seed(908)
  data <- create_realistic_pnadc(n_quarters = 3, n_upas = 20)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # 2. Execute: Pipeline twice with same data (mock targets, no SIDRA)
  mock_targets <- create_mock_pop_targets(data)
  crosswalk1 <- pnadc_identify_periods(data, verbose = FALSE)
  result1 <- pnadc_apply_periods(
    data, crosswalk1,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = mock_targets,
    verbose = FALSE
  )

  crosswalk2 <- pnadc_identify_periods(data, verbose = FALSE)
  result2 <- pnadc_apply_periods(
    data, crosswalk2,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = mock_targets,
    verbose = FALSE
  )

  # 3. Verify: Results should be identical
  expect_equal(result1[determined_month == TRUE]$weight_monthly,
               result2[determined_month == TRUE]$weight_monthly,
               tolerance = 1e-10,
               label = "Pipeline should produce consistent results")

  expect_equal(crosswalk1$determined_month,
               crosswalk2$determined_month,
               label = "Determination should be consistent")
})

#' Identify Reference Periods in PNADC Data
#'
#' Builds a crosswalk containing reference periods (month, fortnight,
#' and week) for PNADC survey data based on IBGE's interview timing rules.
#'
#' @description
#' PNADC is a quarterly survey, but each interview actually refers to a specific
#' temporal period within the quarter. This function identifies which month,
#' fortnight (quinzena), and week each observation belongs to, enabling
#' sub-quarterly time series analysis.
#'
#' The algorithm uses a **nested identification approach**:
#' \itemize{
#'   \item \strong{Phase 1}: Identify MONTHS for all observations using:
#'     \itemize{
#'       \item IBGE's reference week timing rules (first reference week -- ending in a Saturday -- with sufficient days)
#'       \item Respondent birthdates to constrain possible interview dates
#'       \item UPA-panel level aggregation across ALL quarters (panel design)
#'       \item Dynamic exception detection (identifies quarters needing relaxed rules)
#'     }
#'   \item \strong{Phase 2}: Identify FORTNIGHTS for month-determined observations:
#'     \itemize{
#'       \item Search space constrained to 2 fortnights within determined month
#'       \item Household-level aggregation within each quarter
#'     }
#'   \item \strong{Phase 3}: Identify WEEKS for fortnight-determined observations:
#'     \itemize{
#'       \item Search space constrained to ~2 weeks within determined fortnight
#'       \item Household-level aggregation within each quarter
#'     }
#' }
#'
#' @param data A data.frame or data.table with PNADC microdata. Required columns:
#'   \itemize{
#'     \item \code{Ano}: Survey year
#'     \item \code{Trimestre}: Quarter (1-4)
#'     \item \code{UPA}: Primary Sampling Unit
#'     \item \code{V1008}: Household id/sequence within UPA
#'     \item \code{V1014}: Panel identifier
#'     \item \code{V2008}: Birth day (1-31)
#'     \item \code{V20081}: Birth month (1-12)
#'     \item \code{V20082}: Birth year
#'     \item \code{V2009}: Age
#'   }
#'   Optional but recommended:
#'   \itemize{
#'     \item \code{V2003}: Person sequence within household
#'   }
#'
#' @param verbose Logical. If TRUE (default), display progress information.
#'
#' @param store_date_bounds Logical. If TRUE, stores date bounds and exception
#'   flags in the crosswalk for optimization when calling
#'   \code{pnadc_experimental_periods()}. This enables 10-20x speedup for the
#'   probabilistic strategy by avoiding redundant computation. Default FALSE.
#'
#' @return A data.table crosswalk with columns:
#'   \describe{
#'     \item{Ano, Trimestre, UPA, V1008, V1014}{Join keys (year, quarter, UPA, household, panel)}
#'     \item{ref_month_in_quarter}{Integer. Month position in quarter (1, 2, 3) or NA}
#'     \item{ref_month_in_year}{Integer. Month position in year (1-12) or NA}
#'     \item{ref_fortnight_in_month}{Integer. Fortnight position in month (1 or 2) or NA}
#'     \item{ref_fortnight_in_quarter}{Integer. Fortnight position in quarter (1-6) or NA}
#'     \item{ref_week_in_month}{Integer. Week position in month (1-4) or NA}
#'     \item{ref_week_in_quarter}{Integer. Week position in quarter (1-12) or NA}
#'     \item{date_min}{Date. Lower bound of the interview reference date for the individual. Only returned if store_date_bounds = TRUE}
#'     \item{date_max}{Date. Upper bound of the interview reference date for the individual. Only returned if store_date_bounds = TRUE}
#'     \item{week_1_start}{Date. Sunday of the IBGE first reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_1_end}{Date. Saturday of the IBGE first reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_2_start}{Date. Sunday of the IBGE second reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_2_end}{Date. Saturday of the IBGE second reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_3_start}{Date. Sunday of the IBGE third reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_3_end}{Date. Saturday of the IBGE third reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_4_start}{Date. Sunday of the IBGE fourth reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{week_4_end}{Date. Saturday of the IBGE fourth reference week of the month. Only returned if store_date_bounds = TRUE}
#'     \item{month_max_upa}{Integer. Maximum month position across UPA-V1014 group (for debugging). Only returned if store_date_bounds = TRUE}
#'     \item{month_min_upa}{Integer. Minimum month position across UPA-V1014 group (for debugging). Only returned if store_date_bounds = TRUE}
#'     \item{fortnight_max_hh}{Integer. Maximum fortnight position within household (for debugging). Only returned if store_date_bounds = TRUE}
#'     \item{fortnight_min_hh}{Integer. Minimum fortnight position within household (for debugging). Only returned if store_date_bounds = TRUE}
#'     \item{week_min_hh}{Integer. Minimum week position within household (for debugging). Only returned if store_date_bounds = TRUE}
#'     \item{week_max_hh}{Integer. Maximum week position within household (for debugging). Only returned if store_date_bounds = TRUE}
#'     \item{ref_month_yyyymm}{Integer. Identified reference month in the format YYYYMM, where MM follows the IBGE calendar. 1 <= MM  <= 12}
#'     \item{ref_fortnight_yyyyff}{Integer. Identified reference fortnight in the format YYYYFF, where FF follows the IBGE calendar. 1 <= FF  <= 24}
#'     \item{ref_week_yyyyww}{Integer. Identified reference Week in the format YYYYWW, where WW follows the IBGE calendar. 1 <= WW  <= 48}
#'     \item{determined_month}{Logical. Flags if the month was determined.}
#'     \item{determined_fortnight}{Logical. Flags if the fortnight was determined.}
#'     \item{determined_week}{Logical. Flags if the week was determined.}
#'   }
#'
#' @note
#' ## Nested Identification Hierarchy
#'
#' The algorithm enforces strict nesting by construction:
#' \itemize{
#'   \item Fortnights can ONLY be identified for observations with determined months
#'   \item Weeks can ONLY be identified for observations with determined fortnights
#' }
#'
#' This guarantees: \code{determined_week => determined_fortnight => determined_month}
#'
#' ## Aggregation Levels
#'
#' The crosswalk aggregates at different levels:
#' \itemize{
#'   \item \strong{Months}: UPA-V1014 level across ALL quarters
#'     (PNADC panel design ensures same month position)
#'   \item \strong{Fortnights}: Household level within quarter only
#'   \item \strong{Weeks}: Household level within quarter only
#' }
#'
#' @details
#' ## Temporal Granularity
#'
#' The crosswalk contains three levels of temporal granularity:
#' \itemize{
#'   \item \strong{Month}: 3 per quarter, ~97\% determination rate with full series (aggregates across quarters)
#'   \item \strong{Fortnight (quinzena)}: 6 per quarter, ~9\% determination rate (within-quarter only)
#'   \item \strong{Week}: 12 per quarter, ~3\% determination rate (within-quarter only)
#' }
#'
#' ## Cross-Quarter Aggregation (Important!)
#'
#' **For optimal month determination rates, input data should be stacked across
#' multiple quarters** (ideally 4+ years). The algorithm leverages PNADC's
#' rotating panel design where the same UPA-V1014 is interviewed in the same
#' relative position across quarterly visits.
#'
#' ## Fortnight Definition
#'
#' Fortnights are numbered 1-6 per quarter (2 per month), based on the IBGE
#' reference week calendar (not calendar days). Each IBGE "month" consists of
#' exactly 4 reference weeks (28 days), starting on a Sunday:
#' \itemize{
#'   \item Fortnight 1 in month: IBGE weeks 1-2 (days 1-14 of the IBGE month)
#'   \item Fortnight 2 in month: IBGE weeks 3-4 (days 15-28 of the IBGE month)
#' }
#'
#' @examples
#' \dontrun{
#' crosswalk <- pnadc_identify_periods(pnadc_stacked)
#'
#' crosswalk[, .(
#'   month_rate = mean(determined_month),
#'   fortnight_rate = mean(determined_fortnight),
#'   week_rate = mean(determined_week)
#' )]
#'
#' crosswalk[determined_fortnight, all(determined_month)]
#' crosswalk[determined_week, all(determined_fortnight)]
#'
#' result <- pnadc_apply_periods(pnadc_2023, crosswalk,
#'                               weight_var = "V1028",
#'                               anchor = "quarter")
#' }
#'
#' @seealso \code{\link{pnadc_apply_periods}} to apply the crosswalk and
#'   calibrate weights
#'
#' @export
pnadc_identify_periods <- function(data, verbose = TRUE, store_date_bounds = FALSE) {

  # ============================================================================
  # INPUT VALIDATION
  # ============================================================================

  # Validate parameters
  checkmate::assert_flag(verbose)
  checkmate::assert_flag(store_date_bounds)

  # Freeing memory before starting...
  gc()

  if (verbose) {
    cat("Building reference period crosswalk (nested identification)...\n")
  }

  # Validate required columns
  validate_pnadc(data, check_weights = FALSE)

  # Subset to required columns BEFORE copying (80-90% memory reduction)
  # Instead of copying all 50+ columns, only copy the ~9 columns we actually need
  required_cols <- required_vars_ref_month()
  dt            <- subset_and_copy(data, required_cols)

  # Freeing memory after making the copy...
  gc()

  # ============================================================================
  # PHASE 1: MONTH IDENTIFICATION (ALL observations)
  # ============================================================================
  #
  # This phase identifies the reference MONTH for each observation using:
  # - IBGE's reference week timing rules (parada técnica)
  # - Birthday constraints to narrow possible interview dates
  # - Cross-quarter aggregation at UPA-V1014 level (panel design)
  # - Dynamic exception detection for edge cases
  #
  # ============================================================================

  if (verbose) cat("\nPhase 1: Identifying MONTHS (all observations)...\n")

  # --------------------------------------------------------------------------
  # STEP 1.1: Preprocessing - Convert columns and handle special codes
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.1: Preprocessing data...\n")

  # Converts to integer (including V2009 - saves 4 bytes per row vs numeric)
  num_cols = c("Ano", "Trimestre", "V2008", "V20081", "V20082", "V2009")
  dt[ , names(.SD) := lapply(.SD, as.numeric), .SDcols = num_cols]

  dt[V2008  == 99,  V2008  := NA]
  dt[V20081 == 99,  V20081 := NA]
  dt[V20082 == 9999,V20082 := NA]

  # Initialize output columns to ensure they exist even if phases are skipped
  dt[, `:=`(
    ref_month_in_quarter     = NA_integer_,
    ref_month_in_year        = NA_integer_,
    ref_fortnight_in_month   = NA_integer_,
    ref_fortnight_in_quarter = NA_integer_,
    ref_week_in_month        = NA_integer_,
    ref_week_in_quarter      = NA_integer_
  )]

  # --------------------------------------------------------------------------
  # STEP 1.2: Pre-compute first valid Saturdays for each unique (year, quarter)
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.2: Computing valid interview Saturdays...\n")

  unique_quarters <- unique(dt[, .(Ano, Trimestre)])
  unique_quarters[, `:=`(
    month1 = quarter_month_n(Trimestre, 1L),
    month2 = quarter_month_n(Trimestre, 2L),
    month3 = quarter_month_n(Trimestre, 3L),

    is_leap = is_leap_year(Ano)

  )][, `:=`(

    # Standard rule (min_days=4)
    first_sat_m1 = first_valid_saturday(Ano, month1, min_days = 4L),
    first_sat_m2 = first_valid_saturday(Ano, month2, min_days = 4L),
    first_sat_m3 = first_valid_saturday(Ano, month3, min_days = 4L),

    # Exception rule (min_days=3)
    alt_sat_m1 = first_valid_saturday(Ano, month1, min_days = 3L),
    alt_sat_m2 = first_valid_saturday(Ano, month2, min_days = 3L),
    alt_sat_m3 = first_valid_saturday(Ano, month3, min_days = 3L)

  )][, `:=`(

    # Date bounds (for regular first reference weeks, with 4 or more days in the IBGE reference month)
    # date_min: Sunday of first reference week (first_sat - 6 days, but we use Saturday here)
    # date_max: End of 4th week of month 3 (first_sat + 21 days = 3 more weeks after Saturday)
    date_min = lubridate::ymd(paste(Ano, month1, first_sat_m1, sep = "-")),
    date_max = lubridate::ymd(paste(Ano, month3, first_sat_m3, sep = "-")) + 21L,

    # Calculate alternative bounds (for exception handling: reference weeks with 3 or more days in the IBGE reference month)
    alt_date_min = lubridate::ymd(paste(Ano, month1, alt_sat_m1, sep = "-")),
    alt_date_max = lubridate::ymd(paste(Ano, month3, alt_sat_m3, sep = "-")) + 21L
  )]


  # Join pre-computed values to main data
  dt[unique_quarters,
     on = .(Ano, Trimestre),
     `:=`(month1 = i.month1,
          month2 = i.month2,
          month3 = i.month3,

          first_sat_m1 = i.first_sat_m1,
          first_sat_m2 = i.first_sat_m2,
          first_sat_m3 = i.first_sat_m3,

          alt_sat_m1 = i.alt_sat_m1,
          alt_sat_m2 = i.alt_sat_m2,
          alt_sat_m3 = i.alt_sat_m3,

          date_min = i.date_min,
          date_max = i.date_max,

          alt_date_min = i.alt_date_min,
          alt_date_max = i.alt_date_max,

          is_leap = i.is_leap
          )
     ]

  # Remove unique_quarters - no longer needed
  rm(unique_quarters)

  # --------------------------------------------------------------------------
  # STEP 1.3: Calculate birthdays and adjust for leap years
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.3: Calculating birthdays and adjusting for leap years...\n")

  dt[!is.na(V2008), birthday := lubridate::ymd(paste(Ano, V20081, V2008, sep = "-"), quiet = TRUE)]

  # Handling 29th February birthdays on non-leap years
  dt[(!is_leap) & V2008 == 29 & V20081 == 2, birthday := lubridate::ymd(paste(Ano, 3, 1, sep = "-"))]

  # Diagnostic: Warn if many birthdays failed to parse
  n_failed_birthday <- dt[!is.na(V2008) & is.na(birthday), .N]
  n_valid_birthday_input <- dt[!is.na(V2008), .N]
  if (n_failed_birthday > 0 && n_valid_birthday_input > 0) {
    pct_failed <- 100 * n_failed_birthday / n_valid_birthday_input
    if (pct_failed > 5 && verbose) {
      warning(sprintf("%.1f%% of birthdays failed to parse (%d of %d). Check V2008/V20081 data quality.",
                      pct_failed, n_failed_birthday, n_valid_birthday_input), call. = FALSE)
    }
  }

  dt[, first_sat_after_birthday := first_saturday_on_or_after(birthday)]

  # Determine if interview was before or after birthday
  dt[!is.na(V20082), visit_before_birthday := (Ano - V20082) - V2009]

  # --------------------------------------------------------------------------
  # STEP 1.4: Calculate birthday constraints
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.4: Applying birthday constraints...\n")

  # Apply birthday constraints
  dt[visit_before_birthday == 0L &
       !is.na(first_sat_after_birthday) &
       first_sat_after_birthday > date_min &
       first_sat_after_birthday <= date_max,
     date_min := first_sat_after_birthday]

  dt[visit_before_birthday == 1L &
       !is.na(first_sat_after_birthday) &
       (first_sat_after_birthday - 7L) < date_max &
       (first_sat_after_birthday - 7L) >= date_min,
     date_max := first_sat_after_birthday - 7L]

  # Apply birthday constraints to alternative bounds
  dt[visit_before_birthday == 0L &
       !is.na(first_sat_after_birthday) &
       first_sat_after_birthday > alt_date_min &
       first_sat_after_birthday <= alt_date_max,
     alt_date_min := first_sat_after_birthday]

  dt[visit_before_birthday == 1L &
       !is.na(first_sat_after_birthday) &
       (first_sat_after_birthday - 7L) < alt_date_max &
       (first_sat_after_birthday - 7L) >= alt_date_min,
     alt_date_max := first_sat_after_birthday - 7L]

  # Clean up birthday column
  dt[, birthday := NULL]

  # Freeing memory after deleting columns...
  gc()

  # --------------------------------------------------------------------------
  # STEP 1.5: Convert date bounds to MONTH positions
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.5: Converting to month positions...\n")

  dt[, `:=`(
    month_min_pos     = calculate_month_position_min(date = date_min,     year = Ano, quarter = Trimestre, day_threshold = 3L),
    month_max_pos     = calculate_month_position_max(date = date_max,     year = Ano, quarter = Trimestre, day_threshold = 3L),
    alt_month_min_pos = calculate_month_position_min(date = alt_date_min, year = Ano, quarter = Trimestre, day_threshold = 2L),
    alt_month_max_pos = calculate_month_position_max(date = alt_date_max, year = Ano, quarter = Trimestre, day_threshold = 2L)
  )]

  # --------------------------------------------------------------------------
  # STEP 1.6: MONTH aggregation at UPA-V1014 level ACROSS ALL quarters
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.6: Aggregating at UPA-panel level (cross-quarter)...\n")

  # Set key for faster groupby operations
  data.table::setkey(dt, UPA, V1014)

  dt[, `:=`(
    upa_month_min     = max(month_min_pos,     na.rm = TRUE),
    upa_month_max     = min(month_max_pos,     na.rm = TRUE),
    alt_upa_month_min = max(alt_month_min_pos, na.rm = TRUE),
    alt_upa_month_max = min(alt_month_max_pos, na.rm = TRUE)
  ), by = .(UPA, V1014)]

  # Handle all-NA groups (max returns -Inf, min returns Inf)
  dt[is.infinite(upa_month_min),     upa_month_min     := NA_integer_]
  dt[is.infinite(upa_month_max),     upa_month_max     := NA_integer_]
  dt[is.infinite(alt_upa_month_min), alt_upa_month_min := NA_integer_]
  dt[is.infinite(alt_upa_month_max), alt_upa_month_max := NA_integer_]

  # --------------------------------------------------------------------------
  # STEP 1.7: Dynamic exception detection for MONTHS
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.7: Detecting exception cases...\n")

  dt[, requires_exception := (
    upa_month_min > upa_month_max &
      alt_upa_month_min <= alt_upa_month_max &
      ((month_max_pos == upa_month_max & month_max_pos < alt_upa_month_max) |
         (month_min_pos == upa_month_min & month_min_pos > alt_upa_month_min))
  )]

  # Determine which month within quarter requires exception
  dt[, `:=`(
    requires_exc_m1 = requires_exception & month_min_pos == 2L & alt_upa_month_min == 1L,
    requires_exc_m2 = requires_exception & ((month_max_pos == 1L & alt_upa_month_max >= 2L) |
                                              (month_min_pos == 3L & alt_upa_month_min <= 2L)),
    requires_exc_m3 = requires_exception & month_max_pos == 2L & alt_upa_month_max == 3L
  )]

  # Propagate exception requirement to entire quarter
  dt[, `:=`(
    trim_exc_m1 = sum(requires_exc_m1, na.rm = TRUE) > 0L,
    trim_exc_m2 = sum(requires_exc_m2, na.rm = TRUE) > 0L,
    trim_exc_m3 = sum(requires_exc_m3, na.rm = TRUE) > 0L
  ), by = .(Ano, Trimestre)]

  # Apply month exception rules
  exc_condition <- (dt$trim_exc_m1 == 1L | dt$trim_exc_m2 == 1L | dt$trim_exc_m3 == 1L)
  has_any_exception <- any(exc_condition)

  if (has_any_exception) {
    if (verbose) cat("  Step 1.7b: Applying exception rules...\n")


    # Update first Saturday values where exceptions are needed
    dt[trim_exc_m1 == 1L, first_sat_m1 := alt_sat_m1]
    dt[trim_exc_m2 == 1L, first_sat_m2 := alt_sat_m2]
    dt[trim_exc_m3 == 1L, first_sat_m3 := alt_sat_m3]


    # Recalculate date bounds for exception rows
    dt[exc_condition, `:=`(
      date_min = lubridate::ymd(paste(Ano, month1, first_sat_m1, sep = "-")),
      date_max = lubridate::ymd(paste(Ano, month3, first_sat_m3, sep = "-")) + 21L
    )]

    # Re-apply birthday constraints
    dt[exc_condition &
         visit_before_birthday == 0L &
         !is.na(first_sat_after_birthday) &
         first_sat_after_birthday > date_min &
         first_sat_after_birthday <= date_max,
       date_min := first_sat_after_birthday]

    dt[exc_condition &
         visit_before_birthday == 1L &
         !is.na(first_sat_after_birthday) &
         (first_sat_after_birthday - 7L) < date_max &
         (first_sat_after_birthday - 7L) >= date_min,
       date_max := first_sat_after_birthday - 7L]

    # Recalculate month positions with dynamic thresholds
    dt[exc_condition, `:=`(
      month_min_pos = calculate_month_position_min_dynamic(date = date_min,
                                                                          year = Ano,
                                                                          quarter = Trimestre,
                                                                          exc_m1 = trim_exc_m1,
                                                                          exc_m2 = trim_exc_m2,
                                                                          exc_m3 = trim_exc_m3),

      month_max_pos = calculate_month_position_max_dynamic(date = date_max,
                                                                          year = Ano,
                                                                          quarter = Trimestre,
                                                                          exc_m1 = trim_exc_m1,
                                                                          exc_m2 = trim_exc_m2,
                                                                          exc_m3 = trim_exc_m3)
    )]

    # Re-aggregate ALL UPA-V1014 combinations
    dt[, `:=`(
      upa_month_min_final = max(month_min_pos, na.rm = TRUE),
      upa_month_max_final = min(month_max_pos, na.rm = TRUE)
    ), by = .(UPA, V1014)]

    # Handle all-NA groups (max returns -Inf, min returns Inf)
    dt[is.infinite(upa_month_min_final), upa_month_min_final := NA_integer_]
    dt[is.infinite(upa_month_max_final), upa_month_max_final := NA_integer_]
  } else {
    # No exceptions - reuse aggregation
    dt[, `:=`(
      upa_month_min_final = upa_month_min,
      upa_month_max_final = upa_month_max
    )]
  }

  # Clean up intermediate columns no longer needed
  cols_to_remove <- c("is_leap", "alt_sat_m1", "alt_sat_m2", "alt_sat_m3",
                      "alt_date_min", "alt_date_max",
                      "first_sat_after_birthday", "visit_before_birthday",
                      "month_min_pos", "month_max_pos",
                      "alt_month_min_pos", "alt_month_max_pos",
                      "upa_month_min", "upa_month_max",
                      "alt_upa_month_min", "alt_upa_month_max",
                      "requires_exception", "requires_exc_m1", "requires_exc_m2", "requires_exc_m3",
                      "trim_exc_m1", "trim_exc_m2", "trim_exc_m3")
  cols_present <- intersect(cols_to_remove, names(dt))
  if (length(cols_present) > 0) {
    dt[, (cols_present) := NULL]
  }

  # --------------------------------------------------------------------------
  # STEP 1.8: MONTH DETERMINATION
  # --------------------------------------------------------------------------

  if (verbose) cat("  Step 1.8: Determining reference months...\n")

  dt[upa_month_min_final == upa_month_max_final &
       upa_month_min_final >= 1L & upa_month_max_final <= 3L,
     ref_month_in_quarter := upa_month_min_final]

  dt[!is.na(ref_month_in_quarter),
      ref_month_in_year := ref_month_in_quarter + (Trimestre - 1)*3
     ]

  # Freeing memory after finishing the month determination...
  gc()

  # Calculate stats
  n_total            <- nrow(dt)
  n_month_determined <- dt[!is.na(ref_month_in_quarter), .N]
  month_rate         <- n_month_determined / n_total

  if (verbose) {
    cat(sprintf("  -> Month determination: %.1f%% (%s of %s observations)\n",
                month_rate * 100,
                format(n_month_determined, big.mark = ","),
                format(n_total, big.mark = ",")))
  }

  # ============================================================================
  # PHASE 2: FORTNIGHT IDENTIFICATION (ONLY month-determined observations)
  # ============================================================================
  #
  # Fortnights are identified ONLY for observations where the month is already

  # determined. The search space is constrained to 2 fortnights within the
  # determined month:
  #   - Month 1 → fortnights 1-2
  #   - Month 2 → fortnights 3-4
  #   - Month 3 → fortnights 5-6
  #
  # ============================================================================

  if (verbose) cat("\nPhase 2: Identifying FORTNIGHTS (within determined months)...\n")

  # Initialize fortnight columns
  dt[, ref_fortnight_in_quarter := NA_integer_]

  # Initialize counter (may be updated in else branch)
  n_fortnight_determined <- 0L

  # Pre-compute IBGE month boundaries for all unique (Ano, Trimestre, ref_month_in_quarter) combinations
  # Created here (outside conditional) so it's available for both Phase 2 and Phase 3
  month_boundaries <- unique(dt[!is.na(ref_month_in_quarter),
            .(Ano,
              Trimestre,
              ref_month_in_quarter,
              month = ref_month_in_quarter + (Trimestre - 1)*3,
              ibge_first_sat_day  = fcase(ref_month_in_quarter == 1, first_sat_m1,
                                          ref_month_in_quarter == 2, first_sat_m2,
                                          ref_month_in_quarter == 3, first_sat_m3))])
  if (nrow(month_boundaries) > 0) {
    month_boundaries[, ibge_month_start := lubridate::ymd(paste(Ano, month, ibge_first_sat_day, sep = "-")) - 6L]
    data.table::setkey(month_boundaries, Ano, Trimestre, ref_month_in_quarter)
  }

  if (n_month_determined == 0) {
    if (verbose) cat("  -> No months determined - skipping fortnight identification\n")
  } else {
    if (verbose) cat(sprintf("  Processing %s observations with determined months...\n",
                             format(n_month_determined, big.mark = ",")))

    # --------------------------------------------------------------------------
    # STEP 2.1: Calculate fortnight bounds WITHIN the determined month
    # --------------------------------------------------------------------------

    if (verbose) cat("  Step 2.1: Calculate fortnight bounds WITHIN the determined month...\n")

    # Pre-compute IBGE fortnight boundaries
    # IBGE reference months are 28-day periods starting on Sunday of the first reference week
    # Fortnight 1: Days 0-13 (weeks 1-2), Fortnight 2: Days 14-27 (weeks 3-4)
    fortnight_boundaries = copy(month_boundaries)

    fortnight_boundaries[ , `:=`(
      fortnight_1_start = ibge_month_start,           # Day 0 of IBGE month
      fortnight_1_end   = ibge_month_start + 13L,     # Day 13 (14 days: weeks 1-2)
      fortnight_2_start = ibge_month_start + 14L,     # Day 14 (start of weeks 3-4)
      fortnight_2_end   = ibge_month_start + 27L      # Day 27 (28 days total)
      )][,
         `:=`(
           month = NULL,
           ibge_first_sat_day = NULL,
           ibge_month_start   = NULL
         )]

    # Join pre-computed fortnight boundary values to main data
    dt[fortnight_boundaries,
       on = .(Ano, Trimestre, ref_month_in_quarter),
       `:=`(fortnight_1_start = i.fortnight_1_start,
            fortnight_1_end   = i.fortnight_1_end,
            fortnight_2_start = i.fortnight_2_start,
            fortnight_2_end   = i.fortnight_2_end)]


    # Adjust date_min and date_max for those with determined months
    dt[!is.na(ref_month_in_quarter),
       `:=`(
         date_min  = fifelse(is.na(date_min), fortnight_1_start, date_min),
         date_max  = fifelse(is.na(date_max), fortnight_2_end,   date_max))]

    dt[!is.na(ref_month_in_quarter),
       `:=`(
         date_min  = fifelse(date_min < fortnight_1_start, fortnight_1_start, date_min),
         date_max  = fifelse(date_max > fortnight_2_end,   fortnight_2_end,   date_max)
       )]


    # Identifying the fortnight for individuals
    dt[!is.na(ref_month_in_quarter),
       `:=`(
         fortnight_pos = fcase(
           (date_min >= fortnight_1_start) & (date_max <= fortnight_1_end), 1,
           (date_min >= fortnight_2_start) & (date_max <= fortnight_2_end), 2,
           default = NA_real_)
    )]

    # Freeing memory after this step...
    gc()

    # --------------------------------------------------------------------------
    # STEP 2.2: FORTNIGHT aggregation at household-quarter level
    # --------------------------------------------------------------------------

    if (verbose) cat("  Step 2.2: Aggregating at household level (within quarter)...\n")

    # Aggregate ONLY for month-determined rows
    dt[!is.na(ref_month_in_quarter), `:=`(
      hh_fortnight_max     = max(fortnight_pos,     na.rm = TRUE),
      hh_fortnight_min     = min(fortnight_pos,     na.rm = TRUE)
    ), by = .(Ano, Trimestre, UPA, V1008)]

    # Handle all-NA groups (max returns -Inf, min returns Inf)
    dt[is.infinite(hh_fortnight_max), hh_fortnight_max := NA_integer_]
    dt[is.infinite(hh_fortnight_min), hh_fortnight_min := NA_integer_]

    # --------------------------------------------------------------------------
    # STEP 2.3: FORTNIGHT DETERMINATION
    # --------------------------------------------------------------------------

    if (verbose) cat("  Step 2.3: Determining reference fortnights...\n")

    dt[!is.na(ref_month_in_quarter) &
         hh_fortnight_min == hh_fortnight_max,
       `:=`(
         ref_fortnight_in_month   = hh_fortnight_min,
         ref_fortnight_in_quarter = hh_fortnight_min + (ref_month_in_quarter - 1)*2)
       ]

    # Calculate stats
    n_fortnight_determined <- dt[!is.na(ref_fortnight_in_quarter), .N]
    fortnight_rate         <- n_fortnight_determined / n_total

    if (verbose) {
      cat(sprintf("  -> Fortnight determination: %.1f%% (%s of %s observations)\n",
                  fortnight_rate * 100,
                  format(n_fortnight_determined, big.mark = ","),
                  format(n_total, big.mark = ",")))
    }

    # Clean up intermediate fortnight columns no longer needed
    # (hh_fortnight_max/min are kept for store_date_bounds option)
    cols_to_remove <- c("fortnight_pos", "fortnight_1_start", "fortnight_1_end",
                        "fortnight_2_start", "fortnight_2_end")
    cols_present <- intersect(cols_to_remove, names(dt))
    if (length(cols_present) > 0) {
      dt[, (cols_present) := NULL]
    }
  }

  # ============================================================================
  # PHASE 3: WEEK IDENTIFICATION (ONLY fortnight-determined observations)
  # ============================================================================
  #
  # Weeks are identified ONLY for observations where the fortnight is already
  # determined. The search space is constrained to ~2 weeks within the
  # determined fortnight (fortnights typically span 2-3 ISO weeks).
  #
  # ============================================================================

  if (verbose) cat("\nPhase 3: Identifying WEEKS (within determined fortnights)...\n")


  if (n_fortnight_determined == 0) {
    if (verbose) cat("  -> No fortnights determined - skipping week identification\n")
  } else {
    if (verbose) cat(sprintf("  Processing %s observations with determined fortnights...\n",
                             format(n_fortnight_determined, big.mark = ",")))

    # --------------------------------------------------------------------------
    # STEP 3.1: Calculate week bounds WITHIN the determined fortnight
    # --------------------------------------------------------------------------

    # Pre-compute IBGE week boundaries
    # IBGE reference months are 28-day periods (4 weeks x 7 days), starting Sunday
    # Each week runs Sunday (day N) to Saturday (day N+6)
    weeks_boundaries = copy(month_boundaries)

    weeks_boundaries[ , `:=`(
      week_1_start = ibge_month_start,          # Day 0: Week 1 Sunday
      week_1_end   = ibge_month_start + 6L,     # Day 6: Week 1 Saturday

      week_2_start = ibge_month_start + 7L,     # Day 7: Week 2 Sunday
      week_2_end   = ibge_month_start + 13L,    # Day 13: Week 2 Saturday

      week_3_start = ibge_month_start + 14L,    # Day 14: Week 3 Sunday
      week_3_end   = ibge_month_start + 20L,    # Day 20: Week 3 Saturday

      week_4_start = ibge_month_start + 21L,    # Day 21: Week 4 Sunday
      week_4_end   = ibge_month_start + 27L     # Day 27: Week 4 Saturday
    )][,
       `:=`(
         month = NULL,
         ibge_first_sat_day = NULL,
         ibge_month_start   = NULL
       )]

    # Join pre-computed fortnight boundary values to main data
    dt[weeks_boundaries,
       on = .(Ano, Trimestre, ref_month_in_quarter),
       `:=`(
         week_1_start = i.week_1_start,
         week_1_end   = i.week_1_end,
         week_2_start = i.week_2_start,
         week_2_end   = i.week_2_end,
         week_3_start = i.week_3_start,
         week_3_end   = i.week_3_end,
         week_4_start = i.week_4_start,
         week_4_end   = i.week_4_end
       )]

    # Adjust date_min and date_max for those with determined months
    dt[!is.na(ref_fortnight_in_month),
       `:=`(
         date_min  = fifelse(ref_fortnight_in_month == 1L & is.na(date_min), week_1_start, date_min),
         date_max  = fifelse(ref_fortnight_in_month == 1L & is.na(date_max), week_2_end,   date_max))]

    dt[!is.na(ref_fortnight_in_month),
       `:=`(
         date_min  = fifelse(ref_fortnight_in_month == 2L & is.na(date_min), week_3_start, date_min),
         date_max  = fifelse(ref_fortnight_in_month == 2L & is.na(date_max), week_4_end,   date_max)
         )]

    dt[!is.na(ref_fortnight_in_month),
       `:=`(
         date_min  = fifelse(ref_fortnight_in_month == 1L & date_min < week_1_start, week_1_start, date_min),
         date_max  = fifelse(ref_fortnight_in_month == 1L & date_max > week_2_end,   week_2_end,   date_max)
       )]

    dt[!is.na(ref_fortnight_in_month),
       `:=`(
         date_min  = fifelse(ref_fortnight_in_month == 2L & date_min < week_3_start, week_3_start, date_min),
         date_max  = fifelse(ref_fortnight_in_month == 2L & date_max > week_4_end,   week_4_end,   date_max)
         )]

    # Identifying the weeks for individuals
    dt[!is.na(ref_fortnight_in_quarter),
       `:=`(
         week_pos = fcase(
           (ref_fortnight_in_month == 1L) &  (date_min >= week_1_start) & (date_max <= week_1_end), 1,
           (ref_fortnight_in_month == 1L) &  (date_min >= week_2_start) & (date_max <= week_2_end), 2,
           (ref_fortnight_in_month == 2L) &  (date_min >= week_3_start) & (date_max <= week_3_end), 3,
           (ref_fortnight_in_month == 2L) &  (date_min >= week_4_start) & (date_max <= week_4_end), 4,
           default = NA_real_)
       )]

    # Freeing memory after this step...
    gc()

    # --------------------------------------------------------------------------
    # STEP 3.2: WEEK aggregation at household-quarter level
    # --------------------------------------------------------------------------

    if (verbose) cat("  Step 3.2: Aggregating at household level (within quarter)...\n")

    dt[!is.na(ref_fortnight_in_quarter), `:=`(
      hh_week_max = max(week_pos, na.rm = TRUE),
      hh_week_min = min(week_pos, na.rm = TRUE)
    ), by = .(Ano, Trimestre, UPA, V1008)]

    # Handle all-NA groups (max returns -Inf, min returns Inf)
    dt[is.infinite(hh_week_max), hh_week_max := NA_integer_]
    dt[is.infinite(hh_week_min), hh_week_min := NA_integer_]

    # --------------------------------------------------------------------------
    # STEP 3.4: WEEK DETERMINATION (using sequential values for comparison)
    # --------------------------------------------------------------------------

    if (verbose) cat("  Step 3.4: Determining reference weeks...\n")

    # Determine if min == max using sequential values (handles year boundaries)
    dt[!is.na(ref_fortnight_in_quarter) &
         hh_week_min == hh_week_max,
       `:=`(
         ref_week_in_month   = hh_week_min,
         ref_week_in_quarter = hh_week_min + (ref_month_in_quarter - 1)*4)]


    # Calculate stats
    n_week_determined <- dt[!is.na(ref_week_in_month), .N]
    week_rate         <- n_week_determined / n_total

    if (verbose) {
      cat(sprintf("  -> Week determination: %.1f%% (%s of %s observations)\n",
                  week_rate * 100,
                  format(n_week_determined, big.mark = ","),
                  format(n_total, big.mark = ",")))
    }

    # Clean up intermediate week column no longer needed
    # (hh_week_max/min and week_*_start/end are kept for store_date_bounds option)
    if ("week_pos" %in% names(dt)) {
      dt[, week_pos := NULL]
    }
  }

  # ============================================================================
  # PHASE 4: BUILD CROSSWALK
  # ============================================================================

  if (verbose) cat("\nPhase 4: Building crosswalk...\n")


  # Build crosswalk at household-quarter level
  if (!store_date_bounds) {

  dt = dt[,
     .(Ano, Trimestre, UPA, V1008, V1014,

       ref_month_in_quarter,
       ref_month_in_year,

       ref_fortnight_in_month,
       ref_fortnight_in_quarter,

       ref_week_in_month,
       ref_week_in_quarter)
     ]

  } else {
    # Build column list dynamically to handle cases where phases were skipped
    # Base columns that always exist
    cols_to_keep <- c("Ano", "Trimestre", "UPA", "V1008", "V1014",
                      "ref_month_in_quarter", "ref_month_in_year",
                      "ref_fortnight_in_month", "ref_fortnight_in_quarter",
                      "ref_week_in_month", "ref_week_in_quarter",
                      "date_max", "date_min",
                      "upa_month_max_final", "upa_month_min_final")

    # Conditionally add fortnight bounds if they exist
    if ("hh_fortnight_max" %in% names(dt)) {
      cols_to_keep <- c(cols_to_keep, "hh_fortnight_max", "hh_fortnight_min")
    }

    # Conditionally add week bounds if they exist
    if ("hh_week_max" %in% names(dt)) {
      cols_to_keep <- c(cols_to_keep, "hh_week_max", "hh_week_min",
                        "week_1_start", "week_1_end",
                        "week_2_start", "week_2_end",
                        "week_3_start", "week_3_end",
                        "week_4_start", "week_4_end")
    }

    dt <- dt[, ..cols_to_keep]

    # Rename columns for output
    data.table::setnames(dt,
                         old = c("upa_month_max_final", "upa_month_min_final"),
                         new = c("month_max_upa", "month_min_upa"))
    if ("hh_fortnight_max" %in% names(dt)) {
      data.table::setnames(dt,
                           old = c("hh_fortnight_max", "hh_fortnight_min"),
                           new = c("fortnight_max_hh", "fortnight_min_hh"))
    }
    if ("hh_week_max" %in% names(dt)) {
      data.table::setnames(dt,
                           old = c("hh_week_max", "hh_week_min"),
                           new = c("week_max_hh", "week_min_hh"))
    }
  }

  dt[, `:=`(

    ref_month_yyyymm     = Ano * 100 + ref_month_in_year,
    ref_fortnight_yyyyff = Ano * 100 + (ref_fortnight_in_quarter + (Trimestre - 1)*6),
    ref_week_yyyyww      = Ano * 100 + (ref_week_in_month + (ref_month_in_year - 1)*4),

    determined_month     = !is.na(ref_month_in_quarter),
    determined_fortnight = !is.na(ref_fortnight_in_quarter),
    determined_week      = !is.na(ref_week_in_quarter)
    )]


  gc()

  # ============================================================================
  # SUMMARY
  # ============================================================================

  if (verbose) {
    cat("\n========================================\n")
    cat("Crosswalk complete (nested identification):\n")
    cat("========================================\n")
  }

  dt
}

# Tests for experimental period identification strategies
# Tests for pnadc_experimental_periods() - probabilistic and UPA aggregation strategies

# =============================================================================
# PROBABILISTIC STRATEGY TESTS
# =============================================================================

test_that("probabilistic strategy requires store_date_bounds=TRUE", {
  # 1. Setup: Create crosswalk WITHOUT date bounds
  set.seed(400)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = FALSE)

  # 2. Verify: Should error when date bounds not stored
  # Error can be about missing columns like date_min, date_max, or week columns
  expect_error(
    pnadc_experimental_periods(
      crosswalk,
      strategy = "probabilistic",
      verbose = FALSE
    ),
    label = "Probabilistic strategy requires date bounds"
  )

  # 3. Context: Probabilistic strategy needs date ranges to calculate confidence
})


test_that("probabilistic strategy improves month determination", {
  # 1. Setup: Create data with date bounds
  set.seed(401)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # Store strict determination rate
  strict_rate <- crosswalk[, mean(determined_month, na.rm = TRUE)]

  # 2. Execute: Apply probabilistic strategy
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  # Calculate experimental rate
  exp_rate <- result[, mean(determined_month, na.rm = TRUE)]

  # 3. Verify: Experimental rate should be >= strict rate
  expect_true(exp_rate >= strict_rate,
              label = paste0("Probabilistic should maintain or improve rate: ",
                             "strict=", round(strict_rate, 3), ", exp=", round(exp_rate, 3)))

  # 4. Context: Probabilistic adds determinations, never removes strict ones
})


test_that("confidence_threshold parameter affects determination", {
  # 1. Setup: Create data with store_date_bounds
  set.seed(402)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Test with different confidence thresholds
  result_08 <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.8,
    verbose = FALSE
  )

  result_09 <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  result_095 <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.95,
    verbose = FALSE
  )

  # 3. Verify: Lower threshold should give >= determinations than higher threshold
  rate_08 <- result_08[, mean(determined_month, na.rm = TRUE)]
  rate_09 <- result_09[, mean(determined_month, na.rm = TRUE)]
  rate_095 <- result_095[, mean(determined_month, na.rm = TRUE)]

  expect_true(rate_08 >= rate_09 - 0.01,  # Allow tiny tolerance
              label = paste0("threshold=0.8 (", round(rate_08, 3),
                             ") should be >= threshold=0.9 (", round(rate_09, 3), ")"))

  expect_true(rate_09 >= rate_095 - 0.01,
              label = paste0("threshold=0.9 (", round(rate_09, 3),
                             ") should be >= threshold=0.95 (", round(rate_095, 3), ")"))

  # 4. Context: More conservative thresholds assign fewer probabilistic months
})


test_that("probabilistic strategy preserves strict determinations", {
  # 1. Setup: Create data
  set.seed(403)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # Identify strict determinations
  strict_determined <- crosswalk[determined_month == TRUE]

  # 2. Execute: Apply probabilistic
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  # 3. Verify: All strict determinations must be preserved
  result_determined <- result[determined_month == TRUE]

  # All strict IDs should still be determined in result
  # Note: Crosswalk is at household level; V2003 is not in the crosswalk output
  strict_ids <- paste(strict_determined$Ano, strict_determined$Trimestre,
                      strict_determined$UPA, strict_determined$V1008,
                      strict_determined$V1014, sep = "_")

  result_ids <- paste(result_determined$Ano, result_determined$Trimestre,
                      result_determined$UPA, result_determined$V1008,
                      result_determined$V1014, sep = "_")

  all_preserved <- all(strict_ids %in% result_ids)

  expect_true(all_preserved,
              label = "Probabilistic must preserve all strict determinations")

  # 4. Context: Experimental strategies only ADD, never remove determinations
})


test_that("probabilistic assignment only for 2-period ranges", {
  # 1. Setup: Create data
  set.seed(404)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Apply probabilistic
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.8,
    verbose = FALSE
  )

  # 3. Verify: Check the experimental assignment pattern
  # Probabilistic only assigns when there's ambiguity between 2 periods
  # This is verified indirectly - the function should not error

  expect_true(is.data.frame(result))
  expect_true("determined_month" %in% names(result))

  # 4. Context: Probabilistic works on 2-month ranges (implementation detail)
  # Single-month or 3-month ranges are not probabilistically assigned
})


# =============================================================================
# UPA AGGREGATION STRATEGY TESTS
# =============================================================================

test_that("UPA aggregation extends determinations via consensus", {
  # 1. Setup: Create data
  set.seed(500)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # Store strict rate
  strict_rate <- crosswalk[, mean(determined_month, na.rm = TRUE)]

  # 2. Execute: Apply UPA aggregation
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  # Calculate rate after UPA aggregation
  upa_rate <- result[, mean(determined_month, na.rm = TRUE)]

  # 3. Verify: UPA aggregation should maintain or improve rate
  expect_true(upa_rate >= strict_rate,
              label = paste0("UPA aggregation maintains or improves: ",
                             "strict=", round(strict_rate, 3), ", upa=", round(upa_rate, 3)))

  # 4. Context: UPA aggregation extends via UPA-level consensus
})


test_that("upa_proportion_threshold affects determination", {
  # 1. Setup: Create data
  set.seed(501)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Test with different thresholds
  result_05 <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  result_08 <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.8,
    verbose = FALSE
  )

  # 3. Verify: Lower threshold should give >= determinations
  rate_05 <- result_05[, mean(determined_month, na.rm = TRUE)]
  rate_08 <- result_08[, mean(determined_month, na.rm = TRUE)]

  expect_true(rate_05 >= rate_08 - 0.01,
              label = paste0("threshold=0.5 (", round(rate_05, 3),
                             ") should be >= threshold=0.8 (", round(rate_08, 3), ")"))

  # 4. Context: Higher proportion threshold requires stronger UPA consensus
})


# =============================================================================
# "BOTH" STRATEGY TESTS
# =============================================================================

test_that("'both' strategy combines probabilistic and UPA aggregation", {
  # 1. Setup: Create data
  set.seed(600)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Apply each strategy separately and combined
  result_prob <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  result_upa <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  result_both <- pnadc_experimental_periods(
    crosswalk,
    strategy = "both",
    confidence_threshold = 0.9,
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  # 3. Verify: "both" should be >= max(individual strategies)
  rate_prob <- result_prob[, mean(determined_month, na.rm = TRUE)]
  rate_upa <- result_upa[, mean(determined_month, na.rm = TRUE)]
  rate_both <- result_both[, mean(determined_month, na.rm = TRUE)]

  max_individual <- max(rate_prob, rate_upa)

  expect_true(rate_both >= max_individual - 0.01,  # Allow tiny tolerance
              label = paste0("'both' (", round(rate_both, 3),
                             ") should be >= max(individual) (", round(max_individual, 3), ")"))

  # 4. Context: Combined strategy should identify at least as many as either alone
})


test_that("'both' strategy preserves strict determinations", {
  # 1. Setup: Create data
  set.seed(601)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # Count strict determinations
  n_strict <- crosswalk[, sum(determined_month, na.rm = TRUE)]

  # 2. Execute: Apply "both" strategy
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "both",
    verbose = FALSE
  )

  # Count experimental determinations
  n_exp <- result[, sum(determined_month, na.rm = TRUE)]

  # 3. Verify: Should have at least as many determinations as strict
  expect_true(n_exp >= n_strict,
              label = "Combined strategy must preserve strict determinations")

  # 4. Context: All experimental strategies are additive only
})


# =============================================================================
# NESTING PRESERVATION TESTS
# =============================================================================

test_that("experimental strategies maintain nesting (week requires fortnight)", {
  # 1. Setup: Create data
  set.seed(700)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Apply experimental strategy
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "both",
    verbose = FALSE
  )

  # 3. Verify: Nesting violation check
  # All week-determined must have fortnight-determined
  violation <- result[determined_week == TRUE & (is.na(determined_fortnight) | determined_fortnight == FALSE)]

  expect_equal(nrow(violation), 0,
               label = "Week determination requires fortnight determination (nesting)")

  # 4. Context: Nesting is a fundamental invariant
})


test_that("experimental strategies maintain nesting (fortnight requires month)", {
  # 1. Setup: Create data
  set.seed(701)
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Apply experimental strategy
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "both",
    verbose = FALSE
  )

  # 3. Verify: Nesting violation check
  # All fortnight-determined must have month-determined
  violation <- result[determined_fortnight == TRUE & (is.na(determined_month) | determined_month == FALSE)]

  expect_equal(nrow(violation), 0,
               label = "Fortnight determination requires month determination (nesting)")
})


# =============================================================================
# EDGE CASES
# =============================================================================

test_that("experimental strategies handle small data sets", {
  # 1. Setup: Create data with enough observations for experimental strategies
  set.seed(800)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Should not error with reasonable data size
  expect_no_error({
    result <- pnadc_experimental_periods(
      crosswalk,
      strategy = "both",
      verbose = FALSE
    )
  })

  # 3. Context: Implementation should handle various data sizes gracefully
})


test_that("experimental strategies work with multi-quarter data", {
  # 1. Setup: Use multi-quarter data for experimental strategies
  # Single quarter may not have enough variation for experimental strategies
  set.seed(801)
  data <- create_realistic_pnadc(n_quarters = 3, n_upas = 15)

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE, store_date_bounds = TRUE)

  # 2. Execute: Should work with sufficient data
  expect_no_error({
    result <- pnadc_experimental_periods(
      crosswalk,
      strategy = "both",
      verbose = FALSE
    )
  })

  # 3. Verify: Basic structure maintained
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), nrow(crosswalk))

  # 4. Context: Experimental strategies need sufficient data variation
})

# Tests for IBGE Period Calendar Invariants
#
# These tests verify that period identification functions NEVER produce
# invalid IBGE calendar values. All invariants must hold by construction.
#
# IBGE Calendar Invariants:
# - Weeks per month: Always exactly 4
# - Fortnights per month: Always exactly 2
# - Weeks per quarter: Always exactly 12
# - Fortnights per quarter: Always exactly 6
# - Fortnight 1 composition: Weeks 1 + 2 of the month
# - Fortnight 2 composition: Weeks 3 + 4 of the month
# - Week uniqueness: No week belongs to multiple months
# - Fortnight uniqueness: No fortnight spans multiple months
# - Valid ranges: ref_week_in_quarter in [1,12], ref_fortnight_in_quarter in [1,6],
#                 ref_month_in_quarter in [1,3]

# =============================================================================
# HELPER FUNCTIONS
# =============================================================================

#' Create test crosswalk with specific invariant violations for testing
#' @param violation_type Type of violation to introduce
create_crosswalk_with_violation <- function(violation_type = c(
  "invalid_week", "invalid_fortnight", "invalid_month",
  "week_without_fortnight", "fortnight_without_month",
  "week_fortnight_mismatch", "fortnight_month_mismatch",
  "week_month_mismatch"
)) {
  violation_type <- match.arg(violation_type)

  # Create a minimal valid crosswalk
  crosswalk <- data.table::data.table(
    Ano = rep(2023L, 10),
    Trimestre = rep(1L, 10),
    UPA = 1:10,
    V1008 = rep(1L, 10),
    V1014 = rep(1L, 10),
    ref_month_in_quarter = c(1L, 1L, 2L, 2L, 3L, 3L, NA_integer_, NA_integer_, 1L, 2L),
    ref_fortnight_in_quarter = c(1L, 2L, 3L, 4L, 5L, 6L, NA_integer_, NA_integer_, NA_integer_, 4L),
    ref_week_in_quarter = c(1L, 4L, 5L, 8L, 9L, 12L, NA_integer_, NA_integer_, NA_integer_, NA_integer_)
  )

  switch(violation_type,
    "invalid_week" = {
      crosswalk[1, ref_week_in_quarter := 13L]  # Invalid: > 12
    },
    "invalid_fortnight" = {
      crosswalk[1, ref_fortnight_in_quarter := 7L]  # Invalid: > 6
    },
    "invalid_month" = {
      crosswalk[1, ref_month_in_quarter := 4L]  # Invalid: > 3
    },
    "week_without_fortnight" = {
      crosswalk[1, ref_fortnight_in_quarter := NA_integer_]  # Nesting violation
    },
    "fortnight_without_month" = {
      crosswalk[1, ref_month_in_quarter := NA_integer_]  # Nesting violation
    },
    "week_fortnight_mismatch" = {
      # Week 1-2 should be in fortnight 1, but we put week 1 in fortnight 2
      crosswalk[1, `:=`(ref_week_in_quarter = 1L, ref_fortnight_in_quarter = 2L)]
    },
    "fortnight_month_mismatch" = {
      # Fortnight 1-2 should be in month 1, but we put fortnight 1 in month 2
      crosswalk[1, `:=`(ref_fortnight_in_quarter = 1L, ref_month_in_quarter = 2L)]
    },
    "week_month_mismatch" = {
      # Week 1-4 should be in month 1, but we put week 1 in month 2
      crosswalk[1, `:=`(ref_week_in_quarter = 1L, ref_month_in_quarter = 2L)]
    }
  )

  crosswalk
}

# =============================================================================
# VALIDATION FUNCTION TESTS
# =============================================================================

test_that("validate_period_invariants detects invalid week values", {
  crosswalk <- create_crosswalk_with_violation("invalid_week")

  # Should fail with strict=TRUE
  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "ref_week_in_quarter must be 1-12"
  )

  # Should return violations with strict=FALSE
  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("invalid_week_values" %in% names(result$violations))
})

test_that("validate_period_invariants detects invalid fortnight values", {
  crosswalk <- create_crosswalk_with_violation("invalid_fortnight")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "ref_fortnight_in_quarter must be 1-6"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("invalid_fortnight_values" %in% names(result$violations))
})

test_that("validate_period_invariants detects invalid month values", {
  crosswalk <- create_crosswalk_with_violation("invalid_month")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "ref_month_in_quarter must be 1-3"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("invalid_month_values" %in% names(result$violations))
})

test_that("validate_period_invariants detects week without fortnight", {
  crosswalk <- create_crosswalk_with_violation("week_without_fortnight")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "Nesting violation.*week determined but fortnight is NA"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("week_without_fortnight" %in% names(result$violations))
})

test_that("validate_period_invariants detects fortnight without month", {
  crosswalk <- create_crosswalk_with_violation("fortnight_without_month")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "Nesting violation.*fortnight determined but month is NA"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("fortnight_without_month" %in% names(result$violations))
})

test_that("validate_period_invariants detects week-fortnight mismatch", {
  crosswalk <- create_crosswalk_with_violation("week_fortnight_mismatch")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "Week-fortnight inconsistency"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("week_fortnight_mismatch" %in% names(result$violations))
})

test_that("validate_period_invariants detects fortnight-month mismatch", {
  crosswalk <- create_crosswalk_with_violation("fortnight_month_mismatch")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "Fortnight-month inconsistency"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("fortnight_month_mismatch" %in% names(result$violations))
})

test_that("validate_period_invariants detects week-month mismatch", {
  crosswalk <- create_crosswalk_with_violation("week_month_mismatch")

  expect_error(
    validate_period_invariants(crosswalk, strict = TRUE),
    regexp = "Week-month inconsistency"
  )

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_false(result$valid)
  expect_true("week_month_mismatch" %in% names(result$violations))
})

test_that("validate_period_invariants passes for valid crosswalk", {
  # Create a valid crosswalk
  crosswalk <- data.table::data.table(
    Ano = rep(2023L, 6),
    Trimestre = rep(1L, 6),
    UPA = 1:6,
    V1008 = rep(1L, 6),
    V1014 = rep(1L, 6),
    ref_month_in_quarter = c(1L, 1L, 2L, 2L, 3L, NA_integer_),
    ref_fortnight_in_quarter = c(1L, 2L, 3L, 4L, NA_integer_, NA_integer_),
    ref_week_in_quarter = c(1L, 4L, 5L, NA_integer_, NA_integer_, NA_integer_)
  )

  # Should pass without error
  expect_silent(validate_period_invariants(crosswalk, strict = TRUE))

  result <- validate_period_invariants(crosswalk, strict = FALSE)
  expect_true(result$valid)
  expect_length(result$violations, 0)
})

# =============================================================================
# STRICT IDENTIFICATION INVARIANT TESTS
# =============================================================================

test_that("strict identification never produces invalid week values", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check ref_week_in_quarter is always 1-12 or NA
  invalid_weeks <- result[!is.na(ref_week_in_quarter) &
                           (ref_week_in_quarter < 1L | ref_week_in_quarter > 12L)]
  expect_equal(nrow(invalid_weeks), 0L,
               info = paste("Found", nrow(invalid_weeks), "invalid week values"))
})

test_that("strict identification never produces invalid fortnight values", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check ref_fortnight_in_quarter is always 1-6 or NA
  invalid_fortnights <- result[!is.na(ref_fortnight_in_quarter) &
                                 (ref_fortnight_in_quarter < 1L | ref_fortnight_in_quarter > 6L)]
  expect_equal(nrow(invalid_fortnights), 0L,
               info = paste("Found", nrow(invalid_fortnights), "invalid fortnight values"))
})

test_that("strict identification never produces invalid month values", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check ref_month_in_quarter is always 1-3 or NA
  invalid_months <- result[!is.na(ref_month_in_quarter) &
                            (ref_month_in_quarter < 1L | ref_month_in_quarter > 3L)]
  expect_equal(nrow(invalid_months), 0L,
               info = paste("Found", nrow(invalid_months), "invalid month values"))
})

test_that("strict identification maintains week-fortnight consistency", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check: week 1-2 -> fortnight 1, week 3-4 -> fortnight 2, etc.
  # Expected fortnight = ((week - 1) %/% 2) + 1
  inconsistent <- result[
    !is.na(ref_week_in_quarter) & !is.na(ref_fortnight_in_quarter) &
      (((ref_week_in_quarter - 1L) %/% 2L) + 1L) != ref_fortnight_in_quarter
  ]
  expect_equal(nrow(inconsistent), 0L,
               info = paste("Found", nrow(inconsistent), "week-fortnight inconsistencies"))
})

test_that("strict identification maintains fortnight-month consistency", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check: fortnight 1-2 -> month 1, fortnight 3-4 -> month 2, fortnight 5-6 -> month 3
  # Expected month = ((fortnight - 1) %/% 2) + 1
  inconsistent <- result[
    !is.na(ref_fortnight_in_quarter) & !is.na(ref_month_in_quarter) &
      (((ref_fortnight_in_quarter - 1L) %/% 2L) + 1L) != ref_month_in_quarter
  ]
  expect_equal(nrow(inconsistent), 0L,
               info = paste("Found", nrow(inconsistent), "fortnight-month inconsistencies"))
})

test_that("strict identification maintains week-month consistency", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check: weeks 1-4 -> month 1, weeks 5-8 -> month 2, weeks 9-12 -> month 3
  # Expected month = ((week - 1) %/% 4) + 1
  inconsistent <- result[
    !is.na(ref_week_in_quarter) & !is.na(ref_month_in_quarter) &
      (((ref_week_in_quarter - 1L) %/% 4L) + 1L) != ref_month_in_quarter
  ]
  expect_equal(nrow(inconsistent), 0L,
               info = paste("Found", nrow(inconsistent), "week-month inconsistencies"))
})

# =============================================================================
# EXPERIMENTAL STRATEGIES INVARIANT TESTS
# =============================================================================

test_that("experimental strategies never produce invalid week values", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE, store_date_bounds = TRUE)

  # Test probabilistic strategy
  result_prob <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  # Check ref_week_in_month is always 1-4 or NA (experimental strategies update main columns)
  invalid_weeks <- result_prob[!is.na(ref_week_in_month) &
                                 (ref_week_in_month < 1L | ref_week_in_month > 4L)]
  expect_equal(nrow(invalid_weeks), 0L,
               info = paste("Probabilistic strategy produced", nrow(invalid_weeks), "invalid week values"))

  # Test UPA aggregation strategy
  result_upa <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  invalid_weeks_upa <- result_upa[!is.na(ref_week_in_month) &
                                    (ref_week_in_month < 1L | ref_week_in_month > 4L)]
  expect_equal(nrow(invalid_weeks_upa), 0L,
               info = paste("UPA aggregation produced", nrow(invalid_weeks_upa), "invalid week values"))
})

test_that("experimental strategies never produce invalid fortnight values", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE, store_date_bounds = TRUE)

  # Test probabilistic strategy
  result_prob <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  # Check ref_fortnight_in_month is always 1-2 or NA (experimental strategies update main columns)
  invalid_fortnights <- result_prob[!is.na(ref_fortnight_in_month) &
                                      (ref_fortnight_in_month < 1L | ref_fortnight_in_month > 2L)]
  expect_equal(nrow(invalid_fortnights), 0L,
               info = paste("Probabilistic strategy produced", nrow(invalid_fortnights),
                           "invalid fortnight values"))

  # Test UPA aggregation strategy
  result_upa <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  invalid_fortnights_upa <- result_upa[!is.na(ref_fortnight_in_month) &
                                         (ref_fortnight_in_month < 1L | ref_fortnight_in_month > 2L)]
  expect_equal(nrow(invalid_fortnights_upa), 0L,
               info = paste("UPA aggregation produced", nrow(invalid_fortnights_upa),
                           "invalid fortnight values"))
})

test_that("experimental strategies never produce invalid month values", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE, store_date_bounds = TRUE)

  # Test probabilistic strategy
  result_prob <- pnadc_experimental_periods(
    crosswalk,
    strategy = "probabilistic",
    confidence_threshold = 0.9,
    verbose = FALSE
  )

  # Check ref_month_in_quarter is always 1-3 or NA (experimental strategies update main columns)
  invalid_months <- result_prob[!is.na(ref_month_in_quarter) &
                                  (ref_month_in_quarter < 1L | ref_month_in_quarter > 3L)]
  expect_equal(nrow(invalid_months), 0L,
               info = paste("Probabilistic strategy produced", nrow(invalid_months),
                           "invalid month values"))

  # Test UPA aggregation strategy
  result_upa <- pnadc_experimental_periods(
    crosswalk,
    strategy = "upa_aggregation",
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  invalid_months_upa <- result_upa[!is.na(ref_month_in_quarter) &
                                     (ref_month_in_quarter < 1L | ref_month_in_quarter > 3L)]
  expect_equal(nrow(invalid_months_upa), 0L,
               info = paste("UPA aggregation produced", nrow(invalid_months_upa),
                           "invalid month values"))
})

test_that("combined strategy maintains all invariants", {
  test_data <- create_stacked_pnadc(n_quarters = 4, n_upas = 15)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE, store_date_bounds = TRUE)

  # Test combined "both" strategy
  result <- pnadc_experimental_periods(
    crosswalk,
    strategy = "both",
    confidence_threshold = 0.9,
    upa_proportion_threshold = 0.5,
    verbose = FALSE
  )

  # Check all ranges (experimental strategies update main columns)
  expect_equal(
    nrow(result[!is.na(ref_month_in_quarter) & (ref_month_in_quarter < 1L | ref_month_in_quarter > 3L)]),
    0L,
    info = "Invalid month values in combined strategy"
  )
  expect_equal(
    nrow(result[!is.na(ref_fortnight_in_month) & (ref_fortnight_in_month < 1L | ref_fortnight_in_month > 2L)]),
    0L,
    info = "Invalid fortnight values in combined strategy"
  )
  expect_equal(
    nrow(result[!is.na(ref_week_in_month) & (ref_week_in_month < 1L | ref_week_in_month > 4L)]),
    0L,
    info = "Invalid week values in combined strategy"
  )
})

# =============================================================================
# EDGE CASE TESTS
# =============================================================================

test_that("invariants hold for single-quarter data", {
  test_data <- create_stacked_pnadc(n_quarters = 1, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # All invariants should still hold
  result_check <- validate_period_invariants(result, strict = FALSE)
  expect_true(result_check$valid,
              info = paste("Violations found:",
                          paste(names(result_check$violations), collapse = ", ")))
})

test_that("invariants hold for many-quarter data", {
  test_data <- create_stacked_pnadc(n_quarters = 12, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # All invariants should still hold
  result_check <- validate_period_invariants(result, strict = FALSE)
  expect_true(result_check$valid,
              info = paste("Violations found:",
                          paste(names(result_check$violations), collapse = ", ")))
})

test_that("invariants hold per quarter", {
  test_data <- create_stacked_pnadc(n_quarters = 8, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check invariants per quarter
  for (q in unique(result$Trimestre)) {
    quarter_data <- result[Trimestre == q]
    result_check <- validate_period_invariants(quarter_data, strict = FALSE)
    expect_true(result_check$valid,
                info = paste("Quarter", q, "violations:",
                            paste(names(result_check$violations), collapse = ", ")))
  }
})

# Tests for nesting enforcement in period identification
#
# These tests verify the fundamental guarantee of the nested algorithm:
# - Fortnights can ONLY be determined for observations with determined months
# - Weeks can ONLY be determined for observations with determined fortnights
#
# This nesting is enforced BY CONSTRUCTION in the algorithm, not by post-hoc cleanup.

# =============================================================================
# HELPER FUNCTIONS
# =============================================================================

# Note: Using shared test data generators from helper-test-data.R
# - create_realistic_pnadc() for test data generation

# =============================================================================
# NESTING ENFORCEMENT TESTS
# =============================================================================

test_that("fortnight determination requires month determination", {
  # This is the KEY test: no observation should have determined_fortnight = TRUE

  # while determined_month = FALSE
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Count violations
  n_violations <- sum(result$determined_fortnight & !result$determined_month, na.rm = TRUE)

  expect_equal(
    n_violations, 0L,
    info = paste(
      "Found", n_violations, "observations with determined fortnight but undetermined month.",
      "This violates the nesting requirement."
    )
  )
})

test_that("week determination requires fortnight determination", {
  # No observation should have determined_week = TRUE while determined_fortnight = FALSE
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Count violations
  n_violations <- sum(result$determined_week & !result$determined_fortnight, na.rm = TRUE)

  expect_equal(
    n_violations, 0L,
    info = paste(
      "Found", n_violations, "observations with determined week but undetermined fortnight.",
      "This violates the nesting requirement."
    )
  )
})

test_that("week determination implies month determination (transitive)", {
  # By transitivity: determined_week => determined_fortnight => determined_month
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Count violations
  n_violations <- sum(result$determined_week & !result$determined_month, na.rm = TRUE)

  expect_equal(
    n_violations, 0L,
    info = paste(
      "Found", n_violations, "observations with determined week but undetermined month.",
      "This violates the transitive nesting requirement."
    )
  )
})

# =============================================================================
# NESTING CONSISTENCY TESTS
# =============================================================================

test_that("fortnight value is consistent with month value when both determined", {
  # When fortnight is determined, it should fall within the determined month
  # Month 1 = fortnights 1-2, Month 2 = fortnights 3-4, Month 3 = fortnights 5-6
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  both_determined <- result[determined_month & determined_fortnight]

  if (nrow(both_determined) > 0) {
    # Calculate expected fortnight range for determined month
    both_determined[, `:=`(
      expected_fortnight_min = (ref_month_in_quarter - 1L) * 2L + 1L,
      expected_fortnight_max = ref_month_in_quarter * 2L
    )]

    # Check that actual fortnight falls within range
    inconsistent <- both_determined[
      ref_fortnight_in_quarter < expected_fortnight_min |
      ref_fortnight_in_quarter > expected_fortnight_max
    ]

    expect_equal(
      nrow(inconsistent), 0L,
      info = paste(
        "Found", nrow(inconsistent), "observations where fortnight is outside determined month range."
      )
    )
  }
})

test_that("week value is consistent with fortnight value when both determined", {
  # When week is determined, it should fall within the determined fortnight
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  both_determined <- result[determined_fortnight & determined_week]

  if (nrow(both_determined) > 0) {
    # Week should correspond to dates within the fortnight
    # Fortnight 1 = weeks 1-2, Fortnight 2 = weeks 3-4 of each month
    # This is a weaker check - just verify week exists for fortnight-determined obs
    expect_true(all(!is.na(both_determined$ref_week_in_quarter)))
  }
})

# =============================================================================
# NESTING BY CONSTRUCTION TESTS
# =============================================================================

test_that("determination rates follow nesting hierarchy", {
  # By construction: month_rate >= fortnight_rate >= week_rate
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Compute determination rates from result
  n_total <- nrow(result)
  month_rate <- sum(result$determined_month) / n_total
  fortnight_rate <- sum(result$determined_fortnight) / n_total
  week_rate <- sum(result$determined_week) / n_total

  expect_true(
    month_rate >= fortnight_rate,
    info = paste(
      "Month rate", round(month_rate, 4), "should be >= fortnight rate", round(fortnight_rate, 4)
    )
  )

  expect_true(
    fortnight_rate >= week_rate,
    info = paste(
      "Fortnight rate", round(fortnight_rate, 4), "should be >= week rate", round(week_rate, 4)
    )
  )
})

test_that("all determined fortnights have valid month reference", {
  # Every observation with determined fortnight should have valid month columns
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  fortnight_det <- result[determined_fortnight == TRUE]

  if (nrow(fortnight_det) > 0) {
    # All should have valid month values
    expect_true(all(!is.na(fortnight_det$ref_month_in_quarter)))
    expect_true(all(!is.na(fortnight_det$ref_month_in_year)))
    expect_true(all(!is.na(fortnight_det$ref_month_yyyymm)))
  }
})

test_that("all determined weeks have valid fortnight reference", {
  # Every observation with determined week should have valid fortnight columns
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  week_det <- result[determined_week == TRUE]

  if (nrow(week_det) > 0) {
    # All should have valid fortnight values
    expect_true(all(!is.na(week_det$ref_fortnight_in_quarter)))
    expect_true(all(!is.na(week_det$ref_fortnight_in_month)))
    expect_true(all(!is.na(week_det$ref_fortnight_yyyyff)))
  }
})

# =============================================================================
# EDGE CASE TESTS
# =============================================================================

test_that("single quarter data respects nesting", {
  # Even with single quarter (lower determination rate), nesting should hold
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Fortnight requires month
  n_violations_fm <- sum(result$determined_fortnight & !result$determined_month, na.rm = TRUE)
  expect_equal(n_violations_fm, 0L)

  # Week requires fortnight
  n_violations_wf <- sum(result$determined_week & !result$determined_fortnight, na.rm = TRUE)
  expect_equal(n_violations_wf, 0L)
})

test_that("nesting holds across all quarters when stacked", {
  # Test with more quarters to stress the algorithm
  test_data <- create_realistic_pnadc(n_quarters = 8, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Check nesting per quarter
  for (q in unique(result$Trimestre)) {
    quarter_data <- result[Trimestre == q]

    n_violations_fm <- sum(quarter_data$determined_fortnight & !quarter_data$determined_month, na.rm = TRUE)
    n_violations_wf <- sum(quarter_data$determined_week & !quarter_data$determined_fortnight, na.rm = TRUE)

    expect_equal(
      n_violations_fm, 0L,
      info = paste("Quarter", q, "has fortnight without month violations")
    )
    expect_equal(
      n_violations_wf, 0L,
      info = paste("Quarter", q, "has week without fortnight violations")
    )
  }
})

# =============================================================================
# EXPERIMENTAL STRATEGIES NESTING TESTS
# =============================================================================

test_that("experimental fortnight requires month (strict or experimental)", {
  # When fortnight is assigned experimentally, month must already exist
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)
  # Use upa_aggregation strategy which doesn't require original data
  result <- pnadc_experimental_periods(crosswalk, strategy = "upa_aggregation", verbose = FALSE)

  # Check that fortnight requires month (experimental strategies update main columns)
  has_fortnight <- !is.na(result$ref_fortnight_in_month)
  has_month <- !is.na(result$ref_month_in_quarter)

  n_violations <- sum(has_fortnight & !has_month, na.rm = TRUE)

  expect_equal(
    n_violations, 0L,
    info = paste(
      "Found", n_violations, "observations with fortnight but no month.",
      "This violates nesting."
    )
  )
})

test_that("experimental week requires fortnight (strict or experimental)", {
  # When week is assigned experimentally, fortnight must already exist
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)
  # Use upa_aggregation strategy which doesn't require original data
  result <- pnadc_experimental_periods(crosswalk, strategy = "upa_aggregation", verbose = FALSE)

  # Check that week requires fortnight (experimental strategies update main columns)
  has_week <- !is.na(result$ref_week_in_month)
  has_fortnight <- !is.na(result$ref_fortnight_in_month)

  n_violations <- sum(has_week & !has_fortnight, na.rm = TRUE)

  expect_equal(
    n_violations, 0L,
    info = paste(
      "Found", n_violations, "observations with experimental week but no fortnight (strict or exp).",
      "This violates experimental nesting."
    )
  )
})

test_that("experimental fortnight is consistent with month bounds", {
  # When fortnight is assigned, it should be within valid range (1 or 2 within month)
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)
  # Use upa_aggregation strategy which doesn't require original data
  result <- pnadc_experimental_periods(crosswalk, strategy = "upa_aggregation", verbose = FALSE)

  # Check fortnight values are in valid range
  has_fortnight <- result[!is.na(ref_fortnight_in_month)]

  if (nrow(has_fortnight) > 0) {
    # ref_fortnight_in_month should be 1 or 2
    invalid <- has_fortnight[ref_fortnight_in_month < 1 | ref_fortnight_in_month > 2]

    expect_equal(
      nrow(invalid), 0L,
      info = paste(
        "Found", nrow(invalid), "observations where fortnight is outside valid range (1-2)."
      )
    )
  }
})

test_that("experimental strategies extend but don't contradict strict determination", {
  # Experimental strategies may fill in previously NA values but should not

  # change values that were already determined by strict algorithm
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Save original strict values (only where determined)
  original_month <- crosswalk$ref_month_in_quarter
  original_fortnight <- crosswalk$ref_fortnight_in_quarter
  original_week <- crosswalk$ref_week_in_quarter

  # Use upa_aggregation strategy which doesn't require original data
  result <- pnadc_experimental_periods(crosswalk, strategy = "upa_aggregation", verbose = FALSE)

  # Where strict algorithm determined values, they should be preserved
  strict_month_mask <- !is.na(original_month)
  strict_fortnight_mask <- !is.na(original_fortnight)
  strict_week_mask <- !is.na(original_week)

  # Strict values should be unchanged where they existed
  expect_equal(result$ref_month_in_quarter[strict_month_mask], original_month[strict_month_mask])
  expect_equal(result$ref_fortnight_in_quarter[strict_fortnight_mask], original_fortnight[strict_fortnight_mask])
  expect_equal(result$ref_week_in_quarter[strict_week_mask], original_week[strict_week_mask])
})

test_that("experimental determination rates follow nesting hierarchy", {
  # After experimental strategies, determination rates should follow: month >= fortnight >= week
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)
  # Use upa_aggregation strategy which doesn't require original data
  result <- pnadc_experimental_periods(crosswalk, strategy = "upa_aggregation", verbose = FALSE)

  # Calculate determination rates from main columns (experimental strategies update these directly)
  month_rate <- mean(!is.na(result$ref_month_in_quarter))
  fortnight_rate <- mean(!is.na(result$ref_fortnight_in_quarter))
  week_rate <- mean(!is.na(result$ref_week_in_quarter))

  expect_true(
    month_rate >= fortnight_rate,
    info = paste(
      "Month rate", round(month_rate, 4),
      "should be >= fortnight rate", round(fortnight_rate, 4)
    )
  )

  expect_true(
    fortnight_rate >= week_rate,
    info = paste(
      "Fortnight rate", round(fortnight_rate, 4),
      "should be >= week rate", round(week_rate, 4)
    )
  )
})

#' @title Date Utility Functions
#' @description Internal helper functions for date calculations.
#'
#'   This file includes:
#'   \itemize{
#'     \item Day of week calculation (dow)
#'     \item ISO 8601 week utilities (Monday-Sunday weeks)
#'     \item IBGE first Saturday calculation for reference weeks
#'     \item Month position calculation for quarter mapping
#'   }
#'
#' @name utils-dates
#' @keywords internal
NULL

# ============================================================================
# Pre-computed lookup tables for fast date creation
# These are computed once at package load
# ============================================================================

# Days since 1970-01-01 for January 1st of each year (2000-2050)
# Use plain integer vector with offset indexing instead of named lookup
.YEAR_BASE_START <- 2000L
.YEAR_BASE <- local({
  years <- 2000:2050

  # Vectorized approach instead of sapply
  as.integer(as.Date(paste0(years, "-01-01")))
})

# Days since 1970-01-01 for January 4th of each year (2000-2050)
# Used for fast ISO week calculation (Jan 4 is always in ISO week 1)
.JAN4_BASE <- local({
  years <- 2000:2050

  # Vectorized approach instead of sapply
  as.integer(as.Date(paste0(years, "-01-04")))
})

# Cumulative days at the start of each month (0-indexed from Jan 1)
# Regular year (non-leap)
.MONTH_OFFSET_REGULAR <- c(0L, 31L, 59L, 90L, 120L, 151L, 181L, 212L, 243L, 273L, 304L, 334L)

# Leap year (Feb has 29 days)
.MONTH_OFFSET_LEAP <- c(0L, 31L, 60L, 91L, 121L, 152L, 182L, 213L, 244L, 274L, 305L, 335L)

#' Day of Week
#'
#' Returns the day of week for a date.
#' Uses fast integer arithmetic instead of format().
#'
#' @param date A Date object or vector of Dates
#' @return Integer vector: 0 = Sunday, 1 = Monday, ..., 6 = Saturday
#' @keywords internal
#' @noRd
dow <- function(date) {
  # R's origin is 1970-01-01 which was a Thursday (dow=4)
  # Use modular arithmetic: (days_since_origin + 4) %% 7
  (as.integer(date) + 4L) %% 7L
}


#' Is Leap Year
#'
#' Check if a year is a leap year.
#'
#' @param year Integer vector of years
#' @return Logical vector
#' @keywords internal
#' @noRd
is_leap_year <- function(year) {
  (year %% 4 == 0 & year %% 100 != 0) | (year %% 400 == 0)
}


#' Get First Month of Quarter (vectorized)
#'
#' @param quarter Integer vector of quarters (1-4)
#' @return Integer vector of first month numbers
#' @keywords internal
#' @noRd
quarter_first_month <- function(quarter) {
  (quarter - 1L) * 3L + 1L
}

#' Get N-th Month of Quarter (vectorized)
#'
#' @param quarter Integer vector of quarters (1-4)
#' @param n Which month in quarter (1, 2, or 3)
#' @return Integer vector of month numbers
#' @keywords internal
#' @noRd
quarter_month_n <- function(quarter, n) {
  (quarter - 1L) * 3L + n
}

#' Create Year-Month Integer (YYYYMM format)
#'
#' Creates an integer in YYYYMM format from year and month.
#'
#' @param year Integer year
#' @param month Integer month (1-12)
#' @return Integer in YYYYMM format (e.g., 202301 for January 2023)
#' @keywords internal
#' @noRd
yyyymm <- function(year, month) {
  as.integer(year * 100L + month)
}

#' First Saturday of Month (with minimum days constraint)
#'
#' Calculates the day of the month for the first Saturday that has at least
#' `min_days` in the reference week within that month. This implements IBGE's
#' "Parada Tecnica" rule.
#'
#' The reference week ends on Saturday. If Saturday falls on day X of the month,
#' then X days of that week are within the month (days 1 through X). For the
#' first week to be valid, X must be >= min_days.
#'
#' @param year Integer year
#' @param month Integer month (1-12)
#' @param min_days Integer minimum days required (default 4, use 3 for exception quarters)
#' @return Integer day of month for the first valid Saturday
#' @keywords internal
#' @noRd
first_valid_saturday <- function(year, month, min_days = 4L) {

  # Get first day of the month
  first_day <- lubridate::ymd(paste(year = year, month = month, day = 1L, sep = "-"))

  # Day of week for first of month (0=Sun, 6=Sat)
  first_dow <- dow(first_day)

  # Calculate first Saturday
  # If first_dow = 0 (Sunday), first Saturday is day 7
  # If first_dow = 1 (Monday), first Saturday is day 6
  # ...
  # If first_dow = 6 (Saturday), first Saturday is day 1
  days_to_saturday   <- (6L - first_dow) %% 7L
  first_saturday_day <- 1L + days_to_saturday

  # How many days of the reference week (ending on Saturday) are in this month?
  # If Saturday is on day X, then X days (1 through X) are in the month.
  # We need X >= min_days for the week to be valid.
  #
  # Examples for min_days = 4:
  #   If Saturday is day 7: 7 >= 4, valid
  #   If Saturday is day 4: 4 >= 4, valid
  #   If Saturday is day 3: 3 < 4, skip to next week (day 10)
  #   If Saturday is day 1: 1 < 4, skip to next week (day 8)
  #
  # Examples for min_days = 3:
  #   If Saturday is day 3: 3 >= 3, valid
  #   If Saturday is day 2: 2 < 3, skip to next week (day 9)

  # If first Saturday has enough days, use it; otherwise use second Saturday

  data.table::fifelse(first_saturday_day >= min_days, first_saturday_day, first_saturday_day + 7L)
}

#' First Saturday After or On a Date
#'
#' Finds the first Saturday that is on or after the given date.
#'
#' @param date Date vector
#' @return Date vector of Saturdays
#' @keywords internal
#' @noRd
first_saturday_on_or_after <- function(date) {
  dow_date    <- dow(date)
  days_to_add <- (6L - dow_date) %% 7L
  date + days_to_add
}


# ============================================================================
# ISO 8601 Week Utilities
# ISO week 1 is the week containing January 4th (equivalently, the first week
# with at least 4 days in the new year). Weeks start on Monday.
# ============================================================================

#' ISO Week and Year Combined
#'
#' Returns both ISO 8601 week number and week-year for a date in a single pass.
#' This avoids redundant computation when both values are needed.
#'
#' Computes Thursday of the week only once, then derives both
#' week number and year. This is 2x faster than calling iso_week() and iso_year()
#' separately when both values are needed.
#'
#' @param date Date vector
#' @return List with two integer vectors: \code{year} (ISO week-year) and
#'   \code{week} (ISO week number 1-53)
#' @keywords internal
#' @noRd
iso_week_year <- function(date) {
  # Days since epoch
  days <- as.integer(date)

  # Day of week (0=Sun, 1=Mon, ..., 6=Sat) using R's Thursday origin
  date_dow <- (days + 4L) %% 7L

  # ISO weekday (Mon=1, ..., Sun=7)
  date_iso_wday <- data.table::fifelse(date_dow == 0L, 7L, as.integer(date_dow))

  # Thursday of the week containing this date (COMPUTED ONCE)
  thursday_days <- days + (4L - date_iso_wday)

  # ISO year = year of the Thursday
  thursday_date <- structure(thursday_days, class = "Date")
  iso_yr <- data.table::year(thursday_date)

  # Find Monday of week 1 of the ISO year
  # Jan 4 is always in week 1, so find its Monday
  jan4_days <- .JAN4_BASE[iso_yr - .YEAR_BASE_START + 1L]

  # Day of week of Jan 4
  jan4_dow <- (jan4_days + 4L) %% 7L
  jan4_iso_wday <- data.table::fifelse(jan4_dow == 0L, 7L, as.integer(jan4_dow))

  # Monday of week 1
  week1_monday_days <- jan4_days - (jan4_iso_wday - 1L)

  # Monday of the target date's week
  date_monday_days <- days - (date_iso_wday - 1L)

  # ISO week number
  iso_wk <- as.integer((date_monday_days - week1_monday_days) / 7L) + 1L

  list(year = iso_yr, week = iso_wk)
}

#' ISO Week Number
#'
#' Returns the ISO 8601 week number for a date. Week 1 is the week containing
#' January 4th (equivalently, the first week with at least 4 days in January).
#'
#' Uses pure integer arithmetic with pre-computed lookup tables.
#' This is ~300x faster than data.table::isoweek() for large vectors.
#'
#' Note: If you need both week and year, use \code{iso_week_year()} instead
#' to avoid redundant computation.
#'
#' @param date Date vector
#' @return Integer vector of ISO week numbers (1-53)
#' @keywords internal
#' @noRd
iso_week <- function(date) {
  iso_week_year(date)$week
}

#' ISO Week-Year
#'
#' Returns the ISO 8601 week-year for a date. This may differ from the calendar
#' year for dates near year boundaries (e.g., Dec 31 may be in week 1 of the
#' next year, and Jan 1-3 may be in week 52/53 of the previous year).
#'
#' Note: If you need both week and year, use \code{iso_week_year()} instead
#' to avoid redundant computation.
#'
#' @param date Date vector
#' @return Integer vector of ISO week-years
#' @keywords internal
#' @noRd
iso_year <- function(date) {
  iso_week_year(date)$year
}


# ============================================================================
# Month Position Calculation for Quarter Mapping
# ============================================================================

#' Calculate Month Position in Quarter (for date_min)
#'
#' Converts a date_min to its month position within the quarter (1, 2, or 3).
#' For date_min: if day <= threshold and not in first month of quarter, subtract 1.
#'
#' @param date Date vector
#' @param year Integer year vector
#' @param quarter Integer quarter vector (1-4)
#' @param day_threshold Integer threshold for adjustment (3 for standard, 2 for exception)
#' @return Integer vector (1, 2, or 3)
#' @keywords internal
#' @noRd
calculate_month_position_min <- function(date, year, quarter, day_threshold = 3L) {
  first_month <- quarter_first_month(quarter)
  date_month  <- data.table::month(date)
  date_day    <- data.table::mday(date)

  pos <- date_month - first_month + 1L

  # Adjust if day <= threshold and not in first month of quarter
  adjust <- (date_day <= day_threshold & date_month > first_month)
  pos <- pos - as.integer(adjust)

  pmin(pmax(pos, 1L), 3L)
}


#' Calculate Month Position in Quarter (for date_max)
#'
#' Converts a date_max to its month position within the quarter (1, 2, or 3).
#' For date_max: if day <= threshold, use the month of (date - 3 days) instead.
#'
#' @param date Date vector
#' @param year Integer year vector
#' @param quarter Integer quarter vector (1-4)
#' @param day_threshold Integer threshold for adjustment (3 for standard, 2 for exception)
#' @return Integer vector (1, 2, or 3)
#' @keywords internal
#' @noRd
calculate_month_position_max <- function(date, year, quarter, day_threshold = 3L) {
  first_month <- quarter_first_month(quarter)
  date_day    <- data.table::mday(date)

  # When day <= threshold, use the month of (date - 3 days)
  needs_adjust   <- date_day <= day_threshold
  adjusted_date  <- date - needs_adjust * 3L
  adjusted_month <- data.table::month(adjusted_date)

  pos <- adjusted_month - first_month + 1L

  pmin(pmax(pos, 1L), 3L)
}


#' Calculate Month Position (date_min) with Dynamic Per-Month Thresholds
#'
#' Applies different thresholds based on which month the date falls in and
#' whether that month has exception rules enabled.
#'
#' @param date Date vector
#' @param year Integer year vector
#' @param quarter Integer quarter vector (1-4)
#' @param exc_m1 Integer: exception flag for month 1 of quarter (0 or 1)
#' @param exc_m2 Integer: exception flag for month 2 of quarter (0 or 1)
#' @param exc_m3 Integer: exception flag for month 3 of quarter (0 or 1)
#' @return Integer vector (1, 2, or 3)
#' @keywords internal
#' @noRd
calculate_month_position_min_dynamic <- function(date, year, quarter, exc_m1, exc_m2, exc_m3) {
  first_month <- quarter_first_month(quarter)
  date_month <- data.table::month(date)
  date_day <- data.table::mday(date)

  pos <- date_month - first_month + 1L

  # Determine threshold based on which month the date falls in
  # Month 1 of quarter: months 1, 4, 7, 10
  # Month 2 of quarter: months 2, 5, 8, 11
  # Month 3 of quarter: months 3, 6, 9, 12
  # Use modular arithmetic to identify position in quarter
  month_in_quarter <- ((date_month - 1L) %% 3L) + 1L  # 1, 2, or 3

  # Threshold is 2 if exception applies for this month, 3 otherwise
  threshold <- fifelse(
    (month_in_quarter == 1L & exc_m1 == 1L) |
      (month_in_quarter == 2L & exc_m2 == 1L) |
      (month_in_quarter == 3L & exc_m3 == 1L),
    2L, 3L
  )

  # Adjust if day <= threshold and not in first month of quarter
  adjust <- (date_day <= threshold & date_month > first_month)
  pos <- pos - as.integer(adjust)

  pmin(pmax(pos, 1L), 3L)
}


#' Calculate Month Position (date_max) with Dynamic Per-Month Thresholds
#'
#' Applies different thresholds based on which month the date falls in and
#' whether that month has exception rules enabled.
#'
#' @param date Date vector
#' @param year Integer year vector
#' @param quarter Integer quarter vector (1-4)
#' @param exc_m1 Integer: exception flag for month 1 of quarter (0 or 1)
#' @param exc_m2 Integer: exception flag for month 2 of quarter (0 or 1)
#' @param exc_m3 Integer: exception flag for month 3 of quarter (0 or 1)
#' @return Integer vector (1, 2, or 3)
#' @keywords internal
#' @noRd
calculate_month_position_max_dynamic <- function(date, year, quarter, exc_m1, exc_m2, exc_m3) {
  first_month <- quarter_first_month(quarter)
  date_month  <- data.table::month(date)
  date_day    <- data.table::mday(date)

  # Determine threshold based on which month the date falls in
  # Use modular arithmetic to identify position in quarter
  month_in_quarter <- ((date_month - 1L) %% 3L) + 1L  # 1, 2, or 3

  # Threshold is 2 if exception applies for this month, 3 otherwise
  threshold <- fifelse(
    (month_in_quarter == 1L & exc_m1 == 1L) |
      (month_in_quarter == 2L & exc_m2 == 1L) |
      (month_in_quarter == 3L & exc_m3 == 1L),
    2L, 3L
  )

  # When day <= threshold, use the month of (date - 3 days)
  needs_adjust <- date_day <= threshold
  adjusted_date <- date - needs_adjust * 3L
  adjusted_month <- data.table::month(adjusted_date)

  pos <- adjusted_month - first_month + 1L

  pmin(pmax(pos, 1L), 3L)
}

# Tests for date utility functions

test_that("dow returns correct day of week", {
  # Sunday = 0
 expect_equal(dow(as.Date("2024-01-07")), 0)  # Sunday
  # Monday = 1
  expect_equal(dow(as.Date("2024-01-08")), 1)  # Monday
  # Saturday = 6
  expect_equal(dow(as.Date("2024-01-06")), 6)  # Saturday
})

test_that("is_leap_year identifies leap years correctly", {
  expect_true(is_leap_year(2024))
  expect_true(is_leap_year(2000))
  expect_false(is_leap_year(2023))
  expect_false(is_leap_year(1900))
})

test_that("yyyymm creates correct format", {
  expect_equal(yyyymm(2024, 1), 202401L)
  expect_equal(yyyymm(2024, 12), 202412L)
  expect_equal(yyyymm(2023, 6), 202306L)
})

test_that("first_valid_saturday calculates correctly", {
  # January 2024: starts on Monday (dow=1)
  # First Saturday is day 6
  # With min_days=4, need at least 4 days in week, so day 6 is OK (6 days in month)
  expect_equal(first_valid_saturday(2024, 1, min_days = 4), 6)

  # March 2024: starts on Friday (dow=5)
  # First Saturday is day 2 (only 2 days in March)
  # With min_days=4, need second Saturday (day 9)
  expect_equal(first_valid_saturday(2024, 3, min_days = 4), 9)

  # With min_days=3, first Saturday (day 2) might work
  # 2 days is less than 3, so still need second Saturday
  expect_equal(first_valid_saturday(2024, 3, min_days = 3), 9)
})

test_that("first_saturday_on_or_after works correctly", {
  # If date is Saturday, return same date
  expect_equal(first_saturday_on_or_after(as.Date("2024-01-06")), as.Date("2024-01-06"))

  # If date is Sunday, return next Saturday (6 days later)
  expect_equal(first_saturday_on_or_after(as.Date("2024-01-07")), as.Date("2024-01-13"))

  # If date is Monday, return Saturday (5 days later)
  expect_equal(first_saturday_on_or_after(as.Date("2024-01-08")), as.Date("2024-01-13"))
})

# =============================================================================
# DAY OF WEEK EDGE CASES
# =============================================================================

test_that("dow handles year boundaries correctly", {
  # Test year transitions
  expect_equal(dow(as.Date("2023-12-31")), 0)  # Sunday
  expect_equal(dow(as.Date("2024-01-01")), 1)  # Monday

  # Test epoch date (1970-01-01 = Thursday)
  expect_equal(dow(as.Date("1970-01-01")), 4)  # Thursday
})

test_that("dow works for all days of the week", {
  # Test a complete week in January 2024
  expect_equal(dow(as.Date("2024-01-07")), 0)  # Sunday
  expect_equal(dow(as.Date("2024-01-08")), 1)  # Monday
  expect_equal(dow(as.Date("2024-01-09")), 2)  # Tuesday
  expect_equal(dow(as.Date("2024-01-10")), 3)  # Wednesday
  expect_equal(dow(as.Date("2024-01-11")), 4)  # Thursday
  expect_equal(dow(as.Date("2024-01-12")), 5)  # Friday
  expect_equal(dow(as.Date("2024-01-06")), 6)  # Saturday
})

# =============================================================================
# LEAP YEAR EDGE CASES
# =============================================================================

test_that("is_leap_year handles century years correctly", {
  # Century years divisible by 400 are leap years
  expect_true(is_leap_year(2000))
  expect_true(is_leap_year(2400))

  # Century years not divisible by 400 are NOT leap years
  expect_false(is_leap_year(1900))
  expect_false(is_leap_year(2100))
  expect_false(is_leap_year(2200))
  expect_false(is_leap_year(2300))
})

test_that("is_leap_year handles standard leap year pattern", {
  # Regular leap years (divisible by 4, not century)
  expect_true(is_leap_year(2020))
  expect_true(is_leap_year(2024))
  expect_true(is_leap_year(2028))

  # Non-leap years
  expect_false(is_leap_year(2021))
  expect_false(is_leap_year(2022))
  expect_false(is_leap_year(2023))
})

# =============================================================================
# QUARTER HELPER FUNCTIONS
# =============================================================================

test_that("quarter_first_month returns correct first month", {
  expect_equal(PNADCperiods:::quarter_first_month(1), 1)   # Q1 starts in Jan
  expect_equal(PNADCperiods:::quarter_first_month(2), 4)   # Q2 starts in Apr
  expect_equal(PNADCperiods:::quarter_first_month(3), 7)   # Q3 starts in Jul
  expect_equal(PNADCperiods:::quarter_first_month(4), 10)  # Q4 starts in Oct
})

test_that("quarter_month_n returns correct n-th month of quarter", {
  # Q1 (Jan-Mar)
  expect_equal(PNADCperiods:::quarter_month_n(1, 1), 1)  # 1st month = Jan
  expect_equal(PNADCperiods:::quarter_month_n(1, 2), 2)  # 2nd month = Feb
  expect_equal(PNADCperiods:::quarter_month_n(1, 3), 3)  # 3rd month = Mar

  # Q4 (Oct-Dec)
  expect_equal(PNADCperiods:::quarter_month_n(4, 1), 10)  # 1st month = Oct
  expect_equal(PNADCperiods:::quarter_month_n(4, 2), 11)  # 2nd month = Nov
  expect_equal(PNADCperiods:::quarter_month_n(4, 3), 12)  # 3rd month = Dec
})

# =============================================================================
# YYYYMM FORMAT EDGE CASES
# =============================================================================

test_that("yyyymm handles edge cases", {
  # Year boundaries
  expect_equal(yyyymm(1999, 12), 199912L)
  expect_equal(yyyymm(2000, 1), 200001L)

  # Recent years
  expect_equal(yyyymm(2023, 1), 202301L)
  expect_equal(yyyymm(2024, 12), 202412L)
})

# =============================================================================
# ISO WEEK FUNCTIONS
# =============================================================================

test_that("iso_week_year handles year boundary edge cases", {
  # Dec 31, 2020 is week 53 of ISO year 2020
  # (2020 is a leap year starting on Wednesday, so has 53 weeks)
  result_20201231 <- PNADCperiods:::iso_week_year(as.Date("2020-12-31"))
  expect_equal(result_20201231$year, 2020)  # ISO year 2020
  expect_equal(result_20201231$week, 53)     # Week 53

  # Jan 1, 2021 is week 53 of ISO year 2020
  # (belongs to previous ISO year)
  result_20210101 <- PNADCperiods:::iso_week_year(as.Date("2021-01-01"))
  expect_equal(result_20210101$year, 2020)  # ISO year 2020
  expect_equal(result_20210101$week, 53)     # Week 53

  # Jan 4 is always in week 1 of the current year (ISO 8601 rule)
  result_20210104 <- PNADCperiods:::iso_week_year(as.Date("2021-01-04"))
  expect_equal(result_20210104$year, 2021)  # ISO year 2021
  expect_equal(result_20210104$week, 1)      # Week 1
})

test_that("iso_week extracts week number correctly", {
  # Mid-year date
  expect_equal(PNADCperiods:::iso_week(as.Date("2024-06-15")), 24)

  # Jan 4 is always week 1
  expect_equal(PNADCperiods:::iso_week(as.Date("2024-01-04")), 1)

  # Year boundary: 2024-12-31 is Tuesday, Thursday of that week is 2025-01-02
  # So ISO week is 1 of ISO year 2025
  expect_equal(PNADCperiods:::iso_week(as.Date("2024-12-31")), 1L)
})

test_that("iso_year extracts ISO year correctly", {
  # Mid-year - ISO year equals calendar year
  expect_equal(PNADCperiods:::iso_year(as.Date("2024-06-15")), 2024)

  # Jan 1, 2024 is Monday, Thursday of that week is Jan 4 -> ISO year 2024
  expect_equal(PNADCperiods:::iso_year(as.Date("2024-01-01")), 2024L)

  # Dec 31, 2024 is Tuesday, Thursday of that week is Jan 2, 2025 -> ISO year 2025
  expect_equal(PNADCperiods:::iso_year(as.Date("2024-12-31")), 2025L)
})

# =============================================================================
# FIRST SATURDAY EDGE CASES
# =============================================================================

test_that("first_saturday_on_or_after handles month boundaries", {
  # Last day of month
  expect_equal(
    first_saturday_on_or_after(as.Date("2024-01-31")),  # Wednesday
    as.Date("2024-02-03")  # Next Saturday in February
  )

  # Last day of December
  expect_equal(
    first_saturday_on_or_after(as.Date("2024-12-31")),  # Tuesday
    as.Date("2025-01-04")  # Next Saturday in January
  )
})

test_that("first_saturday_on_or_after handles Friday edge case", {
  # Friday should return next day (Saturday)
  expect_equal(
    first_saturday_on_or_after(as.Date("2024-01-05")),  # Friday
    as.Date("2024-01-06")  # Saturday
  )
})

test_that("first_valid_saturday handles leap year February", {
  # February 2024 (leap year) starts on Thursday (dow=4)
  # First Saturday is Feb 3 (day 3)
  # 3 days from start of month, so min_days=4 would skip to day 10
  expect_equal(first_valid_saturday(2024, 2, min_days = 4), 10)

  # February 2023 (non-leap year) starts on Wednesday (dow=3)
  # First Saturday is Feb 4 (day 4)
  # 4 days from start, exactly meets min_days=4
  expect_equal(first_valid_saturday(2023, 2, min_days = 4), 4)
})

test_that("first_valid_saturday handles month starting on Saturday", {
  # April 2023 starts on Saturday
  # First Saturday is day 1, which has only 1 day in month
  # With min_days=4, should skip to second Saturday (day 8)
  expect_equal(first_valid_saturday(2023, 4, min_days = 4), 8)
})

# =============================================================================
# MONTH POSITION FUNCTIONS (INTERNAL)
# =============================================================================

test_that("calculate_month_position_min handles quarter boundaries", {
  # These functions are internal but critical for period identification
  # Test with dates at start of quarters

  # Q1 2024: Jan 1 is a Monday
  # First valid Saturday with min_days=4 determines month position
  result <- PNADCperiods:::calculate_month_position_min(
    as.Date("2024-01-06"),  # First Saturday
    2024, 1
  )
  expect_equal(result, 1)  # Should be month 1

  # Later date in same quarter
  result <- PNADCperiods:::calculate_month_position_min(
    as.Date("2024-03-15"),
    2024, 1
  )
  expect_equal(result, 3)  # Should be month 3
})

test_that("calculate_month_position_max handles quarter boundaries", {
  # Test with dates at end of quarters

  # Early date in quarter
  result <- PNADCperiods:::calculate_month_position_max(
    as.Date("2024-01-15"),
    2024, 1
  )
  expect_equal(result, 1)  # Should be month 1

  # Late date in quarter
  result <- PNADCperiods:::calculate_month_position_max(
    as.Date("2024-03-28"),
    2024, 1
  )
  expect_equal(result, 3)  # Should be month 3
})

# =============================================================================
# VECTORIZED INPUT TESTS
# =============================================================================

test_that("dow is vectorized", {
  dates <- as.Date(c("2024-01-06", "2024-01-07", "2024-01-08"))
  result <- dow(dates)
  expect_equal(result, c(6L, 0L, 1L))  # Sat, Sun, Mon
  expect_length(result, 3)
})

test_that("is_leap_year is vectorized", {
  years <- c(2000, 1900, 2020, 2023, 2024)
  result <- is_leap_year(years)
  expect_equal(result, c(TRUE, FALSE, TRUE, FALSE, TRUE))
  expect_length(result, 5)
})

test_that("iso_week_year is vectorized", {
  dates <- as.Date(c("2024-01-04", "2024-06-15", "2024-12-31"))
  result <- PNADCperiods:::iso_week_year(dates)

  expect_length(result$year, 3)
  expect_length(result$week, 3)
  expect_equal(result$year, c(2024L, 2024L, 2025L))
  expect_equal(result$week, c(1L, 24L, 1L))
})

# =============================================================================
# LOOKUP TABLE VALIDATION
# =============================================================================

test_that(".YEAR_BASE lookup table has correct values for known years", {
  year_base <- PNADCperiods:::.YEAR_BASE
  start <- PNADCperiods:::.YEAR_BASE_START

  # Spot-check known values
  # 2000-01-01 is day 10957 since 1970-01-01
  expect_equal(year_base[2000 - start + 1L], as.integer(as.Date("2000-01-01")))
  # 2024-01-01
  expect_equal(year_base[2024 - start + 1L], as.integer(as.Date("2024-01-01")))
  # 2012-01-01 (PNADC start)
  expect_equal(year_base[2012 - start + 1L], as.integer(as.Date("2012-01-01")))
})

test_that(".JAN4_BASE lookup table has correct values", {
  jan4_base <- PNADCperiods:::.JAN4_BASE
  start <- PNADCperiods:::.YEAR_BASE_START

  # Jan 4 of any year is always in ISO week 1
  expect_equal(jan4_base[2024 - start + 1L], as.integer(as.Date("2024-01-04")))
  expect_equal(jan4_base[2012 - start + 1L], as.integer(as.Date("2012-01-04")))
})

test_that("month offset tables have correct values", {
  # Regular year
  reg <- PNADCperiods:::.MONTH_OFFSET_REGULAR
  expect_equal(reg[1], 0L)   # Jan starts at 0
  expect_equal(reg[2], 31L)  # Feb starts after 31 days of Jan
  expect_equal(reg[3], 59L)  # Mar starts after 31+28=59 days (non-leap)
  expect_equal(reg[12], 334L) # Dec starts after 334 days

  # Leap year
  leap <- PNADCperiods:::.MONTH_OFFSET_LEAP
  expect_equal(leap[1], 0L)
  expect_equal(leap[2], 31L)
  expect_equal(leap[3], 60L)  # Mar starts after 31+29=60 days (leap)
  expect_equal(leap[12], 335L)
})

test_that("first_valid_saturday min_days=3 produces different result than min_days=4", {
  # June 2024 starts on Saturday (dow=6)
  # First Saturday is day 1, which has 1 day in month
  # min_days=4: 1 < 4, skip to day 8
  # min_days=3: 1 < 3, skip to day 8
  # Both skip for June 2024, but try a month where they differ
  #
  # September 2024 starts on Sunday (dow=0)
  # First Saturday is day 7 (7 days from start)
  # Both would accept day 7
  #
  # February 2024 starts on Thursday (dow=4)
  # First Saturday is day 3 (3 days from start)
  # min_days=4: 3 < 4, skip to day 10
  # min_days=3: 3 >= 3, use day 3
  expect_equal(first_valid_saturday(2024, 2, min_days = 4), 10)
  expect_equal(first_valid_saturday(2024, 2, min_days = 3), 3)
})

#' Experimental Period Identification Strategies
#'
#' Provides experimental strategies for improving period identification rates
#' beyond the standard deterministic algorithm. All strategies respect the nested
#' identification hierarchy: weeks require fortnights, fortnights require months.
#'
#' @description
#' Three experimental strategies are available, all properly nested by period:
#' \itemize{
#'   \item \strong{probabilistic}: For narrow ranges (2 possible periods),
#'     classifies based on where most of the date interval falls. Assigns only
#'     when confidence exceeds threshold.
#'   \item \strong{upa_aggregation}: Extends strictly identified periods to other
#'     observations in the same UPA-V1014 within the quarter, if a sufficient
#'     proportion already have strict identification.
#'   \item \strong{both}: Sequentially applies probabilistic strategy first, then
#'     UPA aggregation on top. Guarantees identification rate >= max of individual strategies.
#' }
#'
#' @param crosswalk A crosswalk data.table from \code{pnadc_identify_periods()}
#' @param strategy Character specifying which strategy to apply.
#'   Options: "probabilistic", "upa_aggregation", "both"
#' @param confidence_threshold Numeric (0-1). Minimum confidence required to
#'   assign a probabilistic period. Used by probabilistic and combined strategies.
#'   Default 0.9.
#' @param upa_proportion_threshold Numeric (0-1). Minimum proportion of UPA
#'   observations (within quarter) that must have strict identification with consensus
#'   for extending to unidentified observations. Default 0.5.
#' @param verbose Logical. If TRUE, print progress information.
#'
#' @return A modified crosswalk with additional columns. Output is directly compatible
#'   with \code{pnadc_apply_periods()}:
#'   \itemize{
#'     \item \code{ref_month_in_quarter}, \code{ref_month_in_year}, \code{ref_month_yyyymm}:
#'       Month position (combined strict + experimental, strict takes priority)
#'     \item \code{ref_fortnight_in_month}, \code{ref_fortnight_in_quarter}, \code{ref_fortnight_yyyyff}:
#'       Fortnight position (combined strict + experimental)
#'     \item \code{ref_week_in_month}, \code{ref_week_in_quarter}, \code{ref_week_yyyyww}:
#'       Week position (combined strict + experimental)
#'     \item \code{determined_month}, \code{determined_fortnight}, \code{determined_week}:
#'       TRUE if period is assigned (strictly or experimentally)
#'     \item \code{determined_probable_month}, \code{determined_probable_fortnight},
#'       \code{determined_probable_week}: TRUE if period was assigned by probabilistic strategy
#'     \item \code{probabilistic_assignment}: TRUE if any period was assigned experimentally
#'       (vs strictly deterministic)
#'     \item \code{week_1_start}, \code{week_1_end}, ..., \code{week_4_start}, \code{week_4_end}:
#'       IBGE week boundaries for the assigned month
#'   }
#'
#' @details
#' ## Nesting Enforcement
#'
#' All strategies enforce proper nesting:
#' \itemize{
#'   \item Fortnights can only be assigned if month is identified (strictly OR experimentally)
#'   \item Weeks can only be assigned if fortnight is identified (strictly OR experimentally)
#' }
#'
#' ## Probabilistic Strategy
#'
#' For each period type (processed in order: months, then fortnights, then weeks):
#' \enumerate{
#'   \item Check that the required parent period is identified
#'   \item If bounds are narrowed to exactly 2 sequential periods, calculate
#'     which period contains most of the date interval
#'   \item Calculate confidence based on the proportion of interval in the likely period (0-1)
#'   \item Only assign if confidence >= \code{confidence_threshold}
#' }
#'
#' For months: aggregates at UPA-V1014 level across all quarters (like strict algorithm)
#' For fortnights and weeks: works at household level within quarter
#'
#' ## UPA Aggregation Strategy
#'
#' Extends strictly identified periods based on consensus within geographic groups:
#' \itemize{
#'   \item \strong{Months}: Uses UPA level within quarter
#'   \item \strong{Fortnights/Weeks}: Uses UPA level within quarter (all households in same UPA
#'     are interviewed in same fortnight/week within a quarter)
#' }
#' \enumerate{
#'   \item Calculate proportion of observations with strictly identified period
#'   \item If proportion >= \code{upa_proportion_threshold} AND consensus exists, extend
#'   \item Apply in nested order: months first, then fortnights, then weeks
#' }
#'
#' ## Combined Strategy ("both")
#'
#' Sequentially applies both strategies to maximize identification:
#' \enumerate{
#'   \item First, apply the probabilistic strategy (captures observations with
#'     narrow date ranges and high confidence)
#'   \item Then, apply UPA aggregation (extends based on strict consensus
#'     within UPA/UPA-V1014 groups)
#' }
#'
#' This guarantees that "both" identifies at least as many observations as
#' either individual strategy alone. The strategies operate independently
#' (UPA aggregation considers only strict identifications), so the result
#' is the union of both strategies.
#'
#' ## Integration with Weight Calibration
#'
#' The output can be passed directly to \code{pnadc_apply_periods()} for weight calibration.
#' The derived columns combine strict and experimental assignments, with strict taking priority. Use the
#' \code{probabilistic_assignment} flag to filter if you only want strict determinations.
#'
#' @note
#' These strategies produce "experimental" assignments, not strict determinations.
#' The standard \code{pnadc_identify_periods()} function should be used for
#' rigorous analysis. Experimental outputs are useful for:
#' \itemize{
#'   \item Sensitivity analysis
#'   \item Robustness checks
#'   \item Research into identification algorithm improvements
#' }
#'
#' @seealso
#' \code{\link{pnadc_identify_periods}} to build the crosswalk that this function modifies.
#' \code{\link{pnadc_apply_periods}} to apply period crosswalk and calibrate weights.
#'
#' @examples
#' \dontrun{
#' crosswalk <- pnadc_identify_periods(pnadc_data)
#'
#' crosswalk_exp <- pnadc_experimental_periods(
#'   crosswalk,
#'   strategy = "probabilistic",
#'   confidence_threshold = 0.9
#' )
#'
#' crosswalk_exp[, .(
#'   strict = sum(!is.na(ref_month_in_quarter) & !probabilistic_assignment),
#'   experimental = sum(probabilistic_assignment, na.rm = TRUE),
#'   total = sum(determined_month)
#' )]
#'
#' result <- pnadc_apply_periods(pnadc_data, crosswalk_exp,
#'                               weight_var = "V1028", anchor = "quarter")
#'
#' strict_only <- crosswalk_exp[
#'   probabilistic_assignment == FALSE | is.na(probabilistic_assignment)
#' ]
#' }
#'
#' @export
pnadc_experimental_periods <- function(
    crosswalk,
    strategy = c("probabilistic", "upa_aggregation", "both"),
    confidence_threshold = 0.9,
    upa_proportion_threshold = 0.5,
    verbose = TRUE
) {

  strategy <- match.arg(strategy)

  # Validate
  checkmate::assert_data_frame(crosswalk)
  checkmate::assert_number(confidence_threshold, lower = 0, upper = 1)
  checkmate::assert_number(upa_proportion_threshold, lower = 0, upper = 1)
  checkmate::assert_logical(verbose)


  # Copy crosswalk to avoid modifying original
  result <- data.table::copy(crosswalk)

  # ==========================================================================
  # STRATEGY DISPATCH
  # ==========================================================================

  if (strategy == "probabilistic") {
    result <- .apply_probabilistic(
      crosswalk = result,
      confidence_threshold = confidence_threshold,
      verbose = verbose
    )
  } else if (strategy == "upa_aggregation") {
    result <- .apply_upa_aggregation(
      crosswalk = result,
      threshold = upa_proportion_threshold,
      verbose   = verbose)
  } else if (strategy == "both") {
    result <- .apply_combined_strategy(
      crosswalk = result,
      confidence_threshold = confidence_threshold,
      upa_threshold        = upa_proportion_threshold,
      verbose              = verbose)
  }

  result
}


# =============================================================================
# PROBABILISTIC STRATEGY (Nested) - OPTIMIZED
# =============================================================================

#' Apply Probabilistic Strategy with Proper Nesting (Optimized)
#'
#' Uses date bounds stored in crosswalk from
#' pnadc_identify_periods(store_date_bounds=TRUE). This avoids:
#' - Copying and processing the full data (~90% of computation)
#' - Redundant date calculations
#' - Multiple aggregations
#'
#' @param crosswalk Crosswalk data.table (may contain pre-computed date bounds)
#' @param confidence_threshold Minimum confidence to assign
#' @param verbose Print progress
#' @return Modified crosswalk with probabilistic columns
#'
#' @keywords internal
#' @noRd
.apply_probabilistic <- function(crosswalk, confidence_threshold, verbose) {


  # Helper functions for safe min/max that don't produce warnings on all-NA vectors
  safe_min <- function(x) { x <- x[!is.na(x)]; if (length(x) == 0L) NA_real_ else min(x) }
  safe_max <- function(x) { x <- x[!is.na(x)]; if (length(x) == 0L) NA_real_ else max(x) }

  # ==========================================================================
  # PRE-COMPUTE ALL IBGE BOUNDARIES ONCE
  # ==========================================================================

  calendar_1 <- unique(crosswalk[ref_month_in_quarter == 1,
                                 .(Ano, Trimestre,

                                   week_1_start,
                                   week_1_end,
                                   week_2_start,
                                   week_2_end,
                                   week_3_start,
                                   week_3_end,
                                   week_4_start,
                                   week_4_end)])

  calendar_2 <- unique(crosswalk[ref_month_in_quarter == 2,
                                 .(Ano, Trimestre,

                                   week_5_start = week_1_start,
                                   week_5_end   = week_1_end,
                                   week_6_start = week_2_start,
                                   week_6_end   = week_2_end,
                                   week_7_start = week_3_start,
                                   week_7_end   = week_3_end,
                                   week_8_start = week_4_start,
                                   week_8_end   = week_4_end)])


  calendar_3 <- unique(crosswalk[ref_month_in_quarter ==3,
                                 .(Ano, Trimestre,

                                   week_9_start  = week_1_start,
                                   week_9_end    = week_1_end,
                                   week_10_start = week_2_start,
                                   week_10_end   = week_2_end,
                                   week_11_start = week_3_start,
                                   week_11_end   = week_3_end,
                                   week_12_start = week_4_start,
                                   week_12_end   = week_4_end)])

  data.table::setkey(calendar_1, Ano, Trimestre)
  data.table::setkey(calendar_2, Ano, Trimestre)
  data.table::setkey(calendar_3, Ano, Trimestre)

  calendar_quarter_weeks = calendar_1 |> merge(calendar_2, all.x = TRUE) |> merge(calendar_3, all.x = TRUE)


  calendar_month_weeks <- unique(crosswalk[!is.na(ref_month_in_quarter),
                                           .(Ano, Trimestre, ref_month_in_quarter,

                                             week_1_start,
                                             week_1_end,
                                             week_2_start,
                                             week_2_end,
                                             week_3_start,
                                             week_3_end,
                                             week_4_start,
                                             week_4_end)])


  rm(calendar_1,
     calendar_2,
     calendar_3); gc()

  # ==========================================================================
  # STEP 1: MONTH PROBABILISTIC
  # ==========================================================================


  # Pre-computing the calendar
  # ==========================================================================

  crosswalk[ , week_1_start := NULL]
  crosswalk[ , week_1_end   := NULL]
  crosswalk[ , week_2_start := NULL]
  crosswalk[ , week_2_end   := NULL]
  crosswalk[ , week_3_start := NULL]
  crosswalk[ , week_3_end   := NULL]
  crosswalk[ , week_4_start := NULL]
  crosswalk[ , week_4_end   := NULL]

  if (verbose) cat("  Phase 1: Month probabilistic identification...\n")

  # Set key for faster operations
  data.table::setkey(crosswalk, Ano, Trimestre, UPA, V1014)

  # Calculate month range
  crosswalk[, month_range := month_max_upa - month_min_upa + 1]

  # For range == 2, sequential, not strictly determined: calculate probabilistic assignment
  crosswalk[, month_prob_filter :=
              month_range == 2L &
              is.na(ref_month_in_quarter) &
              !is.na(date_min) &
              !is.na(date_max)
  ]

  # Joining with the calendar
  crosswalk[calendar_quarter_weeks,
            on = .(Ano, Trimestre),
            `:=`(week_1_start = i.week_1_start,
                 week_1_end   = i.week_1_end,
                 week_2_start = i.week_2_start,
                 week_2_end   = i.week_2_end,
                 week_3_start = i.week_3_start,
                 week_3_end   = i.week_3_end,
                 week_4_start = i.week_4_start,
                 week_4_end   = i.week_4_end,

                 week_5_start  = i.week_5_start,
                 week_5_end    = i.week_5_end,
                 week_6_start  = i.week_6_start,
                 week_6_end    = i.week_6_end,
                 week_7_start  = i.week_7_start,
                 week_7_end    = i.week_7_end,
                 week_8_start  = i.week_8_start,
                 week_8_end    = i.week_8_end,

                 week_9_start  = i.week_9_start,
                 week_9_end    = i.week_9_end,
                 week_10_start = i.week_10_start,
                 week_10_end   = i.week_10_end,
                 week_11_start = i.week_11_start,
                 week_11_end   = i.week_11_end,
                 week_12_start = i.week_12_start,
                 week_12_end   = i.week_12_end

            )]

  # When possible months are Months 1 and 2 within quarter
  # ==========================================================================

  # Adjusting minimum and maximum possible dates for observation whose probable months were narrowed down to 1 or 2
  crosswalk[month_prob_filter == 1 & month_max_upa == 2 & month_min_upa == 1,
            `:=`(
              date_min = fifelse(date_min < week_1_start, week_1_start, date_min),
              date_max = fifelse(date_max > week_8_end,   week_8_end,   date_max)
            )]

  # Calculating days within months 1 and 2
  crosswalk[month_prob_filter == 1 & month_max_upa == 2 & month_min_upa == 1,
            `:=`(
              days_within_month_1_min1_max2 = fifelse(is.na(week_4_end) | is.na(date_min), 0L, pmax(0L, as.integer(week_4_end - date_min + 1))),
              days_within_month_2_min1_max2 = fifelse(is.na(date_max) | is.na(week_5_start), 0L, pmax(0L, as.integer(date_max - week_5_start + 1)))
            )]

  # Calculating share of days within months 1 and 2
  crosswalk[month_prob_filter == 1 & month_max_upa == 2 & month_min_upa == 1,
            `:=`(
              prop_month_1_min1_max2 = days_within_month_1_min1_max2/(days_within_month_1_min1_max2 + days_within_month_2_min1_max2 + 1e-10),
              prop_month_2_min1_max2 = days_within_month_2_min1_max2/(days_within_month_1_min1_max2 + days_within_month_2_min1_max2 + 1e-10)
            )]

  # Assigning probable months
  crosswalk[month_prob_filter == 1 & month_max_upa == 2 & month_min_upa == 1,
            `:=`(
              prob_month_min1_max2 = fcase(prop_month_1_min1_max2 >= confidence_threshold, 1,
                                           prop_month_2_min1_max2 >= confidence_threshold, 2,
                                           default = NA),

              determined_probable_month = TRUE)]


  # When possible months are Months 2 and 3 within quarter
  # ==========================================================================

  # Adjusting minimum and maximum possible dates for observation whose probable months were narrowed down to 2 or 3
  crosswalk[month_prob_filter == 1 & month_max_upa == 3 & month_min_upa == 2,
            `:=`(
              date_min = fifelse(date_min < week_5_start, week_5_start, date_min),
              date_max = fifelse(date_max > week_12_end,   week_12_end, date_max)
            )]

  # Calculating days within months 2 and 3
  crosswalk[month_prob_filter == 1 & month_max_upa == 3 & month_min_upa == 2,
            `:=`(
              days_within_month_2_min2_max3 = fifelse(is.na(week_8_end) | is.na(date_min), 0L, pmax(0L, as.integer(week_8_end - date_min + 1))),
              days_within_month_3_min2_max3 = fifelse(is.na(date_max) | is.na(week_9_start), 0L, pmax(0L, as.integer(date_max - week_9_start + 1)))
            )]

  # Calculating share of days within months 2 and 3
  crosswalk[month_prob_filter == 1 & month_max_upa == 3 & month_min_upa == 2,
            `:=`(
              prop_month_2_min2_max3 = days_within_month_2_min2_max3/(days_within_month_2_min2_max3 + days_within_month_3_min2_max3 + 1e-10),
              prop_month_3_min2_max3 = days_within_month_3_min2_max3/(days_within_month_2_min2_max3 + days_within_month_3_min2_max3 + 1e-10)
            )]

  # Assigning probable months
  crosswalk[month_prob_filter == 1 & month_max_upa == 3 & month_min_upa == 2,
            `:=`(
              prob_month_min2_max3 = fcase(prop_month_2_min2_max3 >= confidence_threshold, 2,
                                           prop_month_3_min2_max3 >= confidence_threshold, 3,
                                           default = NA))]


  # Assigning Months
  # ==========================================================================

  crosswalk[month_prob_filter == 1 & determined_month == FALSE,
            `:=`(
              prob_ref_month_in_quarter = fcase(!is.na(prob_month_min1_max2), prob_month_min1_max2,
                                                !is.na(prob_month_min2_max3), prob_month_min2_max3,
                                                default = NA)
            )]


  crosswalk[, determined_probable_month := fifelse(!is.na(prob_ref_month_in_quarter), TRUE, FALSE)]

  # Aggregating by UPA-V1014 across quarters
  crosswalk[month_prob_filter == 1,
            `:=`(
              prob_ref_month_in_quarter_min = safe_min(prob_ref_month_in_quarter),
              prob_ref_month_in_quarter_max = safe_max(prob_ref_month_in_quarter)
            ),
            by = c("UPA", "V1014")]


  crosswalk[month_prob_filter == 1 & prob_ref_month_in_quarter_min == prob_ref_month_in_quarter_max,

            `:=`(ref_month_in_quarter = prob_ref_month_in_quarter_min,
                 determined_month = TRUE,
                 probabilistic_assignment = TRUE)]


  crosswalk[month_prob_filter == 1 & !is.na(ref_month_in_quarter),
            `:=`(
              ref_month_in_year = ref_month_in_quarter + (Trimestre - 1)*3
            )]


  # Cleaning up
  # ==========================================================================

  crosswalk[,
            `:=`(
              days_within_month_1_min1_max2 = NULL,
              days_within_month_2_min1_max2 = NULL,
              prop_month_1_min1_max2        = NULL,
              prop_month_2_min1_max2        = NULL,
              prob_month_min1_max2          = NULL,

              days_within_month_2_min2_max3 = NULL,
              days_within_month_3_min2_max3 = NULL,
              prop_month_2_min2_max3        = NULL,
              prop_month_3_min2_max3        = NULL,
              prob_month_min2_max3          = NULL,
              prob_ref_month_in_quarter     = NULL,
              prob_ref_month_in_quarter_min = NULL,
              prob_ref_month_in_quarter_max = NULL,

              week_5_start  = NULL,
              week_5_end    = NULL,
              week_6_start  = NULL,
              week_6_end    = NULL,
              week_7_start  = NULL,
              week_7_end    = NULL,
              week_8_start  = NULL,
              week_8_end    = NULL,

              week_9_start  = NULL,
              week_9_end    = NULL,
              week_10_start = NULL,
              week_10_end   = NULL,
              week_11_start = NULL,
              week_11_end   = NULL,
              week_12_start = NULL,
              week_12_end   = NULL,

              month_range       = NULL,
              month_prob_filter = NULL
            )]

  gc()


  # Stats
  # ==========================================================================

  n_total      <- crosswalk[, .N]
  n_month_prob <- sum(crosswalk$determined_probable_month, na.rm = TRUE)

  if (verbose) {
    cat(sprintf("    Assigned months to %s observations, representing %.1f%% of the total (confidence >= %.0f%%)\n",
                format(n_month_prob, big.mark = ","),
                n_month_prob/n_total * 100,
                confidence_threshold * 100))
  }


  # ==========================================================================
  # STEP 2: FORTNIGHT PROBABILISTIC (using pre-computed integer bounds)
  # ==========================================================================

  if (verbose) cat("  Phase 2: Fortnight probabilistic identification (nested)...\n")


  # For range == 2, sequential, not strictly determined: calculate probabilistic assignment
  crosswalk[, fortnight_prob_filter :=
              !is.na(ref_month_in_quarter) &
              #probabilistic_assignment == T &
              is.na(ref_fortnight_in_month) &
              !is.na(date_min) &
              !is.na(date_max)
  ]


  # When possible fortnights are fortnights 1 and 2 within months
  # ==========================================================================


  # Freeing memory after deleting columns
  gc()

  # Joining with the calendar
  crosswalk[calendar_month_weeks,
            on = .(Ano, Trimestre, ref_month_in_quarter),
            `:=`(week_1_start = i.week_1_start,
                 week_1_end   = i.week_1_end,
                 week_2_start = i.week_2_start,
                 week_2_end   = i.week_2_end,
                 week_3_start = i.week_3_start,
                 week_3_end   = i.week_3_end,
                 week_4_start = i.week_4_start,
                 week_4_end   = i.week_4_end
            )]

  # Adjusting minimum and maximum possible dates for observation whose probable fortnights were narrowed down to 1 or 2
  crosswalk[fortnight_prob_filter == 1,
            `:=`(
              date_min = fifelse(date_min < week_1_start, week_1_start, date_min),
              date_max = fifelse(date_max > week_4_end,   week_4_end,   date_max)
            )]

  # Calculating days within fortnights 1 and 2
  crosswalk[fortnight_prob_filter == 1,
            `:=`(
              days_within_fortnight_1 = fifelse(is.na(week_2_end) | is.na(date_min), 0L, pmax(0L, as.integer(week_2_end - date_min + 1))),
              days_within_fortnight_2 = fifelse(is.na(date_max) | is.na(week_3_start), 0L, pmax(0L, as.integer(date_max - week_3_start + 1)))
            )]

  # Calculating share of days within fortnights 1 and 2
  crosswalk[fortnight_prob_filter == 1,
            `:=`(
              prop_fortnight_1 = days_within_fortnight_1/(days_within_fortnight_1 + days_within_fortnight_2 + 1e-10),
              prop_fortnight_2 = days_within_fortnight_2/(days_within_fortnight_1 + days_within_fortnight_2 + 1e-10)
            )]


  # Assigning fortnights
  # ==========================================================================

  # Assigning probable fortnights
  crosswalk[fortnight_prob_filter == 1,

            prob_ref_fortnight_in_month := fcase(prop_fortnight_1 >= confidence_threshold, 1,
                                                 prop_fortnight_2 >= confidence_threshold, 2,
                                                 default = NA)]


  # Flagging if fortnight was determined probabilistically
  crosswalk[, determined_probable_fortnight := fifelse(!is.na(prob_ref_fortnight_in_month), TRUE, FALSE)]


  # Aggregating by household within quarter (not by UPA-V1014 across quarters)
  crosswalk[fortnight_prob_filter == 1,
            `:=`(
              prob_ref_fortnight_in_month_min = safe_min(prob_ref_fortnight_in_month),
              prob_ref_fortnight_in_month_max = safe_max(prob_ref_fortnight_in_month)
            ),
            by = c("Ano", "Trimestre", "UPA", "V1008")]


  crosswalk[fortnight_prob_filter == 1 & prob_ref_fortnight_in_month_min == prob_ref_fortnight_in_month_max,

            `:=`(ref_fortnight_in_month = prob_ref_fortnight_in_month_min,
                 determined_fortnight = TRUE,
                 probabilistic_assignment = TRUE)]


  crosswalk[fortnight_prob_filter == 1 & !is.na(ref_fortnight_in_month),
            `:=`(
              ref_fortnight_in_quarter = ref_fortnight_in_month + (ref_month_in_quarter - 1)*2
            )]



  # Cleaning up
  # ==========================================================================

  crosswalk[,
            `:=`(
              days_within_fortnight_1 = NULL,
              days_within_fortnight_2 = NULL,
              prop_fortnight_1 = NULL,
              prop_fortnight_2 = NULL,
              prob_ref_fortnight_in_month     = NULL,
              prob_ref_fortnight_in_month_min = NULL,
              prob_ref_fortnight_in_month_max = NULL,

              fortnight_prob_filter = NULL)]

  gc()


  # Stats
  # ==========================================================================

  n_total          <- crosswalk[, .N]
  n_fortnight_prob <- sum(crosswalk$determined_probable_fortnight, na.rm = TRUE)

  if (verbose) {
    cat(sprintf("    Assigned fortnights to %s observations, representing %.1f%% of the total (confidence >= %.0f%%)\n",
                format(n_fortnight_prob, big.mark = ","),
                n_fortnight_prob/n_total * 100,
                confidence_threshold * 100))
  }


  # ==========================================================================
  # STEP 3: WEEK PROBABILISTIC (using pre-computed integer bounds)
  # ==========================================================================

  if (verbose) cat("  Phase 3: Week probabilistic identification (nested)...\n")

  # For range == 2, sequential, not strictly determined: calculate probabilistic assignment
  crosswalk[, week_prob_filter :=
              !is.na(ref_fortnight_in_month) &
              #probabilistic_assignment == T &
              is.na(ref_week_in_month) &
              !is.na(date_min) &
              !is.na(date_max)
  ]

  # NESTING: Only process observations with identified fortnight

  # When possible weeks are weeks 1 and 2 within month
  # ==========================================================================

  # Adjusting minimum and maximum possible dates for observation whose probable weeks within month were narrowed down to 1 or 2
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 1,
            `:=`(
              date_min = fifelse(date_min < week_1_start, week_1_start, date_min),
              date_max = fifelse(date_max > week_2_end,   week_2_end,   date_max)
            )]

  # Calculating days within weeks 1 and 2
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 1,
            `:=`(
              days_within_week_1 = fifelse(is.na(week_1_end) | is.na(date_min), 0L, pmax(0L, as.integer(week_1_end - date_min + 1))),
              days_within_week_2 = fifelse(is.na(date_max) | is.na(week_2_start), 0L, pmax(0L, as.integer(date_max - week_2_start + 1)))
            )]

  # Calculating share of days within weeks 1 and 2
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 1,
            `:=`(
              prop_week_1 = days_within_week_1/(days_within_week_1 + days_within_week_2 + 1e-10),
              prop_week_2 = days_within_week_2/(days_within_week_1 + days_within_week_2 + 1e-10)
            )]


  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 1,
            `:=`(
              prob_week_1_2 = fcase(prop_week_1 >= confidence_threshold, 1,
                                    prop_week_2 >= confidence_threshold, 2,
                                    default = NA))]


  # When possible weeks are weeks 3 and 4 within month
  # ==========================================================================

  # Adjusting minimum and maximum possible dates for observation whose probable weeks within month were narrowed down to 3 or 4
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 2,
            `:=`(
              date_min = fifelse(date_min < week_3_start, week_3_start, date_min),
              date_max = fifelse(date_max > week_4_end,   week_4_end,   date_max)
            )]

  # Calculating days within weeks 3 and 4
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 2,
            `:=`(
              days_within_week_3 = fifelse(is.na(week_3_end) | is.na(date_min), 0L, pmax(0L, as.integer(week_3_end - date_min + 1))),
              days_within_week_4 = fifelse(is.na(date_max) | is.na(week_4_start), 0L, pmax(0L, as.integer(date_max - week_4_start + 1)))
            )]

  # Calculating share of days within weeks 3 and 4
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 2,
            `:=`(
              prop_week_3 = days_within_week_3/(days_within_week_3 + days_within_week_4 + 1e-10),
              prop_week_4 = days_within_week_4/(days_within_week_3 + days_within_week_4 + 1e-10)
            )]


  # Assigning probable weeks
  crosswalk[week_prob_filter == 1 & ref_fortnight_in_month == 2,
            `:=`(
              prob_week_3_4 = fcase(prop_week_3 >= confidence_threshold, 3,
                                    prop_week_4 >= confidence_threshold, 4,
                                    default = NA))]


  # Assigning Probable weeks
  # ==========================================================================

  crosswalk[week_prob_filter == 1 & determined_week == FALSE,
            `:=`(
              prob_ref_week_in_month = fcase(!is.na(prob_week_1_2), prob_week_1_2,
                                             !is.na(prob_week_3_4), prob_week_3_4,
                                             default = NA)
            )]

  # Flagging if week was determined probabilistically
  crosswalk[, determined_probable_week := fifelse(!is.na(prob_ref_week_in_month), TRUE, FALSE)]

  # Aggregating by households within quarters (not by UPA-V1014 across quarters)
  crosswalk[week_prob_filter == 1,
            `:=`(
              prob_ref_week_in_month_min = safe_min(prob_ref_week_in_month),
              prob_ref_week_in_month_max = safe_max(prob_ref_week_in_month)
            ),
            by = c("Ano", "Trimestre", "UPA", "V1008")]


  crosswalk[week_prob_filter == 1 & prob_ref_week_in_month_min == prob_ref_week_in_month_max,

            `:=`(ref_week_in_month = prob_ref_week_in_month_min,
                 determined_week  = TRUE,
                 probabilistic_assignment = TRUE)]


  crosswalk[week_prob_filter == 1 & !is.na(ref_week_in_month),
            `:=`(
              ref_week_in_quarter = ref_week_in_month + (ref_month_in_quarter - 1)*4
            )]

  # Cleaning up
  # ==========================================================================

  crosswalk[,
            `:=`(
              days_within_week_1 = NULL,
              days_within_week_2 = NULL,
              days_within_week_3 = NULL,
              days_within_week_4 = NULL,

              prop_week_1 = NULL,
              prop_week_2 = NULL,
              prop_week_3 = NULL,
              prop_week_4 = NULL,

              prob_week_1_2 = NULL,
              prob_week_3_4 = NULL,

              week_prob_filter = NULL,

              prob_ref_week_in_month     = NULL,
              prob_ref_week_in_month_min = NULL,
              prob_ref_week_in_month_max = NULL)]

  gc()


  # Stats
  # ==========================================================================

  n_week_prob <- sum(crosswalk$determined_probable_week, na.rm = TRUE)

  if (verbose) {
    cat(sprintf("    Assigned weeks to %s observations, representing %.1f%% of the total (confidence >= %.0f%%)\n",
                format(n_week_prob, big.mark = ","),
                n_week_prob/n_total * 100,
                confidence_threshold * 100))
  }


  # ==========================================================================
  # Re-organizing the crosswalk
  # ==========================================================================

  crosswalk[, `:=`(
    ref_month_yyyymm     = Ano * 100 + ref_month_in_year,
    ref_fortnight_yyyyff = Ano * 100 + ref_fortnight_in_quarter + (Trimestre - 1)*6,
    ref_week_yyyyww      = Ano * 100 + ref_week_in_month + (ref_month_in_year - 1)*4

  )]


  crosswalk <- crosswalk[, .(Ano, Trimestre, UPA, V1008, V1014,

                             ref_month_in_quarter,     ref_month_in_year,
                             ref_fortnight_in_month,   ref_fortnight_in_quarter,
                             ref_week_in_month,        ref_week_in_quarter,

                             date_max,                 date_min,
                             month_max_upa,            month_min_upa,
                             fortnight_max_hh,         fortnight_min_hh,
                             week_min_hh,              week_max_hh,

                             week_1_start, week_1_end,   week_2_start,  week_2_end,
                             week_3_start, week_3_end,   week_4_start,  week_4_end,

                             ref_month_yyyymm, ref_fortnight_yyyyff, ref_week_yyyyww,

                             determined_month, determined_fortnight, determined_week,

                             determined_probable_fortnight,
                             determined_probable_week,
                             determined_probable_month,
                             probabilistic_assignment)]

  gc()

  if (verbose) cat("  Probabilistic strategy complete.\n")

  crosswalk
}


# =============================================================================
# UPA AGGREGATION STRATEGY (Nested)
# =============================================================================

#' Apply UPA Aggregation Strategy with Proper Nesting
#'
#' Extends strictly identified periods to unidentified observations based on
#' consensus within geographic/sampling groups:
#' \itemize{
#'   \item \strong{Months}: Aggregates at UPA-V1014 level (panel design ensures
#'     same month position across quarters)
#'   \item \strong{Fortnights/Weeks}: Aggregates at UPA level (all households in
#'     same UPA are interviewed in same fortnight/week within a quarter,
#'     regardless of panel rotation V1014)
#' }
#'
#' @param crosswalk Crosswalk data.table
#' @param threshold Minimum proportion of observations with strict identification
#'   required before extending to unidentified observations
#' @param verbose Print progress
#' @return Modified crosswalk with UPA-aggregated columns
#' @keywords internal
#' @noRd
.apply_upa_aggregation <- function(crosswalk, threshold, verbose) {

  if (verbose) cat("Applying UPA aggregation strategy (nested)...\n")

  # ==========================================================================
  # STEP 1: MONTH AGGREGATION
  # ==========================================================================

  if (verbose) cat("  Phase 1: Month UPA aggregation...\n")

  # tmp columns
  crosswalk[, `:=`(
    ref_month_in_quarter_aggreg            = NA_integer_,
    ref_month_in_quarter_aggreg_confidence = NA_real_)]


  # Calculate proportion of UPA within-quarter with identified month
  upa_month_stats <- crosswalk[, .(
    n_total = .N,
    n_month = sum(!is.na(ref_month_in_quarter)),
    consensus_month = fifelse(
      uniqueN(ref_month_in_quarter[!is.na(ref_month_in_quarter)]) == 1L,
      ref_month_in_quarter[!is.na(ref_month_in_quarter)][1L],
      NA_integer_
    )
  ), by = .(Ano, Trimestre, UPA)]

  upa_month_stats[, prop_month := n_month / n_total]

  # Only extend if proportion >= threshold AND there's a consensus
  upa_month_qualify <- upa_month_stats[prop_month >= threshold & !is.na(consensus_month)]

  if (nrow(upa_month_qualify) > 0) {
    # Join and assign to observations without strict month
    crosswalk[upa_month_qualify, on = .(Ano, Trimestre, UPA), `:=`(
      ref_month_in_quarter_aggreg            = fifelse(is.na(ref_month_in_quarter), i.consensus_month, ref_month_in_quarter_aggreg),
      ref_month_in_quarter_aggreg_confidence = fifelse(is.na(ref_month_in_quarter), 1.0, ref_month_in_quarter_aggreg_confidence)
    )]
  }

  n_month_upa <- sum(!is.na(crosswalk$ref_month_in_quarter_aggreg) & is.na(crosswalk$ref_month_in_quarter))
  if (verbose) {
    cat(sprintf("    Extended to %s observations via UPA aggregation\n",
                format(n_month_upa, big.mark = ",")))
  }

  # ==========================================================================
  # STEP 2: FORTNIGHT AGGREGATION (NESTED)
  # ==========================================================================

  if (verbose) cat("  Phase 2: Fortnight UPA aggregation (nested)...\n")

  crosswalk[, `:=`(
    ref_fortnight_in_month_aggreg            = NA_integer_,
    ref_fortnight_in_month_aggreg_confidence = NA_real_)]

  # Calculate proportion of UPA (not UPA-V1014) within-quarter with identified fortnight
  upa_fortnight_stats <- crosswalk[, .(
    n_total = .N,
    n_fortnight = sum(!is.na(ref_fortnight_in_month)),
    consensus_fortnight = fifelse(
      uniqueN(ref_fortnight_in_month[!is.na(ref_fortnight_in_month)]) == 1L,
      ref_fortnight_in_month[!is.na(ref_fortnight_in_month)][1L],
      NA_integer_
    )
  ), by = .(Ano, Trimestre, UPA)]

  upa_fortnight_stats[, prop_fortnight := n_fortnight / n_total]

  # Only extend if proportion >= threshold AND there's a consensus
  upa_fortnight_qualify <- upa_fortnight_stats[prop_fortnight >= threshold & !is.na(consensus_fortnight)]

  if (nrow(upa_fortnight_qualify) > 0) {
    # NESTING: Only assign to observations with identified month
    # AND verify consensus_fortnight falls within that month's bounds
    crosswalk[, month_identified := !is.na(ref_month_in_quarter) | !is.na(ref_month_in_quarter_aggreg)]
    crosswalk[month_identified == TRUE, `:=`(
      effective_month =  fifelse(!is.na(ref_month_in_quarter), ref_month_in_quarter, ref_month_in_quarter_aggreg),
      fortnight_lower = 1L,
      fortnight_upper = 2L
    )]

    # Join consensus_fortnight and validate it's within the month's fortnight bounds
    # INVARIANT: Also validate consensus_fortnight is in [1, 2] range (fortnight within month)
    crosswalk[upa_fortnight_qualify, on = .(Ano, Trimestre, UPA), `:=`(
      ref_fortnight_in_month_aggreg = fifelse(
        is.na(ref_fortnight_in_quarter) & month_identified &
          i.consensus_fortnight >= fortnight_lower &
          i.consensus_fortnight <= fortnight_upper &
          i.consensus_fortnight >= 1L & i.consensus_fortnight <= 2L,
        i.consensus_fortnight,
        ref_fortnight_in_month_aggreg
      ),
      ref_fortnight_in_month_aggreg_confidence = fifelse(
        is.na(ref_fortnight_in_quarter) & month_identified &
          i.consensus_fortnight >= fortnight_lower &
          i.consensus_fortnight <= fortnight_upper,
        1.0,
        ref_fortnight_in_month_aggreg_confidence
      )
    )]

    crosswalk[, c("month_identified", "effective_month", "fortnight_lower", "fortnight_upper") := NULL]
  }

  n_fortnight_upa <- sum(!is.na(crosswalk$ref_fortnight_in_month_aggreg) & is.na(crosswalk$ref_fortnight_in_quarter))
  if (verbose) {
    cat(sprintf("    Extended to %s observations via UPA aggregation\n",
                format(n_fortnight_upa, big.mark = ",")))
  }

  # ==========================================================================
  # STEP 3: WEEK AGGREGATION (NESTED)
  # ==========================================================================

  if (verbose) cat("  Phase 3: Week UPA aggregation (nested)...\n")

  crosswalk[, `:=`(
    ref_week_in_month_aggreg            = NA_integer_,
    ref_week_in_month_aggreg_confidence = NA_real_)]

  # Calculate proportion of UPA within-quarter with identified week
  upa_week_stats <- crosswalk[, .(
    n_total = .N,
    n_week = sum(!is.na(ref_week_in_month)),
    consensus_week = fifelse(
      uniqueN(ref_week_in_month[!is.na(ref_week_in_month)]) == 1L,
      ref_week_in_month[!is.na(ref_week_in_month)][1L],
      NA_integer_
    )
  ), by = .(Ano, Trimestre, UPA)]

  upa_week_stats[, prop_week := n_week / n_total]

  # Only extend if proportion >= threshold AND there's a consensus
  upa_week_qualify <- upa_week_stats[prop_week >= threshold & !is.na(consensus_week)]

  if (nrow(upa_week_qualify) > 0) {
    # NESTING: Only assign to observations with identified fortnight
    # Note: Full week-within-fortnight validation would require complex date calculations.
    # Since consensus_week comes from strictly identified weeks (which were already
    # constrained to their fortnight bounds during strict identification), and we require
    # fortnight_identified, the assignment should be valid. We rely on the strict
    # algorithm's nesting enforcement for the source consensus values.
    crosswalk[, fortnight_identified := !is.na(ref_fortnight_in_quarter) | !is.na(ref_fortnight_in_month_aggreg)]

    # INVARIANT: Validate consensus_week is in [1, 4] range
    crosswalk[upa_week_qualify, on = .(Ano, Trimestre, UPA), `:=`(
      ref_week_in_month_aggreg = fifelse(
        is.na(ref_week_in_quarter) & fortnight_identified &
          i.consensus_week >= 1L & i.consensus_week <= 4L,
        i.consensus_week,
        ref_week_in_month_aggreg
      ),
      ref_week_in_month_aggreg_confidence = fifelse(
        is.na(ref_week_in_quarter) & fortnight_identified &
          i.consensus_week >= 1L & i.consensus_week <= 4L,
        1.0,
        ref_week_in_month_aggreg_confidence
      )
    )]

    crosswalk[, fortnight_identified := NULL]
  }

  n_week_upa <- sum(!is.na(crosswalk$ref_week_in_month_aggreg) & is.na(crosswalk$ref_week_in_quarter))
  if (verbose) {
    cat(sprintf("    Extended to %s observations via UPA aggregation\n",
                format(n_week_upa, big.mark = ",")))
  }




  crosswalk[ ,
             `:=`(

               ref_month_in_quarter = fifelse(is.na(ref_month_in_quarter) & !is.na(ref_month_in_quarter_aggreg),
                                              ref_month_in_quarter_aggreg,
                                              ref_month_in_quarter),

               ref_fortnight_in_month = fifelse(is.na(ref_fortnight_in_month) & !is.na(ref_fortnight_in_month_aggreg),
                                                ref_fortnight_in_month_aggreg,
                                                ref_fortnight_in_month),

               ref_week_in_month = fifelse(is.na(ref_week_in_month) & !is.na(ref_week_in_month_aggreg),
                                           ref_week_in_month_aggreg,
                                           ref_week_in_month),

               determined_aggreg_month = fifelse(!is.na(ref_month_in_quarter_aggreg), TRUE, FALSE),
               determined_month        = fifelse(!is.na(ref_month_in_quarter_aggreg), TRUE, determined_month),

               determined_aggreg_fortnight = fifelse(!is.na(ref_fortnight_in_month_aggreg), TRUE, FALSE),
               determined_fortnight        = fifelse(!is.na(ref_fortnight_in_month_aggreg), TRUE, determined_fortnight),

               determined_aggreg_week = fifelse(!is.na(ref_week_in_month_aggreg), TRUE, FALSE),
               determined_week        = fifelse(!is.na(ref_week_in_month_aggreg), TRUE, determined_week)
    )]

  crosswalk[, `:=`(
            ref_month_in_quarter_aggreg               = NULL,
            ref_month_in_quarter_aggreg_confidence    = NULL,
            ref_fortnight_in_month_aggreg             = NULL,
            ref_fortnight_in_month_aggreg_confidence  = NULL,
            ref_week_in_month_aggreg                  = NULL,
            ref_week_in_month_aggreg_confidence       = NULL)]

  crosswalk[, `:=`(
    ref_month_in_year        = ref_month_in_quarter + (Trimestre - 1)*3,
    ref_fortnight_in_quarter = ref_fortnight_in_month + (ref_month_in_quarter - 1)*2,
    ref_week_in_quarter      = ref_week_in_month + (ref_month_in_quarter - 1)*4

    )]

  crosswalk[, `:=`(
    ref_month_yyyymm     = Ano * 100 + ref_month_in_year,
    ref_fortnight_yyyyff = Ano * 100 + (ref_fortnight_in_quarter + (Trimestre - 1)*6),
    ref_week_yyyyww      = Ano * 100 + (ref_week_in_month + (ref_month_in_year - 1)*4)

  )]


  # Update probabilistic_assignment flag only if it already exists (from probabilistic strategy in "both" mode)
  # UPA aggregation standalone intentionally does not create this column
  if ("probabilistic_assignment" %in% names(crosswalk)) {
    crosswalk[, probabilistic_assignment := data.table::fcoalesce(probabilistic_assignment, FALSE) |
                determined_aggreg_month | determined_aggreg_fortnight | determined_aggreg_week]
  }


  if (verbose) {
    cat("  UPA aggregation strategy complete.\n")
  }

  crosswalk
}


# =============================================================================
# COMBINED STRATEGY (Nested)
# =============================================================================

#' Apply Combined Strategy (Probabilistic + UPA Aggregation) with Proper Nesting
#'
#' Sequentially applies both strategies: probabilistic first, then UPA aggregation.
#' This guarantees that "both" identifies at least as many periods as either
#' individual strategy alone.
#'
#' The previous implementation had an extra `prop_narrow` filter that made it
#' more restrictive than probabilistic alone. This refactored version ensures
#' the union behavior users expect from the "both" option.
#'
#' @param crosswalk Crosswalk data.table
#' @param data Original PNADC data
#' @param confidence_threshold Minimum confidence to assign
#' @param upa_threshold Minimum proportion of UPA with narrow ranges
#' @param verbose Print progress
#' @return Modified crosswalk
#' @keywords internal
#' @noRd
.apply_combined_strategy <- function(crosswalk, confidence_threshold, upa_threshold, verbose) {

  if (verbose) cat("Applying combined strategy (probabilistic + UPA aggregation, nested)...\n")

  # ==========================================================================
  # STEP 1: Apply probabilistic strategy first
  # ==========================================================================

  crosswalk <- .apply_probabilistic(crosswalk = crosswalk,
                                    confidence_threshold = confidence_threshold,
                                    verbose = verbose)

  # ==========================================================================
  # STEP 2: Apply UPA aggregation on top
  # ==========================================================================

  crosswalk <- .apply_upa_aggregation(crosswalk = crosswalk,
                                      threshold = upa_threshold,
                                      verbose   = verbose)


  if (verbose) {
    cat("  Combined strategy complete.\n")
  }

  crosswalk
}

#' @title Fetch Monthly Population from SIDRA
#' @description Functions to download and transform IBGE's population estimates
#'   from SIDRA API for use in monthly weight calibration.
#' @name fetch-sidra-population
#' @keywords internal
NULL

# ============================================================================
# NOTE: Caching infrastructure is in fetch-sidra-series.R (.sidra_cache)
# This file uses the unified cache via .is_cache_valid(), .get_cache(), .set_cache()
# ============================================================================

#' Fetch Monthly Population from SIDRA
#'
#' Downloads population estimates from IBGE SIDRA API (table 6022) and
#' transforms from moving-quarter to exact monthly values.
#'
#' @param start_yyyymm Integer. First month to include (YYYYMM format).
#'   If NULL, returns all available months.
#' @param end_yyyymm Integer. Last month to include (YYYYMM format).
#'   If NULL, returns all available months.
#' @param verbose Logical. Print progress messages? Default TRUE.
#' @param use_cache Logical. If TRUE, uses cached data if available and not
#'   expired. Default FALSE (always fetch fresh data for consistency).
#'   Set to TRUE for faster repeated calls during development.
#' @param cache_max_age_hours Numeric. Maximum cache age in hours before
#'   automatic expiration when use_cache=TRUE. Default 24 hours.
#'
#' @return A data.table with columns:
#'   \itemize{
#'     \item \code{ref_month_yyyymm}: Integer in YYYYMM format
#'     \item \code{m_populacao}: Monthly population in thousands
#'   }
#'   Returns \code{NULL} invisibly with an informative message if the SIDRA
#'   API is unreachable (per CRAN policy on Internet resources).
#'
#' @details
#' SIDRA table 6022 provides moving-quarter population estimates. Each value
#' represents the 3-month average centered on the middle month. For example,
#' the value for code 201203 (quarter ending March 2012) represents the
#' population for February 2012.
#'
#' This function:
#' \enumerate{
#'   \item Fetches raw moving-quarter data from SIDRA
#'   \item Transforms to exact monthly values by aligning with middle months
#'   \item Extrapolates boundary months (first and last) using quadratic regression
#' }
#'
#' The extrapolation uses quadratic regression on population differences to
#' estimate the first month (Jan 2012) and the most recent month.
#'
#' @section Dependencies:
#' This function requires the \code{sidrar} package for API access.
#' Install with: \code{install.packages("sidrar")}
#'
#' @examples
#' \donttest{
#' pop <- fetch_monthly_population()
#'
#' pop <- fetch_monthly_population(201301, 201912)
#' }
#'
#' @seealso \code{\link{pnadc_apply_periods}} which uses this function when
#'   \code{calibrate = TRUE}
#'
#' @export
fetch_monthly_population <- function(start_yyyymm = NULL,
                                      end_yyyymm = NULL,
                                      verbose = TRUE,
                                      use_cache = FALSE,
                                      cache_max_age_hours = 24) {

 # OPTIMIZATION: Check cache first to avoid repeated API calls
 if (use_cache && .is_cache_valid("population", max_age_hours = cache_max_age_hours)) {
   if (verbose) message("  Using cached population data...")
   dt <- .get_cache("population")

   # Filter to requested date range if specified
   if (!is.null(start_yyyymm)) {
     dt <- dt[ref_month_yyyymm >= start_yyyymm]
   }
   if (!is.null(end_yyyymm)) {
     dt <- dt[ref_month_yyyymm <= end_yyyymm]
   }

   if (verbose) {
     message("  Population data: ",
             min(dt$ref_month_yyyymm), " to ", max(dt$ref_month_yyyymm),
             " (", nrow(dt), " months, from cache)")
   }
   return(dt)
 }

 # Check for sidrar package
 if (!requireNamespace("sidrar", quietly = TRUE)) {
   stop(
     "Package 'sidrar' is required for fetching population from SIDRA.\n",
     "Install with: install.packages('sidrar')",
     call. = FALSE
   )
 }

 if (verbose) message("  Fetching population from SIDRA API (table 6022)...")

 # Fetch from SIDRA
 # Table 6022: Population estimates from PNADC
 # Variable 606: Population (in thousands)
 # n1/all: National level
 # p/all: All periods
 #
 # CRAN policy: Internet resources must fail gracefully with an informative
 # message (no warning, no error). Return NULL invisibly so callers can
 # detect failure without check()-time errors.
 raw <- tryCatch({
   # suppressMessages to hide sidrar's "All others arguments are desconsidered when 'api' is informed"
   suppressMessages(sidrar::get_sidra(api = "/t/6022/n1/all/v/606/p/all"))
 }, error = function(e) {
   message(
     "fetch_monthly_population: failed to fetch from SIDRA API. ",
     "Check internet connection or try again later. ",
     "Error: ", conditionMessage(e)
   )
   NULL
 })
 if (is.null(raw)) {
   return(invisible(NULL))
 }

 if (verbose) message("  Transforming moving-quarter to exact months...")

 # Convert to data.table and extract relevant columns
 dt <- data.table::as.data.table(raw)

 # The column name may vary; find the moving quarter code column.
 # CRAN policy: if the upstream SIDRA response schema has changed (no
 # matching column), fail gracefully with a message rather than stop().
 code_col <- grep("Trimestre.*vel.*digo|trimestre.*vel.*digo",
                  names(dt), value = TRUE, ignore.case = TRUE)
 if (length(code_col) == 0) {
   # Try alternative pattern
   code_col <- grep("M.*vel.*C.*digo", names(dt), value = TRUE, ignore.case = TRUE)
 }
 if (length(code_col) == 0) {
   message(
     "fetch_monthly_population: SIDRA response schema appears to have ",
     "changed (could not find moving quarter code column). Returning NULL."
   )
   return(invisible(NULL))
 }
 code_col <- code_col[1]

 dt <- dt[, .(
   anomesfinaltrimmovel = as.integer(get(code_col)),
   populacao = as.numeric(Valor)
 )]

 # Remove any rows with NA codes
 dt <- dt[!is.na(anomesfinaltrimmovel)]

 # Sort by moving quarter code
 data.table::setorder(dt, anomesfinaltrimmovel)

 # Transform moving quarter to exact month
 dt <- transform_moving_quarter_to_monthly(dt, verbose = verbose)

 # Apply quadratic extrapolation for boundary months
 if (verbose) message("  Extrapolating boundary months...")
 dt <- extrapolate_boundary_months(dt)

 # Rename to standard output column name
 data.table::setnames(dt, "anomesexato", "ref_month_yyyymm")

 # Keep only final columns
 dt <- dt[, .(ref_month_yyyymm, m_populacao)]

 # OPTIMIZATION: Store FULL unfiltered data in cache for future calls
 if (use_cache) {
   .set_cache("population", dt)
 }

 # Apply date filters AFTER caching (cache stores full data, then we filter)
 if (!is.null(start_yyyymm)) {
   dt <- dt[ref_month_yyyymm >= start_yyyymm]
 }
 if (!is.null(end_yyyymm)) {
   dt <- dt[ref_month_yyyymm <= end_yyyymm]
 }

 if (verbose) {
   message("  Population data: ",
           min(dt$ref_month_yyyymm), " to ", max(dt$ref_month_yyyymm),
           " (", nrow(dt), " months)")
 }

 dt
}


#' Transform Moving Quarter to Monthly Population
#'
#' Internal function that transforms SIDRA moving-quarter population values
#' to exact monthly values.
#'
#' @param dt data.table with columns anomesfinaltrimmovel and populacao
#' @param verbose Logical. Print messages?
#' @return data.table with columns anomesexato and m_populacao
#' @keywords internal
#' @noRd
transform_moving_quarter_to_monthly <- function(dt, verbose = FALSE) {

 # The moving quarter code (e.g., 201203) represents the ENDING month
 # of a 3-month window. The population value is for the MIDDLE month.
 #
 # Example:
 #   Code 201203 = Jan+Feb+Mar window = Feb 2012 population
 #   Code 201204 = Feb+Mar+Apr window = Mar 2012 population
 #
 # To get exact monthly values:
 # 1. Add dummy rows for the first two months (Jan and Feb of first year)
 # 2. Sort by date
 # 3. Shift: m_populacao[n] = populacao[n+1]
 #
 # After this shift:
 #   anomesexato=201201 gets populacao from row with code=201202 -> NA
 #   anomesexato=201202 gets populacao from row with code=201203 -> Feb value
 #   anomesexato=201203 gets populacao from row with code=201204 -> Mar value

 # Get the first moving quarter code (e.g., 201203)
 first_code <- min(dt$anomesfinaltrimmovel)
 year_start <- first_code %/% 100L

 # Add two dummy rows for Jan and Feb of the first year
 # These will have populacao = NA (need extrapolation later)
 dummy_rows <- data.table::data.table(
   anomesfinaltrimmovel = c(year_start * 100L + 1L, year_start * 100L + 2L),
   populacao = NA_real_
 )
 dt <- data.table::rbindlist(list(dummy_rows, dt))

 # Create anomesexato (exact month) = same as anomesfinaltrimmovel for now
 dt[, anomesexato := anomesfinaltrimmovel]

 # Sort by exact month
 data.table::setorder(dt, anomesexato)

 # Shift: m_populacao gets value from NEXT row's populacao
 # This aligns the moving quarter value with its middle month
 # shift(x, n=1, type="lead") gets the value from the next row
 dt[, m_populacao := data.table::shift(populacao, n = 1L, type = "lead")]

 # Clean up
 dt[, populacao := NULL]
 dt[, anomesfinaltrimmovel := NULL]

 dt
}


#' Extrapolate Boundary Months Using Quadratic Regression
#'
#' Internal function that fills in NA values for the first and last months
#' of the population series using quadratic extrapolation on differences.
#'
#' @param dt data.table with columns anomesexato and m_populacao
#' @return data.table with extrapolated values for boundary months
#' @keywords internal
#' @noRd
extrapolate_boundary_months <- function(dt) {

 n <- nrow(dt)

 # Create row indices for regression
 dt[, row_num := .I]
 dt[, row_num2 := row_num^2]

 # First differences of population
 dt[, d_pop := m_populacao - data.table::shift(m_populacao, 1L)]

 # Extrapolate FIRST month (row 1) using quadratic regression on first 26 observations
 # This fills in Jan 2012 which has no moving quarter centered on it
 if (n >= 26 && is.na(dt[1, m_populacao])) {
   # Fit quadratic model on first 26 rows (excluding row 1 which has NA d_pop)
   fit_data <- dt[2:26]
   if (sum(!is.na(fit_data$d_pop)) >= 3) {
     fit_start <- stats::lm(d_pop ~ row_num + row_num2, data = fit_data)
     predicted_diff_2 <- stats::predict(fit_start, newdata = dt[2])

     # Extrapolate: value[1] = value[2] - predicted_diff[2]
     dt[1, m_populacao := dt[2, m_populacao] - round(predicted_diff_2)]
   }
 }

 # Extrapolate LAST month (row n) using quadratic regression on last 25 observations
 # This fills in the final month which has no moving quarter centered on it
 if (n >= 25 && is.na(dt[n, m_populacao])) {
   # Fit quadratic model on last 25 rows (excluding row n which has NA)
   start_row <- max(1, n - 24)
   fit_data <- dt[start_row:(n-1)]
   if (sum(!is.na(fit_data$d_pop)) >= 3) {
     fit_end <- stats::lm(d_pop ~ row_num + row_num2, data = fit_data)
     predicted_diff_n <- stats::predict(fit_end, newdata = dt[n])

     # Extrapolate: value[n] = value[n-1] + predicted_diff[n]
     dt[n, m_populacao := dt[n-1, m_populacao] + round(predicted_diff_n)]
   }
 }

 # Clean up temporary columns
 dt[, c("row_num", "row_num2", "d_pop") := NULL]

 dt
}

# Tests for fetch-sidra-population.R
# Note: Some tests require internet connection and sidrar package

test_that("fetch_monthly_population requires sidrar package", {
  skip_if(requireNamespace("sidrar", quietly = TRUE),
          "sidrar is installed, skipping missing package test")

  # Mock the requireNamespace to return FALSE
  # This test only runs if sidrar is not installed
  expect_error(fetch_monthly_population(), "Package 'sidrar' is required")
})

test_that("fetch_monthly_population returns expected structure", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  result <- fetch_monthly_population(verbose = FALSE)

  # Should be a data.table
  expect_s3_class(result, "data.table")

  # Should have required columns
  expect_true("ref_month_yyyymm" %in% names(result))
  expect_true("m_populacao" %in% names(result))

  # Should have reasonable values
  expect_true(all(result$ref_month_yyyymm >= 201201))
  expect_true(all(result$m_populacao > 0, na.rm = TRUE))
})

test_that("fetch_monthly_population respects date range", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  result <- fetch_monthly_population(
    start_yyyymm = 201501,
    end_yyyymm = 201512,
    verbose = FALSE
  )

  # Should only have 2015 months
  expect_true(all(result$ref_month_yyyymm >= 201501))
  expect_true(all(result$ref_month_yyyymm <= 201512))
  expect_equal(nrow(result), 12)
})

test_that("transform_moving_quarter_to_monthly transforms correctly", {
  # Create sample moving quarter data
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L, 201206L),
    populacao = c(200000, 200100, 200200, 200300)
  )

  result <- transform_moving_quarter_to_monthly(dt, verbose = FALSE)

  # Should have anomesexato and m_populacao columns
  expect_true("anomesexato" %in% names(result))
  expect_true("m_populacao" %in% names(result))

  # Should have correct number of rows (original + 2 dummy rows)
  expect_equal(nrow(result), 6)

  # The shift logic: month N gets population from row N+1
  # So 201202 (Feb 2012) gets value from 201203 row = 200000
  expect_equal(result[anomesexato == 201202L, m_populacao], 200000)

  # First month (201201) should be NA (needs extrapolation)
  expect_true(is.na(result[anomesexato == 201201L, m_populacao]))
})

test_that("extrapolate_boundary_months fills in boundary values", {
  # Create a series with missing first and last values
  set.seed(42)
  n <- 30
  dt <- data.table::data.table(
    anomesexato = generate_yyyymm_seq(201201L, n),
    m_populacao = c(NA, 200000 + cumsum(rnorm(n-2, 100, 10)), NA)
  )

  result <- extrapolate_boundary_months(dt)

  # Should fill in first and last values
  expect_false(is.na(result[1, m_populacao]))
  expect_false(is.na(result[n, m_populacao]))

  # Values should be reasonable (close to neighbors)
  expect_true(abs(result[1, m_populacao] - result[2, m_populacao]) < 500)
  expect_true(abs(result[n, m_populacao] - result[n-1, m_populacao]) < 500)
})

test_that("extrapolate_boundary_months handles short series", {
  # Series too short for extrapolation (< 26 rows)
  dt <- data.table::data.table(
    anomesexato = generate_yyyymm_seq(201201L, 10L),
    m_populacao = c(NA, 200000:200007, NA)
  )

  result <- extrapolate_boundary_months(dt)

  # With only 10 rows, cannot extrapolate using 26-row regression
  # First and last should still be NA
  expect_true(is.na(result[1, m_populacao]))
  expect_true(is.na(result[10, m_populacao]))
})

test_that("extrapolate_boundary_months preserves middle values", {
  set.seed(42)
  n <- 30
  original_middle <- 200000 + cumsum(rnorm(n-2, 100, 10))

  dt <- data.table::data.table(
    anomesexato = generate_yyyymm_seq(201201L, n),
    m_populacao = c(NA, original_middle, NA)
  )

  result <- extrapolate_boundary_months(dt)

  # Middle values should be unchanged
  expect_equal(result$m_populacao[2:(n-1)], original_middle)
})

test_that("extrapolate_boundary_months removes temporary columns", {
  set.seed(42)
  n <- 30
  dt <- data.table::data.table(
    anomesexato = generate_yyyymm_seq(201201L, n),
    m_populacao = c(NA, 200000 + cumsum(rnorm(n-2, 100, 10)), NA)
  )

  result <- extrapolate_boundary_months(dt)

  # Should not have regression columns
  expect_false("row_num" %in% names(result))
  expect_false("row_num2" %in% names(result))
  expect_false("d_pop" %in% names(result))
})


# =============================================================================
# CACHING BEHAVIOR TESTS
# =============================================================================

test_that("clear_sidra_cache clears the cache", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  # 1. Setup: Fetch data to populate cache
  fetch_monthly_population(
    start_yyyymm = 201501,
    end_yyyymm = 201503,
    use_cache = TRUE,
    verbose = FALSE
  )

  # 2. Execute: Clear cache
  clear_sidra_cache()

  # 3. Verify: Cache environment should be empty
  cache_env <- PNADCperiods:::.sidra_cache
  expect_false(exists("population_data", envir = cache_env))
  expect_false(exists("population_time", envir = cache_env))

  # 4. Context: Clearing cache forces fresh API calls
})


test_that("use_cache=FALSE bypasses cache", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  # 1. Setup: Clear cache first
  clear_sidra_cache()

  # 2. Execute: Fetch with use_cache=FALSE
  result1 <- fetch_monthly_population(
    start_yyyymm = 201501,
    end_yyyymm = 201503,
    use_cache = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Cache should still be empty
  cache_env <- PNADCperiods:::.sidra_cache
  expect_false(exists("population_data", envir = cache_env))

  # 4. Context: use_cache=FALSE prevents both reading and writing cache
})


test_that("cached data is returned on second call", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  # 1. Setup: Clear cache
  clear_sidra_cache()

  # 2. Execute: First call (populates cache)
  result1 <- fetch_monthly_population(
    start_yyyymm = 201501,
    end_yyyymm = 201503,
    use_cache = TRUE,
    verbose = FALSE
  )

  # 3. Execute: Second call (should use cache)
  result2 <- fetch_monthly_population(
    start_yyyymm = 201501,
    end_yyyymm = 201503,
    use_cache = TRUE,
    verbose = FALSE
  )

  # 4. Verify: Results should be identical
  expect_equal(nrow(result1), nrow(result2))
  expect_equal(result1$ref_month_yyyymm, result2$ref_month_yyyymm)
  expect_equal(result1$m_populacao, result2$m_populacao)

  # 5. Context: Cache improves performance on repeated calls
})


test_that("cache respects different date ranges", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  # 1. Setup: Clear cache
  clear_sidra_cache()

  # 2. Execute: Cache stores full data, filters are applied after
  result1 <- fetch_monthly_population(
    start_yyyymm = 201501,
    end_yyyymm = 201503,
    use_cache = TRUE,
    verbose = FALSE
  )

  # 3. Execute: Different date range should work correctly
  result2 <- fetch_monthly_population(
    start_yyyymm = 201504,
    end_yyyymm = 201506,
    use_cache = TRUE,
    verbose = FALSE
  )

  # 4. Verify: Should have different months
  expect_false(any(result2$ref_month_yyyymm %in% result1$ref_month_yyyymm))
  expect_true(all(result2$ref_month_yyyymm >= 201504))
  expect_true(all(result2$ref_month_yyyymm <= 201506))

  # 5. Context: This test verifies the Phase 1 bug fix - cache stores full data
})


test_that("cache_max_age_hours parameter accepted", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available")
  skip_if_offline()

  # 1. Setup: Clear cache
  clear_sidra_cache()

  # 2. Execute: Fetch with cache_max_age_hours parameter
  expect_no_error({
    result <- fetch_monthly_population(
      start_yyyymm = 201501,
      end_yyyymm = 201503,
      use_cache = TRUE,
      cache_max_age_hours = 24,
      verbose = FALSE
    )
  })

  # 3. Context: cache_max_age_hours parameter controls cache expiration
})


# =============================================================================
# CRAN POLICY: Internet resources must fail gracefully
# (no warning, no error). Mocked tests run offline.
# =============================================================================

test_that("fetch_monthly_population fails gracefully when SIDRA is unreachable", {
  skip_if_not_installed("sidrar")
  skip_if_not_installed("testthat", "3.0.0")

  # Simulate API down: any call to sidrar::get_sidra throws.
  testthat::local_mocked_bindings(
    get_sidra = function(...) stop("simulated network error"),
    .package = "sidrar"
  )

  expect_message(
    result <- fetch_monthly_population(verbose = FALSE),
    "failed to fetch from SIDRA API"
  )
  expect_null(result)
})


# =============================================================================
# ERROR HANDLING TESTS
# =============================================================================

test_that("transform_moving_quarter_to_monthly handles edge cases", {
  # 1. Setup: Create data.table with moving quarter data
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    populacao = c(200000, 200100, 200200)
  )

  # 2. Execute: Should not error
  expect_no_error({
    result <- transform_moving_quarter_to_monthly(dt, verbose = FALSE)
  })

  # 3. Verify: Should produce output
  expect_true(nrow(result) > 0)

  # 4. Context: Function handles moving quarter transformation
})

#' Mensalize SIDRA Rolling Quarter Series
#'
#' Functions to convert IBGE's rolling quarterly (trimestre movel) series into
#' exact monthly estimates.
#'
#' @name mensalize-sidra-series
#' @keywords internal
NULL

#' Convert Rolling Quarters to Exact Monthly Series
#'
#' Transforms SIDRA rolling quarterly averages into exact monthly values using
#' the mathematical relationship between consecutive rolling quarters.
#'
#' @param rolling_quarters data.table from \code{\link{fetch_sidra_rolling_quarters}}
#'   containing rolling quarter series with columns \code{anomesfinaltrimmovel},
#'   \code{mesnotrim}, and series value columns.
#' @param starting_points Optional data.table with precomputed starting points
#'   (y0 values). If NULL (default), uses bundled \code{pnadc_series_starting_points}.
#'   See Details for format.
#' @param series Character vector of series names to mensalize, or "all" (default)
#'   for all series in the input data (except price indices).
#' @param compute_derived Logical. Compute derived series (rates, aggregates)?
#'   Default TRUE.
#' @param verbose Logical. Print progress messages? Default TRUE.
#'
#' @return A data.table with columns:
#'   \describe{
#'     \item{anomesexato}{Integer. YYYYMM exact month}
#'     \item{m_*}{Numeric. Mensalized value for each series (one column per series)}
#'   }
#'
#' @details
#' The algorithm exploits the mathematical property of rolling quarterly averages:
#'
#' \deqn{RQ_t - RQ_{t-1} = (Month_t - Month_{t-3}) / 3}
#'
#' This means exact 3-month variations can be extracted from consecutive rolling
#' quarters. By accumulating these variations separately for each month-position
#' (1, 2, or 3), we build cumulative variation series. The only unknown is the
#' starting level for Jan, Feb, and Mar 2012.
#'
#' **Starting points** are estimated by:
#' 1. Computing monthly estimates from calibrated microdata (z_ variables)
#' 2. Calculating cumulative variations from SIDRA (cum_ variables)
#' 3. Backprojecting: e0 = z_ - cum_ over calibration period (2013-2019)
#' 4. Averaging e0 by month position to get y0_ for each position
#'
#' **Final adjustment** ensures the average of 3 consecutive mensalized values
#' equals the original rolling quarter value.
#'
#' @section Starting Points Format:
#' If providing custom starting points, the data.table must have columns:
#' \itemize{
#'   \item \code{series_name}: Character. Series name matching rolling_quarters columns
#'   \item \code{mesnotrim}: Integer (1, 2, or 3). Month position in quarter
#'   \item \code{y0}: Numeric. Starting point value
#' }
#'
#' @section Mathematical Foundation:
#' The mensalization algorithm proceeds in steps:
#' \enumerate{
#'   \item Calculate d3 = 3 * (RQ_t - RQ_t-1)
#'   \item Separate d3 by month position: d3m1, d3m2, d3m3
#'   \item Cumulate separately: cum1, cum2, cum3
#'   \item Apply starting points: y = y0 + cum
#'   \item Final adjustment for rolling quarter consistency
#' }
#'
#' @examples
#' \donttest{
#' rq <- fetch_sidra_rolling_quarters(
#'   series = c("taxadesocup", "popocup", "popdesocup")
#' )
#' monthly <- mensalize_sidra_series(rq)
#' head(monthly)
#' }
#'
#' @seealso
#' \code{\link{fetch_sidra_rolling_quarters}} to obtain input data
#' \code{\link{compute_series_starting_points}} for custom calibration
#'
#' @export
mensalize_sidra_series <- function(rolling_quarters,
                                    starting_points = NULL,
                                    series = "all",
                                    compute_derived = TRUE,
                                    verbose = TRUE) {

  # Input validation
  if (!inherits(rolling_quarters, "data.table")) {
    rolling_quarters <- data.table::as.data.table(rolling_quarters)
  }

  required_cols <- c("anomesfinaltrimmovel", "mesnotrim")
  missing <- setdiff(required_cols, names(rolling_quarters))
  if (length(missing) > 0) {
    stop("Missing required columns: ", paste(missing, collapse = ", "))
  }

  # Get series columns (exclude metadata columns)
  meta_cols <- c("anomesfinaltrimmovel", "mesnotrim")
  all_series <- setdiff(names(rolling_quarters), meta_cols)

  # Filter out price indices (they don't need mensalization - already monthly)
  price_indices <- c("ipca100dez1993", "ipcavarmensal",
                     "inpc100dez1993", "inpcvarmensal")
  all_series <- setdiff(all_series, price_indices)

  # Filter out rate series - they are DERIVED from mensalized components
  # (e.g., m_taxadesocup = m_popdesocup / m_popnaforca * 100)
  # Mensalizing rates directly is WRONG - Marcos derives them post-mensalization
  rate_series <- c("taxapartic", "nivelocup", "niveldesocup", "taxadesocup",
                   "perccontribprev", "taxasubocuphoras", "taxacombdesosub",
                   "taxacombdesopot", "taxacompsubutlz", "percdesalento")
  all_series <- setdiff(all_series, rate_series)

  # Filter out average income series - they are DERIVED from mensalized components
  # Marcos: m_rendhabnominaltodos = m_massahabnominaltodos / m_comrendtodos * 1000
  # Mensalizing ratios directly is mathematically WRONG
  # Note: Both nominal and real (INPC-deflated) averages are derived
  avg_income_series <- c("rendhabnominaltodos", "rendefetnominaltodos",
                         "rendhabrealtodos", "rendefetrealtodos",
                         "rendhabrealprinc", "rendefetrealprinc")
  all_series <- setdiff(all_series, avg_income_series)

  # Filter out residual series - they are COMPUTED from mensalized components
  # Residual series are computed from mensalized components (see derived series logic)
  # to ensure subcategories sum exactly to parent totals
  residual_series <- c("popforadaforca", "empregadorsemcnpj",
                       "contapropriasemcnpj", "trabfamauxiliar")
  all_series <- setdiff(all_series, residual_series)

  if (length(all_series) == 0) {
    stop("No series columns found in rolling_quarters")
  }

  # Select series to process
  if (!identical(series, "all")) {
    invalid <- setdiff(series, all_series)
    if (length(invalid) > 0) {
      stop("Series not found in input: ", paste(invalid, collapse = ", "))
    }
    all_series <- series
  }

  # Load bundled starting points if not provided
  if (is.null(starting_points)) {
    # With LazyData: true, data is accessible via getExportedValue or directly
    starting_points <- tryCatch({
      # Try to get from package namespace (works with LazyData: true)
      getExportedValue("PNADCperiods", "pnadc_series_starting_points")
    }, error = function(e) {
      # Fallback: try to load via data()
      tryCatch({
        env <- new.env()
        data("pnadc_series_starting_points", package = "PNADCperiods", envir = env)
        env$pnadc_series_starting_points
      }, error = function(e2) NULL)
    })

    if (!is.null(starting_points)) {
      if (verbose) message("Using bundled starting points")
    } else {
      # Starting points not available - use zeros with warning
      warning("Bundled starting points not found. Using zeros. ",
              "Results will have incorrect levels but correct variations.")
      starting_points <- data.table::data.table(
        series_name = rep(all_series, each = 3),
        mesnotrim = rep(1:3, length(all_series)),
        y0 = 0
      )
    }
  }

  # Validate starting points format
  if (!inherits(starting_points, "data.table")) {
    starting_points <- data.table::as.data.table(starting_points)
  }

  sp_required <- c("series_name", "mesnotrim", "y0")
  sp_missing <- setdiff(sp_required, names(starting_points))
  if (length(sp_missing) > 0) {
    stop("Starting points missing columns: ", paste(sp_missing, collapse = ", "))
  }

  if (verbose) {
    message("Mensalizing ", length(all_series), " series...")
  }

  # Make a copy to avoid modifying input
  dt <- data.table::copy(rolling_quarters)

  # Compute comrendtodos (number of people with income) from SIDRA data
  # This is needed to derive average income series (rendhabnominaltodos, rendefetnominaltodos)
  # Formula: comrendtodos = (massa/rend + massa/rend)/2 * 1000
  # The SIDRA series are: massa in millions, rend in reais, so result is in thousands
  if ("massahabnominaltodos" %in% names(dt) && "rendhabnominaltodos" %in% names(dt) &&
      "massaefetnominaltodos" %in% names(dt) && "rendefetnominaltodos" %in% names(dt)) {
    # Note: massa is in millions, rend is in reais
    # massa/rend gives millions of people, multiply by 1000 to get thousands
    # IMPORTANT: Guard against division by zero - only compute where denominators are positive
    dt[rendhabnominaltodos > 0 & rendefetnominaltodos > 0,
       comrendtodos := (massahabnominaltodos / rendhabnominaltodos +
                        massaefetnominaltodos / rendefetnominaltodos) / 2 * 1000]
    if (verbose) message("Computed comrendtodos from SIDRA massa/rend series")
    # Add comrendtodos to the series list for mensalization
    # (it's derived from SIDRA but needs to be mensalized like other population counts)
    if (!"comrendtodos" %in% all_series) {
      all_series <- c(all_series, "comrendtodos")
    }
  }

  # PNADC started in January 2012; first rolling quarter is March 2012 (201203)

  # Filter to PNADC-era data only. Price indices (IPCA) go back to 1979 but
  # mensalization is only valid for PNADC series. Without this filter, pre-2012
  # rows would just output the y0 starting points cyclically (no actual data).
  # Pre-compute lagged IPCA on the full pre-filter data, so that
  # .compute_derived_series Phase 5 can use IPCA[201112] as the lag
  # for 201201. Without this, the post-filter shift() returns NA for
  # the first PNADC month. This must happen BEFORE the PNADC-era filter.
  data.table::setorder(dt, anomesfinaltrimmovel)
  if ("ipca100dez1993" %in% names(dt)) {
    dt[, .ipca100dez1993_lagged :=
         data.table::shift(ipca100dez1993, n = 1L, type = "lag")]
  }

    pnadc_start <- .PNADC_DATES$PNADC_START
  if (min(dt$anomesfinaltrimmovel) < pnadc_start) {
    n_pre_pnadc <- sum(dt$anomesfinaltrimmovel < pnadc_start)
    if (verbose) {
      message("Filtering out ", n_pre_pnadc, " pre-PNADC rows (before ", pnadc_start, ")")
    }
    dt <- dt[anomesfinaltrimmovel >= pnadc_start]
  }

  # Initialize result with time columns
  result <- data.table::data.table(
    anomesexato = dt$anomesfinaltrimmovel  # Same as end month of rolling quarter
  )

  # Define split-calibration series (require two separate mensalizations stitched together)
  # subocuphoras: VD4004 pre-201509, VD4004A post-201509 (split calibration)
  split_series <- c("subocuphoras")

  # Process each series
  for (v in all_series) {
    if (verbose && length(all_series) > 5) {
      message("  Processing: ", v)
    }

    # Check if this series needs split calibration
    if (v %in% split_series) {
      # Use split mensalization (two separate processes stitched at VD4004 split)
      if (verbose) message("    (using split calibration for ", v, ")")
      m_values <- .mensalize_split_series(
        dt = dt,
        series_name = v,
        starting_points = starting_points,
        split_month = .PNADC_DATES$VD4004_SPLIT
      )
    } else {
      # Standard single mensalization
      m_values <- .mensalize_single_series(
        dt = dt,
        series_name = v,
        starting_points = starting_points
      )
    }

    # Add to result with m_ prefix
    result[, paste0("m_", v) := m_values]
  }

  # Add price indices (pass through - already monthly, no mensalization needed)
  price_indices <- c("ipca100dez1993", "ipcavarmensal",
                     "inpc100dez1993", "inpcvarmensal")
  for (pi in price_indices) {
    if (pi %in% names(dt)) {
      result[, (pi) := dt[[pi]]]
    }
  }


  # Pass-through pre-computed lagged IPCA (for derived series Phase 5)
  if (".ipca100dez1993_lagged" %in% names(dt)) {
    result[, ipca100dez1993_lagged := dt$.ipca100dez1993_lagged]
  }
  # Note: comrendtodos is now mensalized (added to all_series above)
  # and will be available as m_comrendtodos for deriving average income series

  # Compute derived series if requested
  if (compute_derived) {
    if (verbose) message("Computing derived series...")
    result <- .compute_derived_series(result, verbose = verbose)
  }

  if (verbose) {
    message("Mensalization complete: ", nrow(result), " months (",
            min(result$anomesexato), " to ", max(result$anomesexato), ")")
  }

  result
}


# =============================================================================
# MENSALIZATION HELPER FUNCTIONS
# =============================================================================

#' Compute Cumulative Sum by Month Position
#'
#' Internal helper that computes d3 differences and cumsum separated by mesnotrim.
#'
#' @param rq Numeric vector of rolling quarter values
#' @param mesnotrim Integer vector of month positions (1, 2, or 3)
#' @param filter_mask Logical vector. If provided, NA out d3 where FALSE before cumsum.
#' @return Numeric vector of cumulative sums aligned by mesnotrim
#' @keywords internal
#' @noRd
.compute_cumsum_by_mesnotrim <- function(rq, mesnotrim, filter_mask = NULL) {
  n <- length(rq)

  # Step 1: Calculate d3 = 3 × (RQ_t - RQ_{t-1})
  d3 <- 3 * (rq - data.table::shift(rq, 1L))

  # Set first observation of EACH mesnotrim position to NA

  # This ensures each position's cumsum starts at 0, avoiding pre-PNADC contamination
  # for the first few months (201201, 201202, 201203)
  first_pos1 <- which(mesnotrim == 1)[1]
  first_pos2 <- which(mesnotrim == 2)[1]
  first_pos3 <- which(mesnotrim == 3)[1]
  first_positions <- stats::na.omit(c(first_pos1, first_pos2, first_pos3))
  d3[first_positions] <- NA_real_

  # Apply filter mask if provided (for split series)
  if (!is.null(filter_mask)) {
    d3[!filter_mask] <- NA_real_
  }

  # Step 2: Separate by month position
  d3m1 <- ifelse(mesnotrim == 1, d3, NA_real_)
  d3m2 <- ifelse(mesnotrim == 2, d3, NA_real_)
  d3m3 <- ifelse(mesnotrim == 3, d3, NA_real_)

  # Track where we have actual data vs NA from ORIGINAL rq (not d3)
  # This is used to restore NA for series that start later (e.g., CNPJ from 201510)
  rq_has_data <- !is.na(rq)
  has_data1 <- ifelse(mesnotrim == 1, rq_has_data, FALSE)
  has_data2 <- ifelse(mesnotrim == 2, rq_has_data, FALSE)
  has_data3 <- ifelse(mesnotrim == 3, rq_has_data, FALSE)

  # Step 3: Cumulative sum (treating NAs as 0 for accumulation)
  cum1 <- cumsum(ifelse(is.na(d3m1), 0, d3m1))
  cum2 <- cumsum(ifelse(is.na(d3m2), 0, d3m2))
  cum3 <- cumsum(ifelse(is.na(d3m3), 0, d3m3))

  # For series that start later (e.g., CNPJ from 201510), we want NA before first data
  # Find the first index where the series has ANY data
  first_any_data <- which(rq_has_data)[1]

  # If no data at all, all positions are invalid
  if (is.na(first_any_data)) {
    cum1[] <- NA_real_
    cum2[] <- NA_real_
    cum3[] <- NA_real_
  } else {
    # Series start window: first_any_data and 2 positions before (for the 3 mesnotrims)
    # E.g., if first data is at 201512 (idx 48), then 201510 (idx 46) and 201511 (idx 47)
    # should also be valid since they have y0 starting points
    series_start <- max(1L, first_any_data - 2L)

    # Mark all positions at or after series_start as valid for their respective mesnotrim
    valid_range <- seq_len(n) >= series_start

    # For core series with early data, also mark structural first positions
    has_early_data <- any(rq_has_data[1:min(6, n)])
    if (has_early_data) {
      valid_range[first_pos1] <- TRUE
      valid_range[first_pos2] <- TRUE
      valid_range[first_pos3] <- TRUE
    }

    # Apply validity: positions in valid_range with matching mesnotrim get cum, others NA
    first_valid1 <- valid_range & (mesnotrim == 1 | cummax(has_data1))
    first_valid2 <- valid_range & (mesnotrim == 2 | cummax(has_data2))
    first_valid3 <- valid_range & (mesnotrim == 3 | cummax(has_data3))

    # Simpler approach: just mark positions at or after series_start as valid
    first_valid1 <- valid_range | as.logical(cummax(has_data1))
    first_valid2 <- valid_range | as.logical(cummax(has_data2))
    first_valid3 <- valid_range | as.logical(cummax(has_data3))

    cum1[!first_valid1] <- NA_real_
    cum2[!first_valid2] <- NA_real_
    cum3[!first_valid3] <- NA_real_
  }

  # Step 4: Combine based on mesnotrim
  cum <- rep(NA_real_, n)
  cum[mesnotrim == 1] <- cum1[mesnotrim == 1]
  cum[mesnotrim == 2] <- cum2[mesnotrim == 2]
  cum[mesnotrim == 3] <- cum3[mesnotrim == 3]

  cum
}


#' Extract y0 Starting Points Vector
#'
#' Internal helper that extracts y0 values for a series from starting_points.
#'
#' @param starting_points data.table with series_name, mesnotrim, y0 columns
#' @param target_series Character. Series name to extract.
#' @return Numeric vector of length 3 (y0 for mesnotrim 1, 2, 3)
#' @keywords internal
#' @noRd
.extract_y0_vector <- function(starting_points, target_series) {
  sp <- starting_points[get("series_name") == target_series]

  # Return NA for series without calibrated starting points
  # This ensures uncalibrated series show NA instead of 0
  if (nrow(sp) == 0) {
    return(c(NA_real_, NA_real_, NA_real_))
  }

  y0 <- rep(NA_real_, 3)
  for (pos in 1:3) {
    val <- sp[mesnotrim == pos, y0]
    y0[pos] <- if (length(val) > 0) val[1] else NA_real_
  }

  y0
}


#' Apply Final Rolling Quarter Adjustment
#'
#' Internal helper that applies the final adjustment to ensure rolling quarter consistency.
#' For each trio of consecutive months, the average equals the rolling quarter value.
#'
#' Uses fully vectorized operations for performance - no for loops.
#'
#' @param y Numeric vector of y values (y0 + cumsum)
#' @param rq Numeric vector of rolling quarter values
#' @param mesnotrim Integer vector of month positions (1, 2, or 3)
#' @return Numeric vector of final mensalized values
#' @keywords internal
#' @noRd
.apply_final_adjustment <- function(y, rq, mesnotrim) {
  n <- length(y)

  # Shift values for computing averages
  y_lead1 <- data.table::shift(y, 1L, type = "lead")
  y_lead2 <- data.table::shift(y, 2L, type = "lead")
  y_lag1 <- data.table::shift(y, 1L, type = "lag")
  y_lag2 <- data.table::shift(y, 2L, type = "lag")

  # Get rolling quarter value from the mesnotrim==3 position for each trio
  rq_lead1 <- data.table::shift(rq, 1L, type = "lead")
  rq_lead2 <- data.table::shift(rq, 2L, type = "lead")

  # Initialize result with fallback values (y itself)
  m <- y

  # Vectorized computation for mesnotrim==1
  # avg_y = (y + y_lead1 + y_lead2) / 3, m = y + rq_lead2 - avg_y
  valid1 <- mesnotrim == 1L & !is.na(y) & !is.na(y_lead1) & !is.na(y_lead2) & !is.na(rq_lead2)
  avg_y1 <- (y + y_lead1 + y_lead2) / 3
  m[valid1] <- y[valid1] + rq_lead2[valid1] - avg_y1[valid1]

  # Vectorized computation for mesnotrim==2
  # avg_y = (y_lag1 + y + y_lead1) / 3, m = y + rq_lead1 - avg_y
  valid2 <- mesnotrim == 2L & !is.na(y_lag1) & !is.na(y) & !is.na(y_lead1) & !is.na(rq_lead1)
  avg_y2 <- (y_lag1 + y + y_lead1) / 3
  m[valid2] <- y[valid2] + rq_lead1[valid2] - avg_y2[valid2]

  # Vectorized computation for mesnotrim==3
  # avg_y = (y_lag2 + y_lag1 + y) / 3, m = y + rq - avg_y
  valid3 <- mesnotrim == 3L & !is.na(y_lag2) & !is.na(y_lag1) & !is.na(y) & !is.na(rq)
  avg_y3 <- (y_lag2 + y_lag1 + y) / 3
  m[valid3] <- y[valid3] + rq[valid3] - avg_y3[valid3]

  m
}


# =============================================================================
# MENSALIZATION FUNCTIONS
# =============================================================================

#' Mensalize a Single Series
#'
#' Internal function implementing the core mensalization algorithm for one series.
#'
#' @param dt data.table with rolling quarter data
#' @param series_name Name of the series column
#' @param starting_points data.table with y0 values
#' @return Numeric vector of mensalized values
#' @keywords internal
#' @noRd
.mensalize_single_series <- function(dt, series_name, starting_points) {

  rq <- dt[[series_name]]
  mesnotrim <- dt$mesnotrim

  # Step 1-4: Compute cumsum by month position
  cum <- .compute_cumsum_by_mesnotrim(rq, mesnotrim)

  # Step 5: Get starting points
  y0 <- .extract_y0_vector(starting_points, series_name)

  # Step 6: Apply starting points
  y <- y0[mesnotrim] + cum

  # Step 7: Final adjustment for rolling quarter consistency
  m <- .apply_final_adjustment(y, rq, mesnotrim)

  # Step 8: NA-out trailing positions where rq is missing.
  # cumsum-by-mesnotrim coalesces NA as 0; without this mask, trailing
  # rows where SIDRA already published IPCA but not PNADC would receive
  # spurious mensalized values from the apply_final_adjustment fallback.
  #
  # We mask ONLY positions after the last observed rq. Leading positions
  # (e.g., 201201/201202, before the first rolling quarter) are preserved
  # because the algorithm reconstructs them via y0 + lookahead in
  # apply_final_adjustment (valid_k=TRUE thanks to rq_lead2/rq_lead1).
  rq_obs <- which(!is.na(rq))
  if (length(rq_obs) == 0L) {
    m[] <- NA_real_
  } else {
    last_obs <- rq_obs[length(rq_obs)]
    if (last_obs < length(rq)) {
      m[(last_obs + 1L):length(rq)] <- NA_real_
    }
  }
  m
}


#' Mensalize a Split-Calibration Series
#'
#' Internal function for series that require split calibration (like subocuphoras).
#' Performs TWO separate mensalizations and stitches them together at the split point.
#'
#' The key insight is that each period must be processed INDEPENDENTLY to avoid
#' cross-contamination at the boundary. The final adjustment uses lead/lag operations
#' that would otherwise cross the 201509/201510 boundary.
#'
#' @param dt data.table with rolling quarter data
#' @param series_name Name of the series column (e.g., "subocuphoras")
#' @param starting_points data.table with y0 values (must include both "series" and "series_pre")
#' @param split_month Integer. The last month of the pre-split period.
#'   Default NULL uses .PNADC_DATES$VD4004_SPLIT (201509).
#' @return Numeric vector of mensalized values
#' @keywords internal
#' @noRd
.mensalize_split_series <- function(dt, series_name, starting_points, split_month = NULL) {

  # Resolve default from centralized constants

  if (is.null(split_month)) {
    split_month <- .PNADC_DATES$VD4004_SPLIT
  }

  rq <- dt[[series_name]]
  mesnotrim <- dt$mesnotrim
  yyyymm <- dt$anomesfinaltrimmovel
  n <- length(rq)

  # Initialize result vector
  m_result <- numeric(n)

  # ============================================================================
  # PART 1: Process PRE-SPLIT period independently (up to and including split_month)
  # ============================================================================

  pre_idx <- which(yyyymm <= split_month)
  if (length(pre_idx) > 0) {
    # Extract subset for pre-split period
    rq_pre <- rq[pre_idx]
    mesnotrim_pre <- mesnotrim[pre_idx]

    # Compute cumsum for pre-split period only
    cum_pre <- .compute_cumsum_by_mesnotrim(rq_pre, mesnotrim_pre)

    # Get starting points for pre-split
    y0_pre <- .extract_y0_vector(starting_points, paste0(series_name, "_pre"))
    y_pre <- y0_pre[mesnotrim_pre] + cum_pre

    # Apply final adjustment ONLY to pre-split data (no boundary crossing)
    m_pre <- .apply_final_adjustment(y_pre, rq_pre, mesnotrim_pre)

    # Mask only trailing NA in pre-split (see .mensalize_single_series).
    rq_pre_obs <- which(!is.na(rq_pre))
    if (length(rq_pre_obs) == 0L) {
      m_pre[] <- NA_real_
    } else {
      last_pre <- rq_pre_obs[length(rq_pre_obs)]
      if (last_pre < length(rq_pre)) {
        m_pre[(last_pre + 1L):length(rq_pre)] <- NA_real_
      }
    }

    # Store results
    m_result[pre_idx] <- m_pre
  }

  # ============================================================================
  # PART 2: Process POST-SPLIT period independently (after split_month)
  # ============================================================================

  post_idx <- which(yyyymm > split_month)
  if (length(post_idx) > 0) {
    # Extract subset for post-split period
    rq_post <- rq[post_idx]
    mesnotrim_post <- mesnotrim[post_idx]

    # Compute cumsum for post-split period only (cumsum starts fresh from 201510)
    cum_post <- .compute_cumsum_by_mesnotrim(rq_post, mesnotrim_post)

    # Get starting points for post-split
    y0_post <- .extract_y0_vector(starting_points, series_name)
    y_post <- y0_post[mesnotrim_post] + cum_post

    # Apply final adjustment ONLY to post-split data (no boundary crossing)
    m_post <- .apply_final_adjustment(y_post, rq_post, mesnotrim_post)

    # Mask only trailing NA in post-split (see .mensalize_single_series).
    rq_post_obs <- which(!is.na(rq_post))
    if (length(rq_post_obs) == 0L) {
      m_post[] <- NA_real_
    } else {
      last_post <- rq_post_obs[length(rq_post_obs)]
      if (last_post < length(rq_post)) {
        m_post[(last_post + 1L):length(rq_post)] <- NA_real_
      }
    }

    # Store results
    m_result[post_idx] <- m_post
  }

  m_result
}


#' Compute Derived Series (Rates and Aggregates)
#'
#' Internal function to compute rates and aggregates from primary mensalized series.
#'
#' @section Why Derived Series Are Computed Here (Not From Starting Points):
#'
#' Some series have two possible computation paths:
#' \enumerate{
#'   \item **Direct mensalization**: Use z_ aggregates from microdata for starting
#'     points, then mensalize the SIDRA rolling quarter series
#'   \item **Derived computation**: Compute from other mensalized series (sums, ratios)
#' }
#'
#' For certain series, we use path 2 (derived) because:
#'
#' \itemize{
#'   \item **Rate series** (taxadesocup, percdesalento, etc.): Computed as ratios
#'     of mensalized populations. This ensures mathematical consistency -
#'     rate = mensalized_numerator / mensalized_denominator
#'   \item **Aggregate sums** (popnaforca = popocup + popdesocup): Ensures parts
#'     sum exactly to totals after mensalization
#'   \item **Residual series** (empregadorsemcnpj = empregador - empregadorcomcnpj):
#'     Ensures subcategories sum to parent totals (accounting identity)
#' }
#'
#' The alternative (direct mensalization) would require separate z_ aggregates
#' for each rate/aggregate, and the resulting mensalized values wouldn't have
#' guaranteed mathematical relationships with their components.
#'
#' @param dt data.table with mensalized primary series
#' @param verbose Logical. Print progress?
#' @return data.table with additional derived columns
#' @keywords internal
#' @noRd
.compute_derived_series <- function(dt, verbose = FALSE) {

  # Helper to check column existence (with m_ prefix)
  has_col <- function(x) paste0("m_", x) %in% names(dt)
  has_col_raw <- function(x) x %in% names(dt)

  # ============================================================================
  # PHASE 1: AGGREGATES (sums of components)
  # Must be computed first as some rates depend on these aggregates
  # ============================================================================
  for (spec in .DERIVED_SERIES_SPEC$aggregates) {
    if (all(sapply(spec$components, has_col))) {
      cols <- paste0("m_", spec$components)
      dt[, paste0("m_", spec$name) := Reduce(`+`, .SD), .SDcols = cols]
      if (verbose) message("    Computed aggregate: m_", spec$name)
    }
  }

  # ============================================================================
  # PHASE 2: AVERAGE INCOME (massa / comrendtodos * 1000)
  # ============================================================================
  for (spec in .DERIVED_SERIES_SPEC$average_income) {
    if (has_col(spec$numerator) && has_col(spec$denominator)) {
      num_col <- paste0("m_", spec$numerator)
      denom_col <- paste0("m_", spec$denominator)
      out_col <- paste0("m_", spec$name)
      dt[get(denom_col) > 0, (out_col) := round(get(num_col) / get(denom_col) * spec$multiplier, spec$decimals)]
      if (verbose) message("    Computed average income: m_", spec$name)
    }
  }

  # ============================================================================
  # PHASE 3: RESIDUALS (parent - sum of subtracted components)
  # Ensures accounting identities hold
  # ============================================================================
  for (spec in .DERIVED_SERIES_SPEC$residuals) {
    if (has_col(spec$parent) && all(sapply(spec$subtract, has_col))) {
      parent_col <- paste0("m_", spec$parent)
      subtract_cols <- paste0("m_", spec$subtract)
      out_col <- paste0("m_", spec$name)
      dt[, (out_col) := get(parent_col) - Reduce(`+`, .SD), .SDcols = subtract_cols]
      if (verbose) message("    Computed residual: m_", spec$name)
    }
  }

  # ============================================================================
  # PHASE 4: RATES (numerator / denominator * 100)
  # ============================================================================
  for (spec in .DERIVED_SERIES_SPEC$rates) {
    # Build list of numerator components (may be single or multiple)
    num_components <- if (is.list(spec$numerator)) spec$numerator else list(spec$numerator)
    num_is_sum <- length(num_components) > 1 || (is.list(spec$numerator) && length(spec$numerator) > 1)

    # Check if all required columns exist
    all_num_exist <- all(sapply(unlist(num_components), has_col))
    denom_components <- if (is.list(spec$denominator)) spec$denominator else list(spec$denominator)
    all_denom_exist <- all(sapply(unlist(denom_components), has_col))

    if (all_num_exist && all_denom_exist) {
      # Compute numerator (may be sum of multiple components)
      num_cols <- paste0("m_", unlist(num_components))
      denom_cols <- paste0("m_", unlist(denom_components))
      out_col <- paste0("m_", spec$name)
      decimals <- spec$decimals

      # For rates with compound numerator or denominator
      if (length(num_cols) == 1 && length(denom_cols) == 1) {
        dt[, (out_col) := round(get(num_cols) / get(denom_cols) * 100, decimals)]
      } else {
        # Build expression for compound numerator/denominator
        num_sum <- rowSums(dt[, num_cols, with = FALSE], na.rm = TRUE)
        denom_sum <- rowSums(dt[, denom_cols, with = FALSE], na.rm = TRUE)
        dt[, (out_col) := round(num_sum / denom_sum * 100, decimals)]
      }
      if (verbose) message("    Computed rate: m_", spec$name)
    }
  }

  # ============================================================================
  # PHASE 5: DEFLATED SERIES (nominal * deflator)
  # Uses IPCA index with different lag for hab vs efet series
  # ============================================================================
  if (has_col_raw("ipca100dez1993")) {
    latest_ipca <- dt[anomesexato == max(anomesexato), ipca100dez1993][1]

    if (!is.na(latest_ipca) && latest_ipca > 0) {
      # Pre-compute deflators
      dt[, .deflator_hab := latest_ipca / ipca100dez1993]
      if ("ipca100dez1993_lagged" %in% names(dt)) {
        # Use pre-computed lagged IPCA from before the PNADC filter
        dt[, .deflator_efet := latest_ipca / ipca100dez1993_lagged]
      } else {
        # Fallback: compute lag on post-filter data (may have NA for first PNADC month)
        dt[, .ipca_lagged := data.table::shift(ipca100dez1993, n = 1L, type = "lag")]
        dt[, .deflator_efet := latest_ipca / .ipca_lagged]
      }

      for (spec in .DERIVED_SERIES_SPEC$deflated) {
        if (has_col(spec$source)) {
          source_col <- paste0("m_", spec$source)
          out_col <- paste0("m_", spec$name)
          deflator_col <- if (spec$use_lagged_ipca) ".deflator_efet" else ".deflator_hab"
          dt[, (out_col) := round(get(source_col) * get(deflator_col), spec$decimals)]
          if (verbose) message("    Computed deflated: m_", spec$name)
        }
      }

      # Cleanup temporary columns
      if (".ipca_lagged" %in% names(dt)) dt[, .ipca_lagged := NULL]
      dt[, c(".deflator_hab", ".deflator_efet") := NULL]
    }
  }

  dt
}


# =============================================================================
# DERIVED SERIES SPECIFICATION (metadata-driven)
# =============================================================================
# This declarative structure defines all derived series and their computation.
# Processing order: aggregates -> average_income -> residuals -> rates -> deflated
# =============================================================================

.DERIVED_SERIES_SPEC <- list(

  # ==========================================================================
  # AGGREGATES: output = sum of components
  # Order matters: some aggregates depend on others (e.g., empregado needs empregpriv)
  # ==========================================================================
  aggregates = list(
    list(name = "popnaforca",
         components = c("popocup", "popdesocup"),
         description = "Labor force = employed + unemployed"),

    list(name = "empregpriv",
         components = c("empregprivcomcart", "empregprivsemcart"),
         description = "Private sector employees (total)"),

    list(name = "domestico",
         components = c("domesticocomcart", "domesticosemcart"),
         description = "Domestic workers (total)"),

    list(name = "empregpubl",
         components = c("empregpublcomcart", "empregpublsemcart", "estatutmilitar"),
         description = "Public sector employees (total)"),

    list(name = "empregado",
         components = c("empregpriv", "domestico", "empregpubl"),
         description = "All employees (total) - depends on prior aggregates"),

    list(name = "forcaampliada",
         components = c("popnaforca", "forcapotencial"),
         description = "Extended labor force - depends on popnaforca aggregate")
  ),

  # ==========================================================================
  # AVERAGE INCOME: output = numerator / denominator * multiplier
  # Formula: m_rend = m_massa / m_comrendtodos * 1000
  # ==========================================================================
  average_income = list(
    list(name = "rendhabnominaltodos",
         numerator = "massahabnominaltodos",
         denominator = "comrendtodos",
         multiplier = 1000,
         decimals = 0,
         description = "Average habitual income (nominal)"),

    list(name = "rendefetnominaltodos",
         numerator = "massaefetnominaltodos",
         denominator = "comrendtodos",
         multiplier = 1000,
         decimals = 0,
         description = "Average effective income (nominal)"),

    # INPC-deflated average income series
    # These use the mensalized real (INPC-deflated) mass series
    list(name = "rendhabrealtodos",
         numerator = "massahabrealtodos",
         denominator = "comrendtodos",
         multiplier = 1000,
         decimals = 1,
         description = "Average habitual income (INPC-deflated)"),

    list(name = "rendefetrealtodos",
         numerator = "massaefetrealtodos",
         denominator = "comrendtodos",
         multiplier = 1000,
         decimals = 1,
         description = "Average effective income (INPC-deflated)")
  ),

  # ==========================================================================
  # RESIDUALS: output = parent - sum(subtract)
  # Computed as residuals to ensure accounting identities
  # ==========================================================================
  residuals = list(
    list(name = "popforadaforca",
         parent = "pop14mais",
         subtract = c("popocup", "popdesocup"),
         description = "Population outside labor force"),

    list(name = "empregadorsemcnpj",
         parent = "empregador",
         subtract = c("empregadorcomcnpj"),
         description = "Employers without CNPJ"),

    list(name = "contapropriasemcnpj",
         parent = "contapropria",
         subtract = c("contapropriacomcnpj"),
         description = "Self-employed without CNPJ"),

    list(name = "trabfamauxiliar",
         parent = "popocup",
         subtract = c("empregprivcomcart", "empregprivsemcart",
                      "domesticocomcart", "domesticosemcart",
                      "empregpublcomcart", "empregpublsemcart",
                      "estatutmilitar", "empregador", "contapropria"),
         description = "Unpaid family workers (residual)")
  ),

  # ==========================================================================
  # RATES: output = (sum of numerator) / (sum of denominator) * 100
  # All rates use decimals = 1
  # ==========================================================================
  rates = list(
    list(name = "taxapartic",
         numerator = c("popocup", "popdesocup"),
         denominator = c("pop14mais"),
         decimals = 1,
         description = "Participation rate"),

    list(name = "nivelocup",
         numerator = c("popocup"),
         denominator = c("pop14mais"),
         decimals = 1,
         description = "Employment rate"),

    list(name = "niveldesocup",
         numerator = c("popdesocup"),
         denominator = c("pop14mais"),
         decimals = 1,
         description = "Unemployment level"),

    list(name = "taxadesocup",
         numerator = c("popdesocup"),
         denominator = c("popnaforca"),
         decimals = 1,
         description = "Unemployment rate"),

    list(name = "perccontribprev",
         numerator = c("contribuinteprev"),
         denominator = c("popocup"),
         decimals = 1,
         description = "Social security contribution rate"),

    list(name = "taxacombdesosub",
         numerator = c("subocuphoras", "popdesocup"),
         denominator = c("popnaforca"),
         decimals = 1,
         description = "Combined unemployment + underemployment rate"),

    list(name = "taxacombdesopot",
         numerator = c("popdesocup", "forcapotencial"),
         denominator = c("forcaampliada"),
         decimals = 1,
         description = "Combined unemployment + potential labor force rate"),

    list(name = "taxacompsubutlz",
         numerator = c("subocuphoras", "popdesocup", "forcapotencial"),
         denominator = c("forcaampliada"),
         decimals = 1,
         description = "Composite underutilization rate"),

    list(name = "taxasubocuphoras",
         numerator = c("subocuphoras"),
         denominator = c("popocup"),
         decimals = 1,
         description = "Underemployment rate"),

    list(name = "percdesalento",
         numerator = c("desalentado"),
         denominator = c("popnaforca", "desalentado"),
         decimals = 1,
         description = "Discouraged worker percentage")
  ),

  # ==========================================================================
  # DEFLATED: output = source * deflator (rounded)
  # "hab" series use current IPCA, "efet" series use lagged IPCA
  # ==========================================================================
  deflated = list(
    list(name = "massahabtodosipcabr",
         source = "massahabnominaltodos",
         use_lagged_ipca = FALSE,
         decimals = 0,
         description = "Real habitual income mass (IPCA-deflated)"),

    list(name = "rendhabtodosipcabr",
         source = "rendhabnominaltodos",
         use_lagged_ipca = FALSE,
         decimals = 0,
         description = "Real average habitual income (IPCA-deflated)"),

    list(name = "massaefettodosipcabr",
         source = "massaefetnominaltodos",
         use_lagged_ipca = TRUE,
         decimals = 0,
         description = "Real effective income mass (IPCA-deflated, lagged)"),

    list(name = "rendefettodosipcabr",
         source = "rendefetnominaltodos",
         use_lagged_ipca = TRUE,
         decimals = 0,
         description = "Real average effective income (IPCA-deflated, lagged)")
  )
)


#' Compute Starting Points from Microdata
#'
#' For advanced users who want to compute custom starting points using their
#' own calibrated microdata estimates.
#'
#' @param monthly_estimates data.table with columns:
#'   \itemize{
#'     \item \code{anomesexato}: YYYYMM exact month
#'     \item \code{z_*}: Monthly estimates from calibrated microdata (one column per series)
#'   }
#' @param rolling_quarters data.table from \code{fetch_sidra_rolling_quarters}
#' @param calibration_start Integer. Start of calibration period (YYYYMM).
#'   Default NULL uses .PNADC_DATES$DEFAULT_CALIB_START (201301).
#'   Note: CNPJ series automatically use CNPJ_CALIB_START (201601) regardless.
#' @param calibration_end Integer. End of calibration period (YYYYMM).
#'   Default NULL uses .PNADC_DATES$DEFAULT_CALIB_END (201912).
#' @param scale_factor Numeric. Scale factor for z_ values (usually 1000). Default 1000.
#' @param use_series_specific_periods Logical. If TRUE (default), use series-specific
#'   calibration periods for CNPJ series (201601-201912) and cumsum starting dates
#'   (201510). Set to FALSE to use uniform calibration for all series.
#' @param verbose Logical. Print progress? Default TRUE.
#'
#' @return data.table with columns:
#'   \describe{
#'     \item{series_name}{Character. Series name}
#'     \item{mesnotrim}{Integer. Month position (1, 2, or 3)}
#'     \item{y0}{Numeric. Starting point value}
#'   }
#'
#' @details
#' The starting points (y0) are computed by:
#' 1. Calculating cumulative variations from SIDRA rolling quarters
#' 2. Computing backprojection: e0 = z / scale_factor - cum
#' 3. Averaging e0 by mesnotrim over the calibration period
#'
#' @section Series-Specific Handling:
#' When \code{use_series_specific_periods = TRUE}, the following series receive
#' special handling for series-specific data availability:
#' \describe{
#'   \item{CNPJ series}{empregadorcomcnpj, empregadorsemcnpj, contapropriacomcnpj,
#'     contapropriasemcnpj use calibration period 201601-201912 and cumsum starts
#'     from 201510 (when V4019 became available)}
#' }
#'
#' @examples
#' \dontrun{
#' rq <- fetch_sidra_rolling_quarters()
#' z_agg <- compute_z_aggregates(calibrated_data)
#' y0 <- compute_series_starting_points(z_agg, rq)
#' monthly <- mensalize_sidra_series(rq, starting_points = y0)
#' }
#'
#' @export
compute_series_starting_points <- function(monthly_estimates,
                                            rolling_quarters,
                                            calibration_start = NULL,
                                            calibration_end = NULL,
                                            scale_factor = 1000,
                                            use_series_specific_periods = TRUE,
                                            verbose = TRUE) {

  # Resolve defaults from centralized constants
  if (is.null(calibration_start)) {
    calibration_start <- .PNADC_DATES$DEFAULT_CALIB_START
  }
  if (is.null(calibration_end)) {
    calibration_end <- .PNADC_DATES$DEFAULT_CALIB_END
  }

  # Ensure data.tables
  if (!inherits(monthly_estimates, "data.table")) {
    monthly_estimates <- data.table::as.data.table(monthly_estimates)
  }
  if (!inherits(rolling_quarters, "data.table")) {
    rolling_quarters <- data.table::as.data.table(rolling_quarters)
  }

  # Find z_ series in monthly estimates
  z_cols <- grep("^z_", names(monthly_estimates), value = TRUE)
  if (length(z_cols) == 0) {
    stop("No z_ columns found in monthly_estimates")
  }

  series_names <- sub("^z_", "", z_cols)

  # Define series-specific calibration periods
  # CNPJ series: V4019 only available from 201510, use calibration 201601-201912

  cnpj_series <- c("empregadorcomcnpj", "empregadorsemcnpj",
                   "contapropriacomcnpj", "contapropriasemcnpj")

  # subocuphoras requires SPLIT CALIBRATION
  # The VD4004 -> VD4004A transition at 201510 requires TWO separate mensalizations:
  # - Post-201509: cumsum from 201510, calibration 201601-201912 (same as CNPJ)
  # - Pre-201509: cumsum from start, calibration 201301-201412
  # Then the two are stitched together at 201509
  split_series <- c("subocuphoras")

  if (verbose) {
    message("Computing starting points for ", length(series_names), " series")
    message("Calibration period: ", calibration_start, " to ", calibration_end)
    if (use_series_specific_periods) {
      message("  (CNPJ series will use 201601-201912 with cumsum from 201510)")
      message("  (subocuphoras uses split calibration: post-201509 + pre-201509)")
    }
  }

  # Prepare rolling quarters
  rq <- data.table::copy(rolling_quarters)
  data.table::setorder(rq, anomesfinaltrimmovel)

  # Add mesnotrim if missing (use .get_mesnotrim() for consistency)
  if (!"mesnotrim" %in% names(rq)) {
    rq[, mesnotrim := .get_mesnotrim(anomesfinaltrimmovel %% 100L)]
  }

  # Merge with monthly estimates
  me <- data.table::copy(monthly_estimates)
  data.table::setnames(me, "anomesexato", "anomesfinaltrimmovel", skip_absent = TRUE)

  if (!"anomesfinaltrimmovel" %in% names(me)) {
    stop("monthly_estimates must have 'anomesexato' or 'anomesfinaltrimmovel' column")
  }

  dt <- merge(rq, me, by = "anomesfinaltrimmovel", all.x = TRUE)
  data.table::setorder(dt, anomesfinaltrimmovel)

  # Pre-compute lagged INPC before any subsequent deflation, so that
  # the deflator for z_massaefetrealtodos at 201201 can use INPC[201112].
  # This must happen before the actual shift/deflation blocks.
  if ("inpc100dez1993" %in% names(dt)) {
    dt[, .inpc100dez1993_lagged :=
         data.table::shift(inpc100dez1993, n = 1L, type = "lag")]
  }


  # ============================================================================
  # INPC deflation for rhrp* series (hourly wage)
  # ============================================================================
  # SIDRA's rhrp* series are REAL (INPC-deflated to latest period)
  # Our z_rhrp* values are NOMINAL (computed from microdata income/hours)
  # We need to deflate z_rhrp* by INPC to match SIDRA's real values
  # Formula: real = nominal * (latest_inpc / current_inpc)
  # ============================================================================

  rhrp_series <- grep("^z_rhrp", names(dt), value = TRUE)
  if (length(rhrp_series) > 0 && "inpc100dez1993" %in% names(dt)) {
    if (verbose) message("  Deflating ", length(rhrp_series), " rhrp series by INPC...")

    latest_inpc <- dt[anomesfinaltrimmovel == max(anomesfinaltrimmovel), inpc100dez1993][1]

    if (!is.na(latest_inpc) && latest_inpc > 0) {
      for (z_col in rhrp_series) {
        # Deflate: real = nominal * (latest_inpc / current_inpc)
        dt[, (z_col) := get(z_col) * latest_inpc / inpc100dez1993]
      }
    } else {
      warning("INPC values not available, rhrp* series will use nominal values")
    }
  }

  # ============================================================================
  # INPC deflation for real income series (rendhabrealtodos, massahabrealtodos)
  # ============================================================================
  # SIDRA's *real* series are INPC-deflated
  # We need z_ values that match them (deflated nominal values)

  # For rendhabrealtodos etc., we need to deflate the nominal income values
  # These don't have z_ columns directly, but we have z_massahabnominaltodos
  # and z_comrendtodos to compute them

  # Compute real income mass from nominal
  if ("z_massahabnominaltodos" %in% names(dt) && "inpc100dez1993" %in% names(dt)) {
    latest_inpc <- dt[anomesfinaltrimmovel == max(anomesfinaltrimmovel), inpc100dez1993][1]

    if (!is.na(latest_inpc) && latest_inpc > 0) {
      if (verbose) message("  Computing INPC-deflated income series...")

      # Real habitual income mass (INPC)
      dt[, z_massahabrealtodos := z_massahabnominaltodos * latest_inpc / inpc100dez1993]
    }
  }

  if ("z_massaefetnominaltodos" %in% names(dt) && "inpc100dez1993" %in% names(dt)) {
    latest_inpc <- dt[anomesfinaltrimmovel == max(anomesfinaltrimmovel), inpc100dez1993][1]

    if (!is.na(latest_inpc) && latest_inpc > 0) {
      # Real effective income mass (INPC) - use lagged INPC like SIDRA
      inpc_lagged <- dt$.inpc100dez1993_lagged
      dt[, z_massaefetrealtodos := z_massaefetnominaltodos * latest_inpc / inpc_lagged]
    }
  }

  # Update series_names to include newly computed deflated z_ columns
  # (rhrp* and real income series computed above)
  all_z_cols <- grep("^z_", names(dt), value = TRUE)
  series_names <- sub("^z_", "", all_z_cols)

  if (verbose) {
    message("  Total series to process: ", length(series_names))
  }

  # Compute starting points for each series
  results <- list()

  for (v in series_names) {
    z_col <- paste0("z_", v)

    if (!v %in% names(dt)) {
      if (verbose) message("  Skipping ", v, " (not in rolling_quarters)")
      next
    }
    if (!z_col %in% names(dt)) {
      if (verbose) message("  Skipping ", v, " (no z_ column)")
      next
    }

    if (verbose) message("  Processing: ", v)

    rq_values <- dt[[v]]
    z_values <- dt[[z_col]]
    mesnotrim <- dt$mesnotrim
    yyyymm <- dt$anomesfinaltrimmovel

    # Determine series-specific calibration period
    # CNPJ series: use 201601-201912 with cumsum from 201510 (V4019 only available from 201510)
    is_cnpj_series <- use_series_specific_periods && v %in% cnpj_series
    is_split_series <- use_series_specific_periods && v %in% split_series

    # For subocuphoras, use the same post-VD4004_SPLIT parameters as CNPJ (handled below)
    series_cal_start <- if (is_cnpj_series || is_split_series) .PNADC_DATES$CNPJ_CALIB_START else calibration_start
    series_cal_end <- calibration_end
    cumsum_start <- if (is_cnpj_series || is_split_series) .PNADC_DATES$V4019_AVAILABLE else NA_integer_

    # Calculate cumulative variations using shared helper
    # For CNPJ and split series, cumsum should start from 201510
    filter_mask <- if (!is.na(cumsum_start)) (yyyymm >= cumsum_start) else NULL
    cum <- .compute_cumsum_by_mesnotrim(rq_values, mesnotrim, filter_mask = filter_mask)

    # Backprojection error
    e0 <- z_values / scale_factor - cum

    # Filter to calibration period (series-specific)
    in_calibration <- yyyymm >= series_cal_start & yyyymm <= series_cal_end

    # Average by mesnotrim
    for (pos in 1:3) {
      mask <- in_calibration & mesnotrim == pos
      y0_val <- mean(e0[mask], na.rm = TRUE)

      if (!is.finite(y0_val)) y0_val <- 0

      results[[length(results) + 1]] <- data.table::data.table(
        series_name = v,
        mesnotrim = pos,
        y0 = y0_val
      )
    }

    # ========================================================================
    # SPLIT CALIBRATION for subocuphoras (and similar series)
    # Compute ADDITIONAL starting points for pre-VD4004_SPLIT data
    # (split calibration for pre-VD4004_SPLIT period)
    # ========================================================================
    if (is_split_series) {
      split_month <- .PNADC_DATES$VD4004_SPLIT
      if (verbose) message("    Computing pre-", split_month, " starting points for split calibration...")

      # For pre-split: cumsum only for data <= split_month, calibration period before split
      cum_pre <- .compute_cumsum_by_mesnotrim(rq_values, mesnotrim,
                                              filter_mask = (yyyymm <= split_month))

      # Backprojection for pre-split calibration period
      e0_pre <- z_values / scale_factor - cum_pre

      # Pre-split calibration period: DEFAULT_CALIB_START to PRESPLIT_CALIB_END
      in_calib_pre <- yyyymm >= .PNADC_DATES$DEFAULT_CALIB_START & yyyymm <= .PNADC_DATES$PRESPLIT_CALIB_END

      for (pos in 1:3) {
        mask <- in_calib_pre & mesnotrim == pos
        y0_val <- mean(e0_pre[mask], na.rm = TRUE)

        if (!is.finite(y0_val)) y0_val <- 0

        # Store with "_pre" suffix to distinguish from post-201509 y0
        results[[length(results) + 1]] <- data.table::data.table(
          series_name = paste0(v, "_pre"),
          mesnotrim = pos,
          y0 = y0_val
        )
      }
    }
  }

  result <- data.table::rbindlist(results)

  if (verbose) {
    message("Computed starting points for ", length(unique(result$series_name)), " series")
  }

  result
}


# ==============================================================================
# Starting Points Computation from Microdata
# ==============================================================================

#' Compute z_ Aggregates from Monthly Microdata
#'
#' Computes monthly z_ aggregates from PNADC microdata using calibrated monthly
#' weights, with options for different population scaling approaches.
#'
#' @param calibrated_data PNADC microdata output from \code{pnadc_apply_periods()}
#'   with \code{calibrate = TRUE}. Must include \code{weight_monthly},
#'   \code{ref_month_yyyymm}, and \code{ref_month_in_quarter}.
#' @param verbose Print progress messages.
#'
#' @return data.table with columns:
#'   \describe{
#'     \item{anomesexato}{Integer YYYYMM month}
#'     \item{z_*}{Numeric weighted aggregates for each series (one column per series)}
#'   }
#'
#' @details
#' This function creates z_ indicator variables and aggregates them using the
#' calibrated \code{weight_monthly} from \code{pnadc_apply_periods()}.
#'
#' The \code{pnadc_apply_periods()} function implements the calibration
#' methodology as follows:
#' All months are scaled uniformly to SIDRA monthly population totals.
#'
#' This function simply aggregates the indicators using the already-calibrated weights.
#'
#' @examples
#' \dontrun{
#' crosswalk <- pnadc_identify_periods(stacked_data)
#'
#' calibrated <- pnadc_apply_periods(stacked_data, crosswalk,
#'                                    weight_var = "V1028",
#'                                    anchor = "quarter",
#'                                    calibration_unit = "month")
#'
#' z_agg <- compute_z_aggregates(calibrated)
#'
#' rq <- fetch_sidra_rolling_quarters()
#' y0 <- compute_series_starting_points(z_agg, rq)
#' }
#'
#' @seealso
#' \code{\link{pnadc_apply_periods}} for the calibration step
#' \code{\link{compute_series_starting_points}} for the y0 computation
#'
#' @export
compute_z_aggregates <- function(calibrated_data, verbose = TRUE) {

  # Validate input: must be output from pnadc_apply_periods with calibrate = TRUE
  required_cols <- c("weight_monthly", "ref_month_yyyymm", "ref_month_in_quarter")
  missing_cols <- setdiff(required_cols, names(calibrated_data))
  if (length(missing_cols) > 0) {
    stop("Input must be output from pnadc_apply_periods() with calibrate = TRUE. ",
         "Missing columns: ", paste(missing_cols, collapse = ", "))
  }

  # Ensure data.table
  if (!inherits(calibrated_data, "data.table")) {
    calibrated_data <- data.table::as.data.table(calibrated_data)
  }
  dt <- data.table::copy(calibrated_data)

  # Filter to observations with valid weights (determined months)
  dt <- dt[!is.na(weight_monthly) & weight_monthly > 0]
  if (nrow(dt) == 0) {
    stop("No observations with valid weight_monthly")
  }

  if (verbose) {
    message("Computing z_ aggregates using calibrated weights")
    message("  Observations: ", format(nrow(dt), big.mark = ","))
  }

  # ============================================================================
  # Step 1: Create anomesexato from ref_month_yyyymm
  # ============================================================================

  dt[, anomesexato := ref_month_yyyymm]

  # ============================================================================
  # Step 2: Create indicator variables
  # ============================================================================

  if (verbose) message("  Creating indicator variables...")

  # Convert character VD variables to integer (PNADC stores them as "01", "02", etc.)
  vd_vars <- c("VD4001", "VD4002", "VD4003", "VD4004", "VD4004A", "VD4005",
               "VD4009", "VD4010", "VD4012", "V4019")
  for (vv in vd_vars) {
    if (vv %in% names(dt) && is.character(dt[[vv]])) {
      data.table::set(dt, j = vv, value = as.integer(dt[[vv]]))
    }
  }

  # Population indicators
  dt[, z_populacao := 1L]
  dt[, z_pop14mais := as.integer(V2009 >= 14)]

  # Labor force status
  if ("VD4001" %in% names(dt)) {
    dt[, z_popforadaforca := as.integer(VD4001 == 2L)]
  }
  if ("VD4002" %in% names(dt)) {
    dt[, z_popocup := as.integer(VD4002 == 1L)]
    dt[, z_popdesocup := as.integer(VD4002 == 2L)]
  }

  # Employment type (VD4009)
  if ("VD4009" %in% names(dt)) {
    dt[, `:=`(
      z_empregprivcomcart = as.integer(VD4009 == 1L),
      z_empregprivsemcart = as.integer(VD4009 == 2L),
      z_domesticocomcart = as.integer(VD4009 == 3L),
      z_domesticosemcart = as.integer(VD4009 == 4L),
      z_empregpublcomcart = as.integer(VD4009 == 5L),
      z_empregpublsemcart = as.integer(VD4009 == 6L),
      z_estatutmilitar = as.integer(VD4009 == 7L),
      z_empregador = as.integer(VD4009 == 8L),
      z_contapropria = as.integer(VD4009 == 9L),
      z_trabfamauxiliar = as.integer(VD4009 == 10L)
    )]

    # CNPJ indicators - use %in% for NA-safe comparison (NA-safe)
    # V4019 only available from ~201510, so these will be 0/FALSE for earlier periods
    if ("V4019" %in% names(dt)) {
      dt[, z_empregadorcomcnpj := as.integer(VD4009 %in% 8L & V4019 %in% 1L)]
      dt[, z_empregadorsemcnpj := as.integer(VD4009 %in% 8L & V4019 %in% 2L)]
      dt[, z_contapropriacomcnpj := as.integer(VD4009 %in% 9L & V4019 %in% 1L)]
      dt[, z_contapropriasemcnpj := as.integer(VD4009 %in% 9L & V4019 %in% 2L)]
    }
  }

  # Sectors (VD4010)
  if ("VD4010" %in% names(dt)) {
    dt[, `:=`(
      z_agropecuaria = as.integer(VD4010 == 1L),
      z_industria = as.integer(VD4010 == 2L),
      z_construcao = as.integer(VD4010 == 3L),
      z_comercio = as.integer(VD4010 == 4L),
      z_transporte = as.integer(VD4010 == 5L),
      z_alojaliment = as.integer(VD4010 == 6L),
      z_infcomfinimobadm = as.integer(VD4010 == 7L),
      z_adminpublica = as.integer(VD4010 %in% c(8L, 9L)),
      z_outroservico = as.integer(VD4010 == 10L),
      z_servicodomestico = as.integer(VD4010 == 11L)
    )]
  }

  # Other indicators
  if ("VD4012" %in% names(dt)) {
    dt[, z_contribuinteprev := as.integer(VD4012 == 1L)]
  }
  if ("VD4003" %in% names(dt)) {
    dt[, z_forcapotencial := as.integer(VD4003 == 1L)]
  }
  if ("VD4005" %in% names(dt)) {
    dt[, z_desalentado := as.integer(VD4005 == 1L)]
  }

  # Suboccupation (variable changed name over time: VD4004 -> VD4004A at 201510)
  # VD4004 used for anomesexato <= VD4004_SPLIT, VD4004A otherwise
  if ("VD4004A" %in% names(dt) || "VD4004" %in% names(dt)) {
    split_month <- .PNADC_DATES$VD4004_SPLIT
    dt[, z_subocuphoras := 0L]
    if ("VD4004A" %in% names(dt)) {
      dt[anomesexato > split_month, z_subocuphoras := as.integer(VD4004A == 1L)]
    }
    if ("VD4004" %in% names(dt)) {
      dt[anomesexato <= split_month, z_subocuphoras := as.integer(VD4004 == 1L)]
    }
  }

  # ==========================================================================
  # AGGREGATE SERIES (sums of subcategories)
  # These can be computed directly from microdata rather than as post-hoc sums
  # ==========================================================================

  # Labor force (popnaforca = popocup + popdesocup)
  if ("VD4002" %in% names(dt)) {
    dt[, z_popnaforca := as.integer(VD4002 %in% c(1L, 2L))]
  }

  # Aggregate employment types
  if ("VD4009" %in% names(dt)) {
    dt[, z_empregpriv := as.integer(VD4009 %in% c(1L, 2L))]           # Private employees
    dt[, z_domestico := as.integer(VD4009 %in% c(3L, 4L))]            # Domestic workers
    dt[, z_empregpubl := as.integer(VD4009 %in% c(5L, 6L, 7L))]       # Public sector
    dt[, z_empregado := as.integer(VD4009 %in% 1:7)]                   # All employees
  }

  # Extended labor force (forcaampliada = popnaforca + forcapotencial)
  if ("VD4002" %in% names(dt) && "VD4003" %in% names(dt)) {
    dt[, z_forcaampliada := as.integer(VD4002 %in% c(1L, 2L) | VD4003 == 1L)]
  }

  # ==========================================================================
  # INCOME SERIES
  # These are income VALUES (not indicators) - aggregated as sum(value * weight)
  # ==========================================================================

  # Habitual income mass from all jobs (VD4019)
  if ("VD4019" %in% names(dt)) {
    # Convert to numeric if needed
    if (is.character(dt$VD4019)) {
      dt[, VD4019 := as.numeric(VD4019)]
    }
    # z_massahabnominaltodos is the income VALUE itself
    # When aggregated: sum(VD4019 * weight) = total income mass
    dt[, z_massahabnominaltodos := fifelse(is.na(VD4019), 0, VD4019)]

    # Count of people with income (comrendtodos)
    dt[, z_comrendtodos := as.integer(!is.na(VD4019) & VD4019 > 0)]
  }

  # Effective income mass from all jobs (VD4020)
  if ("VD4020" %in% names(dt)) {
    if (is.character(dt$VD4020)) {
      dt[, VD4020 := as.numeric(VD4020)]
    }
    dt[, z_massaefetnominaltodos := fifelse(is.na(VD4020), 0, VD4020)]
  }

  # ==========================================================================
  # HOURS WORKED AND INCOME BY CATEGORY (for rhrp* hourly wage series)
  # ==========================================================================
  # rhrp = Rendimento habitual real por hora efetivamente trabalhada
  # Formula: total_habitual_income / total_effective_hours (then INPC deflated)
  # We need to mensalize numerator and denominator separately, then compute ratio
  #
  # Variables:
  # - VD4016: Habitual income from main job (numerator)
  # - VD4035: Effective hours worked per week - all jobs (denominator)
  # ==========================================================================

  # Convert hours and income variables to numeric
  if ("VD4016" %in% names(dt)) {
    if (is.character(dt$VD4016)) dt[, VD4016 := as.numeric(VD4016)]
    dt[, VD4016 := fifelse(is.na(VD4016), 0, VD4016)]
  }
  if ("VD4017" %in% names(dt)) {
    if (is.character(dt$VD4017)) dt[, VD4017 := as.numeric(VD4017)]
    dt[, VD4017 := fifelse(is.na(VD4017), 0, VD4017)]
  }
  if ("VD4031" %in% names(dt)) {
    if (is.character(dt$VD4031)) dt[, VD4031 := as.numeric(VD4031)]
    dt[, VD4031 := fifelse(is.na(VD4031), 0, VD4031)]
  }
  if ("VD4035" %in% names(dt)) {
    if (is.character(dt$VD4035)) dt[, VD4035 := as.numeric(VD4035)]
    dt[, VD4035 := fifelse(is.na(VD4035), 0, VD4035)]
  }

  # Income and hours by employment type (VD4009) - for rhrp by position
  if ("VD4009" %in% names(dt) && "VD4016" %in% names(dt) && "VD4035" %in% names(dt)) {
    if (verbose) message("  Creating income/hours aggregates by employment type...")

    # Total income from main job (VD4016) by employment type
    dt[, z_income_hab_empregado := VD4016 * as.integer(VD4009 %in% 1:7)]
    dt[, z_income_hab_empregpriv := VD4016 * as.integer(VD4009 %in% c(1L, 2L))]
    dt[, z_income_hab_empregprivcomcart := VD4016 * as.integer(VD4009 == 1L)]
    dt[, z_income_hab_empregprivsemcart := VD4016 * as.integer(VD4009 == 2L)]
    dt[, z_income_hab_domestico := VD4016 * as.integer(VD4009 %in% c(3L, 4L))]
    dt[, z_income_hab_domesticocomcart := VD4016 * as.integer(VD4009 == 3L)]
    dt[, z_income_hab_domesticosemcart := VD4016 * as.integer(VD4009 == 4L)]
    dt[, z_income_hab_empregpubl := VD4016 * as.integer(VD4009 %in% c(5L, 6L, 7L))]
    dt[, z_income_hab_empregpublcomcart := VD4016 * as.integer(VD4009 == 5L)]
    dt[, z_income_hab_empregpublsemcart := VD4016 * as.integer(VD4009 == 6L)]
    dt[, z_income_hab_estatutmilitar := VD4016 * as.integer(VD4009 == 7L)]
    dt[, z_income_hab_empregador := VD4016 * as.integer(VD4009 == 8L)]
    dt[, z_income_hab_contapropria := VD4016 * as.integer(VD4009 == 9L)]

    # CNPJ variants (only available from 201510)
    if ("V4019" %in% names(dt)) {
      dt[, z_income_hab_empregadorcomcnpj := VD4016 * as.integer(VD4009 %in% 8L & V4019 %in% 1L)]
      dt[, z_income_hab_empregadorsemcnpj := VD4016 * as.integer(VD4009 %in% 8L & V4019 %in% 2L)]
      dt[, z_income_hab_contapropriacomcnpj := VD4016 * as.integer(VD4009 %in% 9L & V4019 %in% 1L)]
      dt[, z_income_hab_contapropriasemcnpj := VD4016 * as.integer(VD4009 %in% 9L & V4019 %in% 2L)]
    }

    # Effective hours worked (VD4035) by employment type
    dt[, z_hours_efet_empregado := VD4035 * as.integer(VD4009 %in% 1:7)]
    dt[, z_hours_efet_empregpriv := VD4035 * as.integer(VD4009 %in% c(1L, 2L))]
    dt[, z_hours_efet_empregprivcomcart := VD4035 * as.integer(VD4009 == 1L)]
    dt[, z_hours_efet_empregprivsemcart := VD4035 * as.integer(VD4009 == 2L)]
    dt[, z_hours_efet_domestico := VD4035 * as.integer(VD4009 %in% c(3L, 4L))]
    dt[, z_hours_efet_domesticocomcart := VD4035 * as.integer(VD4009 == 3L)]
    dt[, z_hours_efet_domesticosemcart := VD4035 * as.integer(VD4009 == 4L)]
    dt[, z_hours_efet_empregpubl := VD4035 * as.integer(VD4009 %in% c(5L, 6L, 7L))]
    dt[, z_hours_efet_empregpublcomcart := VD4035 * as.integer(VD4009 == 5L)]
    dt[, z_hours_efet_empregpublsemcart := VD4035 * as.integer(VD4009 == 6L)]
    dt[, z_hours_efet_estatutmilitar := VD4035 * as.integer(VD4009 == 7L)]
    dt[, z_hours_efet_empregador := VD4035 * as.integer(VD4009 == 8L)]
    dt[, z_hours_efet_contapropria := VD4035 * as.integer(VD4009 == 9L)]

    if ("V4019" %in% names(dt)) {
      dt[, z_hours_efet_empregadorcomcnpj := VD4035 * as.integer(VD4009 %in% 8L & V4019 %in% 1L)]
      dt[, z_hours_efet_empregadorsemcnpj := VD4035 * as.integer(VD4009 %in% 8L & V4019 %in% 2L)]
      dt[, z_hours_efet_contapropriacomcnpj := VD4035 * as.integer(VD4009 %in% 9L & V4019 %in% 1L)]
      dt[, z_hours_efet_contapropriasemcnpj := VD4035 * as.integer(VD4009 %in% 9L & V4019 %in% 2L)]
    }
  }

  # Income and hours by sector (VD4010) - for rhrp by sector
  if ("VD4010" %in% names(dt) && "VD4016" %in% names(dt) && "VD4035" %in% names(dt)) {
    if (verbose) message("  Creating income/hours aggregates by sector...")

    # Income by sector
    dt[, z_income_hab_agropecuaria := VD4016 * as.integer(VD4010 == 1L)]
    dt[, z_income_hab_industria := VD4016 * as.integer(VD4010 == 2L)]
    dt[, z_income_hab_construcao := VD4016 * as.integer(VD4010 == 3L)]
    dt[, z_income_hab_comercio := VD4016 * as.integer(VD4010 == 4L)]
    dt[, z_income_hab_transporte := VD4016 * as.integer(VD4010 == 5L)]
    dt[, z_income_hab_alojaliment := VD4016 * as.integer(VD4010 == 6L)]
    dt[, z_income_hab_infcomfinimobadm := VD4016 * as.integer(VD4010 == 7L)]
    dt[, z_income_hab_adminpublica := VD4016 * as.integer(VD4010 %in% c(8L, 9L))]
    dt[, z_income_hab_outroservico := VD4016 * as.integer(VD4010 == 10L)]
    dt[, z_income_hab_servicodomestico := VD4016 * as.integer(VD4010 == 11L)]

    # Hours by sector
    dt[, z_hours_efet_agropecuaria := VD4035 * as.integer(VD4010 == 1L)]
    dt[, z_hours_efet_industria := VD4035 * as.integer(VD4010 == 2L)]
    dt[, z_hours_efet_construcao := VD4035 * as.integer(VD4010 == 3L)]
    dt[, z_hours_efet_comercio := VD4035 * as.integer(VD4010 == 4L)]
    dt[, z_hours_efet_transporte := VD4035 * as.integer(VD4010 == 5L)]
    dt[, z_hours_efet_alojaliment := VD4035 * as.integer(VD4010 == 6L)]
    dt[, z_hours_efet_infcomfinimobadm := VD4035 * as.integer(VD4010 == 7L)]
    dt[, z_hours_efet_adminpublica := VD4035 * as.integer(VD4010 %in% c(8L, 9L))]
    dt[, z_hours_efet_outroservico := VD4035 * as.integer(VD4010 == 10L)]
    dt[, z_hours_efet_servicodomestico := VD4035 * as.integer(VD4010 == 11L)]
  }

  # ============================================================================
  # Step 3: Aggregate by month using weight_monthly
  # ============================================================================

  if (verbose) message("  Aggregating by month...")

  # Get z_ columns
  z_cols <- grep("^z_", names(dt), value = TRUE)

  if (length(z_cols) == 0) {
    stop("No z_ indicator columns created. Check variable availability.")
  }

  # Aggregate: sum(indicator * weight_monthly) by month
  result <- dt[, lapply(.SD, function(x) sum(x * weight_monthly, na.rm = TRUE)),
               by = anomesexato, .SDcols = z_cols]

  data.table::setorder(result, anomesexato)

  # ============================================================================
  # Step 4: Scale income series to match SIDRA units
  # ============================================================================
  # SIDRA reports income mass in MILLIONS of reais, but our z_ aggregation
  # produces raw reais. We scale by 1/1000 here so that the standard
  # scale_factor = 1000 in compute_series_starting_points() works correctly.
  #
  # After this scaling:
  # - z_populacao is in raw (SIDRA in thousands, so z/1000 = SIDRA)
  # - z_massahabnominaltodos is in thousands (SIDRA in millions, so z/1000 = SIDRA)
  # Both use scale_factor = 1000 for y0 computation.

  if (verbose) message("  Scaling income series to SIDRA units...")

  # Scale income mass series (raw reais -> thousands, so /1000 gives SIDRA millions)
  income_mass_series <- c("z_massahabnominaltodos", "z_massaefetnominaltodos")
  for (col in income_mass_series) {
    if (col %in% names(result)) {
      result[, (col) := get(col) / 1000]
    }
  }

  # Scale income by category series (for rhrp computation)
  # These are also in raw reais, need same scaling
  income_by_cat_series <- grep("^z_income_hab_", names(result), value = TRUE)
  for (col in income_by_cat_series) {
    result[, (col) := get(col) / 1000]
  }

  # Note: z_hours_efet_* series stay in raw hours (no scaling needed)

  # ============================================================================
  # Step 5: Compute rhrp (hourly wage) series from income/hours
  # ============================================================================
  # rhrp = rendimento habitual por hora efetivamente trabalhada
  # Formula: rhrp = (total_income / total_hours) * conversion_factor
  # Conversion: income is monthly, hours are weekly -> divide by 4.33 (weeks/month)
  # The z_income_hab_* are in thousands (scaled above), z_hours_efet_* are raw
  # Result: rhrp in reais per hour
  #
  # Note: These z_rhrp_* values are NOMINAL. SIDRA's rhrp series are REAL
  # (INPC-deflated). The y0 computation will need to deflate by INPC.
  # ============================================================================

  if (verbose) message("  Computing hourly wage (rhrp) series...")

  # Helper to compute rhrp safely (avoid division by zero)
  compute_rhrp <- function(income_col, hours_col) {
    if (income_col %in% names(result) && hours_col %in% names(result)) {
      income <- result[[income_col]]
      hours <- result[[hours_col]]
      # Convert: income (thousands/month) / hours (hours/week) / 4.33 (weeks/month)
      # The 1000 factor converts back from thousands
      rhrp <- fifelse(hours > 0, income * 1000 / (hours * 4.33), NA_real_)
      return(rhrp)
    }
    return(NULL)
  }

  # rhrp by employment type
  employment_types <- c("empregado", "empregpriv", "empregprivcomcart", "empregprivsemcart",
                        "domestico", "domesticocomcart", "domesticosemcart",
                        "empregpubl", "empregpublcomcart", "empregpublsemcart", "estatutmilitar",
                        "empregador", "contapropria")

  for (et in employment_types) {
    income_col <- paste0("z_income_hab_", et)
    hours_col <- paste0("z_hours_efet_", et)
    rhrp <- compute_rhrp(income_col, hours_col)
    if (!is.null(rhrp)) {
      result[, paste0("z_rhrp", et) := rhrp]
    }
  }

  # rhrp by employment type (CNPJ variants)
  cnpj_types <- c("empregadorcomcnpj", "empregadorsemcnpj",
                  "contapropriacomcnpj", "contapropriasemcnpj")

  for (et in cnpj_types) {
    income_col <- paste0("z_income_hab_", et)
    hours_col <- paste0("z_hours_efet_", et)
    rhrp <- compute_rhrp(income_col, hours_col)
    if (!is.null(rhrp)) {
      result[, paste0("z_rhrp", et) := rhrp]
    }
  }

  # rhrp by sector
  sectors <- c("agropecuaria", "industria", "construcao", "comercio", "transporte",
               "alojaliment", "infcomfinimobadm", "adminpublica", "outroservico",
               "servicodomestico")

  for (sec in sectors) {
    income_col <- paste0("z_income_hab_", sec)
    hours_col <- paste0("z_hours_efet_", sec)
    rhrp <- compute_rhrp(income_col, hours_col)
    if (!is.null(rhrp)) {
      result[, paste0("z_rhrp", sec) := rhrp]
    }
  }

  # ============================================================================
  # NOTE: Derived series (rates, per-capita income) are NOT computed here
  # ============================================================================
  # Rate series (taxadesocup, taxapartic, etc.) should NOT have starting points.
  # Methodology: mensalize COMPONENT series (popdesocup, popnaforca),
  # then DERIVE rates: m_taxadesocup = m_popdesocup / m_popnaforca * 100
  #
  # Per-capita income series (rendhabnominaltodos, rendefetnominaltodos) are
  # available directly from SIDRA and are mensalized with their own y0 values.
  # They are NOT computed as z_massa / z_comrend here.

  # Count final z_ columns
  z_final <- grep("^z_", names(result), value = TRUE)

  if (verbose) {
    message("  Created ", length(z_final), " z_ series for ",
            nrow(result), " months (",
            min(result$anomesexato), " to ", max(result$anomesexato), ")")
  }

  result
}


#' Compute Starting Points from Raw PNADC Microdata
#'
#' Complete workflow to compute y0 starting points from raw PNADC microdata.
#' This is a convenience wrapper that combines period identification, weight
#' calibration, z_ aggregation, and starting point computation.
#'
#' @param data Stacked PNADC microdata (multiple quarters). Must contain variables
#'   for period identification (see \code{\link{pnadc_identify_periods}}).
#' @param calibration_start Integer. Start of calibration period (YYYYMM).
#'   Default NULL uses .PNADC_DATES$DEFAULT_CALIB_START (201301).
#' @param calibration_end Integer. End of calibration period (YYYYMM).
#'   Default NULL uses .PNADC_DATES$DEFAULT_CALIB_END (201912).
#' @param verbose Print progress messages.
#'
#' @return data.table with columns:
#'   \describe{
#'     \item{series_name}{Character. Series name}
#'     \item{mesnotrim}{Integer. Month position (1, 2, or 3)}
#'     \item{y0}{Numeric. Starting point value}
#'   }
#'
#' @details
#' This function performs the complete workflow:
#' \enumerate{
#'   \item Build crosswalk via \code{pnadc_identify_periods()}
#'   \item Calibrate weights via \code{pnadc_apply_periods()}
#'   \item Compute z_ aggregates via \code{compute_z_aggregates()}
#'   \item Fetch SIDRA rolling quarters
#'   \item Compute starting points via \code{compute_series_starting_points()}
#' }
#'
#' @section Weight Calibration:
#' All months are scaled uniformly to SIDRA monthly population totals.
#'
#' @examples
#' \dontrun{
#' stacked <- fst::read_fst("pnadc_stacked.fst", as.data.table = TRUE)
#'
#' y0 <- compute_starting_points_from_microdata(stacked)
#'
#' bundled <- pnadc_series_starting_points
#' comparison <- merge(y0, bundled, by = c("series_name", "mesnotrim"))
#' }
#'
#' @seealso
#' \code{\link{pnadc_apply_periods}} for the weight calibration step
#' \code{\link{compute_z_aggregates}} for the z_ aggregation step
#' \code{\link{compute_series_starting_points}} for the y0 computation
#' \code{\link{pnadc_identify_periods}} for period identification
#'
#' @export
compute_starting_points_from_microdata <- function(data,
                                                    calibration_start = NULL,
                                                    calibration_end = NULL,
                                                    verbose = TRUE) {

  # Resolve defaults from centralized constants
  if (is.null(calibration_start)) {
    calibration_start <- .PNADC_DATES$DEFAULT_CALIB_START
  }
  if (is.null(calibration_end)) {
    calibration_end <- .PNADC_DATES$DEFAULT_CALIB_END
  }

  if (verbose) {
    message("\n", paste(rep("=", 60), collapse = ""))
    message("Computing Starting Points from Microdata")
    message(paste(rep("=", 60), collapse = ""))
    message("Calibration period: ", calibration_start, " to ", calibration_end)
    message("")
  }

  # Step 1: Build crosswalk
  if (verbose) message("Step 1: Building crosswalk...")
  crosswalk <- pnadc_identify_periods(data, verbose = verbose)

  # Step 2: Calibrate weights (all months -> SIDRA monthly population)
  if (verbose) message("\nStep 2: Calibrating weights...")
  calibrated <- pnadc_apply_periods(
    data = data,
    crosswalk = crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibration_unit = "month",
    verbose = verbose
  )

  # Step 3: Compute z_ aggregates using calibrated weights
  if (verbose) message("\nStep 3: Computing z_ aggregates...")
  z_aggregates <- compute_z_aggregates(calibrated, verbose = verbose)

  # Step 4: Fetch SIDRA rolling quarters
  # exclude_derived=TRUE skips rate series (they're computed from population ratios)
  if (verbose) message("\nStep 4: Fetching SIDRA rolling quarters...")
  rolling_quarters <- fetch_sidra_rolling_quarters(
    verbose = verbose,
    use_cache = TRUE,
    exclude_derived = TRUE
  )

  # Step 5: Compute starting points
  if (verbose) message("\nStep 5: Computing starting points...")
  y0 <- compute_series_starting_points(
    monthly_estimates = z_aggregates,
    rolling_quarters = rolling_quarters,
    calibration_start = calibration_start,
    calibration_end = calibration_end,
    verbose = verbose
  )

  if (verbose) {
    message("\n", paste(rep("=", 60), collapse = ""))
    message("Complete: ", length(unique(y0$series_name)), " series x 3 positions = ",
            nrow(y0), " starting points")
    message(paste(rep("=", 60), collapse = ""))
  }

  y0
}

# Tests for F1 Solution: Pre-PNADC IPCA Lag in rolling_quarters
# EC2 and EC1 revisados para refletir o comportamento real

make_rq_with_ipca <- function(start_yyyymm = 201201L,
                              n_months = 24L,
                              series_name = 'massaefetnominaltodos',
                              base_value = 1000,
                              trend = 100,
                              ipca_start = NULL,
                              ipca_values = NULL) {
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L

  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]

  dt[, (series_name) := NA_real_]
  dt[dt$anomesfinaltrimmovel >= 201201L,
     (series_name) := base_value + trend * (which(dt$anomesfinaltrimmovel >= 201201L) - 1)]

  if (!is.null(ipca_start)) {
    dt[, ipca100dez1993 := NA_real_]
    idx_start <- which(dt$anomesfinaltrimmovel == ipca_start)[1]

    if (!is.na(idx_start) && !is.null(ipca_values)) {
      n_ipca <- length(ipca_values)
      end_idx <- min(idx_start + n_ipca - 1, nrow(dt))
      dt[idx_start:end_idx, ipca100dez1993 := ipca_values[1:(end_idx - idx_start + 1)]]
    }
  }

  dt[, .SD, .SDcols = c('anomesfinaltrimmovel', 'mesnotrim', series_name,
                        if ('ipca100dez1993' %in% names(dt)) 'ipca100dez1993' else NULL)]
}

test_that('F1 EC1: IPCA only 201201+ — lagged IPCA is NA at 201201', {
  dt <- make_rq_with_ipca(start_yyyymm = 201201L, n_months = 24L,
    ipca_start = 201201L,
    ipca_values = c(100, 100.3, 100.6, 100.9, 101.2, 101.5,
                    101.8, 102.1, 102.4, 102.7, 103.0, 103.3,
                    103.6, 103.9, 104.2, 104.5, 104.8, 105.1,
                    105.4, 105.7, 106.0, 106.3, 106.6, 106.9))

  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)

  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = 'massaefetnominaltodos', compute_derived = TRUE, verbose = FALSE)

  row_201201 <- result[anomesexato == 201201L]
  if ('ipca100dez1993_lagged' %in% names(result)) {
    expect_true(is.na(row_201201$ipca100dez1993_lagged),
      label = 'ipca100dez1993_lagged is NA at 201201 (no pre-PNADC data)')
  }
  if ('m_massaefettodosipcabr' %in% names(result)) {
    expect_true(is.na(row_201201$m_massaefettodosipcabr),
      label = 'm_massaefettodosipcabr is NA at 201201 (Phase 5 fallback)')
  }
})

test_that('F1 EC2: IPCA from 201111 — lagged IPCA non-NA at 201201', {
  start_yyyymm <- 201111L
  n_months <- 26L

  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]

  dt[, ipca100dez1993 := 100 + 0.3 * (month_num - 1)]

  dt[, massaefetnominaltodos := NA_real_]
  dt[dt$anomesfinaltrimmovel >= 201201L,
     massaefetnominaltodos := 1000 + 100 * (which(dt$anomesfinaltrimmovel >= 201201L) - 1)]

  dt <- dt[, .(anomesfinaltrimmovel, mesnotrim, ipca100dez1993, massaefetnominaltodos)]

  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)

  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = 'massaefetnominaltodos', compute_derived = TRUE, verbose = FALSE)

  row_201201 <- result[anomesexato == 201201L]
  if ('ipca100dez1993_lagged' %in% names(result)) {
    expect_false(is.na(row_201201$ipca100dez1993_lagged),
      label = 'ipca100dez1993_lagged is non-NA at 201201 (F1 pre-computed from 201112)')
  }
  if ('m_massaefettodosipcabr' %in% names(result)) {
    expect_false(is.na(row_201201$m_massaefettodosipcabr),
      label = 'm_massaefettodosipcabr is non-NA at 201201 (F1 solution)')
  }
})

test_that('F1 EC3: IPCA gap at 201112 — lagged IPCA is NA at 201201', {
  start_yyyymm <- 201110L
  n_months <- 26L

  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]

  dt[, ipca100dez1993 := 100 + 0.3 * (month_num - 1)]
  dt[anomesfinaltrimmovel == 201112L, ipca100dez1993 := NA_real_]

  dt[, massaefetnominaltodos := NA_real_]
  dt[dt$anomesfinaltrimmovel >= 201201L,
     massaefetnominaltodos := 1000 + 100 * (which(dt$anomesfinaltrimmovel >= 201201L) - 1)]

  dt <- dt[, .(anomesfinaltrimmovel, mesnotrim, ipca100dez1993, massaefetnominaltodos)]

  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)

  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = 'massaefetnominaltodos', compute_derived = TRUE, verbose = FALSE)

  row_201201 <- result[anomesexato == 201201L]
  if ('ipca100dez1993_lagged' %in% names(result)) {
    expect_true(is.na(row_201201$ipca100dez1993_lagged),
      label = 'ipca100dez1993_lagged is NA at 201201 (IPCA gap at 201112)')
  }
  if ('m_massaefettodosipcabr' %in% names(result)) {
    expect_true(is.na(row_201201$m_massaefettodosipcabr),
      label = 'm_massaefettodosipcabr is NA at 201201 (lagged IPCA is NA)')
  }
})

test_that('F1 EC4: latest_ipca is NA — Phase 5 guard prevents computation', {
  dt <- make_rq_with_ipca(start_yyyymm = 201201L, n_months = 24L,
    ipca_start = 201201L, ipca_values = rep(NA_real_, 24))

  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)

  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = 'massaefetnominaltodos', compute_derived = TRUE, verbose = FALSE)

  if ('m_massaefettodosipcabr' %in% names(result)) {
    expect_true(all(is.na(result$m_massaefettodosipcabr)),
      label = 'm_massaefettodosipcabr all NA (Phase 5 guard, latest_ipca = NA)')
  }
})

# ============================================================================
# STRESS TESTS: Rolling Quarter Edge Cases and Robustness
# ============================================================================

test_that('STRESS-1: rolling_quarters data.table vazio deve falhar com stop', {
  empty_dt <- data.table::data.table()
  sp <- data.table::data.table(series_name = 'test', mesnotrim = 1, y0 = 100)
  
  expect_error(
    mensalize_sidra_series(empty_dt, starting_points = sp, verbose = FALSE),
    "Missing required columns",
    label = "Empty rolling_quarters should fail"
  )
})

test_that('STRESS-2: rolling_quarters sem series PNADC deve falhar', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    mesnotrim = c(1L, 2L, 3L)
  )
  sp <- data.table::data.table(series_name = 'test', mesnotrim = 1, y0 = 100)
  
  expect_error(
    mensalize_sidra_series(dt, starting_points = sp, verbose = FALSE),
    "No series columns found",
    label = "No PNADC series should fail"
  )
})

test_that('STRESS-3: rolling_quarters apenas IPCA sem PNADC deve falhar', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    mesnotrim = c(1L, 2L, 3L),
    ipca100dez1993 = c(100, 100.5, 101)
  )
  sp <- data.table::data.table(series_name = 'test', mesnotrim = 1, y0 = 100)
  
  expect_error(
    mensalize_sidra_series(dt, starting_points = sp, verbose = FALSE),
    "No series columns found",
    label = "Only IPCA should fail"
  )
})

test_that('STRESS-4: rolling_quarters com anomesfinaltrimmovel duplicado', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201203L, 201204L),
    mesnotrim = c(1L, 1L, 2L),
    massaefetnominaltodos = c(1000, 1000, 1050)
  )
  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)
  
  result <- mensalize_sidra_series(dt, starting_points = sp, verbose = FALSE)
  
  expect_true(nrow(result) > 0, label = "Duplicated anomesfinaltrimmovel should not crash")
  expect_true("m_massaefetnominaltodos" %in% names(result))
})

test_that('STRESS-5: rolling_quarters com mesnotrim inválido', {
  skip("known issue: mesnotrim validation not implemented in mensalize_sidra_series")
})

test_that('STRESS-6: rolling_quarters com anomesfinaltrimmovel desordenado', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201205L, 201203L, 201204L),
    mesnotrim = c(3L, 1L, 2L),
    massaefetnominaltodos = c(1100, 1000, 1050)
  )
  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)
  
  result <- mensalize_sidra_series(dt, starting_points = sp, verbose = FALSE)
  
  expect_true(all(diff(result$anomesexato) > 0),
              label = "Result should be properly ordered")
  expect_true(nrow(result) == 3L)
})

test_that('STRESS-7: starting_points vazio deve produzir m_* todos NA', {
  empty_sp <- data.table::data.table(
    series_name = character(),
    mesnotrim = integer(),
    y0 = numeric()
  )
  
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    mesnotrim = c(1L, 2L, 3L),
    massaefetnominaltodos = c(1000, 1050, 1100)
  )
  
  result <- mensalize_sidra_series(dt, starting_points = empty_sp, verbose = FALSE)
  
  expect_true(all(is.na(result$m_massaefetnominaltodos)),
              label = "Empty starting_points should give all-NA m_*")
})

test_that('STRESS-8: starting_points com series nao em rolling_quarters', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    mesnotrim = c(1L, 2L, 3L),
    massaefetnominaltodos = c(1000, 1050, 1100)
  )
  
  sp <- data.table::data.table(
    series_name = c('massaefetnominaltodos', 'nonexistent_series'),
    mesnotrim = rep(1, 2),
    y0 = c(1000, 5000)
  )
  
  result <- mensalize_sidra_series(dt, starting_points = sp,
                                    series = 'massaefetnominaltodos', verbose = FALSE)
  
  expect_true('m_massaefetnominaltodos' %in% names(result))
  expect_true(!'m_nonexistent_series' %in% names(result))
})

test_that('STRESS-9: rolling_quarters com todos valores NA', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    mesnotrim = c(1L, 2L, 3L),
    massaefetnominaltodos = c(NA_real_, NA_real_, NA_real_)
  )
  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)
  
  result <- mensalize_sidra_series(dt, starting_points = sp, verbose = FALSE)
  
  expect_true(all(is.na(result$m_massaefetnominaltodos)))
})

test_that('STRESS-10: rolling_quarters com NA intercalado', {
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L, 201206L, 201207L),
    mesnotrim = c(1L, 2L, 3L, 1L, 2L),
    massaefetnominaltodos = c(1000, 1050, NA_real_, 1100, 1150)
  )
  sp <- data.table::data.table(series_name = 'massaefetnominaltodos', mesnotrim = 1, y0 = 1000)
  
  result <- mensalize_sidra_series(dt, starting_points = sp, verbose = FALSE)
  
  expect_true(nrow(result) == 5L)
  expect_false(is.na(result$m_massaefetnominaltodos[4]),
               label = "KNOWN ISSUE: NA gap not properly handled")
})

# Test NA propagation in derived series - EXTENDED WITH REAL DATA

# Helper to load real rolling quarters
load_real_rolling_quarters <- function() {
  testthat::skip_if_not_installed("qs2")
  data_path <- "d:/Dropbox/Artigos/mensalizacao_pnad/PNADCperiods-dashboard/data/rolling_quarters.qs2"
  if (!file.exists(data_path)) {
    skip("rolling_quarters.qs2 not found")
  }
  getNamespace("qs2")$qs_read(data_path)
}

# D1: Basic aggregates
test_that("D1: popnaforca = popocup + popdesocup - synthetic data", {
  dt <- data.table::data.table(
    anomesexato = 202301:202310,
    m_popocup = c(100, 105, 110, 108, 112, 115, 118, 120, 122, 125),
    m_popdesocup = c(10, 12, 11, 13, 9, 8, 10, 11, 12, 10)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  expect_equal(result$m_popnaforca, dt$m_popocup + dt$m_popdesocup)
})

test_that("D1.NA: popnaforca NA when any component NA", {
  dt <- data.table::data.table(
    anomesexato = c(202301, 202302, 202303, 202304),
    m_popocup = c(100, NA, 110, 108),
    m_popdesocup = c(10, 12, NA, 13)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  # Row 2: popocup NA -> aggregate NA
  expect_true(is.na(result$m_popnaforca[2]))
  # Row 3: popdesocup NA -> aggregate NA
  expect_true(is.na(result$m_popnaforca[3]))
  # Row 1: both valid -> aggregate valid
  expect_false(is.na(result$m_popnaforca[1]))
})

# D2: Rates
test_that("D2: taxadesocup rate computation", {
  dt <- data.table::data.table(
    anomesexato = 202301:202310,
    m_popocup = c(100, 105, 110, 108, 112, 115, 118, 120, 122, 125),
    m_popdesocup = c(10, 12, 11, 13, 9, 8, 10, 11, 12, 10)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  popnaforca <- dt$m_popocup + dt$m_popdesocup
  expected <- round(dt$m_popdesocup / popnaforca * 100, 1)
  expect_equal(result$m_taxadesocup, expected)
})

test_that("D2.NA: taxadesocup NA when denominator NA", {
  dt <- data.table::data.table(
    anomesexato = c(202301, 202302, 202303),
    m_popocup = c(100, 105, 110),
    m_popdesocup = c(10, NA, 11)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  # Row 2: popdesocup NA -> popnaforca NA -> taxadesocup NA
  expect_true(is.na(result$m_popnaforca[2]))
  expect_true(is.na(result$m_taxadesocup[2]) || is.nan(result$m_taxadesocup[2]))
})

# D3: Residuals
test_that("D3: residual computation", {
  dt <- data.table::data.table(
    anomesexato = 202301:202310,
    m_pop14mais = c(170, 172, 174, 176, 178, 180, 182, 184, 186, 188),
    m_popocup = c(100, 102, 104, 106, 108, 110, 112, 114, 116, 118),
    m_popdesocup = c(12, 13, 11, 14, 10, 9, 11, 12, 13, 10)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  expected <- dt$m_pop14mais - dt$m_popocup - dt$m_popdesocup
  expect_equal(result$m_popforadaforca, expected)
})

test_that("D3.NA: residual NA when any subtracted component NA", {
  dt <- data.table::data.table(
    anomesexato = c(202301, 202302, 202303),
    m_pop14mais = c(170, 172, 174),
    m_popocup = c(100, NA, 104),
    m_popdesocup = c(12, 13, NA)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  # Row 2: popocup NA -> residual NA
  expect_true(is.na(result$m_popforadaforca[2]))
  # Row 3: popdesocup NA -> residual NA
  expect_true(is.na(result$m_popforadaforca[3]))
})

# D4: Average income
test_that("D4: average income computation", {
  dt <- data.table::data.table(
    anomesexato = 202301:202310,
    m_massahabnominaltodos = c(250e6, 260e6, 255e6, 265e6, 270e6,
                               275e6, 280e6, 285e6, 290e6, 295e6),
    m_comrendtodos = c(90e3, 91e3, 92e3, 93e3, 94e3,
                       95e3, 96e3, 97e3, 98e3, 99e3)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  expected <- round(dt$m_massahabnominaltodos / dt$m_comrendtodos * 1000, 0)
  expect_equal(result$m_rendhabnominaltodos, expected)
})

test_that("D4.NA: average income NA when denominator NA", {
  dt <- data.table::data.table(
    anomesexato = c(202301, 202302, 202303),
    m_massahabnominaltodos = c(250e6, 260e6, 255e6),
    m_comrendtodos = c(90e3, NA, 92e3)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  # Row 2: denominator NA -> average income NA
  expect_true(is.na(result$m_rendhabnominaltodos[2]))
  # Row 1,3: denominator valid -> average income valid
  expect_false(is.na(result$m_rendhabnominaltodos[1]))
  expect_false(is.na(result$m_rendhabnominaltodos[3]))
})

# D5: Deflated series
test_that("D5: deflated series with IPCA", {
  ipca_values <- seq(5000, 5450, by = 50)
  dt <- data.table::data.table(
    anomesexato = 202301:202310,
    m_massahabnominaltodos = c(250e6, 260e6, 255e6, 265e6, 270e6,
                               275e6, 280e6, 285e6, 290e6, 295e6),
    m_comrendtodos = c(90e3, 91e3, 92e3, 93e3, 94e3,
                       95e3, 96e3, 97e3, 98e3, 99e3),
    ipca100dez1993 = ipca_values
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  expect_true("m_massahabtodosipcabr" %in% names(result))
  # All values should be non-NA since all sources are valid
  expect_true(sum(!is.na(result$m_massahabtodosipcabr)) == 10)
})

test_that("D5.NA: deflated series NA when nominal source NA", {
  ipca_values <- seq(5000, 5450, by = 50)
  dt <- data.table::data.table(
    anomesexato = c(202301, 202302, 202303),
    m_massahabnominaltodos = c(250e6, NA, 255e6),
    m_comrendtodos = c(90e3, 91e3, 92e3),
    ipca100dez1993 = c(5000, 5050, 5100)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))
  # Row 2: nominal source NA -> deflated NA
  expect_true(is.na(result$m_massahabtodosipcabr[2]))
  # Row 1,3: nominal source valid -> deflated valid
  expect_false(is.na(result$m_massahabtodosipcabr[1]))
  expect_false(is.na(result$m_massahabtodosipcabr[3]))
})

# D1-D5 Summary: comprehensive edge case
test_that("D1-D5: complex dependency chain with partial NA", {
  dt <- data.table::data.table(
    anomesexato = c(202301, 202302, 202303, 202304, 202305),
    m_popocup = c(100, 105, NA, 108, 112),
    m_popdesocup = c(10, NA, 11, 13, 9),
    m_pop14mais = c(170, 172, 174, 176, 178),
    m_massahabnominaltodos = c(250e6, 260e6, 255e6, 265e6, 270e6),
    m_comrendtodos = c(90e3, 91e3, NA, 93e3, 94e3),
    ipca100dez1993 = c(5000, 5050, 5100, 5150, 5200)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  # Row 1: all valid
  expect_false(is.na(result$m_popnaforca[1]))
  expect_false(is.na(result$m_taxadesocup[1]))
  expect_false(is.na(result$m_popforadaforca[1]))
  expect_false(is.na(result$m_rendhabnominaltodos[1]))

  # Row 2: popdesocup NA -> affects popnaforca, taxadesocup
  expect_true(is.na(result$m_popnaforca[2]))
  expect_true(is.na(result$m_taxadesocup[2]) || is.nan(result$m_taxadesocup[2]))
  # comrendtodos valid -> average income valid
  expect_false(is.na(result$m_rendhabnominaltodos[2]))

  # Row 3: popocup NA -> affects popnaforca
  #        comrendtodos NA -> affects average income
  expect_true(is.na(result$m_popnaforca[3]))
  expect_true(is.na(result$m_rendhabnominaltodos[3]))

  # Row 4: all valid
  expect_false(is.na(result$m_popnaforca[4]))
  expect_false(is.na(result$m_taxadesocup[4]))

  # Row 5: all valid
  expect_false(is.na(result$m_popnaforca[5]))
  expect_false(is.na(result$m_taxadesocup[5]))
})

# Tests for .compute_derived_series() and .DERIVED_SERIES_SPEC
# These tests verify the derived series computation engine (aggregates, average
# income, residuals, rates, deflated) and the structural integrity of the
# declarative specification that drives it.

library(data.table)

# =============================================================================
# PHASE 1: AGGREGATE TESTS
# =============================================================================

test_that("aggregate: popocup + popdesocup = popnaforca", {
  dt <- data.table(
    anomesexato = 202301:202310,
    m_popocup    = c(100, 105, 110, 108, 112, 115, 118, 120, 122, 125),
    m_popdesocup = c(10, 12, 11, 13, 9, 8, 10, 11, 12, 10)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  expect_true("m_popnaforca" %in% names(result))
  expect_equal(result$m_popnaforca, dt$m_popocup + dt$m_popdesocup)
})

test_that("aggregate dependency chain: empregpriv built first, then empregado uses it", {
  dt <- data.table(
    anomesexato = 202301:202310,
    # Components for empregpriv
    m_empregprivcomcart = seq(40, 49),
    m_empregprivsemcart = seq(10, 19),
    # Components for domestico
    m_domesticocomcart = seq(5, 14),
    m_domesticosemcart = seq(2, 11),
    # Components for empregpubl
    m_empregpublcomcart = seq(8, 17),
    m_empregpublsemcart = seq(3, 12),
    m_estatutmilitar    = seq(1, 10)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  # empregpriv should be computed first
  expect_true("m_empregpriv" %in% names(result))
  expected_empregpriv <- dt$m_empregprivcomcart + dt$m_empregprivsemcart
  expect_equal(result$m_empregpriv, expected_empregpriv)

  # domestico should also be computed
  expect_true("m_domestico" %in% names(result))
  expected_domestico <- dt$m_domesticocomcart + dt$m_domesticosemcart
  expect_equal(result$m_domestico, expected_domestico)

  # empregpubl depends on 3 components

  expect_true("m_empregpubl" %in% names(result))
  expected_empregpubl <- dt$m_empregpublcomcart + dt$m_empregpublsemcart + dt$m_estatutmilitar
  expect_equal(result$m_empregpubl, expected_empregpubl)

  # empregado = empregpriv + domestico + empregpubl (all computed above)
  expect_true("m_empregado" %in% names(result))
  expected_empregado <- expected_empregpriv + expected_domestico + expected_empregpubl
  expect_equal(result$m_empregado, expected_empregado)
})

# =============================================================================
# PHASE 2: AVERAGE INCOME TESTS
# =============================================================================

test_that("average income: rendhabnominaltodos = massahabnominaltodos / comrendtodos * 1000", {
  dt <- data.table(
    anomesexato        = 202301:202310,
    m_massahabnominaltodos = c(250e6, 260e6, 255e6, 265e6, 270e6,
                               275e6, 280e6, 285e6, 290e6, 295e6),
    m_comrendtodos         = c(90e3, 91e3, 92e3, 93e3, 94e3,
                               95e3, 96e3, 97e3, 98e3, 99e3)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  expect_true("m_rendhabnominaltodos" %in% names(result))
  expected <- round(dt$m_massahabnominaltodos / dt$m_comrendtodos * 1000, 0)
  expect_equal(result$m_rendhabnominaltodos, expected)
})

# =============================================================================
# PHASE 3: RESIDUAL TESTS
# =============================================================================

test_that("residual: popforadaforca = pop14mais - popocup - popdesocup", {
  dt <- data.table(
    anomesexato  = 202301:202310,
    m_pop14mais  = c(170, 172, 174, 176, 178, 180, 182, 184, 186, 188),
    m_popocup    = c(100, 102, 104, 106, 108, 110, 112, 114, 116, 118),
    m_popdesocup = c(12, 13, 11, 14, 10, 9, 11, 12, 13, 10)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  expect_true("m_popforadaforca" %in% names(result))
  expected <- dt$m_pop14mais - dt$m_popocup - dt$m_popdesocup
  expect_equal(result$m_popforadaforca, expected)
})

# =============================================================================
# PHASE 4: RATE TESTS
# =============================================================================

test_that("rate: taxadesocup = popdesocup / popnaforca * 100", {
  dt <- data.table(
    anomesexato  = 202301:202310,
    m_popocup    = c(100, 105, 110, 108, 112, 115, 118, 120, 122, 125),
    m_popdesocup = c(10, 12, 11, 13, 9, 8, 10, 11, 12, 10)
  )
  # Phase 1 computes m_popnaforca = popocup + popdesocup
  # Phase 4 computes taxadesocup = popdesocup / popnaforca * 100
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  expect_true("m_taxadesocup" %in% names(result))
  popnaforca <- dt$m_popocup + dt$m_popdesocup
  expected <- round(dt$m_popdesocup / popnaforca * 100, 1)
  expect_equal(result$m_taxadesocup, expected)
})

test_that("rate with compound denominator: percdesalento uses (popnaforca + desalentado)", {
  dt <- data.table(
    anomesexato    = 202301:202310,
    m_popocup      = c(100, 105, 110, 108, 112, 115, 118, 120, 122, 125),
    m_popdesocup   = c(10, 12, 11, 13, 9, 8, 10, 11, 12, 10),
    m_desalentado  = c(5, 6, 4, 7, 3, 4, 5, 6, 7, 4),
    m_forcapotencial = c(8, 9, 7, 10, 6, 7, 8, 9, 10, 7)
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  expect_true("m_percdesalento" %in% names(result))

  # percdesalento spec: numerator = "desalentado", denominator = c("popnaforca", "desalentado")
  # popnaforca is computed in Phase 1 as popocup + popdesocup
  popnaforca <- dt$m_popocup + dt$m_popdesocup
  # Compound denominator: rowSums of m_popnaforca + m_desalentado
  denom <- popnaforca + dt$m_desalentado
  expected <- round(dt$m_desalentado / denom * 100, 1)
  expect_equal(result$m_percdesalento, expected)
})

# =============================================================================
# PHASE 5: DEFLATED SERIES TESTS
# =============================================================================

test_that("deflated series: verify IPCA deflation formula for hab and efet", {
  # 10 months with rising IPCA index
  ipca_values <- seq(5000, 5450, by = 50)
  dt <- data.table(
    anomesexato            = 202301:202310,
    m_massahabnominaltodos = c(250e6, 260e6, 255e6, 265e6, 270e6,
                               275e6, 280e6, 285e6, 290e6, 295e6),
    m_massaefetnominaltodos = c(240e6, 250e6, 245e6, 255e6, 260e6,
                                265e6, 270e6, 275e6, 280e6, 285e6),
    m_comrendtodos         = c(90e3, 91e3, 92e3, 93e3, 94e3,
                               95e3, 96e3, 97e3, 98e3, 99e3),
    ipca100dez1993         = ipca_values
  )
  result <- PNADCperiods:::.compute_derived_series(copy(dt))

  latest_ipca <- ipca_values[10]  # max anomesexato

  # hab series use current IPCA (use_lagged_ipca = FALSE)
  # deflator_hab = latest_ipca / ipca100dez1993
  expect_true("m_massahabtodosipcabr" %in% names(result))
  deflator_hab <- latest_ipca / ipca_values
  expected_hab <- round(dt$m_massahabnominaltodos * deflator_hab, 0)
  expect_equal(result$m_massahabtodosipcabr, expected_hab)

  # efet series use lagged IPCA (use_lagged_ipca = TRUE)
  # deflator_efet = latest_ipca / shift(ipca100dez1993, 1, type="lag")
  expect_true("m_massaefettodosipcabr" %in% names(result))
  ipca_lagged <- c(NA, ipca_values[1:9])
  deflator_efet <- latest_ipca / ipca_lagged
  expected_efet <- round(dt$m_massaefetnominaltodos * deflator_efet, 0)
  # Row 1 is NA because lagged IPCA is NA
  expect_true(is.na(result$m_massaefettodosipcabr[1]))
  expect_equal(result$m_massaefettodosipcabr[2:10], expected_efet[2:10])
})

# =============================================================================
# EDGE CASE TESTS
# =============================================================================

test_that("missing component column: derived series gracefully skipped", {
  # Only provide popocup, not popdesocup -- popnaforca cannot be computed
  dt <- data.table(
    anomesexato = 202301:202310,
    m_popocup   = seq(100, 109)
  )
  # Should not error
  result <- expect_silent(PNADCperiods:::.compute_derived_series(copy(dt)))
  # popnaforca should NOT be created (missing component)
  expect_false("m_popnaforca" %in% names(result))
  # taxadesocup also should NOT be created (missing numerator and denominator)
  expect_false("m_taxadesocup" %in% names(result))
})

test_that("division by zero: denominator=0 does not crash", {
  dt <- data.table(
    anomesexato  = 202301:202310,
    m_popocup    = c(100, 105, 110, 108, 0, 115, 118, 120, 122, 125),
    m_popdesocup = c(10, 12, 11, 13, 0, 8, 10, 11, 12, 10)
  )
  # Row 5: popnaforca = 0, so taxadesocup = 0/0
  # Should not error -- NaN or Inf is acceptable, just no crash
  result <- expect_silent(PNADCperiods:::.compute_derived_series(copy(dt)))
  expect_true("m_taxadesocup" %in% names(result))
  # The function completes without error
  expect_s3_class(result, "data.table")
})

# =============================================================================
# .DERIVED_SERIES_SPEC STRUCTURAL VALIDATION
# =============================================================================

test_that("spec: all aggregate specs have 'name' and non-empty 'components'", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  for (s in spec$aggregates) {
    expect_true(!is.null(s$name), info = paste("Aggregate missing 'name'"))
    expect_true(is.character(s$name) && nchar(s$name) > 0,
                info = paste("Aggregate has empty name"))
    expect_true(!is.null(s$components),
                info = paste("Aggregate", s$name, "missing 'components'"))
    expect_true(length(s$components) >= 2,
                info = paste("Aggregate", s$name, "has fewer than 2 components"))
  }
})

test_that("spec: all rate specs have 'name', 'numerator', 'denominator'", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  for (s in spec$rates) {
    expect_true(!is.null(s$name),
                info = "Rate missing 'name'")
    expect_true(is.character(s$name) && nchar(s$name) > 0,
                info = "Rate has empty name")
    expect_true(!is.null(s$numerator),
                info = paste("Rate", s$name, "missing 'numerator'"))
    expect_true(!is.null(s$denominator),
                info = paste("Rate", s$name, "missing 'denominator'"))
    expect_true(length(unlist(s$numerator)) >= 1,
                info = paste("Rate", s$name, "has empty numerator"))
    expect_true(length(unlist(s$denominator)) >= 1,
                info = paste("Rate", s$name, "has empty denominator"))
  }
})

test_that("spec: all residual specs have 'name', 'parent', 'subtract'", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  for (s in spec$residuals) {
    expect_true(!is.null(s$name),
                info = "Residual missing 'name'")
    expect_true(is.character(s$name) && nchar(s$name) > 0,
                info = "Residual has empty name")
    expect_true(!is.null(s$parent),
                info = paste("Residual", s$name, "missing 'parent'"))
    expect_true(is.character(s$parent) && nchar(s$parent) > 0,
                info = paste("Residual", s$name, "has empty parent"))
    expect_true(!is.null(s$subtract),
                info = paste("Residual", s$name, "missing 'subtract'"))
    expect_true(length(s$subtract) >= 1,
                info = paste("Residual", s$name, "has empty subtract list"))
  }
})

test_that("spec: no duplicate names across all spec sections", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  all_names <- c(
    sapply(spec$aggregates, `[[`, "name"),
    sapply(spec$average_income, `[[`, "name"),
    sapply(spec$residuals, `[[`, "name"),
    sapply(spec$rates, `[[`, "name"),
    sapply(spec$deflated, `[[`, "name")
  )
  duplicated_names <- all_names[duplicated(all_names)]
  expect_equal(length(duplicated_names), 0,
               info = paste("Duplicate derived series names:", paste(duplicated_names, collapse = ", ")))
})

test_that("spec: aggregate dependencies in correct order (empregpriv before empregado)", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  agg_names <- sapply(spec$aggregates, `[[`, "name")

  # empregpriv must appear before empregado (empregado depends on empregpriv)
  idx_empregpriv <- which(agg_names == "empregpriv")
  idx_empregado  <- which(agg_names == "empregado")
  expect_true(length(idx_empregpriv) == 1, info = "empregpriv not found in aggregates")
  expect_true(length(idx_empregado) == 1, info = "empregado not found in aggregates")
  expect_true(idx_empregpriv < idx_empregado,
              info = "empregpriv must be computed before empregado")

  # popnaforca must appear before forcaampliada (forcaampliada depends on popnaforca)
  idx_popnaforca   <- which(agg_names == "popnaforca")
  idx_forcaampliada <- which(agg_names == "forcaampliada")
  expect_true(length(idx_popnaforca) == 1, info = "popnaforca not found in aggregates")
  expect_true(length(idx_forcaampliada) == 1, info = "forcaampliada not found in aggregates")
  expect_true(idx_popnaforca < idx_forcaampliada,
              info = "popnaforca must be computed before forcaampliada")

  # domestico must appear before empregado
  idx_domestico <- which(agg_names == "domestico")
  expect_true(idx_domestico < idx_empregado,
              info = "domestico must be computed before empregado")

  # empregpubl must appear before empregado
  idx_empregpubl <- which(agg_names == "empregpubl")
  expect_true(idx_empregpubl < idx_empregado,
              info = "empregpubl must be computed before empregado")
})

test_that("spec: all deflated specs have 'source' and 'use_lagged_ipca'", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  for (s in spec$deflated) {
    expect_true(!is.null(s$name),
                info = "Deflated missing 'name'")
    expect_true(!is.null(s$source),
                info = paste("Deflated", s$name, "missing 'source'"))
    expect_true(is.character(s$source) && nchar(s$source) > 0,
                info = paste("Deflated", s$name, "has empty source"))
    expect_true(!is.null(s$use_lagged_ipca),
                info = paste("Deflated", s$name, "missing 'use_lagged_ipca'"))
    expect_true(is.logical(s$use_lagged_ipca),
                info = paste("Deflated", s$name, "'use_lagged_ipca' is not logical"))
  }
})

test_that("spec: phase ordering ensures aggregates computed before rates that use them", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC

  # Collect all names produced by aggregates (Phase 1)
  agg_names <- sapply(spec$aggregates, `[[`, "name")

  # Rates that reference aggregate outputs in their numerator or denominator
  # e.g., taxadesocup uses popnaforca (aggregate) as denominator
  for (s in spec$rates) {
    # Check denominator references
    denom_components <- unlist(s$denominator)
    for (d in denom_components) {
      if (d %in% agg_names) {
        # This is fine -- the aggregate is computed in Phase 1, rate in Phase 4
        expect_true(d %in% agg_names,
                    info = paste("Rate", s$name, "references aggregate", d,
                                 "which should be computed before rates"))
      }
    }
    # Check numerator references
    num_components <- unlist(s$numerator)
    for (n in num_components) {
      if (n %in% agg_names) {
        expect_true(n %in% agg_names,
                    info = paste("Rate", s$name, "references aggregate", n,
                                 "which should be computed before rates"))
      }
    }
  }

  # Specifically verify that taxadesocup denominator (popnaforca) is an aggregate
  taxadesocup <- Filter(function(s) s$name == "taxadesocup", spec$rates)
  expect_true(length(taxadesocup) == 1)
  denom <- unlist(taxadesocup[[1]]$denominator)
  expect_true("popnaforca" %in% agg_names,
              info = "popnaforca must be in aggregates since taxadesocup depends on it")
  expect_true(all(denom %in% c(agg_names, "popnaforca")),
              info = "taxadesocup denominator must reference known aggregates or primaries")
})

test_that("spec: all average_income specs have 'multiplier' and 'decimals'", {
  spec <- PNADCperiods:::.DERIVED_SERIES_SPEC
  for (s in spec$average_income) {
    expect_true(!is.null(s$name),
                info = "Average income spec missing 'name'")
    expect_true(!is.null(s$numerator),
                info = paste("Average income", s$name, "missing 'numerator'"))
    expect_true(!is.null(s$denominator),
                info = paste("Average income", s$name, "missing 'denominator'"))
    expect_true(!is.null(s$multiplier),
                info = paste("Average income", s$name, "missing 'multiplier'"))
    expect_true(is.numeric(s$multiplier) && s$multiplier > 0,
                info = paste("Average income", s$name, "'multiplier' must be positive numeric"))
    expect_true(!is.null(s$decimals),
                info = paste("Average income", s$name, "missing 'decimals'"))
    expect_true(is.numeric(s$decimals) && s$decimals >= 0,
                info = paste("Average income", s$name, "'decimals' must be non-negative numeric"))
  }
})

# Tests for trailing-NA propagation in mensalize_sidra_series().
#
# Regression coverage for the "phantom row" pattern: when SIDRA publishes
# a price index (IPCA/INPC) for a month before PNADC publishes the rolling
# quarter ending in that same month, fetch_sidra_rolling_quarters() returns
# rows where only the price-index columns are filled and every PNADC column
# is NA. The mensalization must propagate NA for those positions; otherwise
# .compute_cumsum_by_mesnotrim() (which intentionally treats NA as 0 to
# accommodate late-starting series like CNPJ post-201510) and the fallback
# branch of .apply_final_adjustment() collude to produce numerically
# plausible but spurious mensalized values.

# Local synthetic generator (kept here to avoid refactoring helper-test-data.R;
# pattern mirrors make_rolling_quarter_dt in test-mensalize-sidra-series.R)
make_rq_with_trailing_na <- function(start_yyyymm = 201201L,
                                     n_months = 36L,
                                     series_name = "popocup",
                                     base_value = 100000,
                                     trend = 100,
                                     n_trailing_na = 1L) {
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L

  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  dt[, (series_name) := base_value + trend * (month_num - 1) +
       500 * sin(2 * pi * month_num / 12)]

  # Inject trailing NAs in the last n_trailing_na positions
  if (n_trailing_na > 0L) {
    dt[seq.int(n_months - n_trailing_na + 1L, n_months),
       (series_name) := NA_real_]
  }

  dt[, .SD, .SDcols = c("anomesfinaltrimmovel", "mesnotrim", series_name)]
}


# =============================================================================
# Single-series: trailing NA in rq must propagate as NA in m
# =============================================================================

test_that(".mensalize_single_series propagates trailing NA from rq to m", {
  dt <- make_rq_with_trailing_na(
    start_yyyymm = 201201L, n_months = 36L,
    series_name = "popocup", base_value = 90000, trend = 100,
    n_trailing_na = 1L
  )
  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(89500, 89800, 90100)
  )

  m <- PNADCperiods:::.mensalize_single_series(dt, "popocup", sp)

  # Last position has rq=NA; mensalized output must be NA.
  expect_true(is.na(m[length(m)]),
              label = "Last m must be NA when last rq is NA")

  # All positions where rq is non-NA must have non-NA m (with valid
  # starting points and 36 months, no first_any_data masking applies).
  rq_vec <- dt[["popocup"]]
  expect_true(all(!is.na(m[!is.na(rq_vec)])),
              label = "Non-trailing positions remain non-NA")
})


test_that(".mensalize_single_series propagates multiple trailing NAs", {
  dt <- make_rq_with_trailing_na(
    start_yyyymm = 201201L, n_months = 36L,
    series_name = "popocup", base_value = 90000, trend = 100,
    n_trailing_na = 3L
  )
  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(89500, 89800, 90100)
  )

  m <- PNADCperiods:::.mensalize_single_series(dt, "popocup", sp)

  # Last 3 positions have rq=NA; all 3 mensalized values must be NA.
  n <- length(m)
  expect_true(all(is.na(m[(n - 2L):n])),
              label = "Last 3 m values must be NA when last 3 rq are NA")
})


# =============================================================================
# Split-series: trailing NA in post-split rq must propagate as NA in m
# =============================================================================

test_that(".mensalize_split_series propagates trailing NA in post-split", {
  dt <- make_rq_with_trailing_na(
    start_yyyymm = 201201L, n_months = 60L,
    series_name = "subocuphoras", base_value = 5000, trend = 10,
    n_trailing_na = 1L
  )
  sp <- data.table::data.table(
    series_name = c(rep("subocuphoras_pre", 3), rep("subocuphoras", 3)),
    mesnotrim = c(1:3, 1:3),
    y0 = c(4800, 4900, 5000, 5400, 5500, 5600)
  )

  m <- PNADCperiods:::.mensalize_split_series(
    dt, "subocuphoras", sp, split_month = 201509L
  )

  # Last position is in post-split (201601 onward) with rq=NA;
  # mensalized must be NA.
  expect_true(is.na(m[length(m)]),
              label = "Last m must be NA when last post-split rq is NA")
})


# =============================================================================
# Full pipeline: derived series must be NA when primaries are NA on trailing
# =============================================================================

test_that("leading NA before first observed rq is reconstructed via y0 (regression guard)", {
  # The leading positions for which no rolling quarter ever existed
  # (e.g., 201201 = mesnotrim 1, 201202 = mesnotrim 2 — before the first
  # rolling quarter ending in 201203) MUST receive a non-NA mensalized
  # value derived from y0 + apply_final_adjustment lookahead. A previous
  # blanket fix `m[is.na(rq)] <- NA_real_` regressed this behaviour and
  # produced NA; this test guards against re-introducing that bug.
  start_yyyymm <- 201201L
  n_months <- 24L
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  # Series starts ONLY at month 3 (mesnotrim=3) — first 2 are NA, like
  # IPCA-only leading rows for 201201/201202.
  dt[, popocup := 90000 + 100 * (month_num - 1)]
  dt[1L:2L, popocup := NA_real_]
  dt <- dt[, .(anomesfinaltrimmovel, mesnotrim, popocup)]

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(89500, 89800, 90100)
  )

  m <- PNADCperiods:::.mensalize_single_series(dt, "popocup", sp)

  # Leading positions (1, 2) MUST be non-NA (reconstructed from y0).
  expect_false(is.na(m[1L]),
               label = "Leading position 1 (rq=NA) reconstructed from y0")
  expect_false(is.na(m[2L]),
               label = "Leading position 2 (rq=NA) reconstructed from y0")
  # And they must be reasonably close to y0 (sanity check; exact value
  # depends on apply_final_adjustment's lookahead computation).
  expect_true(abs(m[1L] - 89500) < 10000,
              label = "Leading m[1] is in the y0 ballpark")
  expect_true(abs(m[2L] - 89800) < 10000,
              label = "Leading m[2] is in the y0 ballpark")
})


test_that("mensalize_sidra_series produces NA derived when primaries trail-NA", {
  # Build a multi-series RQ with trailing NA on both popocup and popdesocup
  # so that the derived popnaforca = popocup + popdesocup and rate
  # taxadesocup = popdesocup/popnaforca*100 receive NA inputs.
  start_yyyymm <- 201201L
  n_months <- 36L

  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  dt[, popocup := 90000 + 100 * (month_num - 1)]
  dt[, popdesocup := 10000 + 30 * (month_num - 1)]
  # Inject trailing NA in last position for both primaries
  dt[n_months, `:=`(popocup = NA_real_, popdesocup = NA_real_)]
  dt <- dt[, .(anomesfinaltrimmovel, mesnotrim, popocup, popdesocup)]

  sp <- data.table::data.table(
    series_name = c(rep("popocup", 3), rep("popdesocup", 3)),
    mesnotrim = rep(1:3, 2),
    y0 = c(89500, 89800, 90100, 9800, 9900, 10000)
  )

  out <- mensalize_sidra_series(dt, starting_points = sp,
                                 series = c("popocup", "popdesocup"),
                                 compute_derived = TRUE,
                                 verbose = FALSE)

  last_row <- out[anomesexato == max(anomesexato)]
  # Primaries must be NA
  expect_true(is.na(last_row$m_popocup),
              label = "m_popocup is NA on trailing")
  expect_true(is.na(last_row$m_popdesocup),
              label = "m_popdesocup is NA on trailing")
  # Derived rate: NA (or NaN — both are detected by is.na in R)
  if ("m_taxadesocup" %in% names(last_row)) {
    expect_true(is.na(last_row$m_taxadesocup),
                label = "m_taxadesocup is NA/NaN on trailing")
  }
})


# =============================================================================


# =============================================================================
# F1 Solution Tests - Corrected
# =============================================================================

test_that("F1: Pre-computed IPCA lag is available in result", {
  # Create synthetic rolling quarter data with IPCA from pre-PNADC era
  start_yyyymm <- 201101L
  n_months <- 24L
  
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  
  # IPCA from pre-PNADC
  dt[, ipca100dez1993 := 100 + 0.3 * (month_num - 1)]
  
  # PNADC series from 201201 onward
  dt[, massaefetnominaltodos := NA_real_]
  dt[3:n_months, massaefetnominaltodos := 1000 + 100 * (month_num - 3)]
  
  dt <- dt[, .(anomesfinaltrimmovel, mesnotrim, ipca100dez1993, massaefetnominaltodos)]
  
  sp <- data.table::data.table(
    series_name = "massaefetnominaltodos",
    mesnotrim = 1,
    y0 = 1000
  )
  
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "massaefetnominaltodos",
    compute_derived = TRUE,
    verbose = FALSE
  )
  
  # Check that ipca100dez1993_lagged was copied to result
  expect_true("ipca100dez1993_lagged" %in% names(result),
              label = "ipca100dez1993_lagged column exists in result")
  
  # 201201 should have a lagged IPCA value (from 201112)
  row_201201 <- result[anomesexato == 201201L]
  expect_false(is.na(row_201201$ipca100dez1993_lagged),
               label = "ipca100dez1993_lagged is non-NA in 201201 (pre-PNADC available)")
})


test_that("F1: Deflated efet series is non-NA in 201201 with pre-PNADC IPCA lag", {
  # Test that the actual deflated series are computed correctly in 201201
  # when pre-PNADC IPCA lag is available (F1 solution)
  
  start_yyyymm <- 201101L
  n_months <- 24L
  
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  
  dt[, ipca100dez1993 := 100 + 0.3 * (month_num - 1)]
  
  # For deflated series, we need BOTH hab and efet nominal series
  dt[, massahabnominaltodos := NA_real_]
  dt[, massaefetnominaltodos := NA_real_]
  dt[3:n_months, `:=`(
    massahabnominaltodos = 1000 + 100 * (month_num - 3),
    massaefetnominaltodos = 1000 + 100 * (month_num - 3)
  )]
  
  dt <- dt[, .(anomesfinaltrimmovel, mesnotrim, ipca100dez1993, 
               massahabnominaltodos, massaefetnominaltodos)]
  
  sp <- data.table::data.table(
    series_name = c(rep("massahabnominaltodos", 3), rep("massaefetnominaltodos", 3)),
    mesnotrim = c(1:3, 1:3),
    y0 = c(1000, 1000, 1000, 1000, 1000, 1000)
  )
  
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = c("massahabnominaltodos", "massaefetnominaltodos"),
    compute_derived = TRUE,
    verbose = FALSE
  )
  
  row_201201 <- result[anomesexato == 201201L]
  
  # hab deflated (uses current IPCA, no lag)
  expect_false(is.na(row_201201$m_massahabtodosipcabr),
               label = "m_massahabtodosipcabr is non-NA in 201201 (hab, no lag needed)")
  
  # efet deflated (uses lagged IPCA - the F1 fix)
  # BEFORE F1, this would be NA
  # AFTER F1, this should be non-NA because lagged IPCA from pre-PNADC is available
  expect_false(is.na(row_201201$m_massaefettodosipcabr),
               label = "m_massaefettodosipcabr is non-NA in 201201 (efet, with F1 lag available)")
})

# Tests for Fix Interactions: V1, V2, F1 operating together
# 
# Context:
# V1 (trailing fantasma): m[is.na(rq)] <- NA_real_ — REPLACED by V2
# V2 (trailing-only): NA-out only after last non-NA of rq (preserves leading via lookahead)
# F1 (pre-PNADC lag IPCA/INPC): pre-compute lag before PNADC filter
#
# These tests confirm all 3 fixes work together without interference.

# Helper: Create synthetic rolling quarters with IPCA for interaction testing
make_rq_interaction_test <- function(
    start_yyyymm = 201201L,
    n_months = 24L,
    series_name = "massaefetnominaltodos",
    base_value = 1000,
    trend = 100,
    ipca_start = NULL,
    ipca_values = NULL,
    na_pattern = NULL) {
  
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  
  # Initialize series with NA, then fill where specified
  dt[, (series_name) := NA_real_]
  
  # Apply NA pattern if provided (indices to mark as NA)
  if (!is.null(na_pattern)) {
    # na_pattern is list(leading = c(...), trailing = c(...))
    if (!is.null(na_pattern$leading)) {
      dt[na_pattern$leading, (series_name) := NA_real_]
    }
    if (!is.null(na_pattern$trailing)) {
      dt[na_pattern$trailing, (series_name) := NA_real_]
    }
  }
  
  # Fill non-NA positions with trend
  dt[!is.na(get(series_name)) | (is.na(na_pattern) | (is.null(na_pattern$leading) && is.null(na_pattern$trailing))),
     (series_name) := base_value + trend * (seq_len(.N) - 1L)]
  
  # Add IPCA if specified
  if (!is.null(ipca_start)) {
    dt[, ipca100dez1993 := NA_real_]
    idx_start <- which(dt$anomesfinaltrimmovel == ipca_start)[1]
    
    if (!is.na(idx_start) && !is.null(ipca_values)) {
      n_ipca <- length(ipca_values)
      end_idx <- min(idx_start + n_ipca - 1, nrow(dt))
      dt[idx_start:end_idx, ipca100dez1993 := ipca_values[1:(end_idx - idx_start + 1)]]
    }
  }
  
  dt[, .SD, .SDcols = c("anomesfinaltrimmovel", "mesnotrim", series_name,
                        if ("ipca100dez1993" %in% names(dt)) "ipca100dez1993" else NULL)]
}


# =============================================================================
# T1: Rolling quarters with trailing NA (V2) AND pre-PNADC IPCA (F1)
# =============================================================================
test_that("T1: V2 trailing + F1 pre-PNADC IPCA work together", {
  # Setup: 26 months (201111-201312)
  # - 201111 to 201112: IPCA only (pre-PNADC), pop NA
  # - 201201 to 201212: pop data + IPCA (PNADC era)
  # - 202603 (last row): trailing NA in pop, but IPCA continues
  
  start_year <- 201111L
  n_months <- 26L
  
  start_year_num <- start_year %/% 100L
  start_month_num <- start_year %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year_num + (start_month_num + month_num - 2L) %/% 12L,
    month = ((start_month_num + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  
  # IPCA from 201111 throughout
  dt[, ipca100dez1993 := 100 + 0.3 * (month_num - 1)]
  
  # massaefetnominaltodos: NA for 201111-201112, then data from 201201+
  dt[, massaefetnominaltodos := NA_real_]
  pnadc_start_idx <- which(dt$anomesfinaltrimmovel >= 201201L)[1]
  if (!is.na(pnadc_start_idx)) {
    dt[pnadc_start_idx:nrow(dt), massaefetnominaltodos := 1000 + 50 * seq_len(nrow(dt) - pnadc_start_idx + 1)]
  }
  
  # Mark last row as trailing NA (V2 behavior)
  dt[.N, massaefetnominaltodos := NA_real_]
  
  sp <- data.table::data.table(
    series_name = "massaefetnominaltodos",
    mesnotrim = 1:3,
    y0 = c(1000, 1010, 1020)
  )
  
  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = "massaefetnominaltodos", compute_derived = TRUE, verbose = FALSE)
  
  # Assert F1: m_massaefettodosipcabr[201201] is non-NA (lagged IPCA from 201112 exists)
  row_201201 <- result[anomesexato == 201201L]
  if ("m_massaefettodosipcabr" %in% names(result)) {
    expect_false(is.na(row_201201$m_massaefettodosipcabr),
      label = "T1-F1: m_massaefettodosipcabr[201201] is non-NA (F1 lag from pre-PNADC)")
  }
  
  # Assert V2: m_massaefetnominaltodos[last] is NA (trailing NA)
  last_row <- result[.N]
  expect_true(is.na(last_row$m_massaefetnominaltodos),
    label = "T1-V2: m_massaefetnominaltodos[trailing] is NA (V2 trailing)")
})


# =============================================================================
# T2: Leading NA reconstruction (V2 lookahead) + F1 pre-PNADC lag
# =============================================================================
test_that("T2: V2 lookahead (leading NA) + F1 lag work together", {
  # Setup: 26 months (201111-201312)
  # - 201111-201112: IPCA present but pop NA (leading)
  # - 201201 onwards: pop data present
  # Expected: m_popocup[201201] reconstructed via lookahead, m_*ipcabr[201201] via F1 lag
  
  start_year <- 201111L
  n_months <- 26L
  
  start_year_num <- start_year %/% 100L
  start_month_num <- start_year %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year_num + (start_month_num + month_num - 2L) %/% 12L,
    month = ((start_month_num + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  
  # IPCA from 201111 throughout (pre-PNADC available)
  dt[, ipca100dez1993 := 100 + 0.2 * (month_num - 1)]
  
  # popocup: NA for 201111-201112, then data from 201201+
  dt[, popocup := NA_real_]
  pnadc_start_idx <- which(dt$anomesfinaltrimmovel >= 201201L)[1]
  if (!is.na(pnadc_start_idx)) {
    dt[pnadc_start_idx:nrow(dt), popocup := 80000 + 200 * seq_len(nrow(dt) - pnadc_start_idx + 1)]
  }
  
  # popdesocup: same pattern
  dt[, popdesocup := NA_real_]
  if (!is.na(pnadc_start_idx)) {
    dt[pnadc_start_idx:nrow(dt), popdesocup := 5000 + 30 * seq_len(nrow(dt) - pnadc_start_idx + 1)]
  }
  
  sp <- data.table::data.table(
    series_name = c(rep("popocup", 3), rep("popdesocup", 3)),
    mesnotrim = rep(1:3, 2),
    y0 = c(80000, 80100, 80200, 5000, 5100, 5200)
  )
  
  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = c("popocup", "popdesocup"), compute_derived = TRUE, verbose = FALSE)
  
  # Assert V2 lookahead: m_popocup[201201] is non-NA
  # (Algorithm reconstructs leading position via y0 + lookahead in final adjustment)
  row_201201 <- result[anomesexato == 201201L]
  expect_false(is.na(row_201201$m_popocup),
    label = "T2-V2: m_popocup[201201] is non-NA (V2 lookahead reconstruction)")
  expect_false(is.na(row_201201$m_popdesocup),
    label = "T2-V2: m_popdesocup[201201] is non-NA (V2 lookahead reconstruction)")
  
  # Assert F1: if IPCA-deflated series exist, they should also be non-NA at 201201
  # (because F1 pre-computed lag before PNADC filter)
  # Note: These are only created if compute_derived=TRUE AND ipca series present
  if ("m_popocupipcabr" %in% names(result)) {
    expect_false(is.na(row_201201$m_popocupipcabr),
      label = "T2-F1: m_popocupipcabr[201201] is non-NA (F1 lag from pre-PNADC)")
  }
})


# =============================================================================
# T3: Combined trailing + leading NA + IPCA pre-2012 (all 3 fixes interact)
# =============================================================================
test_that("T3: V2 trailing + V2 leading + F1 IPCA all together", {
  # Setup: 28 months (201111-202003)
  # - 201111-201112: leading NA in pop, IPCA present
  # - 201201 to 201912: full data
  # - 202001 to 202003: trailing NA in pop (last 3 months), IPCA continues
  # Expected: 
  # - m_popocup[201201] non-NA (V2 lookahead)
  # - m_*ipcabr[201201] non-NA (F1 lag from 201112)
  # - m_popocup[202001-202003] NA (V2 trailing)
  
  start_year <- 201111L
  n_months <- 28L
  
  start_year_num <- start_year %/% 100L
  start_month_num <- start_year %% 100L
  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year_num + (start_month_num + month_num - 2L) %/% 12L,
    month = ((start_month_num + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]
  
  # IPCA from 201111 throughout
  dt[, ipca100dez1993 := 100 + 0.15 * (month_num - 1)]
  
  # popocup: NA for 201111-201112 (leading), NA for 202001-202003 (trailing)
  dt[, popocup := NA_real_]
  pnadc_start_idx <- which(dt$anomesfinaltrimmovel >= 201201L)[1]
  pnadc_end_idx <- which(dt$anomesfinaltrimmovel <= 201912L)
  pnadc_end_idx <- if (length(pnadc_end_idx) > 0) max(pnadc_end_idx) else nrow(dt)
  
  if (!is.na(pnadc_start_idx) && pnadc_start_idx <= pnadc_end_idx) {
    dt[pnadc_start_idx:pnadc_end_idx, popocup := 80000 + 100 * seq_len(pnadc_end_idx - pnadc_start_idx + 1)]
  }
  
  sp <- data.table::data.table(
    series_name = "popocup",
    mesnotrim = 1:3,
    y0 = c(80000, 80050, 80100)
  )
  
  result <- mensalize_sidra_series(dt, starting_points = sp,
    series = "popocup", compute_derived = TRUE, verbose = FALSE)
  
  # Assert V2 lookahead: m_popocup[201201] non-NA
  row_201201 <- result[anomesexato == 201201L]
  expect_false(is.na(row_201201$m_popocup),
    label = "T3-V2-lead: m_popocup[201201] is non-NA (V2 lookahead)")
  
  # Assert V2 trailing: m_popocup[202001-202003] NA
  trailing_rows <- result[anomesexato %in% c(202001L, 202002L, 202003L)]
  expect_true(all(is.na(trailing_rows$m_popocup)),
    label = "T3-V2-trail: m_popocup[202001-202003] all NA (V2 trailing)")
  
  # Assert F1: if deflated series exist, m_popocupipcabr[201201] non-NA
  if ("m_popocupipcabr" %in% names(result)) {
    expect_false(is.na(row_201201$m_popocupipcabr),
      label = "T3-F1: m_popocupipcabr[201201] is non-NA (F1 lag from pre-PNADC)")
  }
})

# Tests for starting points functions and bundled dataset
# Covers: pnadc_series_starting_points (bundled data),
#          compute_z_aggregates(), compute_series_starting_points()

# =============================================================================
# BUNDLED DATASET TESTS (pnadc_series_starting_points)
# =============================================================================

test_that("bundled dataset loads successfully via data()", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  expect_s3_class(pnadc_series_starting_points, "data.table")
  expect_true(nrow(pnadc_series_starting_points) > 0,
              label = "Dataset has rows")
})

test_that("bundled dataset has exactly 3 columns: series_name, mesnotrim, y0", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  expect_equal(ncol(pnadc_series_starting_points), 3)
  expect_named(pnadc_series_starting_points,
               c("series_name", "mesnotrim", "y0"),
               ignore.order = FALSE)
})

test_that("mesnotrim only contains values 1, 2, 3", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  expect_true(all(pnadc_series_starting_points$mesnotrim %in% c(1L, 2L, 3L)),
              label = "All mesnotrim values are 1, 2, or 3")
  # Confirm all three positions are present

  expect_setequal(unique(pnadc_series_starting_points$mesnotrim), c(1L, 2L, 3L))
})

test_that("no NA in series_name or mesnotrim", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  expect_false(any(is.na(pnadc_series_starting_points$series_name)),
               label = "No NA in series_name")
  expect_false(any(is.na(pnadc_series_starting_points$mesnotrim)),
               label = "No NA in mesnotrim")
})

test_that("all y0 values are finite (no NA, NaN, Inf)", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  expect_true(all(is.finite(pnadc_series_starting_points$y0)),
              label = "All y0 values are finite")
})

test_that("key labor market series are present in bundled dataset", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  available <- unique(pnadc_series_starting_points$series_name)

  key_series <- c("popocup", "popdesocup", "popnaforca", "pop14mais")
  for (s in key_series) {
    expect_true(s %in% available,
                label = paste0("Series '", s, "' is present"))
  }
})

test_that("each series has exactly 3 rows (one per mesnotrim)", {
  data("pnadc_series_starting_points", package = "PNADCperiods")
  counts <- pnadc_series_starting_points[, .N, by = series_name]

  expect_true(all(counts$N == 3L),
              label = "Every series has exactly 3 rows")

  # Verify each series has all three positions
  for (s in unique(pnadc_series_starting_points$series_name)) {
    positions <- pnadc_series_starting_points[series_name == s, mesnotrim]
    expect_setequal(positions, c(1L, 2L, 3L))
  }
})

# =============================================================================
# compute_z_aggregates() TESTS
# =============================================================================

# Helper: create mock calibrated microdata for compute_z_aggregates()
create_mock_calibrated <- function(n_per_month = 10L,
                                    months = c(202301L, 202302L, 202303L),
                                    seed = 100L) {
  set.seed(seed)
  n <- n_per_month * length(months)

  dt <- data.table::data.table(
    weight_monthly = rep(1000, n),
    ref_month_yyyymm = rep(months, each = n_per_month),
    ref_month_in_quarter = rep(1:3, each = n_per_month),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = sample(c(5L, 20L, 35L, 50L, 70L), n, replace = TRUE),
    VD4001 = sample(c(1L, 2L), n, replace = TRUE),
    VD4002 = sample(c(1L, 2L, NA_integer_), n, replace = TRUE, prob = c(0.45, 0.15, 0.40)),
    VD4009 = sample(c(1L:10L, NA_integer_), n, replace = TRUE),
    VD4010 = sample(c(1L:11L, NA_integer_), n, replace = TRUE),
    V4019 = sample(c(1L, 2L, NA_integer_), n, replace = TRUE),
    VD4004 = sample(c(1L, 2L, NA_integer_), n, replace = TRUE),
    VD4004A = sample(c(1L, 2L, NA_integer_), n, replace = TRUE),
    VD4019 = round(runif(n, 1000, 5000), 2),
    VD4020 = round(runif(n, 900, 4500), 2),
    VD4016 = round(runif(n, 500, 3000), 2),
    VD4017 = round(runif(n, 500, 3000), 2),
    VD4031 = round(runif(n, 20, 44), 0),
    VD4035 = round(runif(n, 20, 50), 0)
  )
  dt
}


test_that("compute_z_aggregates errors when weight_monthly is missing", {
  dt <- data.table::data.table(
    ref_month_yyyymm = c(202301L, 202302L),
    ref_month_in_quarter = c(1L, 2L),
    V2009 = c(30L, 40L)
  )
  expect_error(
    compute_z_aggregates(dt, verbose = FALSE),
    "weight_monthly"
  )
})

test_that("compute_z_aggregates computes correct z_popocup and z_popdesocup", {
  # Construct a deterministic example:
  # 4 obs in one month, weights all 1000
  # VD4002: 1, 1, 2, 2  -> 2 occupied, 2 unemployed
  dt <- data.table::data.table(
    weight_monthly = rep(1000, 4),
    ref_month_yyyymm = rep(202301L, 4),
    ref_month_in_quarter = rep(1L, 4),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = c(30L, 35L, 25L, 40L),
    VD4001 = c(1L, 1L, 1L, 1L),
    VD4002 = c(1L, 1L, 2L, 2L)
  )

  result <- compute_z_aggregates(dt, verbose = FALSE)

  # z_popocup = sum(indicator * weight): 2 obs with VD4002==1, weight=1000 each
  expect_equal(result$z_popocup, 2 * 1000)
  # z_popdesocup = sum(indicator * weight): 2 obs with VD4002==2
  expect_equal(result$z_popdesocup, 2 * 1000)
})

test_that("z_populacao equals 1 for every observation (total pop = sum of weights)", {
  # 6 obs across 2 months, various weights
  dt <- data.table::data.table(
    weight_monthly = c(500, 800, 1200, 600, 900, 1100),
    ref_month_yyyymm = c(rep(202301L, 3), rep(202302L, 3)),
    ref_month_in_quarter = c(rep(1L, 3), rep(2L, 3)),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = c(30L, 35L, 25L, 40L, 50L, 20L)
  )

  result <- compute_z_aggregates(dt, verbose = FALSE)
  data.table::setorder(result, anomesexato)

  # z_populacao should be sum of weights per month
  # Month 1: 500 + 800 + 1200 = 2500
  # Month 2: 600 + 900 + 1100 = 2600
  expect_equal(result[anomesexato == 202301L, z_populacao], 2500)
  expect_equal(result[anomesexato == 202302L, z_populacao], 2600)
})

test_that("compute_z_aggregates groups correctly by month across multiple months", {
  dt <- create_mock_calibrated(n_per_month = 20L)
  result <- compute_z_aggregates(dt, verbose = FALSE)

  # Should have one row per month
  expect_equal(nrow(result), 3)
  expect_true(all(c(202301L, 202302L, 202303L) %in% result$anomesexato))

  # All z_ columns should be present and numeric
  z_cols <- grep("^z_", names(result), value = TRUE)
  expect_true(length(z_cols) > 0, label = "z_ columns exist")
  for (col in z_cols) {
    expect_true(is.numeric(result[[col]]),
                label = paste0(col, " is numeric"))
  }
})

test_that("compute_z_aggregates converts character VD variables to integer", {
  # VD variables stored as character in some PNADC files ("01", "02")
  dt <- data.table::data.table(
    weight_monthly = rep(1000, 4),
    ref_month_yyyymm = rep(202301L, 4),
    ref_month_in_quarter = rep(1L, 4),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = c(30L, 35L, 25L, 40L),
    VD4001 = c("01", "01", "02", "02"),
    VD4002 = c("01", "01", "02", "02")
  )

  # Should not error, and should produce correct results
  result <- compute_z_aggregates(dt, verbose = FALSE)
  expect_equal(result$z_popocup, 2 * 1000)
  expect_equal(result$z_popdesocup, 2 * 1000)
})

# =============================================================================
# compute_series_starting_points() TESTS
# =============================================================================

# Helper: create paired mock data for compute_series_starting_points()
# Returns a list with $monthly_estimates and $rolling_quarters
create_mock_for_starting_points <- function(n_months = 36L,
                                             start_yyyymm = 201301L,
                                             base_rq = 100,
                                             scale_factor = 1000,
                                             seed = 200L) {
  set.seed(seed)

  # Generate YYYYMM sequence
  months_seq <- generate_yyyymm_seq(start_yyyymm, n_months)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  # Rolling quarters: base value with slight trend
  rq_values <- base_rq * (1 + 0.001 * seq_len(n_months))

  rolling_quarters <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rq_values,
    popdesocup = rq_values * 0.1,
    pop14mais = rq_values * 1.2
  )

  # Monthly estimates: z_ columns = scale_factor * rq + noise
  # (so that z/scale_factor is close to rq, giving small y0)
  monthly_estimates <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = rq_values * scale_factor + rnorm(n_months, 0, 50),
    z_popdesocup = rq_values * 0.1 * scale_factor + rnorm(n_months, 0, 5),
    z_pop14mais = rq_values * 1.2 * scale_factor + rnorm(n_months, 0, 60)
  )

  list(
    monthly_estimates = monthly_estimates,
    rolling_quarters = rolling_quarters
  )
}


test_that("compute_series_starting_points returns correct columns", {
  mock <- create_mock_for_starting_points()
  result <- compute_series_starting_points(
    mock$monthly_estimates,
    mock$rolling_quarters,
    calibration_start = 201301L,
    calibration_end = 201912L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  expect_s3_class(result, "data.table")
  expect_named(result, c("series_name", "mesnotrim", "y0"), ignore.order = FALSE)
})

test_that("compute_series_starting_points returns 3 rows per series", {
  mock <- create_mock_for_starting_points()
  result <- compute_series_starting_points(
    mock$monthly_estimates,
    mock$rolling_quarters,
    calibration_start = 201301L,
    calibration_end = 201912L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  counts <- result[, .N, by = series_name]
  expect_true(all(counts$N == 3L),
              label = "Each series has exactly 3 rows")

  # Each series should have all 3 mesnotrim values
  for (s in unique(result$series_name)) {
    positions <- result[series_name == s, mesnotrim]
    expect_setequal(positions, c(1L, 2L, 3L))
  }
})

test_that("custom calibration_start/end are respected", {
  # Create data spanning a wide range
  mock <- create_mock_for_starting_points(n_months = 120L, start_yyyymm = 201201L)

  # Use narrow calibration period
  result_narrow <- compute_series_starting_points(
    mock$monthly_estimates,
    mock$rolling_quarters,
    calibration_start = 201601L,
    calibration_end = 201712L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # Use wider calibration period
  result_wide <- compute_series_starting_points(
    mock$monthly_estimates,
    mock$rolling_quarters,
    calibration_start = 201301L,
    calibration_end = 201912L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # Both should produce valid results
  expect_true(all(is.finite(result_narrow$y0)))
  expect_true(all(is.finite(result_wide$y0)))

  # y0 values should differ because calibration periods differ
  merged <- merge(result_narrow, result_wide,
                  by = c("series_name", "mesnotrim"),
                  suffixes = c("_narrow", "_wide"))
  # Not all y0 pairs should be identical (noise + different averaging window)
  expect_false(all(merged$y0_narrow == merged$y0_wide),
               label = "Different calibration periods produce different y0")
})

test_that("use_series_specific_periods=FALSE uses uniform calibration for all series", {
  # Create data with a "CNPJ-like" series name to verify it does NOT get special treatment
  months_seq <- generate_yyyymm_seq(201201L, 120)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  rq_vals <- 100 * (1 + 0.001 * seq_along(months_seq))

  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rq_vals,
    empregadorcomcnpj = rq_vals * 0.05
  )

  me <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = rq_vals * 1000 + rnorm(120, 0, 10),
    z_empregadorcomcnpj = rq_vals * 0.05 * 1000 + rnorm(120, 0, 1)
  )

  result <- compute_series_starting_points(
    me, rq,
    calibration_start = 201301L,
    calibration_end = 201912L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # Both series should be present with 3 rows each
  expect_true("popocup" %in% result$series_name)
  expect_true("empregadorcomcnpj" %in% result$series_name)
  expect_equal(nrow(result[series_name == "empregadorcomcnpj"]), 3)
})

test_that("scale_factor=1 produces correct y0 (no scaling applied)", {
  mock <- create_mock_for_starting_points(scale_factor = 1)

  # Rebuild monthly estimates with scale_factor=1 (z_ values close to rq)
  months_seq <- mock$rolling_quarters$anomesfinaltrimmovel
  rq_popocup <- mock$rolling_quarters$popocup

  me <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = rq_popocup + rnorm(length(months_seq), 0, 0.1)
  )

  result <- compute_series_starting_points(
    me,
    mock$rolling_quarters,
    calibration_start = 201301L,
    calibration_end = 201912L,
    scale_factor = 1,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # With scale_factor=1 and z ~ rq, y0 should be close to the base rq value
  # (y0 is the starting level, not zero; ~100 for base_rq=100)
  expect_true(all(is.finite(result$y0)))
  # The three y0 values per mesnotrim should be consistent with each other
  y0_vals <- result[series_name == "popocup", y0]
  expect_true(sd(y0_vals) < 5,
              label = "y0 values consistent across mesnotrim positions")
})

test_that("mathematical property: backprojection e0 averaged gives y0", {
  # The defining formula:
  #   e0 = z / scale_factor - cumsum_by_mesnotrim(rq)
  #   y0[pos] = mean(e0[mesnotrim == pos & in_calibration])
  #
  # We verify this by constructing data where we can compute e0 manually.

  set.seed(42)
  n <- 12L  # 12 months = 4 quarters
  months_seq <- generate_yyyymm_seq(201301L, n)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  rq_vals <- rep(100, n)  # Constant rolling quarter
  z_vals <- rep(100000, n)  # Constant z values (scale 1000)
  scale_factor <- 1000

  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rq_vals
  )
  me <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = z_vals
  )

  result <- compute_series_starting_points(
    me, rq,
    calibration_start = 201301L,
    calibration_end = 201312L,
    scale_factor = scale_factor,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # With constant rq, d3 = 3*(rq - lag(rq)) = 0, so cumsum = 0 for all periods
  # e0 = z/scale_factor - cum = 100000/1000 - 0 = 100
  # y0 should be 100 for all positions
  expect_equal(result[series_name == "popocup", y0],
               rep(100, 3),
               tolerance = 0.001)
})

test_that("compute_series_starting_points errors when no z_ columns present", {
  months_seq <- generate_yyyymm_seq(201301L, 12)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rep(100, 12)
  )
  # monthly_estimates with NO z_ columns
  me <- data.table::data.table(
    anomesexato = months_seq,
    popocup = rep(100000, 12)
  )

  expect_error(
    compute_series_starting_points(me, rq, verbose = FALSE),
    "No z_ columns"
  )
})

test_that("compute_series_starting_points skips series not in rolling_quarters", {
  months_seq <- generate_yyyymm_seq(201301L, 12)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rep(100, 12)
    # NOTE: no "popdesocup" column here
  )
  me <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = rep(100000, 12),
    z_popdesocup = rep(10000, 12)  # z_ column exists but series not in rq
  )

  result <- compute_series_starting_points(
    me, rq,
    calibration_start = 201301L,
    calibration_end = 201312L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # Only popocup should appear (popdesocup skipped because not in rq)
  expect_true("popocup" %in% result$series_name)
  expect_false("popdesocup" %in% result$series_name)
})

test_that("compute_series_starting_points handles anomesexato column name in monthly_estimates", {
  # The function renames "anomesexato" to "anomesfinaltrimmovel" internally
  months_seq <- generate_yyyymm_seq(201301L, 12)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rep(100, 12)
  )
  me <- data.table::data.table(
    anomesexato = months_seq,  # Using anomesexato, not anomesfinaltrimmovel
    z_popocup = rep(100000, 12)
  )

  # Should not error
  result <- compute_series_starting_points(
    me, rq,
    calibration_start = 201301L,
    calibration_end = 201312L,
    scale_factor = 1000,
    verbose = FALSE
  )
  expect_s3_class(result, "data.table")
  expect_true(nrow(result) > 0)
})

test_that("non-finite y0 values are replaced with 0", {
  # If all z_ values are NA for a calibration period, e0 would be NaN
  # The function should replace non-finite y0 with 0
  months_seq <- generate_yyyymm_seq(201301L, 12)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L

  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rep(100, 12)
  )

  # z_ values are all NA
  me <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = rep(NA_real_, 12)
  )

  result <- compute_series_starting_points(
    me, rq,
    calibration_start = 201301L,
    calibration_end = 201312L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  # All y0 should be 0 (non-finite replaced)
  expect_true(all(result$y0 == 0),
              label = "Non-finite y0 replaced with 0")
})

# =============================================================================
# INTEGRATION: compute_z_aggregates() -> compute_series_starting_points()
# =============================================================================

test_that("z_aggregates output is compatible with compute_series_starting_points input", {
  # Build calibrated mock data -> compute z_aggregates -> feed into starting points
  dt <- create_mock_calibrated(n_per_month = 30L,
                                months = generate_yyyymm_seq(201301L, 12))
  z_agg <- compute_z_aggregates(dt, verbose = FALSE)

  # Verify z_aggregates has anomesexato column

  expect_true("anomesexato" %in% names(z_agg))

  # Create matching rolling quarter data with same series
  z_cols <- grep("^z_", names(z_agg), value = TRUE)
  series_names <- sub("^z_", "", z_cols)

  rq <- data.table::data.table(
    anomesfinaltrimmovel = z_agg$anomesexato,
    mesnotrim = ((z_agg$anomesexato %% 100L - 1L) %% 3L) + 1L
  )
  # Add matching series columns with plausible rolling quarter values
  for (s in series_names) {
    z_col <- paste0("z_", s)
    if (z_col %in% names(z_agg)) {
      rq[, (s) := z_agg[[z_col]] / 1000]
    }
  }

  # Should not error
  result <- compute_series_starting_points(
    z_agg, rq,
    calibration_start = 201301L,
    calibration_end = 201312L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )

  expect_s3_class(result, "data.table")
  expect_true(nrow(result) > 0, label = "Starting points computed from z_aggregates")
  expect_true(all(is.finite(result$y0)),
              label = "All y0 from pipeline are finite")
})

test_that("compute_z_aggregates filters out zero and NA weights", {
  dt <- data.table::data.table(
    weight_monthly = c(1000, 0, NA_real_, 1000),
    ref_month_yyyymm = rep(202301L, 4),
    ref_month_in_quarter = rep(1L, 4),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = c(30L, 35L, 25L, 40L)
  )

  result <- compute_z_aggregates(dt, verbose = FALSE)

  # z_populacao should only count obs with valid weight (2 obs with weight=1000)
  expect_equal(result$z_populacao, 2 * 1000)
})

test_that("compute_z_aggregates errors when all weights are zero/NA", {
  dt <- data.table::data.table(
    weight_monthly = c(0, NA_real_, 0),
    ref_month_yyyymm = rep(202301L, 3),
    ref_month_in_quarter = rep(1L, 3),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = c(30L, 35L, 25L)
  )

  expect_error(
    compute_z_aggregates(dt, verbose = FALSE),
    "No observations with valid weight_monthly"
  )
})

test_that("compute_z_aggregates produces z_pop14mais only for age >= 14", {
  dt <- data.table::data.table(
    weight_monthly = rep(1000, 6),
    ref_month_yyyymm = rep(202301L, 6),
    ref_month_in_quarter = rep(1L, 6),
    Ano = 2023L,
    Trimestre = 1L,
    V2009 = c(5L, 10L, 13L, 14L, 30L, 65L)  # 3 under 14, 3 at/above 14
  )

  result <- compute_z_aggregates(dt, verbose = FALSE)

  # z_pop14mais: only ages 14, 30, 65 -> 3 obs * 1000 weight
  expect_equal(result$z_pop14mais, 3 * 1000)
  # z_populacao: all 6 obs
  expect_equal(result$z_populacao, 6 * 1000)
})

# =============================================================================
# INPC LAG PRE-COMPUTE TEST
# =============================================================================

test_that("compute_series_starting_points pre-computes lagged INPC", {
  skip("requires proper INPC data sequence with pre-PNADC values")
  
  # This test validates that .inpc100dez1993_lagged is computed BEFORE
  # any deflation operations, ensuring that z_massaefetrealtodos can use
  # the lagged INPC for the first PNADC month (201201 uses INPC[201112]).
  #
  # Mock scenario:
  # - rolling_quarters: 201112, 201201 (at least)
  # - inpc100dez1993: values for both months
  # - z_massaefetnominaltodos: values for both months
  #
  # Expected: z_massaefetrealtodos[201201] uses INPC[201112] as lag
  # (not NA from post-filter shift)
})

test_that("compute_series_starting_points produces non-NA y0 with lagged INPC", {
  # Lightweight version: validate that the pre-compute doesn't break existing tests
  # and that y0 values remain finite when inpc columns are present
  
  months_seq <- generate_yyyymm_seq(201301L, 12)
  mesnotrim_seq <- ((months_seq %% 100L - 1L) %% 3L) + 1L
  
  rq <- data.table::data.table(
    anomesfinaltrimmovel = months_seq,
    mesnotrim = mesnotrim_seq,
    popocup = rep(100, 12),
    inpc100dez1993 = 100 + seq_len(12)  # Simple increasing sequence
  )
  
  me <- data.table::data.table(
    anomesexato = months_seq,
    z_popocup = rep(100000, 12)  # Nominal values
  )
  
  result <- compute_series_starting_points(
    me, rq,
    calibration_start = 201301L,
    calibration_end = 201312L,
    scale_factor = 1000,
    use_series_specific_periods = FALSE,
    verbose = FALSE
  )
  
  # Verify result is valid (covers the pre-compute block)
  expect_s3_class(result, "data.table")
  expect_true(nrow(result) > 0)
  expect_true(all(is.finite(result$y0)))
})

# Tests for mensalize-sidra-series.R
# Covers the exported mensalize_sidra_series() function and its internal helpers:
#   .compute_cumsum_by_mesnotrim(), .extract_y0_vector(),
#   .apply_final_adjustment(), .mensalize_single_series(), .mensalize_split_series()
#
# All tests use synthetic mock data (no API calls).

# =============================================================================
# HELPER: generate proper YYYYMM sequence and mesnotrim
# =============================================================================

make_rolling_quarter_dt <- function(start_yyyymm = 201201L, n_months = 36L,
                                    series_name = "popocup",
                                    base_value = 100000, trend = 100) {

  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L

  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]

  # Deterministic series: base + linear trend + small seasonal component
  dt[, (series_name) := base_value + trend * (month_num - 1) +
       500 * sin(2 * pi * month_num / 12)]

  dt[, .SD, .SDcols = c("anomesfinaltrimmovel", "mesnotrim", series_name)]
}


# =============================================================================
# .compute_cumsum_by_mesnotrim() TESTS
# =============================================================================

test_that(".compute_cumsum_by_mesnotrim computes d3 correctly for simple case", {
  # 1. Setup: 6 months of constant rolling quarters (no change expected)
  rq <- c(300, 300, 300, 300, 300, 300)
  mesnotrim <- c(1L, 2L, 3L, 1L, 2L, 3L)

  # 2. Execute
  cum <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim)

  # 3. Verify: d3 = 3*(RQ_t - RQ_{t-1}) = 0 for all interior,
  #    first of each mesnotrim position is set to NA (d3=0 for cumsum)

  #    All cumsums should be 0 since no variation exists
  expect_equal(cum[4], 0, label = "Cumsum for second pos1 should be 0 (no variation)")
  expect_equal(cum[5], 0, label = "Cumsum for second pos2 should be 0 (no variation)")
  expect_equal(cum[6], 0, label = "Cumsum for second pos3 should be 0 (no variation)")
})


test_that(".compute_cumsum_by_mesnotrim separates 3 mesnotrim positions", {
  # 1. Setup: Different step sizes at each month position
  #    RQ sequence: 100, 200, 300, 110, 210, 310, 120, 220, 320
  rq <- c(100, 200, 300, 110, 210, 310, 120, 220, 320)
  mesnotrim <- c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L)

  # 2. Execute
  cum <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim)

  # 3. Verify: Each position accumulates independently
  #    For pos1: first d3 = NA (forced 0), d3[4] = 3*(110-300) = -570, d3[7] = 3*(120-310) = -570
  #    For pos2: first d3 = NA (forced 0), d3[5] = 3*(210-110) = 300, d3[8] = 3*(220-120) = 300
  #    For pos3: first d3 = NA (forced 0), d3[6] = 3*(310-210) = 300, d3[9] = 3*(320-220) = 300
  # Pos1 cumsums:
  expect_equal(cum[1], 0, label = "First pos1 cumsum is 0")
  # Pos2 cumsums:
  expect_equal(cum[2], 0, label = "First pos2 cumsum is 0")
  # Pos3 cumsums:
  expect_equal(cum[3], 0, label = "First pos3 cumsum is 0")

  # The second occurrence of each position should have the correct d3 accumulated
  # pos1: cum[4] = d3 at index 4 = 3*(110 - 300) = -570
  expect_equal(cum[4], -570, label = "Second pos1 d3 accumulated correctly")
  # pos2: cum[5] = d3 at index 5 = 3*(210 - 110) = 300
  expect_equal(cum[5], 300, label = "Second pos2 d3 accumulated correctly")
  # pos3: cum[6] = d3 at index 6 = 3*(310 - 210) = 300
  expect_equal(cum[6], 300, label = "Second pos3 d3 accumulated correctly")
})


test_that(".compute_cumsum_by_mesnotrim first obs of each position starts at 0", {
  # 1. Setup: 9 months with non-trivial values
  rq <- c(1000, 2000, 3000, 1100, 2100, 3100, 1200, 2200, 3200)
  mesnotrim <- c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L)

  # 2. Execute
  cum <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim)

  # 3. Verify: First occurrence of each position = 0
  expect_equal(cum[1], 0, label = "First pos1 starts at 0")
  expect_equal(cum[2], 0, label = "First pos2 starts at 0")
  expect_equal(cum[3], 0, label = "First pos3 starts at 0")
})


test_that(".compute_cumsum_by_mesnotrim respects filter_mask", {
  # 1. Setup: 9 months but mask out the middle 3
  rq <- c(100, 200, 300, 110, 210, 310, 120, 220, 320)
  mesnotrim <- c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L)
  mask <- c(TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, TRUE, TRUE, TRUE)

  # 2. Execute
  cum <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim, filter_mask = mask)

  # 3. Verify: d3 at masked positions (4,5,6) treated as 0
  #    pos1: d3[1]=0, d3[4]=0(masked), d3[7] = 3*(120-310) = -570
  #    So cum[7] = -570 (only one unmasked step)
  #    Compare to unmasked: cum[7] would be -570 + (-570) = -1140
  cum_unmasked <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim)
  expect_true(cum[7] != cum_unmasked[7],
              label = "Filtered cumsum differs from unfiltered")
})


test_that(".compute_cumsum_by_mesnotrim handles all-NA input", {
  # 1. Setup: All NA rolling quarter values
  rq <- rep(NA_real_, 9)
  mesnotrim <- c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L)

  # 2. Execute
  cum <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim)

  # 3. Verify: All cumsums should be NA (no data to cumulate)
  expect_true(all(is.na(cum)),
              label = "All-NA input produces all-NA cumsum")
})


# =============================================================================
# .extract_y0_vector() TESTS
# =============================================================================

test_that(".extract_y0_vector returns 3 values for found series", {
  # 1. Setup: Starting points table with 3 positions for one series
  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(90000, 91000, 92000)
  )

  # 2. Execute
  y0 <- PNADCperiods:::.extract_y0_vector(sp, "popocup")

  # 3. Verify: 3-element vector with correct values

  expect_length(y0, 3)
  expect_equal(y0[1], 90000)
  expect_equal(y0[2], 91000)
  expect_equal(y0[3], 92000)
})


test_that(".extract_y0_vector returns 3 NAs for missing series", {
  # 1. Setup: Starting points table without the target series
  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(90000, 91000, 92000)
  )

  # 2. Execute
  y0 <- PNADCperiods:::.extract_y0_vector(sp, "popdesocup")

  # 3. Verify: 3 NAs when series not found
  expect_length(y0, 3)
  expect_true(all(is.na(y0)),
              label = "Missing series returns c(NA, NA, NA)")
})


test_that(".extract_y0_vector handles partial starting points", {
  # 1. Setup: Starting points with only 2 of 3 positions
  sp <- data.table::data.table(
    series_name = c("popocup", "popocup"),
    mesnotrim = c(1L, 3L),
    y0 = c(90000, 92000)
  )

  # 2. Execute
  y0 <- PNADCperiods:::.extract_y0_vector(sp, "popocup")

  # 3. Verify: Position 2 is NA, others filled
  expect_length(y0, 3)
  expect_equal(y0[1], 90000)
  expect_true(is.na(y0[2]), label = "Missing position 2 is NA")
  expect_equal(y0[3], 92000)
})


# =============================================================================
# .apply_final_adjustment() TESTS
# =============================================================================

test_that(".apply_final_adjustment preserves rolling quarter consistency", {
  # 1. Setup: 12 months where mean(m[t], m[t+1], m[t+2]) should == rq[t+2]
  #    Use known y values and rq values
  n <- 12
  mesnotrim <- rep(1:3, 4)
  # Create y values that differ from rq (final adjustment corrects them)
  y <- c(100, 200, 300, 110, 210, 310, 120, 220, 320, 130, 230, 330)
  # rq values: rolling quarter = average of 3 consecutive months
  # We want rq[3] = mean(m[1],m[2],m[3]), rq[4] = mean(m[2],m[3],m[4]), etc.
  # For testing, set rq = average of y with small shift
  rq <- c(NA, NA, mean(y[1:3]) + 5,
          mean(y[2:4]) + 3, mean(y[3:5]) + 2, mean(y[4:6]) + 1,
          mean(y[5:7]) + 4, mean(y[6:8]) + 2, mean(y[7:9]) + 3,
          mean(y[8:10]) + 1, mean(y[9:11]) + 5, mean(y[10:12]) + 2)

  # 2. Execute
  m <- PNADCperiods:::.apply_final_adjustment(y, rq, mesnotrim)

  # 3. Verify: For mesnotrim==3 positions where all neighbors are available,
  #    mean(m[i-2], m[i-1], m[i]) should equal rq[i]
  for (i in seq(3, 12, by = 3)) {
    if (i >= 3 && !is.na(rq[i])) {
      trio_mean <- mean(m[(i - 2):i])
      expect_equal(trio_mean, rq[i], tolerance = 1e-10,
                   label = paste0("Rolling quarter consistency at position ", i))
    }
  }
})


test_that(".apply_final_adjustment falls back to y at boundaries", {
  # 1. Setup: Short sequence where lead/lag values are missing
  y <- c(100, 200, 300)
  rq <- c(NA, NA, 200)
  mesnotrim <- c(1L, 2L, 3L)

  # 2. Execute
  m <- PNADCperiods:::.apply_final_adjustment(y, rq, mesnotrim)

  # 3. Verify: mesnotrim==1 cannot look ahead 2 positions beyond length
  #    (y_lead1 and y_lead2 will be NA for last elements),
  #    mesnotrim==1 at position 1: needs y_lead1[1], y_lead2[1] which exist
  #    mesnotrim==3 at position 3: needs y_lag1 and y_lag2 which exist
  expect_length(m, 3)
  # For this 3-element case, all trios are complete
  # pos1 (i=1): valid1 = needs y[1], y_lead1=y[2], y_lead2=y[3], rq_lead2=rq[3]
  # All available, so adjustment is applied
  expect_false(is.na(m[1]), label = "Position 1 gets adjusted value (complete trio)")
  expect_false(is.na(m[3]), label = "Position 3 gets adjusted value (complete trio)")
})


test_that(".apply_final_adjustment works with 12-month sequence", {
  # 1. Setup: 12 months with linearly increasing rq
  mesnotrim <- rep(1:3, 4)
  rq <- seq(1000, 1110, by = 10)  # 1000, 1010, ..., 1110
  # Set y to be rq itself (adjustment should be minimal when y is already close)
  y <- rq

  # 2. Execute
  m <- PNADCperiods:::.apply_final_adjustment(y, rq, mesnotrim)

  # 3. Verify: For complete trios at mesnotrim==3 positions,
  #    the rolling quarter relationship holds
  # At i=3: rq[3] should == mean(m[1], m[2], m[3])
  expect_equal(mean(m[1:3]), rq[3], tolerance = 1e-10,
               label = "First complete trio matches rq")
  # At i=6: rq[6] should == mean(m[4], m[5], m[6])
  expect_equal(mean(m[4:6]), rq[6], tolerance = 1e-10,
               label = "Second complete trio matches rq")
  # At i=9: rq[9] should == mean(m[7], m[8], m[9])
  expect_equal(mean(m[7:9]), rq[9], tolerance = 1e-10,
               label = "Third complete trio matches rq")
})


# =============================================================================
# .mensalize_single_series() TESTS
# =============================================================================

test_that(".mensalize_single_series produces standard output", {
  # 1. Setup: Build a data.table with 36 months of rolling quarter data
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 36L,
    series_name = "popocup", base_value = 90000, trend = 100
  )

  # Create starting points for this series
  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(89500, 89800, 90100)
  )

  # 2. Execute
  m_values <- PNADCperiods:::.mensalize_single_series(dt, "popocup", sp)

  # 3. Verify: Output is numeric vector with same length as input
  expect_length(m_values, nrow(dt))
  expect_type(m_values, "double")
  # Non-NA values should exist for most of the series (at least after first few)
  # With 36 months and valid starting points, all values should be non-NA
  # (cumsum produces 0 at first positions, y0 is non-NA, so y is non-NA everywhere)
  expect_true(sum(!is.na(m_values)) > nrow(dt) * 0.5,
              label = "Most values should be non-NA")
})


test_that(".mensalize_single_series returns all NA when series not in starting_points", {
  # 1. Setup: Data has series "popocup" but starting points are for a different series
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 12L,
    series_name = "popocup", base_value = 90000
  )

  sp <- data.table::data.table(
    series_name = rep("popdesocup", 3),
    mesnotrim = 1:3,
    y0 = c(10000, 10500, 11000)
  )

  # 2. Execute
  m_values <- PNADCperiods:::.mensalize_single_series(dt, "popocup", sp)

  # 3. Verify: y0 is c(NA,NA,NA), so y = NA + cum => all NA
  expect_true(all(is.na(m_values)),
              label = "Missing starting points produce all-NA output")
})


# =============================================================================
# .mensalize_split_series() TESTS
# =============================================================================

test_that(".mensalize_split_series handles spanning split correctly", {
  # 1. Setup: Series spanning from 201201 to 201612 (5 years, crosses 201509 split)
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 60L,
    series_name = "subocuphoras", base_value = 5000, trend = 10
  )

  # Starting points for pre-split and post-split
  sp <- data.table::data.table(
    series_name = c(rep("subocuphoras_pre", 3), rep("subocuphoras", 3)),
    mesnotrim = c(1:3, 1:3),
    y0 = c(4800, 4900, 5000, 5400, 5500, 5600)
  )

  # 2. Execute
  m_values <- PNADCperiods:::.mensalize_split_series(
    dt, "subocuphoras", sp, split_month = 201509L
  )

  # 3. Verify: Output covers full range
  expect_length(m_values, nrow(dt))
  expect_type(m_values, "double")

  # Pre-split values (up to and including 201509) should be mostly non-NA
  # With valid starting points, all values in each segment should be non-NA
  pre_idx <- which(dt$anomesfinaltrimmovel <= 201509L)
  expect_true(sum(!is.na(m_values[pre_idx])) > length(pre_idx) * 0.5,
              label = "Pre-split values mostly non-NA")

  # Post-split values (after 201509) should be mostly non-NA
  post_idx <- which(dt$anomesfinaltrimmovel > 201509L)
  expect_true(sum(!is.na(m_values[post_idx])) > length(post_idx) * 0.5,
              label = "Post-split values mostly non-NA")
})


test_that(".mensalize_split_series handles data entirely before split", {
  # 1. Setup: Series ending before the split point
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 36L,  # ends Dec 2014, before split
    series_name = "subocuphoras", base_value = 5000, trend = 10
  )

  sp <- data.table::data.table(
    series_name = c(rep("subocuphoras_pre", 3), rep("subocuphoras", 3)),
    mesnotrim = c(1:3, 1:3),
    y0 = c(4800, 4900, 5000, 5400, 5500, 5600)
  )

  # 2. Execute
  m_values <- PNADCperiods:::.mensalize_split_series(
    dt, "subocuphoras", sp, split_month = 201509L
  )

  # 3. Verify: Only pre-split logic is used
  expect_length(m_values, nrow(dt))
  # Should produce values using pre-split starting points
  expect_true(sum(!is.na(m_values)) > nrow(dt) * 0.5,
              label = "Before-split data fully processed")
})


test_that(".mensalize_split_series handles data entirely after split", {
  # 1. Setup: Series starting after the split point
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201601L, n_months = 24L,
    series_name = "subocuphoras", base_value = 5500, trend = 10
  )

  sp <- data.table::data.table(
    series_name = c(rep("subocuphoras_pre", 3), rep("subocuphoras", 3)),
    mesnotrim = c(1:3, 1:3),
    y0 = c(4800, 4900, 5000, 5400, 5500, 5600)
  )

  # 2. Execute
  m_values <- PNADCperiods:::.mensalize_split_series(
    dt, "subocuphoras", sp, split_month = 201509L
  )

  # 3. Verify: Only post-split logic is used
  expect_length(m_values, nrow(dt))
  expect_true(sum(!is.na(m_values)) > nrow(dt) * 0.5,
              label = "After-split data fully processed")
})


test_that(".mensalize_split_series resets cumsum at split boundary", {
  # 1. Setup: Data spanning the split with a jump in values
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201501L, n_months = 24L,  # Jan 2015 - Dec 2016
    series_name = "subocuphoras", base_value = 5000, trend = 10
  )
  # Introduce a level shift after the split to test independence
  post_rows <- dt$anomesfinaltrimmovel > 201509L
  data.table::set(dt, i = which(post_rows), j = "subocuphoras",
                  value = dt[["subocuphoras"]][post_rows] + 2000)

  sp <- data.table::data.table(
    series_name = c(rep("subocuphoras_pre", 3), rep("subocuphoras", 3)),
    mesnotrim = c(1:3, 1:3),
    y0 = c(4800, 4900, 5000, 7400, 7500, 7600)
  )

  # 2. Execute
  m_values <- PNADCperiods:::.mensalize_split_series(
    dt, "subocuphoras", sp, split_month = 201509L
  )

  # 3. Verify: Post-split values should reflect the level shift
  #    Pre-split values should be around 5000 level
  #    Post-split values should be around 7000+ level
  pre_mean <- mean(m_values[!post_rows], na.rm = TRUE)
  post_mean <- mean(m_values[post_rows], na.rm = TRUE)
  expect_true(post_mean > pre_mean + 1000,
              label = "Post-split reflects level shift (cumsums reset)")
})


# =============================================================================
# mensalize_sidra_series() EXPORTED FUNCTION TESTS
# =============================================================================

test_that("mensalize_sidra_series errors on missing anomesfinaltrimmovel", {
  # 1. Setup: data.table without the required column
  dt <- data.table::data.table(
    mesnotrim = c(1L, 2L, 3L),
    popocup = c(100, 200, 300)
  )

  # 2. Verify: Error about missing column
  expect_error(
    mensalize_sidra_series(dt, verbose = FALSE),
    "Missing required columns.*anomesfinaltrimmovel",
    label = "Missing anomesfinaltrimmovel raises error"
  )
})


test_that("mensalize_sidra_series errors on missing mesnotrim", {
  # 1. Setup: data.table without mesnotrim
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201203L, 201204L, 201205L),
    popocup = c(100, 200, 300)
  )

  # 2. Verify: Error about missing column
  expect_error(
    mensalize_sidra_series(dt, verbose = FALSE),
    "Missing required columns.*mesnotrim",
    label = "Missing mesnotrim raises error"
  )
})


test_that("mensalize_sidra_series auto-converts data.frame to data.table", {
  # 1. Setup: plain data.frame input with starting points
  n <- 36L
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = n, series_name = "popocup"
  )
  df <- as.data.frame(dt)  # convert to plain data.frame

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute: Should not error
  expect_no_error({
    result <- mensalize_sidra_series(
      df,
      starting_points = sp,
      series = "popocup",
      compute_derived = FALSE,
      verbose = FALSE
    )
  })

  # 3. Verify: Result is data.table with expected columns
  result <- mensalize_sidra_series(
    df,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )
  expect_s3_class(result, "data.table")
  expect_true("m_popocup" %in% names(result))
})


test_that("mensalize_sidra_series respects specific series selection", {
  # 1. Setup: Data with multiple series
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 36L, series_name = "popocup"
  )
  # Add additional series columns using data.table::set to avoid scoping issues
  data.table::set(dt, j = "popdesocup",
                  value = dt[["popocup"]] * 0.1 + rnorm(nrow(dt), 0, 50))
  data.table::set(dt, j = "popnaforca",
                  value = dt[["popocup"]] + dt[["popdesocup"]])

  sp <- data.table::data.table(
    series_name = c(rep("popocup", 3), rep("popdesocup", 3), rep("popnaforca", 3)),
    mesnotrim = rep(1:3, 3),
    y0 = c(99000, 99500, 100000, 9000, 9500, 10000, 108000, 109000, 110000)
  )

  # 2. Execute: Select only one series
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Only selected series in output
  expect_true("m_popocup" %in% names(result))
  expect_false("m_popdesocup" %in% names(result),
               label = "Non-selected series excluded")
  expect_false("m_popnaforca" %in% names(result),
               label = "Non-selected series excluded")
})


test_that("mensalize_sidra_series errors on invalid series name", {
  # 1. Setup
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 12L, series_name = "popocup"
  )

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Verify: Invalid series name triggers error
  expect_error(
    mensalize_sidra_series(
      dt,
      starting_points = sp,
      series = "nonexistent_series",
      verbose = FALSE
    ),
    "Series not found",
    label = "Invalid series name raises error"
  )
})


test_that("mensalize_sidra_series compute_derived=FALSE omits derived columns", {
  # 1. Setup: Provide two component series that normally produce a derived aggregate
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 36L, series_name = "popocup"
  )
  data.table::set(dt, j = "popdesocup", value = dt[["popocup"]] * 0.1)

  sp <- data.table::data.table(
    series_name = c(rep("popocup", 3), rep("popdesocup", 3)),
    mesnotrim = rep(1:3, 2),
    y0 = c(99000, 99500, 100000, 9000, 9500, 10000)
  )

  # 2. Execute: compute_derived = FALSE
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: No derived series like m_popnaforca (= popocup + popdesocup)
  expect_false("m_popnaforca" %in% names(result),
               label = "Derived aggregate not computed when compute_derived=FALSE")
  # Primary series should still be there
  expect_true("m_popocup" %in% names(result))
  expect_true("m_popdesocup" %in% names(result))
})


test_that("mensalize_sidra_series output has anomesexato as first column", {
  # 1. Setup
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: anomesexato is the first column
  expect_equal(names(result)[1], "anomesexato",
               label = "anomesexato is first column of output")
})


test_that("mensalize_sidra_series filters pre-PNADC rows", {
  # 1. Setup: Data starting before PNADC era (before 201201)
  # Build manually to include pre-2012 data
  dt_pre <- data.table::data.table(
    anomesfinaltrimmovel = c(201101L, 201102L, 201103L),
    mesnotrim = c(1L, 2L, 3L),
    popocup = c(80000, 80100, 80200)
  )
  dt_post <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 12L, series_name = "popocup"
  )
  dt <- data.table::rbindlist(list(dt_pre, dt_post))

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: No pre-2012 rows in output
  expect_true(min(result$anomesexato) >= 201201L,
              label = "Pre-PNADC rows (before 201201) filtered out")
  expect_equal(nrow(result), 12L,
               label = "Only PNADC-era rows remain")
})


test_that("mensalize_sidra_series accepts custom starting_points", {
  # 1. Setup
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )

  # Custom starting points with specific values
  sp_custom <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(95000, 96000, 97000)
  )

  # 2. Execute: Should use our custom starting points without error
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp_custom,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Result produced without trying to load bundled data
  expect_s3_class(result, "data.table")
  expect_true("m_popocup" %in% names(result))
  expect_true(any(!is.na(result$m_popocup)),
              label = "Custom starting points produce non-NA values")
})


test_that("mensalize_sidra_series errors on wrong starting_points format", {
  # 1. Setup: Starting points missing required column
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 12L, series_name = "popocup"
  )

  # Missing 'y0' column
  sp_bad <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    value = c(99000, 99500, 100000)  # wrong column name
  )

  # 2. Verify: Error about missing columns
  expect_error(
    mensalize_sidra_series(dt, starting_points = sp_bad, verbose = FALSE),
    "Starting points missing columns.*y0",
    label = "Malformed starting_points raises error"
  )
})


test_that("mensalize_sidra_series passes through price index columns", {
  # 1. Setup: Data with price index column alongside a regular series
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )
  # Add a price index column (these are not mensalized, just passed through)
  data.table::set(dt, j = "ipca100dez1993",
                  value = seq(100, by = 0.5, length.out = nrow(dt)))

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Price index column present (no m_ prefix, no mensalization)
  expect_true("ipca100dez1993" %in% names(result),
              label = "Price index column passed through")
  # Verify values are the same as input (no transformation)
  expect_equal(result$ipca100dez1993, dt$ipca100dez1993,
               label = "Price index values unchanged")
})


# =============================================================================
# ADDITIONAL PROPERTY-BASED AND EDGE CASE TESTS
# =============================================================================

test_that("mensalize_sidra_series excludes rate series from direct mensalization", {
  # 1. Setup: Data containing a rate series (should be excluded from mensalization)
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )
  data.table::set(dt, j = "taxadesocup", value = runif(nrow(dt), 5, 15))

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute: Request "all" series (default)
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Rate series is filtered out, not directly mensalized
  expect_false("m_taxadesocup" %in% names(result),
               label = "Rate series not directly mensalized")
  expect_true("m_popocup" %in% names(result),
              label = "Population series is mensalized")
})


test_that("mensalize_sidra_series errors when no valid series columns exist", {
  # 1. Setup: Only metadata columns, no series data
  dt <- data.table::data.table(
    anomesfinaltrimmovel = c(201201L, 201202L, 201203L),
    mesnotrim = c(1L, 2L, 3L)
  )

  # 2. Verify: Error about no series columns
  expect_error(
    mensalize_sidra_series(dt, verbose = FALSE),
    "No series columns found",
    label = "No series columns raises error"
  )
})


test_that(".apply_final_adjustment handles NA values in y gracefully", {
  # 1. Setup: y with some NA values interspersed
  y <- c(100, NA, 300, 110, 210, NA, 120, 220, 320)
  rq <- c(NA, NA, 200, 180, 220, 210, 190, 230, 220)
  mesnotrim <- c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L)

  # 2. Execute
  m <- PNADCperiods:::.apply_final_adjustment(y, rq, mesnotrim)

  # 3. Verify: Where y is NA, m should remain NA (no adjustment possible)
  expect_true(is.na(m[2]), label = "NA in y propagates to m")
  expect_true(is.na(m[6]), label = "NA in y propagates to m")

  # Where y is not NA and trio is incomplete due to NA neighbor, fallback = y
  # Position 1 (mesnotrim=1): needs y[1], y_lead1=y[2]=NA -> invalid, m=y
  expect_equal(m[1], y[1], label = "Boundary fallback to y when neighbor is NA")
})


test_that("mensalize_sidra_series output length matches PNADC-era input rows", {
  # 1. Setup: Exactly 12 months of PNADC-era data
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 12L, series_name = "popocup"
  )

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: One output row per input row
  expect_equal(nrow(result), 12L,
               label = "Output has same number of rows as PNADC-era input")
})


test_that(".compute_cumsum_by_mesnotrim produces correct d3 for linear rq", {
  # 1. Setup: Linearly increasing RQ
  #    rq = 100, 101, 102, 103, 104, ...
  #    d3 = 3*(rq[t] - rq[t-1]) = 3*1 = 3 for all t > first of position
  rq <- 100 + 0:11
  mesnotrim <- rep(1:3, 4)

  # 2. Execute
  cum <- PNADCperiods:::.compute_cumsum_by_mesnotrim(rq, mesnotrim)

  # 3. Verify: For position 1, second occurrence is index 4
  #    d3[4] = 3*(103-102) = 3, cum[4] = 3
  #    d3[7] = 3*(106-105) = 3, cum[7] = 6
  #    d3[10] = 3*(109-108) = 3, cum[10] = 9
  expect_equal(cum[1], 0, label = "First pos1 cum=0")
  expect_equal(cum[4], 3, label = "Second pos1 d3=3")
  expect_equal(cum[7], 6, label = "Third pos1 cum=6")
  expect_equal(cum[10], 9, label = "Fourth pos1 cum=9")

  # For position 2:
  #    d3[5] = 3*(104-103) = 3, cum[5] = 3
  #    d3[8] = 3*(107-106) = 3, cum[8] = 6
  expect_equal(cum[2], 0, label = "First pos2 cum=0")
  expect_equal(cum[5], 3, label = "Second pos2 d3=3")
  expect_equal(cum[8], 6, label = "Third pos2 cum=6")
})


test_that("mensalize_sidra_series excludes avg income series from direct mensalization", {
  # 1. Setup: Data with an average income column (should be derived, not mensalized)
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )
  data.table::set(dt, j = "rendhabnominaltodos", value = runif(nrow(dt), 2000, 3000))

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Average income not directly mensalized
  expect_false("m_rendhabnominaltodos" %in% names(result),
               label = "Average income series not directly mensalized")
})


test_that("mensalize_sidra_series excludes residual series from direct mensalization", {
  # 1. Setup: Data with a residual series column
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )
  data.table::set(dt, j = "popforadaforca", value = runif(nrow(dt), 50000, 60000))

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Residual series not directly mensalized
  expect_false("m_popforadaforca" %in% names(result),
               label = "Residual series not directly mensalized")
})


test_that("mensalize_sidra_series does not modify input data", {
  # 1. Setup
  dt <- make_rolling_quarter_dt(
    start_yyyymm = 201201L, n_months = 24L, series_name = "popocup"
  )
  dt_copy <- data.table::copy(dt)

  sp <- data.table::data.table(
    series_name = rep("popocup", 3),
    mesnotrim = 1:3,
    y0 = c(99000, 99500, 100000)
  )

  # 2. Execute
  result <- mensalize_sidra_series(
    dt,
    starting_points = sp,
    series = "popocup",
    compute_derived = FALSE,
    verbose = FALSE
  )

  # 3. Verify: Original input unchanged
  expect_equal(names(dt), names(dt_copy),
               label = "Input columns unchanged")
  expect_equal(nrow(dt), nrow(dt_copy),
               label = "Input row count unchanged")
  expect_equal(dt$popocup, dt_copy$popocup,
               label = "Input values unchanged")
})

# Helper functions for creating realistic PNADC test data
# These functions ensure consistency between birthday and age variables

#' Create Realistic PNADC Test Data
#'
#' Generates synthetic PNADC data with internally consistent fields.
#' Age is correctly derived from birth date and survey date.
#'
#' @param n_quarters Number of quarters to generate (default 4)
#' @param n_upas Number of UPAs per quarter (default 10)
#' @param persons_per_household Average persons per household (default 3)
#' @param start_year Starting year (default 2023)
#' @param seed Random seed for reproducibility
#' @return data.table with realistic PNADC structure
create_realistic_pnadc <- function(n_quarters = 4,
                                    n_upas = 10,
                                    persons_per_household = 3,
                                    start_year = 2023L,
                                    seed = 42L) {
  set.seed(seed)

  # Generate quarter combinations
  quarters <- data.table::data.table(
    q = seq_len(n_quarters)
  )
  quarters[, `:=`(
    Ano = start_year + (q - 1L) %/% 4L,
    Trimestre = ((q - 1L) %% 4L) + 1L
  )]

  # Generate UPAs (each UPA has a panel group V1014)
  upas <- data.table::data.table(
    UPA = seq_len(n_upas),
    V1014 = sample(1:8, n_upas, replace = TRUE)  # Panel groups 1-8
  )

  # Generate households per UPA
  households_per_upa <- sample(2:5, n_upas, replace = TRUE)
  households <- data.table::rbindlist(lapply(seq_len(n_upas), function(u) {
    data.table::data.table(
      UPA = u,
      V1008 = seq_len(households_per_upa[u])
    )
  }))
  households <- merge(households, upas, by = "UPA")

  # Generate persons per household
  persons <- data.table::rbindlist(lapply(seq_len(nrow(households)), function(h) {
    n_persons <- max(1L, rpois(1, persons_per_household))
    hh <- households[h]
    data.table::data.table(
      UPA = hh$UPA,
      V1008 = hh$V1008,
      V1014 = hh$V1014,
      V2003 = seq_len(n_persons)
    )
  }))

  # Generate birth dates with realistic age distribution
  # PNADC covers all ages, with typical Brazilian population distribution
  n_persons <- nrow(persons)

  # Generate birth years (wider range for realistic age distribution)
  # Ages typically 0-100, median around 35
  birth_years <- start_year - sample(0:85, n_persons, replace = TRUE,
                                      prob = dnorm(0:85, mean = 35, sd = 20) + 0.01)
  birth_months <- sample(1:12, n_persons, replace = TRUE)
  birth_days <- sapply(birth_months, function(m) {
    max_day <- c(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)[m]
    sample(1:max_day, 1)
  })

  persons[, `:=`(
    V20082 = as.integer(birth_years),
    V20081 = as.integer(birth_months),
    V2008 = as.integer(birth_days)
  )]

  # Properly create the cross-product of persons and quarters
  dt <- data.table::rbindlist(lapply(seq_len(nrow(quarters)), function(q) {
    qdata <- quarters[q]
    persons_copy <- data.table::copy(persons)
    persons_copy[, `:=`(
      Ano = qdata$Ano,
      Trimestre = qdata$Trimestre
    )]
    persons_copy
  }))

  # Calculate age based on survey date
  # Survey is conducted in months corresponding to quarter
  # Q1 = Jan-Mar, Q2 = Apr-Jun, Q3 = Jul-Sep, Q4 = Oct-Dec
  # Assume interview at middle of quarter (2nd month)
  dt[, survey_month := (Trimestre - 1L) * 3L + 2L]

  # Calculate age correctly
  dt[, V2009 := {
    age <- Ano - V20082
    # Adjust if birthday hasn't occurred yet this year
    had_birthday <- (survey_month > V20081) |
                    (survey_month == V20081 & 15L >= V2008)  # Assume interview on 15th
    age <- age - as.integer(!had_birthday)
    pmax(0L, age)  # Ensure non-negative
  }]

  # Add some realistic NA values for unknown birthdays (~2%)
  unknown_idx <- sample(seq_len(nrow(dt)), size = ceiling(nrow(dt) * 0.02))
  dt[unknown_idx, `:=`(
    V2008 = 99L,    # PNADC code for unknown
    V20081 = 99L,
    V20082 = 9999L
  )]

  # Clean up
  dt[, survey_month := NULL]

  # Add weight and calibration columns (commonly needed for apply_periods tests)
  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  dt[, `:=`(
    V1028 = runif(.N, 500, 2000),
    UF = sample(valid_ufs, .N, replace = TRUE),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # Reorder columns to match PNADC structure
  data.table::setcolorder(dt, c("Ano", "Trimestre", "UPA", "V1008", "V1014", "V2003",
                                 "V2008", "V20081", "V20082", "V2009",
                                 "V1028", "UF", "posest", "posest_sxi"))

  dt
}


#' Create Minimal Valid PNADC Data
#'
#' Creates a minimal dataset that passes validation for testing.
#'
#' @param n Number of observations
#' @param year Survey year
#' @param quarter Quarter (1-4)
#' @return data.table with minimal required columns
create_minimal_pnadc <- function(n = 10L, year = 2023L, quarter = 1L) {
  # Generate birth dates
  birth_years <- year - sample(18:65, n, replace = TRUE)
  birth_months <- sample(1:12, n, replace = TRUE)
  birth_days <- sapply(birth_months, function(m) {
    max_day <- c(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)[m]
    sample(1:max_day, 1)
  })

  dt <- data.table::data.table(
    Ano = rep(year, n),
    Trimestre = rep(quarter, n),
    UPA = seq_len(n),
    V1008 = rep(1L, n),
    V1014 = sample(1:8, n, replace = TRUE),
    V2003 = rep(1L, n),
    V2008 = as.integer(birth_days),
    V20081 = as.integer(birth_months),
    V20082 = as.integer(birth_years)
  )

  # Calculate age correctly based on survey date
  # Assume interview at middle of quarter (2nd month)
  dt[, survey_month := (Trimestre - 1L) * 3L + 2L]
  dt[, V2009 := {
    age <- Ano - V20082
    # Adjust if birthday hasn't occurred yet this year
    had_birthday <- (survey_month > V20081) |
                    (survey_month == V20081 & 15L >= V2008)  # Assume interview on 15th
    age <- age - as.integer(!had_birthday)
    pmax(0L, age)  # Ensure non-negative
  }]
  dt[, survey_month := NULL]

  dt
}


#' Create PNADC Data with Calibration Columns
#'
#' Creates test data including columns needed for weight calibration.
#'
#' @param n Number of observations
#' @param year Survey year
#' @param quarter Quarter (1-4)
#' @return data.table with calibration-ready structure
create_pnadc_for_calibration <- function(n = 100L, year = 2023L, quarter = 1L) {
  dt <- create_minimal_pnadc(n, year, quarter)

  # Add calibration columns
  # Valid Brazilian UF codes: 11-17 (North), 21-29 (Northeast), 31-35 (Southeast), 41-43 (South), 50-53 (Central-West)
  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  dt[, `:=`(
    UF = sample(valid_ufs, n, replace = TRUE),  # Brazilian state codes
    V1028 = runif(n, 500, 2000),                # Quarterly weights
    V1032 = runif(n, 500, 2000),                # Annual weights
    posest = sample(1:500, n, replace = TRUE),
    posest_sxi = sample(100:999, n, replace = TRUE)
  )]

  dt
}


#' Create Stacked Multi-Quarter PNADC Data
#'
#' Creates data spanning multiple quarters for testing cross-quarter aggregation.
#'
#' @param n_quarters Number of quarters
#' @param start_year Starting year
#' @param n_upas Number of unique UPAs
#' @return data.table with stacked quarterly data
create_stacked_pnadc <- function(n_quarters = 8L, start_year = 2022L, n_upas = 20L) {
  create_realistic_pnadc(
    n_quarters = n_quarters,
    n_upas = n_upas,
    start_year = start_year
  )
}


#' Create Monthly Population Targets for Testing
#'
#' Creates mock monthly population targets matching SIDRA format.
#'
#' @param start_yyyymm Starting month (YYYYMM format)
#' @param n_months Number of months
#' @param base_pop Base population (in thousands)
#' @return data.table with monthly population targets
create_monthly_targets <- function(start_yyyymm = 202301L,
                                    n_months = 12L,
                                    base_pop = 215000) {
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L

  months <- data.table::data.table(
    month_num = seq_len(n_months)
  )
  months[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  months[, ref_month_yyyymm := year * 100L + month]

  # Add slight growth trend
  months[, m_populacao := base_pop * (1 + 0.001 * (month_num - 1))]

  months[, .(ref_month_yyyymm, m_populacao)]
}


#' Create Mock Rolling Quarter Data for Testing
#'
#' Creates synthetic rolling quarter data matching SIDRA format.
#' Useful for testing mensalize_sidra_series() without API calls.
#'
#' @param n_months Number of months to generate
#' @param series Character vector of series names to include
#' @param start_yyyymm Starting YYYYMM (default 201201)
#' @param base_values Named list of base values per series
#' @return data.table with anomesfinaltrimmovel, mesnotrim, and series columns
create_mock_rolling_quarters <- function(n_months = 36L,
                                          series = "popocup",
                                          start_yyyymm = 201201L,
                                          base_values = NULL) {
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L

  dt <- data.table::data.table(month_num = seq_len(n_months))
  dt[, `:=`(
    year = start_year + (start_month + month_num - 2L) %/% 12L,
    month = ((start_month + month_num - 2L) %% 12L) + 1L
  )]
  dt[, anomesfinaltrimmovel := year * 100L + month]
  dt[, mesnotrim := ((month - 1L) %% 3L) + 1L]

  # Generate series columns with realistic rolling quarter patterns
  for (s in series) {
    base <- if (!is.null(base_values) && s %in% names(base_values)) {
      base_values[[s]]
    } else {
      100000  # default base value
    }
    # Slight trend + seasonal pattern for realism
    dt[, (s) := base * (1 + 0.001 * (month_num - 1) +
                          0.005 * sin(2 * pi * month_num / 12))]
  }

  dt[, .(anomesfinaltrimmovel, mesnotrim,
         .SD), .SDcols = series]
}


#' Generate proper YYYYMM sequence
#'
#' @param start_yyyymm Starting YYYYMM integer
#' @param n Number of months
#' @return Integer vector of YYYYMM values
generate_yyyymm_seq <- function(start_yyyymm, n) {
  start_year <- start_yyyymm %/% 100L
  start_month <- start_yyyymm %% 100L
  vapply(seq_len(n), function(i) {
    total_months <- start_month + i - 2L
    y <- start_year + total_months %/% 12L
    m <- (total_months %% 12L) + 1L
    y * 100L + m
  }, integer(1))
}


#' Create mock monthly population targets for tests (avoids SIDRA fetch).
#'
#' Builds a target_totals data.table by aggregating V1028 weights of the
#' input PNADC dataset across quarters and expanding the quarterly sum to
#' three months. The result has the structure expected by
#' pnadc_apply_periods() for calibration_unit = "month". For "fortnight"
#' or "week", apply derive_fortnight_population() or
#' derive_weekly_population() to the output.
#'
#' Using V1028 sums ensures the mock is realistic relative to the
#' synthetic dataset (so calibration converges and test assertions are
#' meaningful), without making any HTTP request to SIDRA.
#'
#' @param data PNADC data.table with columns Ano, Trimestre, V1028.
#' @return data.table with columns ref_month_yyyymm and m_populacao.
create_mock_pop_targets <- function(data) {
  qtr_wsum <- data[, .(qtr_wsum = sum(V1028, na.rm = TRUE)),
                    by = .(Ano, Trimestre)]
  qtr_wsum[, {
    months <- (Trimestre - 1L) * 3L + 1:3
    data.table::data.table(
      ref_month_yyyymm = Ano * 100L + months,
      m_populacao = qtr_wsum / 1000
    )
  }, by = .(Ano, Trimestre)][, .(ref_month_yyyymm, m_populacao)]
}

# Tests for pnadc_apply_periods() - apply crosswalk and optionally calibrate weights

# =============================================================================
# HELPER FUNCTIONS
# =============================================================================

# Note: Using shared test data generators from helper-test-data.R
# - create_realistic_pnadc() for test data generation
# - Real crosswalks from pnadc_identify_periods() instead of mocks

# =============================================================================
# INPUT VALIDATION TESTS
# =============================================================================

test_that("pnadc_apply_periods requires weight_var argument", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  expect_error(
    pnadc_apply_periods(test_data, crosswalk, anchor = "quarter"),
    "weight_var"
  )
})

test_that("pnadc_apply_periods requires anchor argument", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  expect_error(
    pnadc_apply_periods(test_data, crosswalk, weight_var = "V1028"),
    "anchor"
  )
})

test_that("pnadc_apply_periods validates anchor values", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  expect_error(
    pnadc_apply_periods(test_data, crosswalk,
                        weight_var = "V1028",
                        anchor = "invalid"),
    "anchor"
  )
})

test_that("pnadc_apply_periods validates calibration_unit values", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  # match.arg error message format
  expect_error(
    pnadc_apply_periods(test_data, crosswalk,
                        weight_var = "V1028",
                        anchor = "quarter",
                        calibration_unit = "invalid"),
    "should be one of"
  )
})

test_that("pnadc_apply_periods validates weight_var exists", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  expect_error(
    pnadc_apply_periods(test_data, crosswalk,
                        weight_var = "NONEXISTENT",
                        anchor = "quarter",
                        calibrate = FALSE),
    "not found in data"
  )
})

test_that("pnadc_apply_periods validates crosswalk has required columns", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  # Crosswalk missing V1014 (required minimal key)
  bad_crosswalk <- data.frame(UPA = 1:3, something_else = 1:3)

  expect_error(
    pnadc_apply_periods(test_data, bad_crosswalk,
                        weight_var = "V1028",
                        anchor = "quarter",
                        calibrate = FALSE),
    "missing"  # More general pattern to match error
  )
})

# =============================================================================
# OUTPUT STRUCTURE TESTS (NO CALIBRATION)
# =============================================================================

test_that("pnadc_apply_periods without calibration returns correct structure", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  result <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = FALSE,
    verbose = FALSE
  )

  # Should be data.table
  expect_s3_class(result, "data.table")

  # Should have reference period columns from crosswalk (current implementation)
  expect_true("ref_month_in_quarter" %in% names(result))
  expect_true("ref_month_in_year" %in% names(result))
  expect_true("ref_fortnight_in_month" %in% names(result))
  expect_true("ref_fortnight_in_quarter" %in% names(result))
  expect_true("ref_week_in_month" %in% names(result))
  expect_true("ref_week_in_quarter" %in% names(result))

  # Should have determination flags
  expect_true("determined_month" %in% names(result))
  expect_true("determined_fortnight" %in% names(result))
  expect_true("determined_week" %in% names(result))

  # Should NOT have calibrated weight column when calibrate = FALSE
  expect_false("weight_monthly" %in% names(result))
})

test_that("pnadc_apply_periods preserves original data columns", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  test_data[, custom_column := "test_value"]

  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  result <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = FALSE,
    verbose = FALSE
  )

  expect_true("custom_column" %in% names(result))
  expect_true("Ano" %in% names(result))
  expect_true("Trimestre" %in% names(result))
  expect_true("V1028" %in% names(result))
})

# =============================================================================
# CALIBRATION TESTS
# =============================================================================

test_that("pnadc_apply_periods with calibration adds weight column", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  result <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = create_mock_pop_targets(test_data),
    verbose = FALSE
  )

  expect_true("weight_monthly" %in% names(result))
  expect_type(result$weight_monthly, "double")
})

test_that("pnadc_apply_periods respects calibration_unit parameter", {
  skip_on_cran()
  skip_if_not(requireNamespace("sidrar", quietly = TRUE),
              "sidrar package not available for fetching population targets")
  skip_if_offline()

  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Month calibration (with explicit targets)
  result_month <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "month",
    target_totals = NULL,  # Auto-derive from SIDRA
    verbose = FALSE
  )
  expect_true("weight_monthly" %in% names(result_month))

  # Fortnight calibration (let it derive targets from SIDRA)
  result_fortnight <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "fortnight",
    target_totals = NULL,  # Auto-derive from SIDRA
    verbose = FALSE
  )
  expect_true("weight_fortnight" %in% names(result_fortnight))

  # Week calibration (let it derive targets from SIDRA)
  result_week <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    calibration_unit = "week",
    target_totals = NULL,  # Auto-derive from SIDRA
    verbose = FALSE
  )
  expect_true("weight_weekly" %in% names(result_week))
})

# =============================================================================
# KEEP_ALL PARAMETER TESTS
# =============================================================================

test_that("pnadc_apply_periods keep_all = TRUE includes undetermined rows", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  result <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    target_totals = create_mock_pop_targets(test_data),
    keep_all = TRUE,
    verbose = FALSE
  )

  # Should include all rows from original data
  expect_equal(nrow(result), nrow(test_data))

  # Undetermined rows should have NA weight
  undetermined <- result[is.na(ref_month_in_quarter)]
  if (nrow(undetermined) > 0) {
    expect_true(all(is.na(undetermined$weight_monthly)))
  }
})

test_that("pnadc_apply_periods keep_all = FALSE excludes undetermined rows", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  result <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = TRUE,
    target_totals = create_mock_pop_targets(test_data),
    keep_all = FALSE,
    verbose = FALSE
  )

  # Should only include determined rows
  expect_true(all(!is.na(result$ref_month_in_quarter)))
  expect_true(nrow(result) <= nrow(test_data))
})

# =============================================================================
# ANCHOR PARAMETER TESTS
# =============================================================================

test_that("pnadc_apply_periods accepts anchor = 'quarter'", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  expect_no_error(
    result <- pnadc_apply_periods(
      test_data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = TRUE,
      target_totals = create_mock_pop_targets(test_data),
      verbose = FALSE
    )
  )
})

test_that("pnadc_apply_periods accepts anchor = 'year'", {
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  expect_no_error(
    result <- pnadc_apply_periods(
      test_data, crosswalk,
      weight_var = "V1028",
      anchor = "year",
      calibrate = TRUE,
      target_totals = create_mock_pop_targets(test_data),
      verbose = FALSE
    )
  )
})

# =============================================================================
# VERBOSE OUTPUT TESTS
# =============================================================================

test_that("pnadc_apply_periods respects verbose parameter", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  # verbose = FALSE should produce no output
  expect_silent(
    result <- pnadc_apply_periods(
      test_data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = FALSE,
      verbose = FALSE
    )
  )

  # verbose = TRUE should produce output
  expect_output(
    result <- pnadc_apply_periods(
      test_data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = FALSE,
      verbose = TRUE
    ),
    "Applying|crosswalk"
  )
})

# =============================================================================
# EDGE CASE TESTS
# =============================================================================

test_that("pnadc_identify_periods rejects empty data", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  test_data <- test_data[0]  # Empty data

  # Empty data should error at identify_periods stage (validation rejects empty data)
  expect_error(
    pnadc_identify_periods(test_data, verbose = FALSE),
    "at least 1 rows"
  )
})

test_that("pnadc_apply_periods handles mismatched data keys", {
  # Create data and crosswalk with different UPA sets
  data1 <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)
  data2 <- create_realistic_pnadc(n_quarters = 1, n_upas = 5, seed = 999)

  # Create crosswalk from data1 (only UPAs 1-3)
  crosswalk <- pnadc_identify_periods(data1, verbose = FALSE)

  # Apply to data2 (has UPAs 1-5) - some rows won't match
  result <- pnadc_apply_periods(
    data2, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = FALSE,
    keep_all = TRUE,
    verbose = FALSE
  )

  # Result should have same number of rows as data2 (keep_all=TRUE)
  expect_equal(nrow(result), nrow(data2))

  # But some rows may have NA for period columns if they don't match crosswalk
  expect_true(any(is.na(result$ref_month_in_quarter)))
})

# =============================================================================
# INTEGRATION TESTS
# =============================================================================

test_that("pnadc_apply_periods works with real identify function output", {
  # Create data and run the identify function
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)
  crosswalk <- pnadc_identify_periods(test_data, verbose = FALSE)

  result <- pnadc_apply_periods(
    test_data, crosswalk,
    weight_var = "V1028",
    anchor = "quarter",
    calibrate = FALSE,
    verbose = FALSE
  )

  expect_s3_class(result, "data.table")
  expect_true("ref_month_in_quarter" %in% names(result))
  expect_true("ref_fortnight_in_quarter" %in% names(result))
  expect_true("ref_week_in_quarter" %in% names(result))
})

# Tests for edge cases and boundary conditions
# Systematic coverage of unusual inputs and boundary transitions

# =============================================================================
# EMPTY AND MINIMAL DATA TESTS
# =============================================================================

test_that("identify_periods rejects empty data", {
  # 1. Setup: Create empty data.table with correct structure
  empty_data <- data.table::data.table(
    Ano = integer(0),
    Trimestre = integer(0),
    UPA = integer(0),
    V1008 = integer(0),
    V1014 = integer(0),
    V2008 = integer(0),
    V20081 = integer(0),
    V20082 = integer(0),
    V2009 = integer(0)
  )

  # 2. Verify: Should error on empty data (validation fails)
  expect_error(
    pnadc_identify_periods(empty_data, verbose = FALSE),
    "at least 1 rows",
    label = "Empty data should be rejected by validation"
  )

  # 3. Context: Input validation prevents processing of empty data
})


test_that("identify_periods handles single observation", {
  # 1. Setup: Create data with just one person
  single_obs <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1L,
    V1008 = 1L,
    V1014 = 1L,
    V2008 = 15L,
    V20081 = 6L,
    V20082 = 1990L,
    V2009 = 33L
  )

  # 2. Execute: Should not error
  expect_no_error({
    result <- pnadc_identify_periods(single_obs, verbose = FALSE)
  })

  # 3. Verify: Should return one row
  expect_equal(nrow(result), 1)
  expect_true("determined_month" %in% names(result))

  # 4. Context: Minimal valid input
})


test_that("apply_periods handles all NA weights gracefully", {
  # 1. Setup: Create data with NA weights
  data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  data[, V1028 := NA_real_]

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Execute: Apply without calibration (NA weights)
  expect_no_error({
    result <- pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = FALSE,
      verbose = FALSE
    )
  })

  # 3. Context: Missing weight data should not crash
})


# =============================================================================
# YEAR AND QUARTER BOUNDARY TESTS
# =============================================================================

test_that("year boundary transition handled correctly", {

  # 1. Setup: Create data spanning year boundary (Q4 2023 + Q1 2024)
  # Use 4-quarter data from each year and filter to get correct Q4/Q1 with valid ages
  set.seed(100)
  data_2023 <- create_realistic_pnadc(n_quarters = 4, n_upas = 10, start_year = 2023)
  data_q4 <- data_2023[Trimestre == 4L]

  data_2024 <- create_realistic_pnadc(n_quarters = 4, n_upas = 10, start_year = 2024, seed = 99L)
  data_q1 <- data_2024[Trimestre == 1L]

  # Combine
  data <- rbind(data_q4, data_q1)

  # 2. Execute: Should handle year transition
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Both years should be present
  expect_true(2023L %in% result$Ano)
  expect_true(2024L %in% result$Ano)
  expect_true(4L %in% result$Trimestre)
  expect_true(1L %in% result$Trimestre)

  # 4. Context: Date calculations must handle year boundaries
})


test_that("December 31 to January 1 transition", {
  # 1. Setup: Create observation with interview on Dec 31
  data <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 4L,
    UPA = 1L,
    V1008 = 1L,
    V1014 = 1L,
    V2008 = 31L,  # Day 31
    V20081 = 12L, # December
    V20082 = 1990L,
    V2009 = 33L
  )

  # 2. Execute: Should not error
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Should produce valid crosswalk
  expect_equal(nrow(result), 1)

  # 4. Context: Edge case for last day of year
})


# =============================================================================
# LEAP YEAR TESTS
# =============================================================================

test_that("February 29 birthday in non-leap year", {
  # 1. Setup: Person born Feb 29, interviewed in non-leap year (2023)
  data <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1L,
    V1008 = 1L,
    V1014 = 1L,
    V2008 = 29L,  # Day 29
    V20081 = 2L,  # February
    V20082 = 1992L,  # Leap year birth
    V2009 = 31L  # Age
  )

  # 2. Execute: Should handle gracefully (March 1 used as substitute in non-leap years)
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Should produce valid crosswalk
  expect_equal(nrow(result), 1)

  # 4. Context: Implementation substitutes March 1 for Feb 29 in non-leap years
})


test_that("February 29 birthday in leap year", {
  # 1. Setup: Person born Feb 29, interviewed in leap year (2024)
  data <- data.table::data.table(
    Ano = 2024L,
    Trimestre = 1L,
    UPA = 1L,
    V1008 = 1L,
    V1014 = 1L,
    V2008 = 29L,
    V20081 = 2L,
    V20082 = 1992L,
    V2009 = 32L
  )

  # 2. Execute: Should handle correctly
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Valid crosswalk
  expect_equal(nrow(result), 1)

  # 4. Context: Feb 29 exists in leap years
})


# =============================================================================
# UNKNOWN BIRTHDAY CODES
# =============================================================================

test_that("all unknown birthdays (99/9999) handled", {
  # 1. Setup: Create data where all observations have unknown birthdays
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)
  data[, `:=`(
    V2008 = 99L,
    V20081 = 99L,
    V20082 = 9999L
  )]

  # 2. Execute: Should not error
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Determination still works (based on other constraints)
  expect_true(nrow(result) > 0)
  expect_true("determined_month" %in% names(result))

  # 4. Context: Birthday constraints skipped when unknown
})


test_that("mixed known and unknown birthdays", {
  # 1. Setup: Half known, half unknown birthdays
  set.seed(200)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)

  n <- nrow(data)
  unknown_idx <- sample(seq_len(n), size = n %/% 2)
  data[unknown_idx, `:=`(
    V2008 = 99L,
    V20082 = 9999L
  )]

  # 2. Execute: Should not error
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Both types processed
  expect_equal(nrow(result), n)

  # 4. Context: Mixed data should work
})


# =============================================================================
# EXTREME AGE VALUES
# =============================================================================

test_that("very young ages handled (age 0)", {
  # 1. Setup: Create data with age 0 (infant)
  data <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1L,
    V1008 = 1L,
    V1014 = 1L,
    V2008 = 1L,
    V20081 = 1L,
    V20082 = 2023L,  # Born same year
    V2009 = 0L  # Age 0
  )

  # 2. Execute: Should not error
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Context: PNADC includes all ages
})


test_that("very old ages handled (age 100+)", {
  # 1. Setup: Create data with age 100+
  data <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1L,
    V1008 = 1L,
    V1014 = 1L,
    V2008 = 15L,
    V20081 = 6L,
    V20082 = 1920L,  # Very old
    V2009 = 103L  # Age 103
  )

  # 2. Execute: Should not error
  expect_no_error({
    result <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Context: Centenarians exist in data
})


# =============================================================================
# SINGLE VS MULTI-QUARTER BEHAVIOR
# =============================================================================

test_that("single quarter produces lower determination rate", {
  # 1. Setup: Same UPAs across 1 quarter vs 4 quarters
  set.seed(300)
  data_1q <- create_realistic_pnadc(n_quarters = 1, n_upas = 20)
  data_4q <- create_realistic_pnadc(n_quarters = 4, n_upas = 20)

  # 2. Execute: Identify periods
  cw_1q <- pnadc_identify_periods(data_1q, verbose = FALSE)
  cw_4q <- pnadc_identify_periods(data_4q, verbose = FALSE)

  # 3. Verify: More quarters should give higher determination rate
  rate_1q <- cw_1q[, mean(determined_month, na.rm = TRUE)]
  rate_4q <- cw_4q[, mean(determined_month, na.rm = TRUE)]

  expect_true(rate_4q > rate_1q,
              label = paste0("4Q (", round(rate_4q, 3),
                             ") should be > 1Q (", round(rate_1q, 3), ")"))

  # 4. Context: Cross-quarter aggregation improves determination
})


# =============================================================================
# ZERO WEIGHT TESTS
# =============================================================================

test_that("zero weights in input handled", {
  # 1. Setup: Create data with some zero weights
  set.seed(400)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)

  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  # Set some weights to zero
  zero_idx <- sample(1:nrow(data), size = 5)
  data[zero_idx, V1028 := 0]

  # 2. Execute: Apply with calibration (mock targets, no SIDRA)
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  result <- NULL
  expect_no_error({
    result <- pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = TRUE,
      calibration_unit = "month",
      target_totals = create_mock_pop_targets(data),
      verbose = FALSE
    )
  })

  # 3. Verify: No negative weights produced
  expect_true(all(result$weight_monthly >= 0, na.rm = TRUE))

  # 4. Context: Zero weights edge case
})


# =============================================================================
# ALL PERIODS INDETERMINATE TEST
# =============================================================================

test_that("data with no determinations handled", {
  # 1. Setup: Create data that's very hard to determine
  # Single quarter, wide date ranges, no birthday info
  data <- data.table::data.table(
    Ano = rep(2023L, 10),
    Trimestre = rep(1L, 10),
    UPA = 1:10,  # All different UPAs
    V1008 = 1L,
    V1014 = 1:10,  # All different V1014
    V2008 = 99L,  # Unknown birthday
    V20081 = 99L,
    V20082 = 9999L,
    V2009 = 30L
  )

  # 2. Execute: Identify periods
  result <- pnadc_identify_periods(data, verbose = FALSE)

  # 3. Verify: Should return crosswalk (even if determination rate is low)
  expect_equal(nrow(result), 10)
  expect_true("determined_month" %in% names(result))

  # 4. Context: Low determination rate is valid outcome
})


# =============================================================================
# PARAMETER VALIDATION TESTS
# =============================================================================

test_that("invalid anchor parameter rejected", {
  # 1. Setup: Create data
  data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Verify: Invalid anchor should error
  expect_error(
    pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",
      anchor = "invalid_anchor",
      calibrate = TRUE,
      calibration_unit = "month",
      verbose = FALSE
    ),
    "anchor.*quarter.*year",
    label = "Invalid anchor parameter should be rejected"
  )
})


test_that("invalid calibration_unit rejected", {
  # 1. Setup: Create data
  data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  valid_ufs <- c(11:17, 21:29, 31:35, 41:43, 50:53)
  data[, `:=`(
    UF = sample(valid_ufs, .N, replace = TRUE),
    V1028 = runif(.N, 500, 2000),
    posest = sample(1:500, .N, replace = TRUE),
    posest_sxi = sample(100:999, .N, replace = TRUE)
  )]

  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 2. Verify: Invalid calibration_unit should error
  # Note: match.arg() throws error before custom message
  expect_error(
    pnadc_apply_periods(
      data, crosswalk,
      weight_var = "V1028",
      anchor = "quarter",
      calibrate = TRUE,
      calibration_unit = "invalid_unit",
      verbose = FALSE
    ),
    label = "Invalid calibration_unit should be rejected"
  )
})

# Tests for pnadc_identify_periods() - main crosswalk builder

# =============================================================================
# HELPER FUNCTIONS
# =============================================================================

# Note: Using shared test data generators from helper-test-data.R
# to ensure consistent age/birthday calculations across all tests

# =============================================================================
# INPUT VALIDATION TESTS
# =============================================================================

test_that("pnadc_identify_periods validates required columns", {
  bad_data <- data.frame(Ano = 2023, Trimestre = 1)

  expect_error(
    pnadc_identify_periods(bad_data, verbose = FALSE),
    "missing"
  )
})

test_that("pnadc_identify_periods handles character columns", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 5)
  test_data[, Ano := as.character(Ano)]
  test_data[, Trimestre := as.character(Trimestre)]

  result <- pnadc_identify_periods(test_data, verbose = FALSE)
  expect_s3_class(result, "data.table")
})

# =============================================================================
# OUTPUT STRUCTURE TESTS
# =============================================================================

test_that("pnadc_identify_periods returns correct crosswalk structure", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Should be data.table

  expect_s3_class(result, "data.table")

  # Should have household-quarter level keys
  expect_true("Ano" %in% names(result))
  expect_true("Trimestre" %in% names(result))
  expect_true("UPA" %in% names(result))
  expect_true("V1008" %in% names(result))
  expect_true("V1014" %in% names(result))

  # Should NOT have person-level keys
  expect_false("V2003" %in% names(result))

  # Should have month columns (IBGE-based)
  expect_true("ref_month_in_quarter" %in% names(result))
  expect_true("ref_month_in_year" %in% names(result))
  expect_true("ref_month_yyyymm" %in% names(result))
  expect_true("determined_month" %in% names(result))

  # Should have fortnight columns (IBGE-based)
  expect_true("ref_fortnight_in_month" %in% names(result))
  expect_true("ref_fortnight_in_quarter" %in% names(result))
  expect_true("ref_fortnight_yyyyff" %in% names(result))
  expect_true("determined_fortnight" %in% names(result))

  # Should have week columns (IBGE-based)
  expect_true("ref_week_in_month" %in% names(result))
  expect_true("ref_week_in_quarter" %in% names(result))
  expect_true("ref_week_yyyyww" %in% names(result))
  expect_true("determined_week" %in% names(result))
})

test_that("pnadc_identify_periods returns same number of rows as input", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Result should have same number of rows as input
  # (crosswalk preserves person-level rows for joining)
  expect_equal(nrow(result), nrow(test_data))

  # Each household-quarter-panel combination should have consistent reference values
  # (all persons in same household should have same determined period)
  dt_check <- result[, .(
    n_unique_month = data.table::uniqueN(ref_month_in_quarter)
  ), by = .(Ano, Trimestre, UPA, V1008, V1014)]
  expect_true(all(dt_check$n_unique_month == 1))
})

test_that("pnadc_identify_periods has determination flags that allow computing rates", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Compute determination rates from the result
  n_total <- nrow(result)
  month_rate <- sum(result$determined_month) / n_total
  fortnight_rate <- sum(result$determined_fortnight) / n_total
  week_rate <- sum(result$determined_week) / n_total

  # Rates should be between 0 and 1

  expect_true(month_rate >= 0 && month_rate <= 1)
  expect_true(fortnight_rate >= 0 && fortnight_rate <= 1)
  expect_true(week_rate >= 0 && week_rate <= 1)
})

# =============================================================================
# DETERMINATION FLAG TESTS
# =============================================================================

test_that("determination flags are consistent with ref values", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # determined_month should be TRUE iff ref_month_in_quarter is not NA
  expect_equal(
    result$determined_month,
    !is.na(result$ref_month_in_quarter)
  )

  # determined_fortnight should be TRUE iff ref_fortnight_in_quarter is not NA
  expect_equal(
    result$determined_fortnight,
    !is.na(result$ref_fortnight_in_quarter)
  )

  # determined_week should be TRUE iff ref_week_in_quarter is not NA
  expect_equal(
    result$determined_week,
    !is.na(result$ref_week_in_quarter)
  )
})

# =============================================================================
# REFERENCE VALUE CONSISTENCY TESTS
# =============================================================================
test_that("ref_month_in_quarter values are 1, 2, 3, or NA", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  valid_values <- c(1L, 2L, 3L, NA_integer_)
  expect_true(all(result$ref_month_in_quarter %in% valid_values))
})

test_that("ref_fortnight_in_quarter values are 1-6 or NA", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  non_na <- result$ref_fortnight_in_quarter[!is.na(result$ref_fortnight_in_quarter)]
  if (length(non_na) > 0) {
    expect_true(all(non_na >= 1L & non_na <= 6L))
  }
})

test_that("ref_week_in_quarter values are 1-12 or NA", {
  # IBGE quarters always have exactly 12 reference weeks (4 weeks × 3 months)
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  non_na <- result$ref_week_in_quarter[!is.na(result$ref_week_in_quarter)]
  if (length(non_na) > 0) {
    expect_true(all(non_na >= 1L & non_na <= 12L))
  }
})

test_that("ref_month_yyyymm is consistent with ref_month_in_year", {
  test_data <- create_realistic_pnadc(n_quarters = 2, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  determined <- result[determined_month == TRUE]
  if (nrow(determined) > 0) {
    # ref_month_yyyymm should be Ano * 100 + ref_month_in_year
    expected_yyyymm <- determined$Ano * 100L + determined$ref_month_in_year
    expect_equal(determined$ref_month_yyyymm, expected_yyyymm)
  }
})

# =============================================================================
# FORTNIGHT FORMAT TESTS
# =============================================================================

test_that("ref_fortnight_yyyyff follows YYYY01-YYYY24 format", {
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  determined <- result[determined_fortnight == TRUE]
  if (nrow(determined) > 0) {
    years <- determined$ref_fortnight_yyyyff %/% 100L
    fortnights <- determined$ref_fortnight_yyyyff %% 100L

    # Fortnights should be 1-24
    expect_true(all(fortnights >= 1L & fortnights <= 24L))

    # Years should be reasonable
    expect_true(all(years >= 2000L & years <= 2100L))
  }
})

test_that("ref_fortnight_in_month is 1 or 2", {
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 5)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  determined <- result[determined_fortnight == TRUE]
  if (nrow(determined) > 0) {
    # ref_fortnight_in_month should be 1 (first fortnight) or 2 (second fortnight)
    expect_true(all(determined$ref_fortnight_in_month %in% c(1L, 2L)))
  }
})

# =============================================================================
# VERBOSE OUTPUT TESTS
# =============================================================================

test_that("pnadc_identify_periods respects verbose parameter", {
  test_data <- create_realistic_pnadc(n_quarters = 1, n_upas = 3)

  # verbose = FALSE should produce no output
  expect_silent(result <- pnadc_identify_periods(test_data, verbose = FALSE))

  # verbose = TRUE should produce output
  expect_output(
    result <- pnadc_identify_periods(test_data, verbose = TRUE),
    "Building|Step|determination"
  )
})

# =============================================================================
# DETERMINATION RATE HIERARCHY TESTS
# =============================================================================

test_that("month determination rate >= fortnight rate >= week rate", {
  # This tests the expected hierarchy: coarser granularity = higher determination
  test_data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10)

  result <- pnadc_identify_periods(test_data, verbose = FALSE)

  # Compute determination rates from the result
  n_total <- nrow(result)
  month_rate <- sum(result$determined_month) / n_total
  fortnight_rate <- sum(result$determined_fortnight) / n_total
  week_rate <- sum(result$determined_week) / n_total

  # Month should have highest (or equal) rate
  expect_true(month_rate >= fortnight_rate ||
              abs(month_rate - fortnight_rate) < 0.01)  # Allow small tolerance

  # Fortnight should have higher (or equal) rate than week
  expect_true(fortnight_rate >= week_rate ||
              abs(fortnight_rate - week_rate) < 0.01)
})

# =============================================================================
# CROSS-QUARTER AGGREGATION TESTS
# =============================================================================

test_that("cross-quarter aggregation improves month determination rate", {
  # 1. Context: The algorithm's core feature is aggregating by UPA-V1014 ACROSS quarters
  # Documentation states: 1 quarter ~70% determination, 8 quarters ~94% determination

  set.seed(200)

  # 2. Test with 1 quarter - expect lower determination rate
  data_1q <- create_realistic_pnadc(n_quarters = 1, n_upas = 25, start_year = 2023)
  cw_1q <- pnadc_identify_periods(data_1q, verbose = FALSE)
  rate_1q <- cw_1q[, mean(determined_month, na.rm = TRUE)]

  # 3. Test with 8 quarters - expect higher determination rate
  data_8q <- create_realistic_pnadc(n_quarters = 8, n_upas = 25, start_year = 2023, seed = 200)
  cw_8q <- pnadc_identify_periods(data_8q, verbose = FALSE)
  rate_8q <- cw_8q[, mean(determined_month, na.rm = TRUE)]

  # 4. Verify: More quarters should improve determination significantly
  # Documentation benchmarks: 70% (1Q) vs 94% (8Q) = 24pp improvement
  # We'll be conservative and expect at least 10pp improvement
  improvement <- rate_8q - rate_1q

  expect_true(improvement > 0.10,
              label = paste0("8 quarters should improve determination by >10pp. ",
                             "Actual: 1Q=", round(rate_1q, 3), ", 8Q=", round(rate_8q, 3),
                             " (", round(improvement, 3), "pp)"))

  # 5. Context: Cross-quarter aggregation is the key algorithmic innovation
  # Same UPA-V1014 across quarters converge to consistent month constraints
})


test_that("same UPA-V1014 across quarters converge to same month", {
  # 1. Setup: Create data where same UPA-V1014 appears in multiple quarters
  set.seed(201)

  # Use small number of UPAs and panel groups to ensure overlaps
  data <- create_realistic_pnadc(n_quarters = 4, n_upas = 10, start_year = 2023)

  # 2. Execute: Identify periods with cross-quarter aggregation
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 3. Verify: Within same UPA-V1014, determined months should be consistent
  # Group by UPA-V1014 and check if ref_month_in_quarter is consistent when determined
  consistency_check <- crosswalk[determined_month == TRUE, .(
    n_obs = .N,
    n_unique_months = uniqueN(ref_month_in_quarter, na.rm = TRUE),
    months = paste(unique(ref_month_in_quarter), collapse = ",")
  ), by = .(UPA, V1014)]

  # Filter to UPA-V1014 with multiple determinations
  multi_det <- consistency_check[n_obs > 1]

  if (nrow(multi_det) > 0) {
    # All multi-observation UPA-V1014 should have consistent month
    all_consistent <- all(multi_det$n_unique_months == 1)

    expect_true(all_consistent,
                label = "Same UPA-V1014 across quarters should converge to same month")
  }

  # 4. Context: This is the fundamental mechanism of cross-quarter aggregation
  # Phase 1 aggregates by .(UPA, V1014) across ALL quarters, not within-quarter
})


test_that("determination rate improves monotonically with more quarters", {
  # 1. Setup: Test with 2, 4, 6 quarters to verify monotonic improvement
  set.seed(202)

  rates <- sapply(c(2, 4, 6), function(nq) {
    data <- create_realistic_pnadc(n_quarters = nq, n_upas = 20, start_year = 2023, seed = 202)
    cw <- pnadc_identify_periods(data, verbose = FALSE)
    cw[, mean(determined_month, na.rm = TRUE)]
  })

  # 2. Verify: Each increase in quarters should improve or maintain rate
  # rates[1] = 2Q, rates[2] = 4Q, rates[3] = 6Q
  expect_true(rates[2] >= rates[1] - 0.01,  # Allow tiny tolerance for randomness
              label = paste0("4Q (", round(rates[2], 3), ") should be >= 2Q (", round(rates[1], 3), ")"))

  expect_true(rates[3] >= rates[2] - 0.01,
              label = paste0("6Q (", round(rates[3], 3), ") should be >= 4Q (", round(rates[2], 3), ")"))

  # 3. Context: Stacking more data consistently improves month determination
  # This is why CLAUDE.md recommends "8+ quarters recommended" for best results
})

# =============================================================================
# BIRTHDAY CONSTRAINT TESTS
# =============================================================================

test_that("unknown birthdays (99/9999) are handled gracefully", {
  # 1. Setup: Create data with unknown birthdays (PNADC codes)
  set.seed(300)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 10)

  # Make 20% have unknown birthdays
  unknown_idx <- sample(seq_len(nrow(data)), size = ceiling(nrow(data) * 0.2))
  data[unknown_idx, `:=`(
    V2008 = 99L,    # PNADC code for unknown day
    V20081 = 99L,   # PNADC code for unknown month
    V20082 = 9999L  # PNADC code for unknown year
  )]

  # 2. Execute: Should not error with unknown birthdays
  expect_no_error({
    crosswalk <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Determination still works (based on other constraints like V1014)
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # Some observations should still be determined despite missing birthdays
  expect_true(crosswalk[, sum(determined_month, na.rm = TRUE)] > 0,
              label = "Determination works even with unknown birthdays")

  # 4. Context: Birthday constraints are helpful but not required
  # Algorithm can determine months using V1014 patterns alone
})


test_that("February 29 birthdays are handled correctly", {
  # 1. Setup: Create person born on Feb 29 (leap year birthday)
  # Interview in non-leap year (2023)
  data <- data.table::data.table(
    Ano = rep(2023L, 4),
    Trimestre = rep(1L, 4),  # Q1 = Jan-Mar
    UPA = rep(1L, 4),
    V1008 = rep(1L, 4),
    V1014 = rep(1L, 4),
    V2003 = 1:4,
    V2008 = 29L,      # Born on 29th
    V20081 = 2L,      # February
    V20082 = 2000L,   # Leap year
    V2009 = 23L       # Age 23 in 2023
  )

  # 2. Execute: Should not error with Feb 29 birthday in non-leap year
  expect_no_error({
    crosswalk <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 3. Verify: Returns valid structure
  expect_true(is.data.frame(crosswalk))
  expect_equal(nrow(crosswalk), nrow(data))

  # 4. Context: Implementation substitutes March 1 for Feb 29 in non-leap years
  # See pnadc-identify-periods.R line 307
})


test_that("birthday constraints narrow date ranges correctly", {
  # 1. Setup: Create controlled data to test birthday constraint logic
  # Person born March 15, interviewed in Q1 (Jan-Mar)
  set.seed(301)

  # Create Q1 2023 data
  data <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1:10,
    V1008 = 1L,
    V1014 = 1:10,
    V2003 = 1L,
    V2008 = 15L,      # Born on 15th of month
    V20081 = 3L,      # March
    V20082 = 1990L,
    V2009 = 32L       # Age 32 (depends on if birthday passed)
  )

  # 2. Execute: Identify periods
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 3. Verify: Some observations should be determined
  # With birthday on March 15, and interview in Q1, the month determination
  # may depend on whether interview was before or after birthday
  expect_true(is.data.frame(crosswalk))
  expect_true("ref_month_in_quarter" %in% names(crosswalk))

  # 4. Context: Birthday constraints help narrow date ranges
  # visit_before_birthday calculation adjusts date_min/date_max
  # This is tested indirectly here - direct testing would require accessing internals
})


test_that("age and birthday consistency is maintained", {
  # 1. Setup: Use realistic data generator which ensures age/birthday consistency
  set.seed(302)
  data <- create_realistic_pnadc(n_quarters = 2, n_upas = 15)

  # 2. Verify: Age is consistent with birth year
  # Calculate expected age range based on birth year and survey year
  data[V20082 != 9999, expected_age_range := Ano - V20082]  # Rough age

  # Age should be within 1 year of expected (depends on birthday)
  age_consistent <- data[V20082 != 9999,
                         all(abs(V2009 - expected_age_range) <= 1)]

  expect_true(age_consistent,
              label = "Age should be consistent with birth year (within 1 year)")

  # 3. Execute: Identification should work correctly with consistent data
  expect_no_error({
    crosswalk <- pnadc_identify_periods(data, verbose = FALSE)
  })

  # 4. Context: Fixing create_minimal_pnadc() ensures tests use realistic data
  # This test verifies the fix from Phase 1.1
})


test_that("mixed birthday data (some known, some unknown) is handled", {
  # 1. Setup: Create data with mix of known and unknown birthdays
  set.seed(303)
  data <- create_realistic_pnadc(n_quarters = 3, n_upas = 15)

  # Make 50% have unknown birthdays
  n_unknown <- ceiling(nrow(data) * 0.5)
  unknown_idx <- sample(seq_len(nrow(data)), size = n_unknown)
  data[unknown_idx, `:=`(
    V2008 = 99L,
    V20081 = 99L,
    V20082 = 9999L
  )]

  # 2. Execute: Should handle mixed data gracefully
  crosswalk <- pnadc_identify_periods(data, verbose = FALSE)

  # 3. Verify: Both groups (known and unknown birthdays) can be determined
  known_birthday_idx <- data[V20082 != 9999, which = TRUE]
  unknown_birthday_idx <- data[V20082 == 9999, which = TRUE]

  # Some with known birthdays should be determined
  known_det_rate <- crosswalk[known_birthday_idx, mean(determined_month, na.rm = TRUE)]

  # Some with unknown birthdays should also be determined (via V1014 patterns)
  unknown_det_rate <- crosswalk[unknown_birthday_idx, mean(determined_month, na.rm = TRUE)]

  # Both groups should have non-zero determination
  expect_true(known_det_rate > 0,
              label = "Observations with known birthdays can be determined")

  expect_true(unknown_det_rate > 0,
              label = "Observations with unknown birthdays can still be determined")

  # 4. Context: Algorithm uses multiple constraints, not just birthdays
})

#' @title Input Validation Utilities
#' @description Internal helper functions for validating PNADC input data.
#' @name utils-validation
#' @keywords internal
NULL

# ============================================================================
# IBGE PERIOD INVARIANT VALIDATION
# ============================================================================

#' Validate IBGE Period Crosswalk Invariants
#'
#' Ensures that period identification functions never produce invalid IBGE
#' calendar values. This function checks all invariants that must hold for
#' valid period assignments.
#'
#' @param crosswalk A data.table crosswalk with period columns
#' @param strict Logical. If TRUE (default), stops with an error on violations.
#'   If FALSE, issues warnings and returns a validation report.
#' @param context Character string describing where validation is being called
#'   from (for error messages). Default is "crosswalk".
#'
#' @return If \code{strict = TRUE}, returns invisibly if valid or stops with error.
#'   If \code{strict = FALSE}, returns a list with:
#'   \itemize{
#'     \item \code{valid}: Logical indicating if all invariants passed
#'     \item \code{violations}: Named list of violations found (empty if none)
#'   }
#'
#' @details
#' The following IBGE calendar invariants are checked:
#' \itemize{
#'   \item \strong{Valid ranges}: ref_week_in_quarter in [1,12] or NA,
#'     ref_fortnight_in_quarter in [1,6] or NA, ref_month_in_quarter in [1,3] or NA
#'   \item \strong{Nesting}: week requires fortnight, fortnight requires month
#'   \item \strong{Fortnight-month consistency}: fortnights 1-2 belong to month 1,
#'     3-4 to month 2, 5-6 to month 3
#'   \item \strong{Week-fortnight consistency}: weeks 1-2 belong to fortnight 1,
#'     3-4 to fortnight 2, etc.
#' }
#'
#' @keywords internal
#' @noRd
validate_period_invariants <- function(crosswalk, strict = TRUE, context = "crosswalk") {

  violations <- list()

  # -------------------------------------------------------------------------
  # INVARIANT 1: Valid ranges
  # -------------------------------------------------------------------------

  # Check ref_month_in_quarter in [1,3] or NA
  if ("ref_month_in_quarter" %in% names(crosswalk)) {
    invalid_months <- crosswalk[!is.na(ref_month_in_quarter) &
                                  (ref_month_in_quarter < 1L | ref_month_in_quarter > 3L)]
    if (nrow(invalid_months) > 0) {
      violations$invalid_month_values <- list(
        count = nrow(invalid_months),
        values = unique(invalid_months$ref_month_in_quarter),
        message = sprintf(
          "ref_month_in_quarter must be 1-3 or NA. Found %d rows with invalid values: %s",
          nrow(invalid_months),
          paste(unique(invalid_months$ref_month_in_quarter), collapse = ", ")
        )
      )
    }
  }

  # Check ref_fortnight_in_quarter in [1,6] or NA
  if ("ref_fortnight_in_quarter" %in% names(crosswalk)) {
    invalid_fortnights <- crosswalk[!is.na(ref_fortnight_in_quarter) &
                                      (ref_fortnight_in_quarter < 1L | ref_fortnight_in_quarter > 6L)]
    if (nrow(invalid_fortnights) > 0) {
      violations$invalid_fortnight_values <- list(
        count = nrow(invalid_fortnights),
        values = unique(invalid_fortnights$ref_fortnight_in_quarter),
        message = sprintf(
          "ref_fortnight_in_quarter must be 1-6 or NA. Found %d rows with invalid values: %s",
          nrow(invalid_fortnights),
          paste(unique(invalid_fortnights$ref_fortnight_in_quarter), collapse = ", ")
        )
      )
    }
  }

  # Check ref_week_in_quarter in [1,12] or NA
  if ("ref_week_in_quarter" %in% names(crosswalk)) {
    invalid_weeks <- crosswalk[!is.na(ref_week_in_quarter) &
                                 (ref_week_in_quarter < 1L | ref_week_in_quarter > 12L)]
    if (nrow(invalid_weeks) > 0) {
      violations$invalid_week_values <- list(
        count = nrow(invalid_weeks),
        values = unique(invalid_weeks$ref_week_in_quarter),
        message = sprintf(
          "ref_week_in_quarter must be 1-12 or NA. Found %d rows with invalid values: %s",
          nrow(invalid_weeks),
          paste(unique(invalid_weeks$ref_week_in_quarter), collapse = ", ")
        )
      )
    }
  }

  # -------------------------------------------------------------------------
  # INVARIANT 2: Nesting - week requires fortnight, fortnight requires month
  # -------------------------------------------------------------------------

  # Check: if week is determined, fortnight must be determined
  if (all(c("ref_week_in_quarter", "ref_fortnight_in_quarter") %in% names(crosswalk))) {
    week_without_fortnight <- crosswalk[!is.na(ref_week_in_quarter) &
                                          is.na(ref_fortnight_in_quarter)]
    if (nrow(week_without_fortnight) > 0) {
      violations$week_without_fortnight <- list(
        count = nrow(week_without_fortnight),
        message = sprintf(
          "Nesting violation: %d rows have week determined but fortnight is NA",
          nrow(week_without_fortnight)
        )
      )
    }
  }

  # Check: if fortnight is determined, month must be determined
  if (all(c("ref_fortnight_in_quarter", "ref_month_in_quarter") %in% names(crosswalk))) {
    fortnight_without_month <- crosswalk[!is.na(ref_fortnight_in_quarter) &
                                           is.na(ref_month_in_quarter)]
    if (nrow(fortnight_without_month) > 0) {
      violations$fortnight_without_month <- list(
        count = nrow(fortnight_without_month),
        message = sprintf(
          "Nesting violation: %d rows have fortnight determined but month is NA",
          nrow(fortnight_without_month)
        )
      )
    }
  }

  # -------------------------------------------------------------------------
  # INVARIANT 3: Fortnight-month consistency
  # Fortnights 1-2 belong to month 1, 3-4 to month 2, 5-6 to month 3
  # -------------------------------------------------------------------------

  if (all(c("ref_fortnight_in_quarter", "ref_month_in_quarter") %in% names(crosswalk))) {
    # Calculate expected month from fortnight: ((fortnight - 1) %/% 2) + 1
    fortnight_month_inconsistent <- crosswalk[
      !is.na(ref_fortnight_in_quarter) & !is.na(ref_month_in_quarter) &
        (((ref_fortnight_in_quarter - 1L) %/% 2L) + 1L) != ref_month_in_quarter
    ]
    if (nrow(fortnight_month_inconsistent) > 0) {
      violations$fortnight_month_mismatch <- list(
        count = nrow(fortnight_month_inconsistent),
        message = sprintf(
          "Fortnight-month inconsistency: %d rows have fortnight that doesn't match month",
          nrow(fortnight_month_inconsistent)
        )
      )
    }
  }

  # -------------------------------------------------------------------------
  # INVARIANT 4: Week-fortnight consistency
  # Weeks 1-2 belong to fortnight 1, 3-4 to fortnight 2, etc.
  # Formula: expected_fortnight = ((week - 1) %/% 2) + 1
  # -------------------------------------------------------------------------

  if (all(c("ref_week_in_quarter", "ref_fortnight_in_quarter") %in% names(crosswalk))) {
    # Calculate expected fortnight from week
    week_fortnight_inconsistent <- crosswalk[
      !is.na(ref_week_in_quarter) & !is.na(ref_fortnight_in_quarter) &
        (((ref_week_in_quarter - 1L) %/% 2L) + 1L) != ref_fortnight_in_quarter
    ]
    if (nrow(week_fortnight_inconsistent) > 0) {
      violations$week_fortnight_mismatch <- list(
        count = nrow(week_fortnight_inconsistent),
        message = sprintf(
          "Week-fortnight inconsistency: %d rows have week that doesn't match fortnight",
          nrow(week_fortnight_inconsistent)
        )
      )
    }
  }

  # -------------------------------------------------------------------------
  # INVARIANT 5: Week-month consistency (derived from above, but direct check)
  # Weeks 1-4 belong to month 1, 5-8 to month 2, 9-12 to month 3
  # -------------------------------------------------------------------------

  if (all(c("ref_week_in_quarter", "ref_month_in_quarter") %in% names(crosswalk))) {
    # Calculate expected month from week: ((week - 1) %/% 4) + 1
    week_month_inconsistent <- crosswalk[
      !is.na(ref_week_in_quarter) & !is.na(ref_month_in_quarter) &
        (((ref_week_in_quarter - 1L) %/% 4L) + 1L) != ref_month_in_quarter
    ]
    if (nrow(week_month_inconsistent) > 0) {
      violations$week_month_mismatch <- list(
        count = nrow(week_month_inconsistent),
        message = sprintf(
          "Week-month inconsistency: %d rows have week that doesn't match month",
          nrow(week_month_inconsistent)
        )
      )
    }
  }

  # -------------------------------------------------------------------------
  # RETURN RESULT
  # -------------------------------------------------------------------------

  result <- list(
    valid = length(violations) == 0,
    violations = violations
  )

  if (strict && !result$valid) {
    # Build error message
    error_msgs <- vapply(violations, function(v) v$message, character(1))
    msg <- paste0(
      "IBGE period invariant violations in ", context, ":\n",
      paste("  - ", error_msgs, collapse = "\n")
    )
    stop(msg, call. = FALSE)
  }

  if (!strict && !result$valid) {
    # Issue warnings
    for (violation in violations) {
      warning(paste0(context, ": ", violation$message), call. = FALSE)
    }
  }

  invisible(result)
}

#' Required Variables for Reference Period Identification
#'
#' Returns the minimum required column names for identifying reference periods
#' (month, fortnight, and week).
#'
#' @return Character vector of required column names
#' @keywords internal
#' @noRd
required_vars_ref_month <- function() {
  # V1008 is required for week identification (household-level aggregation)
  c("Ano", "Trimestre", "UPA", "V1008", "V1014", "V2008", "V20081", "V20082", "V2009")
}

#' Required Variables for Crosswalk Join Keys
#'
#' Returns the column names used as join keys in the output crosswalk.
#' The crosswalk is at household-quarter level (not person level),
#' so V2003 (person sequence) is NOT included.
#'
#' @return Character vector of join key column names
#' @keywords internal
#' @noRd
join_key_vars <- function() {
  # Note: Crosswalk is at household level (V1008), not person level (V2003)
  # All persons in a household share the same reference period
  c("Ano", "Trimestre", "UPA", "V1008", "V1014")
}

#' Required Variables for Weight Calibration
#'
#' Returns the additional column names required for computing calibrated weights.
#'
#' Note: Weight calibration operates at **person level** (not household level)
#' because calibration cells are based on individual attributes like age (V2009).
#' Although the crosswalk assigns reference periods at household level (all
#' household members share the same reference period), each person receives
#' a potentially different calibrated weight based on their age group.
#'
#' @return Character vector of required column names
#' @keywords internal
#' @noRd
required_vars_weights <- function() {
  c(
    # Survey design - V1028 (quarterly) or V1032 (annual) weights
    "V1028", "UF", "posest", "posest_sxi"
    # V2009 (age) is already required for ref_month identification
    # and is also used to define calibration cells (person-level)
  )
}

#' Validate PNADC Input Data
#'
#' Checks that input data has required columns for the specified processing.
#'
#' @param data A data.frame or data.table with PNADC microdata
#' @param check_weights Logical. If TRUE, also check for weight-related variables.
#' @param stop_on_error Logical. If TRUE, stops with an error. If FALSE, returns
#'   a validation report list.
#'
#' @return If \code{stop_on_error = TRUE}, returns invisibly if valid or stops with error.
#'   If \code{stop_on_error = FALSE}, returns a list with:
#'   \itemize{
#'     \item \code{valid}: Logical indicating if data passed all validations
#'     \item \code{issues}: Named list of validation issues found (empty if none)
#'     \item \code{n_rows}: Number of rows in input data
#'     \item \code{n_cols}: Number of columns in input data
#'     \item \code{join_keys_available}: Character vector of available join key columns
#'   }
#'
#' @details
#' The function performs the following validations:
#' \itemize{
#'   \item Checks for required columns for reference period identification:
#'     \code{Ano}, \code{Trimestre}, \code{UPA}, \code{V1008}, \code{V1014},
#'     \code{V2008}, \code{V20081}, \code{V20082}, \code{V2009}
#'   \item Validates year range (2012-2100 for PNADC coverage)
#'   \item Validates quarter values (must be 1-4)
#'   \item Validates birth day values (must be 1-31 or 99 for unknown)
#'   \item Validates birth month values (must be 1-12 or 99 for unknown)
#'   \item Warns about unusual ages (outside 0-130 range)
#'   \item If \code{check_weights = TRUE}, also validates weight-related columns:
#'     \code{V1028}, \code{UF}, \code{posest}, \code{posest_sxi}
#' }
#'
#' @seealso \code{\link{pnadc_identify_periods}} which calls this function
#'   internally to validate input data.
#'
#' @examples
#' # Minimal valid data (all 9 required columns)
#' sample_data <- data.frame(
#'   Ano = 2023L, Trimestre = 1L, UPA = 110000001L,
#'   V1008 = 1L, V1014 = 1L,
#'   V2008 = 15L, V20081 = 3L, V20082 = 1990L, V2009 = 33L
#' )
#' validate_pnadc(sample_data)
#'
#' # Data with missing columns returns issues (non-stop mode)
#' incomplete_data <- data.frame(Ano = 2023L, Trimestre = 1L)
#' result <- validate_pnadc(incomplete_data, stop_on_error = FALSE)
#' result$valid    # FALSE
#' result$issues   # lists missing columns
#'
#' @export
validate_pnadc <- function(data, check_weights = FALSE, stop_on_error = TRUE) {
  checkmate::assert_data_frame(data, min.rows = 1)

  issues <- list()

  # Check required columns for reference month
  required <- required_vars_ref_month()
  missing  <- setdiff(required, names(data))
  if (length(missing) > 0) {
    issues$missing_ref_month <- missing
  }

  # Check weight-related columns if requested
  if (check_weights) {
    required_wt <- required_vars_weights()
    missing_wt  <- setdiff(required_wt, names(data))
    if (length(missing_wt) > 0) {
      issues$missing_weights <- missing_wt
    }
  }

  # Check join key columns (V1008, V2003 may be optional in some cases)
  join_keys      <- join_key_vars()
  join_available <- intersect(join_keys, names(data))

  # Validate data types and ranges
  # Note: PNADC data may have character columns that need conversion
  if ("Ano" %in% names(data)) {
    years <- suppressWarnings(as.integer(unique(data$Ano)))
    years <- years[!is.na(years)]
    invalid_years <- years[years < 2012 | years > 2100]
    if (length(invalid_years) > 0) {
      issues$invalid_years <- invalid_years
    }
  }

  if ("Trimestre" %in% names(data)) {
    quarters <- suppressWarnings(as.integer(unique(data$Trimestre)))
    quarters <- quarters[!is.na(quarters)]
    invalid_quarters <- quarters[!quarters %in% 1:4]
    if (length(invalid_quarters) > 0) {
      issues$invalid_quarters <- invalid_quarters
    }
  }

  if ("V2008" %in% names(data)) {
    days <- suppressWarnings(as.integer(unique(data$V2008)))
    # Exclude NA and special codes (99 = unknown)
    days <- days[!is.na(days) & days != 99]
    invalid_days <- days[days < 1 | days > 31]
    if (length(invalid_days) > 0) {
      issues$invalid_birth_days <- invalid_days
    }
  }

  if ("V20081" %in% names(data)) {
    months <- suppressWarnings(as.integer(unique(data$V20081)))
    # Exclude NA and special codes (99 = unknown)
    months <- months[!is.na(months) & months != 99]
    invalid_months <- months[months < 1 | months > 12]
    if (length(invalid_months) > 0) {
      issues$invalid_birth_months <- invalid_months
    }
  }

  if ("V2009" %in% names(data)) {
    ages <- suppressWarnings(as.numeric(unique(data$V2009)))
    ages <- ages[!is.na(ages)]
    # Allow ages up to 130 (rare but possible; higher values are likely data errors)
    # These extreme values will be handled gracefully during processing
    invalid_ages <- ages[ages < 0 | ages > 130]
    if (length(invalid_ages) > 0) {
      issues$warning_ages <- paste("Unusual ages found:", paste(head(invalid_ages, 5), collapse = ", "),
                                    "- these will be processed but may affect results")
    }
  }

  # Prepare result
  result <- list(
    valid = length(issues) == 0 || all(grepl("^warning", names(issues))),
    issues = issues,
    n_rows = nrow(data),
    n_cols = ncol(data),
    join_keys_available = join_available
  )

  if (stop_on_error && !result$valid) {
    error_issues <- issues[!grepl("^warning", names(issues))]
    msg <- paste0(
      "PNADC data validation failed:\n",
      paste(
        vapply(names(error_issues), function(nm) {
          paste0("  - ", nm, ": ", paste(error_issues[[nm]], collapse = ", "))
        }, character(1)),
        collapse = "\n"
      )
    )
    stop(msg, call. = FALSE)
  }

  if (stop_on_error) {
    # Print warnings if any
    warnings <- issues[grepl("^warning", names(issues))]
    if (length(warnings) > 0) {
      for (nm in names(warnings)) {
        warning(warnings[[nm]], call. = FALSE)
      }
    }
    invisible(result)
  } else {
    result
  }
}

#' Validate Monthly Totals Data
#'
#' Checks that monthly totals data has required structure.
#'
#' @param monthly_totals A data.frame with monthly population totals
#' @param stop_on_error Logical. If TRUE, stops with an error.
#'
#' @return Validation result or stops with error
#' @keywords internal
#' @noRd
validate_monthly_totals <- function(monthly_totals, stop_on_error = TRUE) {
  checkmate::assert_data_frame(monthly_totals, min.rows = 1)

  issues <- list()

  # Check for required column (either ref_month_yyyymm or anomesexato)
  has_yyyymm <- "ref_month_yyyymm" %in% names(monthly_totals)
  has_anomesexato <- "anomesexato" %in% names(monthly_totals)

  if (!has_yyyymm && !has_anomesexato) {
    issues$missing_date <- "Need either 'ref_month_yyyymm' or 'anomesexato' column"
  }

  # Check for population column
  has_pop <- "m_populacao" %in% names(monthly_totals)
  if (!has_pop) {
    issues$missing_population <- "Need 'm_populacao' column with monthly population totals"
  }

  if (stop_on_error && length(issues) > 0) {
    msg <- paste0(
      "Monthly totals validation failed:\n",
      paste(
        vapply(names(issues), function(nm) {
          paste0("  - ", nm, ": ", issues[[nm]])
        }, character(1)),
        collapse = "\n"
      )
    )
    stop(msg, call. = FALSE)
  }

  invisible(length(issues) == 0)
}

#' Check for Data.table
#'
#' Ensures data is a data.table, converting if necessary.
#'
#' @param data A data.frame or data.table
#' @param copy Logical. If TRUE, always returns a copy. If FALSE, may return
#'   the original object if already a data.table.
#' @return data.table
#' @keywords internal
#' @noRd
ensure_data_table <- function(data, copy = FALSE) {
  if (data.table::is.data.table(data)) {
    if (copy) {
      data.table::copy(data)
    } else {
      data
    }
  } else {
    data.table::as.data.table(data)
  }
}


#' Subset Data to Required Columns and Copy
#'
#' Instead of copying the entire data.table (potentially 50+ columns),
#' this function first subsets to only the required columns, then copies.
#'
#' @param data A data.frame or data.table
#' @param required_cols Character vector of required column names
#' @param optional_cols Character vector of optional column names (included if present)
#' @return data.table with only the specified columns (always a copy)
#' @keywords internal
#' @noRd
subset_and_copy <- function(data, required_cols, optional_cols = NULL) {
  # Check required columns exist
  missing <- setdiff(required_cols, names(data))
  if (length(missing) > 0) {
    stop("Missing required columns: ", paste(missing, collapse = ", "), call. = FALSE)
  }

  # Build column list: required + available optional
  cols_to_keep <- required_cols
  if (!is.null(optional_cols)) {
    cols_to_keep <- unique(c(cols_to_keep, intersect(optional_cols, names(data))))
  }

  # Subset and copy in one operation
  # Use .SDcols to avoid R CMD check NOTE about ..cols_to_keep
  if (data.table::is.data.table(data)) {
    data.table::copy(data[, .SD, .SDcols = cols_to_keep])
  } else {
    data.table::as.data.table(data[, cols_to_keep, drop = FALSE])
  }
}

# Tests for validation utility functions

test_that("required_vars_ref_month returns expected columns", {
  vars <- required_vars_ref_month()
  expect_type(vars, "character")
  expect_true("Ano" %in% vars)
  expect_true("Trimestre" %in% vars)
  expect_true("UPA" %in% vars)
  expect_true("V2008" %in% vars)  # Birth day
  expect_true("V20081" %in% vars)  # Birth month
  expect_true("V20082" %in% vars)  # Birth year
  expect_true("V2009" %in% vars)   # Age
})

test_that("join_key_vars returns expected columns", {
  vars <- join_key_vars()
  expect_type(vars, "character")
  expect_true("Ano" %in% vars)
  expect_true("Trimestre" %in% vars)
  expect_true("UPA" %in% vars)
  expect_true("V1008" %in% vars)
  expect_true("V1014" %in% vars)
  # Note: V2003 (person number) is NOT in join_key_vars because
  # crosswalk is at household level (V1008), not person level
  expect_false("V2003" %in% vars)
})

test_that("required_vars_weights returns expected columns", {
  vars <- required_vars_weights()
  expect_type(vars, "character")
  expect_true("V1028" %in% vars)
  expect_true("UF" %in% vars)
  expect_true("posest" %in% vars)
  expect_true("posest_sxi" %in% vars)
})

test_that("validate_pnadc detects missing columns", {
  # Missing required columns
  bad_data <- data.frame(Ano = 2023, Trimestre = 1)
  expect_error(validate_pnadc(bad_data), "missing_ref_month")
})

test_that("validate_pnadc accepts valid minimal data", {
  valid_data <- data.frame(
    Ano = 2023,
    Trimestre = 1,
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = 33
  )
  # Should not error
 expect_silent(validate_pnadc(valid_data))
})

# Parameterized test for invalid field values
test_cases <- list(
  list(field = "Ano", value = 2000, error_pattern = "invalid_years",
       desc = "year (< 2012)"),
  list(field = "Trimestre", value = 5, error_pattern = "invalid_quarters",
       desc = "quarter (> 4)"),
  list(field = "V2008", value = 35, error_pattern = "invalid_birth_days",
       desc = "birth day (> 31)"),
  list(field = "V20081", value = 15, error_pattern = "invalid_birth_months",
       desc = "birth month (> 12)")
)

for (tc in test_cases) {
  test_that(paste("validate_pnadc detects invalid", tc$desc), {
    # Create valid base data
    bad_data <- data.frame(
      Ano = 2023,
      Trimestre = 1,
      UPA = 1,
      V1008 = 1,
      V1014 = 1,
      V2008 = 15,
      V20081 = 6,
      V20082 = 1990,
      V2009 = 33
    )
    # Override with invalid value
    bad_data[[tc$field]] <- tc$value

    expect_error(validate_pnadc(bad_data), tc$error_pattern)
  })
}

test_that("validate_pnadc returns report when stop_on_error = FALSE", {
  bad_data <- data.frame(Ano = 2023, Trimestre = 1)
  result <- validate_pnadc(bad_data, stop_on_error = FALSE)

  expect_type(result, "list")
  expect_false(result$valid)
  expect_true("missing_ref_month" %in% names(result$issues))
})

test_that("validate_pnadc reports valid data correctly", {
  valid_data <- data.frame(
    Ano = 2023,
    Trimestre = 1,
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = 33
  )
  result <- validate_pnadc(valid_data, stop_on_error = FALSE)

  expect_type(result, "list")
  expect_true(result$valid)
  expect_equal(result$n_rows, 1)
})

test_that("validate_pnadc checks weight columns when requested", {
  # Valid ref month data but missing weight columns
  data_no_weights <- data.frame(
    Ano = 2023,
    Trimestre = 1,
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = 33
  )

  # Should pass without check_weights
  expect_silent(validate_pnadc(data_no_weights, check_weights = FALSE))

  # Should fail with check_weights
  expect_error(validate_pnadc(data_no_weights, check_weights = TRUE), "missing_weights")
})

test_that("validate_monthly_totals accepts valid data", {
  valid_totals <- data.frame(
    ref_month_yyyymm = c(202301, 202302, 202303),
    m_populacao = c(200000, 200100, 200200)
  )
  # Should not error
  expect_silent(validate_monthly_totals(valid_totals))
})

test_that("validate_monthly_totals accepts anomesexato column", {
  valid_totals <- data.frame(
    anomesexato = c(202301, 202302, 202303),
    m_populacao = c(200000, 200100, 200200)
  )
  # Should not error
  expect_silent(validate_monthly_totals(valid_totals))
})

test_that("validate_monthly_totals detects missing date column", {
  bad_totals <- data.frame(
    m_populacao = c(200000, 200100, 200200)
  )
  expect_error(validate_monthly_totals(bad_totals), "missing_date")
})

test_that("validate_monthly_totals detects missing population column", {
  bad_totals <- data.frame(
    ref_month_yyyymm = c(202301, 202302, 202303)
  )
  expect_error(validate_monthly_totals(bad_totals), "missing_population")
})

test_that("ensure_data_table converts data.frame", {
  df <- data.frame(x = 1:3, y = letters[1:3])
  result <- ensure_data_table(df)

  expect_s3_class(result, "data.table")
  expect_equal(nrow(result), 3)
})

test_that("ensure_data_table returns data.table unchanged when copy = FALSE", {
  dt <- data.table::data.table(x = 1:3)
  result <- ensure_data_table(dt, copy = FALSE)

  # Should be the same object (not a copy)
  expect_identical(data.table::address(result), data.table::address(dt))
})

test_that("ensure_data_table returns copy when copy = TRUE", {
  dt <- data.table::data.table(x = 1:3)
  result <- ensure_data_table(dt, copy = TRUE)

  # Should be a different object
  expect_false(identical(data.table::address(result), data.table::address(dt)))
  expect_equal(result$x, dt$x)
})

# =============================================================================
# VALIDATE_PERIOD_INVARIANTS TESTS
# =============================================================================

test_that("validate_period_invariants accepts valid crosswalk", {
  # 1. Setup: Create valid crosswalk with proper nesting
  # Weeks 1-2 → Fortnight 1, Weeks 3-4 → Fortnight 2, etc.
  # Fortnights 1-2 → Month 1, Fortnights 3-4 → Month 2, Fortnights 5-6 → Month 3
  crosswalk <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1:10,
    V1008 = 1L,
    V2003 = 1L,
    ref_month_in_quarter = c(1L, 1L, 2L, 2L, 3L, 3L, NA, NA, NA, NA),
    ref_fortnight_in_quarter = c(1L, 1L, 3L, 3L, 5L, 5L, NA, NA, NA, NA),  # Match months
    ref_week_in_quarter = c(1L, 2L, 5L, 6L, 9L, 10L, NA, NA, NA, NA),      # Match fortnights
    determined_month = c(rep(TRUE, 6), rep(FALSE, 4)),
    determined_fortnight = c(rep(TRUE, 6), rep(FALSE, 4)),
    determined_week = c(rep(TRUE, 6), rep(FALSE, 4))
  )

  # 2. Verify: Should not error
  expect_no_error(
    PNADCperiods:::validate_period_invariants(crosswalk, context = "test")
  )
})

test_that("validate_period_invariants detects invalid month ranges", {
  # 1. Setup: Create crosswalk with invalid month (4, should be 1-3)
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, 2L, 3L, 4L),  # 4 is invalid!
    ref_fortnight_in_quarter = c(1L, 3L, 5L, NA),
    ref_week_in_quarter = c(1L, 3L, 5L, NA),
    determined_month = c(TRUE, TRUE, TRUE, TRUE),
    determined_fortnight = c(TRUE, TRUE, TRUE, FALSE),
    determined_week = c(TRUE, TRUE, TRUE, FALSE)
  )

  # 2. Verify: Should error on invalid month value
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "ref_month_in_quarter must be 1-3",
    label = "Invalid month value (4) should be detected"
  )
})

test_that("validate_period_invariants detects invalid fortnight ranges", {
  # 1. Setup: Create crosswalk with invalid fortnight (7, should be 1-6)
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, 2L, 3L),
    ref_fortnight_in_quarter = c(1L, 3L, 7L),  # 7 is invalid!
    ref_week_in_quarter = c(1L, 3L, 5L),
    determined_month = c(TRUE, TRUE, TRUE),
    determined_fortnight = c(TRUE, TRUE, TRUE),
    determined_week = c(TRUE, TRUE, TRUE)
  )

  # 2. Verify: Should error on invalid fortnight value
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "ref_fortnight_in_quarter must be 1-6",
    label = "Invalid fortnight value (7) should be detected"
  )
})

test_that("validate_period_invariants detects invalid week ranges", {
  # 1. Setup: Create crosswalk with invalid week (13, should be 1-12)
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, 2L, 3L),
    ref_fortnight_in_quarter = c(1L, 3L, 5L),
    ref_week_in_quarter = c(1L, 5L, 13L),  # 13 is invalid!
    determined_month = c(TRUE, TRUE, TRUE),
    determined_fortnight = c(TRUE, TRUE, TRUE),
    determined_week = c(TRUE, TRUE, TRUE)
  )

  # 2. Verify: Should error on invalid week value
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "ref_week_in_quarter must be 1-12",
    label = "Invalid week value (13) should be detected"
  )
})

test_that("validate_period_invariants detects week without fortnight (nesting violation)", {
  # 1. Setup: Week determined but fortnight is NA (violates nesting)
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, 2L),
    ref_fortnight_in_quarter = c(1L, NA),  # Fortnight NA
    ref_week_in_quarter = c(1L, 3L),       # But week determined!
    determined_month = c(TRUE, TRUE),
    determined_fortnight = c(TRUE, FALSE),  # Fortnight not determined
    determined_week = c(TRUE, TRUE)         # But week is determined - VIOLATION
  )

  # 2. Verify: Should error on nesting violation
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "Nesting violation.*week determined but fortnight is NA",
    label = "Week without fortnight violates nesting"
  )
})

test_that("validate_period_invariants detects fortnight without month (nesting violation)", {
  # 1. Setup: Fortnight determined but month is NA
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, NA),       # Month NA
    ref_fortnight_in_quarter = c(1L, 3L),   # But fortnight determined!
    ref_week_in_quarter = c(1L, NA),
    determined_month = c(TRUE, FALSE),       # Month not determined
    determined_fortnight = c(TRUE, TRUE),    # But fortnight is - VIOLATION
    determined_week = c(TRUE, FALSE)
  )

  # 2. Verify: Should error on nesting violation
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "Nesting violation.*fortnight determined but month is NA",
    label = "Fortnight without month violates nesting"
  )
})

test_that("validate_period_invariants detects week-fortnight mismatch", {
  # 1. Setup: Week and fortnight inconsistent
  # Week 1-2 should be in fortnight 1, but marked as fortnight 3
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L),
    ref_fortnight_in_quarter = c(3L),  # Fortnight 3
    ref_week_in_quarter = c(1L),       # Week 1 - should be in fortnight 1!
    determined_month = c(TRUE),
    determined_fortnight = c(TRUE),
    determined_week = c(TRUE)
  )

  # 2. Verify: Should error on inconsistency
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "Week-fortnight inconsistency",
    label = "Week-fortnight mismatch should be detected"
  )
})

test_that("validate_period_invariants detects fortnight-month mismatch", {
  # 1. Setup: Fortnight and month inconsistent
  # Fortnights 1-2 should be in month 1, but marked as month 3
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(3L),       # Month 3
    ref_fortnight_in_quarter = c(1L),   # Fortnight 1 - should be in month 1!
    ref_week_in_quarter = c(1L),
    determined_month = c(TRUE),
    determined_fortnight = c(TRUE),
    determined_week = c(TRUE)
  )

  # 2. Verify: Should error on inconsistency
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk),
    "Fortnight-month inconsistency",
    label = "Fortnight-month mismatch should be detected"
  )
})

test_that("validate_period_invariants strict=FALSE allows warnings", {
  # 1. Setup: Create crosswalk with minor issue
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, 2L, 4L),  # 4 is invalid
    ref_fortnight_in_quarter = c(1L, 3L, NA),
    ref_week_in_quarter = c(1L, 3L, NA),
    determined_month = c(TRUE, TRUE, TRUE),
    determined_fortnight = c(TRUE, TRUE, FALSE),
    determined_week = c(TRUE, TRUE, FALSE)
  )

  # 2. Verify: strict=FALSE should warn, not error
  expect_warning(
    PNADCperiods:::validate_period_invariants(crosswalk, strict = FALSE),
    "ref_month_in_quarter must be 1-3",
    label = "strict=FALSE should warn instead of error"
  )
})

test_that("validate_period_invariants provides context in error messages", {
  # 1. Setup: Invalid crosswalk
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(4L),  # Invalid
    ref_fortnight_in_quarter = c(NA),
    ref_week_in_quarter = c(NA),
    determined_month = c(TRUE),
    determined_fortnight = c(FALSE),
    determined_week = c(FALSE)
  )

  # 2. Verify: Context appears in error message
  expect_error(
    PNADCperiods:::validate_period_invariants(crosswalk, context = "my_test"),
    "my_test.*ref_month_in_quarter",
    label = "Context should appear in error message"
  )
})

# =============================================================================
# VALIDATE_PNADC ADDITIONAL TESTS
# =============================================================================

test_that("validate_pnadc with stop_on_error=FALSE returns issues for invalid years", {
  # 1. Setup: Data with invalid year
  bad_data <- data.frame(
    Ano = 2000,  # Before 2012 (PNADC started 2012)
    Trimestre = 1,
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = 33
  )

  # 2. Execute: validate with stop_on_error=FALSE
  result <- validate_pnadc(bad_data, stop_on_error = FALSE)

  # 3. Verify: Should return list with issues
  expect_type(result, "list")
  expect_false(result$valid)
  expect_true("invalid_years" %in% names(result$issues))
})

test_that("validate_pnadc with stop_on_error=FALSE returns issues for invalid quarters", {
  # 1. Setup: Data with invalid quarter
  bad_data <- data.frame(
    Ano = 2023,
    Trimestre = 5,  # Invalid (should be 1-4)
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = 33
  )

  # 2. Execute: validate with stop_on_error=FALSE
  result <- validate_pnadc(bad_data, stop_on_error = FALSE)

  # 3. Verify: Should return list with issues
  expect_false(result$valid)
  expect_true("invalid_quarters" %in% names(result$issues))
})

test_that("validate_pnadc detects unusual ages as warnings only", {
  # 1. Setup: Data with unusual age
  data_unusual_age <- data.frame(
    Ano = 2023,
    Trimestre = 1,
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = 150  # Unusual age (>130)
  )

  # 2. Execute: Should produce warning but still validate
  expect_warning(
    result <- validate_pnadc(data_unusual_age, stop_on_error = TRUE),
    "Unusual ages|warning_ages"
  )

  # 3. With stop_on_error=FALSE, should still return valid=TRUE (only warning)
  result <- suppressWarnings(validate_pnadc(data_unusual_age, stop_on_error = FALSE))
  expect_true(result$valid,
              label = "Unusual ages are warnings only, not validation failures")
})

test_that("validate_pnadc detects negative ages", {
  # 1. Setup: Data with negative age
  data_negative_age <- data.frame(
    Ano = 2023,
    Trimestre = 1,
    UPA = 1,
    V1008 = 1,
    V1014 = 1,
    V2008 = 15,
    V20081 = 6,
    V20082 = 1990,
    V2009 = -5  # Negative age
  )

  # 2. Execute: Should produce warning
  expect_warning(
    validate_pnadc(data_negative_age, stop_on_error = TRUE),
    "Unusual ages|warning_ages"
  )
})

# =============================================================================
# SUBSET_AND_COPY TESTS
# =============================================================================

test_that("subset_and_copy errors on missing required columns", {
  dt <- data.table::data.table(a = 1:3, b = 4:6)

  expect_error(
    PNADCperiods:::subset_and_copy(dt, required_cols = c("a", "c")),
    "Missing required columns.*c"
  )
})

test_that("subset_and_copy selects required columns from data.table", {
  dt <- data.table::data.table(a = 1:3, b = 4:6, c = 7:9)

  result <- PNADCperiods:::subset_and_copy(dt, required_cols = c("a", "b"))

  expect_s3_class(result, "data.table")
  expect_equal(names(result), c("a", "b"))
  expect_equal(nrow(result), 3)

  # Should be a copy (not same address)
  expect_false(identical(data.table::address(result), data.table::address(dt)))
})

test_that("subset_and_copy converts data.frame to data.table", {
  df <- data.frame(a = 1:3, b = 4:6, c = 7:9)

  result <- PNADCperiods:::subset_and_copy(df, required_cols = c("a", "b"))

  expect_s3_class(result, "data.table")
  expect_equal(names(result), c("a", "b"))
  expect_equal(nrow(result), 3)
})

test_that("subset_and_copy includes available optional columns", {
  dt <- data.table::data.table(a = 1:3, b = 4:6, c = 7:9)

  # Optional cols: c is available, d is not
  result <- PNADCperiods:::subset_and_copy(
    dt,
    required_cols = c("a"),
    optional_cols = c("c", "d")
  )

  expect_equal(sort(names(result)), c("a", "c"))
  expect_false("d" %in% names(result))
})

test_that("subset_and_copy with NULL optional_cols returns only required", {
  dt <- data.table::data.table(a = 1:3, b = 4:6)

  result <- PNADCperiods:::subset_and_copy(dt, required_cols = c("a"))

  expect_equal(names(result), "a")
})

# =============================================================================
# VALIDATE_MONTHLY_TOTALS ADDITIONAL TESTS
# =============================================================================

test_that("validate_monthly_totals stop_on_error=FALSE returns FALSE for invalid data", {
  bad_totals <- data.frame(m_populacao = c(200000, 200100))

  # Function returns invisible(logical): FALSE when issues found
  result <- validate_monthly_totals(bad_totals, stop_on_error = FALSE)

  expect_type(result, "logical")
  expect_false(result)
})

test_that("validate_monthly_totals stop_on_error=FALSE returns TRUE for valid data", {
  valid_totals <- data.frame(
    ref_month_yyyymm = c(202301, 202302, 202303),
    m_populacao = c(200000, 200100, 200200)
  )

  # Function returns invisible(logical): TRUE when valid
  result <- validate_monthly_totals(valid_totals, stop_on_error = FALSE)

  expect_type(result, "logical")
  expect_true(result)
})

# =============================================================================
# VALIDATE_PERIOD_INVARIANTS ADDITIONAL TESTS
# =============================================================================

test_that("validate_period_invariants handles crosswalk missing period columns", {
  # Crosswalk without any period columns
  crosswalk <- data.table::data.table(
    Ano = 2023L,
    Trimestre = 1L,
    UPA = 1:5
  )

  # Should not error when period columns are absent (gracefully skips checks)
  expect_no_error(
    PNADCperiods:::validate_period_invariants(crosswalk, context = "test")
  )
})

test_that("validate_period_invariants strict=FALSE returns structured report", {
  # Create invalid crosswalk
  crosswalk <- data.table::data.table(
    ref_month_in_quarter = c(1L, 4L),  # 4 is invalid
    ref_fortnight_in_quarter = c(1L, NA),
    ref_week_in_quarter = c(1L, NA),
    determined_month = c(TRUE, TRUE),
    determined_fortnight = c(TRUE, FALSE),
    determined_week = c(TRUE, FALSE)
  )

  result <- suppressWarnings(
    PNADCperiods:::validate_period_invariants(crosswalk, strict = FALSE)
  )

  expect_type(result, "list")
  expect_false(result$valid)
  expect_true(length(result$violations) > 0)
})

test_that("validate_pnadc handles empty data gracefully", {
  # 1. Setup: Empty data.frame with correct columns
  empty_data <- data.frame(
    Ano = integer(0),
    Trimestre = integer(0),
    UPA = integer(0),
    V1008 = integer(0),
    V1014 = integer(0),
    V2008 = integer(0),
    V20081 = integer(0),
    V20082 = integer(0),
    V2009 = integer(0)
  )

  # 2. Verify: Should error on min.rows constraint
  # checkmate produces error about "at least 1 rows"
  expect_error(
    validate_pnadc(empty_data),
    "at least 1 rows|min.rows",
    label = "Empty data should fail min.rows check"
  )
})

#' SIDRA Series Metadata for PNADC Mensalization
#'
#' This file contains metadata definitions for all 86 SIDRA series used in the
#' mensalization process. It maps series names to their SIDRA API endpoints,
#' table IDs, variable codes, and hierarchical categorization.
#'
#' @name sidra-series-metadata
#' @keywords internal
NULL

#' Get SIDRA Series Metadata
#'
#' Returns a data.table with metadata for all PNADC rolling quarter series
#' available from IBGE's SIDRA API.
#'
#' @param series Character vector of series names to retrieve, or "all" (default)
#'   for all series.
#' @param theme Character vector of themes to filter by. Valid themes:
#'   "labor_market", "earnings", "demographics", "social_protection", "prices".
#'   Use NULL (default) for no filtering.
#' @param theme_category Character vector of theme categories to filter by.
#'   Use NULL (default) for no filtering.
#' @param subcategory Character vector of subcategories to filter by.
#'   Use NULL (default) for no filtering.
#' @param lang Character. Language for descriptions: "pt" (Portuguese, default)
#'   or "en" (English). When "en", the `description` column contains English text.
#'
#' @return A data.table with columns:
#'   \describe{
#'     \item{series_name}{Character. Internal name used in the package}
#'     \item{api_path}{Character. SIDRA API path for get_sidra()}
#'     \item{table_id}{Integer. SIDRA table number}
#'     \item{variable_id}{Integer. SIDRA variable code}
#'     \item{theme}{Character. Top-level theme}
#'     \item{theme_category}{Character. Middle-level category within theme}
#'     \item{subcategory}{Character. Optional subcategory for filtering}
#'     \item{description_pt}{Character. Portuguese description}
#'     \item{description_en}{Character. English description}
#'     \item{description}{Character. Description in the requested language}
#'     \item{unit}{Character. Unit of measurement}
#'     \item{unit_label_pt}{Character. Unit label in Portuguese}
#'     \item{unit_label_en}{Character. Unit label in English}
#'     \item{is_derived}{Logical. TRUE if computed from other series}
#'     \item{requires_deflation}{Logical. TRUE if needs IPCA deflation}
#'   }
#'
#' @examples
#' meta <- get_sidra_series_metadata()
#' head(meta)
#'
#' labor <- get_sidra_series_metadata(theme = "labor_market")
#'
#' unemp <- get_sidra_series_metadata(theme = "labor_market",
#'                                     theme_category = "unemployment")
#'
#' meta_en <- get_sidra_series_metadata(series = c("taxadesocup", "popocup"),
#'                                      lang = "en")
#'
#' @export
get_sidra_series_metadata <- function(series = "all",
                                       theme = NULL,
                                       theme_category = NULL,
                                       subcategory = NULL,
                                       lang = "pt") {

  # Build complete metadata table

  meta <- data.table::data.table(
    series_name = character(),
    api_path = character(),
    table_id = integer(),
    variable_id = integer(),
    classification_id = character(),
    classification_value = character(),
    theme = character(),
    theme_category = character(),
    subcategory = character(),
    description_pt = character(),
    description_en = character(),
    unit = character(),
    unit_label_pt = character(),
    unit_label_en = character(),
    is_derived = logical(),
    requires_deflation = logical()
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Participation (Rates)
  # ============================================================================

  participation_rates <- data.table::data.table(
    series_name = c("taxapartic", "nivelocup", "niveldesocup"),
    api_path = c(
      "/t/5944/n1/all/v/4096/p/all/d/v4096%201",
      "/t/6379/n1/all/v/4097/p/all/d/v4097%201",
      "/t/6380/n1/all/v/4098/p/all/d/v4098%201"
    ),
    table_id = c(5944L, 6379L, 6380L),
    variable_id = c(4096L, 4097L, 4098L),
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "labor_market",
    theme_category = "participation",
    subcategory = "rates",
    description_pt = c(
      "Taxa de participacao na forca de trabalho",
      "Nivel de ocupacao",
      "Nivel de desocupacao"
    ),
    description_en = c(
      "Labor force participation rate",
      "Employment-population ratio",
      "Unemployment-population ratio"
    ),
    unit = "percent",
    unit_label_pt = "%",
    unit_label_en = "%",
    is_derived = TRUE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Participation (Levels)
  # ============================================================================

  participation_levels <- data.table::data.table(
    series_name = c("popnaforca", "popocup", "popdesocup", "popforadaforca"),
    api_path = c(
      "/t/6318/n1/all/v/1641/p/all/c629/32386",
      "/t/6318/n1/all/v/1641/p/all/c629/32387",
      "/t/6318/n1/all/v/1641/p/all/c629/32446",
      "/t/6318/n1/all/v/1641/p/all/c629/32447"
    ),
    table_id = 6318L,
    variable_id = 1641L,
    classification_id = "c629",
    classification_value = c("32386", "32387", "32446", "32447"),
    theme = "labor_market",
    theme_category = "participation",
    subcategory = "levels",
    description_pt = c(
      "Populacao na forca de trabalho",
      "Populacao ocupada",
      "Populacao desocupada",
      "Populacao fora da forca de trabalho"
    ),
    description_en = c(
      "Labor force",
      "Employed population",
      "Unemployed population",
      "Population not in labor force"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Unemployment (Rates)
  # ============================================================================

  unemployment_rates <- data.table::data.table(
    series_name = "taxadesocup",
    api_path = "/t/6381/n1/all/v/4099/p/all/d/v4099%201",
    table_id = 6381L,
    variable_id = 4099L,
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "labor_market",
    theme_category = "unemployment",
    subcategory = "rates",
    description_pt = "Taxa de desocupacao",
    description_en = "Unemployment rate",
    unit = "percent",
    unit_label_pt = "%",
    unit_label_en = "%",
    is_derived = TRUE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Underutilization (Rates)
  # ============================================================================

  underutilization_rates <- data.table::data.table(
    series_name = c("perccontribprev", "taxacombdesosub", "taxacombdesopot",
                    "taxacompsubutlz", "taxasubocuphoras", "percdesalento"),
    api_path = c(
      "/t/3919/n1/all/v/8463/p/all/d/v8463%201",
      "/t/6439/n1/all/v/4114/p/all/d/v4114%201",
      "/t/6440/n1/all/v/4116/p/all/d/v4116%201",
      "/t/6441/n1/all/v/4118/p/all/d/v4118%201",
      "/t/6785/n1/all/v/9819/p/all/d/v9819%201",
      "/t/6807/n1/all/v/9869/p/all/d/v9869%201"
    ),
    table_id = c(3919L, 6439L, 6440L, 6441L, 6785L, 6807L),
    variable_id = c(8463L, 4114L, 4116L, 4118L, 9819L, 9869L),
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "labor_market",
    theme_category = "underutilization",
    subcategory = "rates",
    description_pt = c(
      "Percentual de contribuintes para a previdencia",
      "Taxa combinada de desocupacao e subocupacao",
      "Taxa combinada de desocupacao e forca potencial",
      "Taxa composta de subutilizacao",
      "Taxa de subocupacao por insuficiencia de horas",
      "Percentual de desalentados na forca potencial"
    ),
    description_en = c(
      "Social security contribution rate",
      "Combined unemployment and underemployment rate",
      "Combined unemployment and potential labor force rate",
      "Composite underutilization rate",
      "Time-related underemployment rate",
      "Discouraged workers as share of potential labor force"
    ),
    unit = "percent",
    unit_label_pt = "%",
    unit_label_en = "%",
    is_derived = TRUE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Underutilization (Levels)
  # ============================================================================

  underutilization_levels <- data.table::data.table(
    series_name = c("subocuphoras", "forcapotencial", "forcaampliada", "desalentado"),
    api_path = c(
      "/t/6438/n1/all/v/1641/p/all/c604/31751",
      "/t/6438/n1/all/v/1641/p/all/c604/31752",
      "/t/6438/n1/all/v/1641/p/all/c604/40287",
      "/t/6438/n1/all/v/1641/p/all/c604/46254"
    ),
    table_id = 6438L,
    variable_id = 1641L,
    classification_id = "c604",
    classification_value = c("31751", "31752", "40287", "46254"),
    theme = "labor_market",
    theme_category = "underutilization",
    subcategory = "levels",
    description_pt = c(
      "Subocupados por insuficiencia de horas",
      "Forca de trabalho potencial",
      "Forca de trabalho ampliada",
      "Desalentados"
    ),
    description_en = c(
      "Time-related underemployed",
      "Potential labor force",
      "Extended labor force",
      "Discouraged workers"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Employment Type (Employees)
  # ============================================================================

  employment_employees <- data.table::data.table(
    series_name = c(
      "empregado", "empregpriv", "empregprivcomcart", "empregprivsemcart",
      "empregpubl", "empregpublcomcart", "empregpublsemcart", "estatutmilitar"
    ),
    api_path = c(
      "/t/6320/n1/all/v/4090/p/all/c11913/96166",
      "/t/6320/n1/all/v/4090/p/all/c11913/31721",
      "/t/6320/n1/all/v/4090/p/all/c11913/31722",
      "/t/6320/n1/all/v/4090/p/all/c11913/31723",
      "/t/6320/n1/all/v/4090/p/all/c11913/31727",
      "/t/6320/n1/all/v/4090/p/all/c11913/31728",
      "/t/6320/n1/all/v/4090/p/all/c11913/31729",
      "/t/6320/n1/all/v/4090/p/all/c11913/31730"
    ),
    table_id = 6320L,
    variable_id = 4090L,
    classification_id = "c11913",
    classification_value = c("96166", "31721", "31722", "31723", "31727", "31728", "31729", "31730"),
    theme = "labor_market",
    theme_category = "employment_type",
    subcategory = "employees",
    description_pt = c(
      "Empregado (total)",
      "Empregado no setor privado",
      "Empregado no setor privado com carteira",
      "Empregado no setor privado sem carteira",
      "Empregado no setor publico",
      "Empregado no setor publico com carteira",
      "Empregado no setor publico sem carteira",
      "Militar e servidor estatutario"
    ),
    description_en = c(
      "Employees (total)",
      "Private sector employees",
      "Private sector employees with formal contract",
      "Private sector employees without formal contract",
      "Public sector employees",
      "Public sector employees with formal contract",
      "Public sector employees without formal contract",
      "Military and statutory civil servants"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Employment Type (Domestic Workers)
  # ============================================================================

  employment_domestic <- data.table::data.table(
    series_name = c("domestico", "domesticocomcart", "domesticosemcart"),
    api_path = c(
      "/t/6320/n1/all/v/4090/p/all/c11913/31724",
      "/t/6320/n1/all/v/4090/p/all/c11913/31725",
      "/t/6320/n1/all/v/4090/p/all/c11913/31726"
    ),
    table_id = 6320L,
    variable_id = 4090L,
    classification_id = "c11913",
    classification_value = c("31724", "31725", "31726"),
    theme = "labor_market",
    theme_category = "employment_type",
    subcategory = "domestic",
    description_pt = c(
      "Trabalhador domestico",
      "Trabalhador domestico com carteira",
      "Trabalhador domestico sem carteira"
    ),
    description_en = c(
      "Domestic workers",
      "Domestic workers with formal contract",
      "Domestic workers without formal contract"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Employment Type (Employers)
  # ============================================================================

  employment_employers <- data.table::data.table(
    series_name = c("empregador", "empregadorcomcnpj", "empregadorsemcnpj"),
    api_path = c(
      "/t/6320/n1/all/v/4090/p/all/c11913/96170",
      "/t/6320/n1/all/v/4090/p/all/c11913/45934",
      "/t/6320/n1/all/v/4090/p/all/c11913/45935"
    ),
    table_id = 6320L,
    variable_id = 4090L,
    classification_id = "c11913",
    classification_value = c("96170", "45934", "45935"),
    theme = "labor_market",
    theme_category = "employment_type",
    subcategory = "employers",
    description_pt = c(
      "Empregador (total)",
      "Empregador com CNPJ",
      "Empregador sem CNPJ"
    ),
    description_en = c(
      "Employers (total)",
      "Employers with business registration (CNPJ)",
      "Employers without business registration"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Employment Type (Self-Employed)
  # ============================================================================

  employment_self_employed <- data.table::data.table(
    series_name = c("contapropria", "contapropriacomcnpj", "contapropriasemcnpj"),
    api_path = c(
      "/t/6320/n1/all/v/4090/p/all/c11913/96171",
      "/t/6320/n1/all/v/4090/p/all/c11913/45936",
      "/t/6320/n1/all/v/4090/p/all/c11913/45937"
    ),
    table_id = 6320L,
    variable_id = 4090L,
    classification_id = "c11913",
    classification_value = c("96171", "45936", "45937"),
    theme = "labor_market",
    theme_category = "employment_type",
    subcategory = "self_employed",
    description_pt = c(
      "Conta propria (total)",
      "Conta propria com CNPJ",
      "Conta propria sem CNPJ"
    ),
    description_en = c(
      "Self-employed (total)",
      "Self-employed with business registration (CNPJ)",
      "Self-employed without business registration"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Employment Type (Family Workers)
  # ============================================================================

  employment_family <- data.table::data.table(
    series_name = "trabfamauxiliar",
    api_path = "/t/6320/n1/all/v/4090/p/all/c11913/31731",
    table_id = 6320L,
    variable_id = 4090L,
    classification_id = "c11913",
    classification_value = "31731",
    theme = "labor_market",
    theme_category = "employment_type",
    subcategory = "family_workers",
    description_pt = "Trabalhador familiar auxiliar",
    description_en = "Unpaid family workers",
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: LABOR_MARKET - Economic Sector
  # ============================================================================

  economic_sector <- data.table::data.table(
    series_name = c(
      "agropecuaria", "industria", "construcao", "comercio", "transporte",
      "alojaliment", "infcomfinimobadm", "adminpublica", "outroservico", "servicodomestico"
    ),
    api_path = c(
      "/t/6323/n1/all/v/4090/p/all/c888/47947",
      "/t/6323/n1/all/v/4090/p/all/c888/47948",
      "/t/6323/n1/all/v/4090/p/all/c888/47949",
      "/t/6323/n1/all/v/4090/p/all/c888/47950",
      "/t/6323/n1/all/v/4090/p/all/c888/56622",
      "/t/6323/n1/all/v/4090/p/all/c888/56623",
      "/t/6323/n1/all/v/4090/p/all/c888/56624",
      "/t/6323/n1/all/v/4090/p/all/c888/60032",
      "/t/6323/n1/all/v/4090/p/all/c888/56627",
      "/t/6323/n1/all/v/4090/p/all/c888/56628"
    ),
    table_id = 6323L,
    variable_id = 4090L,
    classification_id = "c888",
    classification_value = c(
      "47947", "47948", "47949", "47950", "56622",
      "56623", "56624", "60032", "56627", "56628"
    ),
    theme = "labor_market",
    theme_category = "economic_sector",
    subcategory = NA_character_,
    description_pt = c(
      "Ocupados na agropecuaria",
      "Ocupados na industria",
      "Ocupados na construcao",
      "Ocupados no comercio",
      "Ocupados em transporte, armazenagem e correio",
      "Ocupados em alojamento e alimentacao",
      "Ocupados em informacao, comunicacao, financeiras, imobiliarias e administrativas",
      "Ocupados na administracao publica, defesa, seguridade, educacao, saude e servicos sociais",
      "Ocupados em outros servicos",
      "Ocupados em servicos domesticos"
    ),
    description_en = c(
      "Employed in agriculture",
      "Employed in industry",
      "Employed in construction",
      "Employed in commerce",
      "Employed in transport, storage and mail",
      "Employed in accommodation and food services",
      "Employed in information, communication, finance, real estate and admin",
      "Employed in public admin, defense, social security, education, health and social services",
      "Employed in other services",
      "Employed in domestic services"
    ),
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: EARNINGS - Average Usual (All Jobs)
  # ============================================================================

  earnings_usual_all <- data.table::data.table(
    series_name = c("rendhabnominaltodos", "rendhabrealtodos",
                    "rendhabrealprinc", "rendefetrealprinc"),
    api_path = c(
      "/t/6390/n1/all/v/5929/p/all",
      "/t/6390/n1/all/v/5933/p/all",
      "/t/6389/n1/all/v/5932/p/all/c11913/96165",
      "/t/6388/n1/all/v/5934/p/all"
    ),
    table_id = c(6390L, 6390L, 6389L, 6388L),
    variable_id = c(5929L, 5933L, 5932L, 5934L),
    classification_id = c(NA_character_, NA_character_, "c11913", NA_character_),
    classification_value = c(NA_character_, NA_character_, "96165", NA_character_),
    theme = "earnings",
    theme_category = "average_usual",
    subcategory = "all_jobs",
    description_pt = c(
      "Rendimento habitual nominal medio de todos os trabalhos",
      "Rendimento habitual real medio de todos os trabalhos",
      "Rendimento habitual real medio do trabalho principal",
      "Rendimento efetivo real medio do trabalho principal"
    ),
    description_en = c(
      "Average usual nominal earnings from all jobs",
      "Average usual real earnings from all jobs",
      "Average usual real earnings from main job",
      "Average effective real earnings from main job"
    ),
    unit = "currency",
    unit_label_pt = "R$",
    unit_label_en = "BRL",
    is_derived = FALSE,
    requires_deflation = c(FALSE, TRUE, TRUE, TRUE)
  )

  # ============================================================================
  # THEME: EARNINGS - Average Usual (By Employment Type)
  # ============================================================================

  earnings_usual_by_type <- data.table::data.table(
    series_name = c(
      "rhrpempregado", "rhrpempregpriv", "rhrpempregprivcomcart", "rhrpempregprivsemcart",
      "rhrpdomestico", "rhrpdomesticocomcart", "rhrpdomesticosemcart",
      "rhrpempregpubl", "rhrpempregpublcomcart", "rhrpempregpublsemcart", "rhrpestatutmilitar",
      "rhrpempregador", "rhrpempregadorcomcnpj", "rhrpempregadorsemcnpj",
      "rhrpcontapropria", "rhrpcontapropriacomcnpj", "rhrpcontapropriasemcnpj"
    ),
    api_path = c(
      "/t/6389/n1/all/v/5932/p/all/c11913/96166",
      "/t/6389/n1/all/v/5932/p/all/c11913/31721",
      "/t/6389/n1/all/v/5932/p/all/c11913/31722",
      "/t/6389/n1/all/v/5932/p/all/c11913/31723",
      "/t/6389/n1/all/v/5932/p/all/c11913/31724",
      "/t/6389/n1/all/v/5932/p/all/c11913/31725",
      "/t/6389/n1/all/v/5932/p/all/c11913/31726",
      "/t/6389/n1/all/v/5932/p/all/c11913/31727",
      "/t/6389/n1/all/v/5932/p/all/c11913/31728",
      "/t/6389/n1/all/v/5932/p/all/c11913/31729",
      "/t/6389/n1/all/v/5932/p/all/c11913/31730",
      "/t/6389/n1/all/v/5932/p/all/c11913/96170",
      "/t/6389/n1/all/v/5932/p/all/c11913/45934",
      "/t/6389/n1/all/v/5932/p/all/c11913/45935",
      "/t/6389/n1/all/v/5932/p/all/c11913/96171",
      "/t/6389/n1/all/v/5932/p/all/c11913/45936",
      "/t/6389/n1/all/v/5932/p/all/c11913/45937"
    ),
    table_id = 6389L,
    variable_id = 5932L,
    classification_id = "c11913",
    classification_value = c(
      "96166", "31721", "31722", "31723",
      "31724", "31725", "31726",
      "31727", "31728", "31729", "31730",
      "96170", "45934", "45935",
      "96171", "45936", "45937"
    ),
    theme = "earnings",
    theme_category = "average_usual",
    subcategory = "by_employment_type",
    description_pt = c(
      "Rend. hab. real principal - Empregado",
      "Rend. hab. real principal - Empregado setor privado",
      "Rend. hab. real principal - Empregado setor privado com carteira",
      "Rend. hab. real principal - Empregado setor privado sem carteira",
      "Rend. hab. real principal - Trabalhador domestico",
      "Rend. hab. real principal - Trabalhador domestico com carteira",
      "Rend. hab. real principal - Trabalhador domestico sem carteira",
      "Rend. hab. real principal - Empregado setor publico",
      "Rend. hab. real principal - Empregado setor publico com carteira",
      "Rend. hab. real principal - Empregado setor publico sem carteira",
      "Rend. hab. real principal - Militar e servidor estatutario",
      "Rend. hab. real principal - Empregador",
      "Rend. hab. real principal - Empregador com CNPJ",
      "Rend. hab. real principal - Empregador sem CNPJ",
      "Rend. hab. real principal - Conta propria",
      "Rend. hab. real principal - Conta propria com CNPJ",
      "Rend. hab. real principal - Conta propria sem CNPJ"
    ),
    description_en = c(
      "Usual real earnings (main job) - Employees",
      "Usual real earnings (main job) - Private sector employees",
      "Usual real earnings (main job) - Private sector with formal contract",
      "Usual real earnings (main job) - Private sector without formal contract",
      "Usual real earnings (main job) - Domestic workers",
      "Usual real earnings (main job) - Domestic workers with formal contract",
      "Usual real earnings (main job) - Domestic workers without formal contract",
      "Usual real earnings (main job) - Public sector employees",
      "Usual real earnings (main job) - Public sector with formal contract",
      "Usual real earnings (main job) - Public sector without formal contract",
      "Usual real earnings (main job) - Military and statutory servants",
      "Usual real earnings (main job) - Employers",
      "Usual real earnings (main job) - Employers with CNPJ",
      "Usual real earnings (main job) - Employers without CNPJ",
      "Usual real earnings (main job) - Self-employed",
      "Usual real earnings (main job) - Self-employed with CNPJ",
      "Usual real earnings (main job) - Self-employed without CNPJ"
    ),
    unit = "currency",
    unit_label_pt = "R$",
    unit_label_en = "BRL",
    is_derived = FALSE,
    requires_deflation = TRUE
  )

  # ============================================================================
  # THEME: EARNINGS - Average Usual (By Economic Sector)
  # ============================================================================

  earnings_usual_by_sector <- data.table::data.table(
    series_name = c(
      "rhrpagropecuaria", "rhrpindustria", "rhrpconstrucao", "rhrpcomercio", "rhrptransporte",
      "rhrpalojaliment", "rhrpinfcomfinimobadm", "rhrpadminpublica", "rhrpoutroservico", "rhrpservicodomestico"
    ),
    api_path = c(
      "/t/6391/n1/all/v/5932/p/all/c888/47947",
      "/t/6391/n1/all/v/5932/p/all/c888/47948",
      "/t/6391/n1/all/v/5932/p/all/c888/47949",
      "/t/6391/n1/all/v/5932/p/all/c888/47950",
      "/t/6391/n1/all/v/5932/p/all/c888/56622",
      "/t/6391/n1/all/v/5932/p/all/c888/56623",
      "/t/6391/n1/all/v/5932/p/all/c888/56624",
      "/t/6391/n1/all/v/5932/p/all/c888/60032",
      "/t/6391/n1/all/v/5932/p/all/c888/56627",
      "/t/6391/n1/all/v/5932/p/all/c888/56628"
    ),
    table_id = 6391L,
    variable_id = 5932L,
    classification_id = "c888",
    classification_value = c(
      "47947", "47948", "47949", "47950", "56622",
      "56623", "56624", "60032", "56627", "56628"
    ),
    theme = "earnings",
    theme_category = "average_usual",
    subcategory = "by_economic_sector",
    description_pt = c(
      "Rend. hab. real principal - Agropecuaria",
      "Rend. hab. real principal - Industria",
      "Rend. hab. real principal - Construcao",
      "Rend. hab. real principal - Comercio",
      "Rend. hab. real principal - Transporte, armazenagem e correio",
      "Rend. hab. real principal - Alojamento e alimentacao",
      "Rend. hab. real principal - Informacao, comunicacao, financeiras, imobiliarias e administrativas",
      "Rend. hab. real principal - Administracao publica, defesa, seguridade, educacao, saude e servicos sociais",
      "Rend. hab. real principal - Outros servicos",
      "Rend. hab. real principal - Servicos domesticos"
    ),
    description_en = c(
      "Usual real earnings (main job) - Agriculture",
      "Usual real earnings (main job) - Industry",
      "Usual real earnings (main job) - Construction",
      "Usual real earnings (main job) - Commerce",
      "Usual real earnings (main job) - Transport, storage and mail",
      "Usual real earnings (main job) - Accommodation and food services",
      "Usual real earnings (main job) - Information, communication, finance, real estate and admin",
      "Usual real earnings (main job) - Public admin, defense, education, health and social services",
      "Usual real earnings (main job) - Other services",
      "Usual real earnings (main job) - Domestic services"
    ),
    unit = "currency",
    unit_label_pt = "R$",
    unit_label_en = "BRL",
    is_derived = FALSE,
    requires_deflation = TRUE
  )

  # ============================================================================
  # THEME: EARNINGS - Average Effective (All Jobs)
  # ============================================================================

  earnings_effective_all <- data.table::data.table(
    series_name = c("rendefetnominaltodos", "rendefetrealtodos"),
    api_path = c(
      "/t/6387/n1/all/v/5931/p/all",
      "/t/6387/n1/all/v/5935/p/all"
    ),
    table_id = 6387L,
    variable_id = c(5931L, 5935L),
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "earnings",
    theme_category = "average_effective",
    subcategory = "all_jobs",
    description_pt = c(
      "Rendimento efetivo nominal medio de todos os trabalhos",
      "Rendimento efetivo real medio de todos os trabalhos"
    ),
    description_en = c(
      "Average effective nominal earnings from all jobs",
      "Average effective real earnings from all jobs"
    ),
    unit = "currency",
    unit_label_pt = "R$",
    unit_label_en = "BRL",
    is_derived = FALSE,
    requires_deflation = c(FALSE, TRUE)
  )

  # ============================================================================
  # THEME: EARNINGS - Wage Mass (Usual)
  # ============================================================================

  wage_mass_usual <- data.table::data.table(
    series_name = c("massahabnominaltodos", "massahabrealtodos"),
    api_path = c(
      "/t/6392/n1/all/v/6288/p/all",
      "/t/6392/n1/all/v/6293/p/all"
    ),
    table_id = 6392L,
    variable_id = c(6288L, 6293L),
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "earnings",
    theme_category = "wage_mass",
    subcategory = "usual",
    description_pt = c(
      "Massa de rendimento habitual nominal de todos os trabalhos",
      "Massa de rendimento habitual real de todos os trabalhos"
    ),
    description_en = c(
      "Usual nominal wage mass from all jobs",
      "Usual real wage mass from all jobs"
    ),
    unit = "currency_millions",
    unit_label_pt = "R$ milhoes",
    unit_label_en = "BRL millions",
    is_derived = FALSE,
    requires_deflation = c(FALSE, TRUE)
  )

  # ============================================================================
  # THEME: EARNINGS - Wage Mass (Effective)
  # ============================================================================

  wage_mass_effective <- data.table::data.table(
    series_name = c("massaefetnominaltodos", "massaefetrealtodos"),
    api_path = c(
      "/t/6393/n1/all/v/6291/p/all",
      "/t/6393/n1/all/v/6295/p/all"
    ),
    table_id = 6393L,
    variable_id = c(6291L, 6295L),
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "earnings",
    theme_category = "wage_mass",
    subcategory = "effective",
    description_pt = c(
      "Massa de rendimento efetivo nominal de todos os trabalhos",
      "Massa de rendimento efetivo real de todos os trabalhos"
    ),
    description_en = c(
      "Effective nominal wage mass from all jobs",
      "Effective real wage mass from all jobs"
    ),
    unit = "currency_millions",
    unit_label_pt = "R$ milhoes",
    unit_label_en = "BRL millions",
    is_derived = FALSE,
    requires_deflation = c(FALSE, TRUE)
  )

  # ============================================================================
  # THEME: DEMOGRAPHICS - Total Population
  # ============================================================================

  demographics_total <- data.table::data.table(
    series_name = "populacao",
    api_path = "/t/6022/n1/all/v/606/p/all",
    table_id = 6022L,
    variable_id = 606L,
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "demographics",
    theme_category = "total",
    subcategory = NA_character_,
    description_pt = "Populacao total residente",
    description_en = "Total resident population",
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: DEMOGRAPHICS - Working Age Population
  # ============================================================================

  demographics_working_age <- data.table::data.table(
    series_name = "pop14mais",
    api_path = "/t/6318/n1/all/v/1641/p/all/c629/32385",
    table_id = 6318L,
    variable_id = 1641L,
    classification_id = "c629",
    classification_value = "32385",
    theme = "demographics",
    theme_category = "working_age",
    subcategory = NA_character_,
    description_pt = "Populacao de 14 anos ou mais de idade",
    description_en = "Working age population (14 years and older)",
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: SOCIAL_PROTECTION - Social Security
  # ============================================================================

  social_security <- data.table::data.table(
    series_name = "contribuinteprev",
    api_path = "/t/3918/n1/all/v/4090/p/all/c12027/allxt",
    table_id = 3918L,
    variable_id = 4090L,
    classification_id = "c12027",
    classification_value = "allxt",
    theme = "social_protection",
    theme_category = "social_security",
    subcategory = NA_character_,
    description_pt = "Contribuintes para a previdencia",
    description_en = "Social security contributors",
    unit = "thousands",
    unit_label_pt = "mil pessoas",
    unit_label_en = "thousands",
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # THEME: PRICES - Deflators
  # ============================================================================

  price_deflators <- data.table::data.table(
    series_name = c("ipca100dez1993", "ipcavarmensal", "inpc100dez1993", "inpcvarmensal"),
    api_path = c(
      "/t/1737/n1/all/v/2266/p/all/d/v2266%2013",
      "/t/1737/n1/all/v/63/p/all/d/v63%202",
      "/t/1736/n1/all/v/2289/p/all/d/v2289%2013",
      "/t/1736/n1/all/v/44/p/all/d/v44%202"
    ),
    table_id = c(1737L, 1737L, 1736L, 1736L),
    variable_id = c(2266L, 63L, 2289L, 44L),
    classification_id = NA_character_,
    classification_value = NA_character_,
    theme = "prices",
    theme_category = "deflators",
    subcategory = NA_character_,
    description_pt = c(
      "IPCA - indice (base dez/1993 = 100)",
      "IPCA - variacao mensal (%)",
      "INPC - indice (base dez/1993 = 100)",
      "INPC - variacao mensal (%)"
    ),
    description_en = c(
      "IPCA - Consumer Price Index (Dec/1993 = 100)",
      "IPCA - Monthly variation (%)",
      "INPC - National Consumer Price Index (Dec/1993 = 100)",
      "INPC - Monthly variation (%)"
    ),
    unit = c("index", "percent", "index", "percent"),
    unit_label_pt = c("indice", "%", "indice", "%"),
    unit_label_en = c("index", "%", "index", "%"),
    is_derived = FALSE,
    requires_deflation = FALSE
  )

  # ============================================================================
  # Combine all categories
  # ============================================================================

  meta <- data.table::rbindlist(list(
    participation_rates, participation_levels,
    unemployment_rates,
    underutilization_rates, underutilization_levels,
    employment_employees, employment_domestic, employment_employers,
    employment_self_employed, employment_family,
    economic_sector,
    earnings_usual_all, earnings_usual_by_type, earnings_usual_by_sector,
    earnings_effective_all,
    wage_mass_usual, wage_mass_effective,
    demographics_total, demographics_working_age,
    social_security,
    price_deflators
  ), fill = TRUE)

  # Add description column based on language
  if (lang == "en") {
    meta[, description := description_en]
  } else {
    meta[, description := description_pt]
  }

  # ============================================================================
  # Filter by hierarchical criteria
  # ============================================================================

  # Filter by theme if specified
  if (!is.null(theme)) {
    valid_themes <- c("labor_market", "earnings", "demographics", "social_protection", "prices")
    invalid <- setdiff(theme, valid_themes)
    if (length(invalid) > 0) {
      stop("Invalid theme: ", paste(invalid, collapse = ", "),
           ". Valid themes: ", paste(valid_themes, collapse = ", "))
    }
    filter_themes <- theme
    meta <- meta[get("theme") %in% filter_themes]
  }

  # Filter by theme_category if specified
  if (!is.null(theme_category)) {
    filter_categories <- theme_category
    meta <- meta[get("theme_category") %in% filter_categories]
  }

  # Filter by subcategory if specified
  if (!is.null(subcategory)) {
    filter_subcategories <- subcategory
    # Include rows where subcategory matches OR is NA (for series without subcategories)
    meta <- meta[get("subcategory") %in% filter_subcategories | is.na(get("subcategory"))]
  }

  # Filter by series if specified
  if (!identical(series, "all")) {
    invalid <- setdiff(series, meta$series_name)
    if (length(invalid) > 0) {
      stop("Unknown series: ", paste(invalid, collapse = ", "))
    }
    meta <- meta[series_name %in% series]
  }

  return(meta)
}

# =============================================================================
# Internal Helper Functions
# =============================================================================

#' Get Month Position in Quarter (mesnotrim)
#'
#' Given a month (1-12), returns its position in the rolling quarter (1, 2, or 3).
#' - January, April, July, October = position 1
#' - February, May, August, November = position 2
#' - March, June, September, December = position 3
#'
#' @param month Integer. Month number (1-12).
#'
#' @return Integer. Position in quarter (1, 2, or 3).
#'
#' @keywords internal
.get_mesnotrim <- function(month) {
  ((month - 1) %% 3) + 1
}


# =============================================================================
# PNADC Survey Date Constants
# =============================================================================
#
# Centralized constants for PNADC survey dates to ensure consistency across
# all mensalization functions. These reflect IBGE methodology changes.
#
# Usage: Access via .PNADC_DATES$<name>
# =============================================================================

.PNADC_DATES <- list(
  # ==========================================================================
  # Survey inception
  # ==========================================================================
  PNADC_START = 201201L,         # First PNADC month (Jan 2012)

  # ==========================================================================
  # Variable transitions
  # ==========================================================================
  VD4004_SPLIT = 201509L,        # Last month using VD4004 (underemployment)
                                  # VD4004A used from 201510 onwards
  V4019_AVAILABLE = 201510L,     # First month V4019 (CNPJ status) available

  # ==========================================================================
  # Calibration periods
  # ==========================================================================
  DEFAULT_CALIB_START = 201301L, # Default calibration period start
  DEFAULT_CALIB_END = 201912L,   # Default calibration period end (pre-COVID)
  CNPJ_CALIB_START = 201601L,    # Calibration start for CNPJ series
  PRESPLIT_CALIB_END = 201412L   # Calibration end for pre-VD4004A period
)

# Tests for sidra-series-metadata.R
# Note: All tests use the metadata function's own output (no external data needed)

# =============================================================================
# DEFAULT CALL: STRUCTURE AND COMPLETENESS
# =============================================================================

test_that("default call returns 86+ rows data.table with all required columns", {
  # 1. Setup: Call with defaults
  meta <- get_sidra_series_metadata()

  # 2. Verify: Structure

  expect_s3_class(meta, "data.table")
  expect_gte(nrow(meta), 86)

  # 3. Verify: All documented columns are present
  required_cols <- c(
    "series_name", "api_path", "table_id", "variable_id",
    "classification_id", "classification_value",
    "theme", "theme_category", "subcategory",
    "description_pt", "description_en", "description",
    "unit", "unit_label_pt", "unit_label_en",
    "is_derived", "requires_deflation"
  )
  for (col in required_cols) {
    expect_true(col %in% names(meta),
                label = paste("Column", col, "must be present"))
  }
})

test_that("no duplicate series_name values", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: All series names are unique
  expect_equal(length(unique(meta$series_name)), nrow(meta),
               label = "series_name values must be unique")

  # 3. Context: Duplicates would cause ambiguous lookups
  dup_names <- meta$series_name[duplicated(meta$series_name)]
  expect_equal(length(dup_names), 0,
               label = paste("Duplicated names:", paste(dup_names, collapse = ", ")))
})

test_that("all api_path values start with /t/", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: SIDRA API paths follow the /t/{table_id}/... pattern
  expect_true(all(grepl("^/t/", meta$api_path)),
              label = "All api_path values must start with /t/")

  # 3. Context: Malformed paths will cause SIDRA API failures
  bad_paths <- meta$api_path[!grepl("^/t/", meta$api_path)]
  expect_equal(length(bad_paths), 0,
               label = paste("Invalid paths:", paste(bad_paths, collapse = "; ")))
})

test_that("all table_id and variable_id are positive integers", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: table_id
  expect_type(meta$table_id, "integer")
  expect_true(all(meta$table_id > 0L),
              label = "All table_id must be positive integers")

  # 3. Verify: variable_id
  expect_type(meta$variable_id, "integer")
  expect_true(all(meta$variable_id > 0L),
              label = "All variable_id must be positive integers")
})

# =============================================================================
# THEME FILTERING
# =============================================================================

test_that("theme filtering: labor_market returns a subset", {
  # 1. Setup
  all_meta <- get_sidra_series_metadata()
  labor <- get_sidra_series_metadata(theme = "labor_market")

  # 2. Verify: Filtered result is a proper subset
  expect_s3_class(labor, "data.table")
  expect_gt(nrow(labor), 0)
  expect_lt(nrow(labor), nrow(all_meta))

  # 3. Verify: All returned rows have the correct theme
  expect_true(all(labor$theme == "labor_market"),
              label = "All rows must have theme == 'labor_market'")
})

test_that("invalid theme raises error with informative message", {
  # 1. Execute + Verify: Invalid theme name
  expect_error(
    get_sidra_series_metadata(theme = "nonexistent_theme"),
    "Invalid theme.*nonexistent_theme"
  )

  # 2. Context: Check that valid themes are listed in the error
  expect_error(
    get_sidra_series_metadata(theme = "fake"),
    "labor_market"
  )
})

test_that("multiple themes can be specified", {
  # 1. Setup
  combined <- get_sidra_series_metadata(theme = c("labor_market", "earnings"))

  # 2. Verify: Contains rows from both themes
  expect_true("labor_market" %in% combined$theme)
  expect_true("earnings" %in% combined$theme)

  # 3. Verify: No rows from other themes
  expect_true(all(combined$theme %in% c("labor_market", "earnings")),
              label = "Only requested themes should appear")
})

# =============================================================================
# THEME_CATEGORY FILTERING
# =============================================================================

test_that("theme_category filtering works", {
  # 1. Setup
  unemployment <- get_sidra_series_metadata(theme_category = "unemployment")

  # 2. Verify: Returns non-empty subset
  expect_gt(nrow(unemployment), 0)
  expect_true(all(unemployment$theme_category == "unemployment"))

  # 3. Context: taxadesocup should be in unemployment category
  expect_true("taxadesocup" %in% unemployment$series_name)
})

test_that("theme_category with no matches returns zero rows", {
  # 1. Setup: Use a category that does not exist
  result <- get_sidra_series_metadata(theme_category = "nonexistent_category")

  # 2. Verify: Returns valid data.table with zero rows
  expect_s3_class(result, "data.table")
  expect_equal(nrow(result), 0)
})

# =============================================================================
# SUBCATEGORY FILTERING
# =============================================================================

test_that("subcategory filtering includes NA subcategory rows", {
  # 1. Setup: economic_sector has subcategory = NA; filter for "levels"
  #    should return rows with subcategory == "levels" OR subcategory == NA
  result <- get_sidra_series_metadata(
    theme = "labor_market",
    subcategory = "levels"
  )

  # 2. Verify: Contains explicit "levels" rows
  expect_true("levels" %in% result$subcategory)

  # 3. Verify: Also includes NA subcategory rows (economic_sector, etc.)
  expect_true(any(is.na(result$subcategory)),
              label = "NA subcategory rows should be included when filtering by subcategory")
})

test_that("subcategory filtering returns correct subset", {
  # 1. Setup: Get only "rates" subcategory from labor_market
  rates <- get_sidra_series_metadata(theme = "labor_market", subcategory = "rates")

  # 2. Verify: All non-NA subcategories are "rates"
  non_na_subcats <- rates$subcategory[!is.na(rates$subcategory)]
  expect_true(all(non_na_subcats == "rates"),
              label = "Non-NA subcategories should all be 'rates'")
})

# =============================================================================
# SPECIFIC SERIES SELECTION
# =============================================================================

test_that("specific series selection: taxadesocup returns exactly 1 row", {
  # 1. Setup
  result <- get_sidra_series_metadata(series = "taxadesocup")

  # 2. Verify: Exactly one row
  expect_equal(nrow(result), 1)
  expect_equal(result$series_name, "taxadesocup")

  # 3. Verify: Known metadata values
  expect_equal(result$table_id, 6381L)
  expect_equal(result$variable_id, 4099L)
  expect_equal(result$theme, "labor_market")
  expect_equal(result$theme_category, "unemployment")
  expect_equal(result$unit, "percent")
})

test_that("multiple specific series selection works", {
  # 1. Setup
  result <- get_sidra_series_metadata(series = c("taxadesocup", "popocup", "populacao"))

  # 2. Verify: Exactly 3 rows, one per requested series
  expect_equal(nrow(result), 3)
  expect_true(all(c("taxadesocup", "popocup", "populacao") %in% result$series_name))
})

test_that("invalid series name raises error", {
  # 1. Execute + Verify: Unknown series name
  expect_error(
    get_sidra_series_metadata(series = "this_series_does_not_exist"),
    "Unknown series.*this_series_does_not_exist"
  )

  # 2. Verify: Partially valid set still errors on the invalid entry
  expect_error(
    get_sidra_series_metadata(series = c("taxadesocup", "fake_series")),
    "Unknown series.*fake_series"
  )
})

# =============================================================================
# COMBINED FILTERS
# =============================================================================

test_that("combined theme + theme_category filters work correctly", {
  # 1. Setup: Filter labor_market -> participation
  result <- get_sidra_series_metadata(
    theme = "labor_market",
    theme_category = "participation"
  )

  # 2. Verify: All rows match both criteria
  expect_true(all(result$theme == "labor_market"))
  expect_true(all(result$theme_category == "participation"))

  # 3. Verify: Known series are present
  expect_true("taxapartic" %in% result$series_name)
  expect_true("popocup" %in% result$series_name)
  expect_true("popdesocup" %in% result$series_name)
})

test_that("combined theme + subcategory + series filters work", {
  # 1. Setup: Narrow filter that should return 1 row
  result <- get_sidra_series_metadata(
    theme = "labor_market",
    subcategory = "rates",
    series = "taxadesocup"
  )

  # 2. Verify
  expect_equal(nrow(result), 1)
  expect_equal(result$series_name, "taxadesocup")
})

# =============================================================================
# UNIT CONSISTENCY
# =============================================================================

test_that("population/level series have unit = thousands", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: Known population-level series use "thousands"
  pop_series <- c("popocup", "popdesocup", "popnaforca", "popforadaforca",
                   "populacao", "pop14mais")
  pop_meta <- meta[series_name %in% pop_series]
  expect_true(all(pop_meta$unit == "thousands"),
              label = "All population/level series must use unit = 'thousands'")
})

test_that("rate series have unit = percent", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: Known rate series use "percent"
  rate_series <- c("taxadesocup", "taxapartic", "nivelocup", "niveldesocup",
                    "taxacombdesosub", "taxacompsubutlz", "perccontribprev")
  rate_meta <- meta[series_name %in% rate_series]
  expect_true(all(rate_meta$unit == "percent"),
              label = "All rate series must use unit = 'percent'")
})

test_that("income series have unit = currency or currency_millions", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: Earnings series use "currency"
  earnings_avg <- c("rendhabnominaltodos", "rendhabrealtodos",
                     "rendhabrealprinc", "rendefetrealprinc")
  earn_meta <- meta[series_name %in% earnings_avg]
  expect_true(all(earn_meta$unit == "currency"),
              label = "Average earnings series must use unit = 'currency'")

  # 3. Verify: Wage mass series use "currency_millions"
  mass_series <- c("massahabnominaltodos", "massahabrealtodos",
                    "massaefetnominaltodos", "massaefetrealtodos")
  mass_meta <- meta[series_name %in% mass_series]
  expect_true(all(mass_meta$unit == "currency_millions"),
              label = "Wage mass series must use unit = 'currency_millions'")
})

# =============================================================================
# IS_DERIVED CONSISTENCY
# =============================================================================

test_that("is_derived is TRUE for rate/percentage series", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: Known rate series (which are derived from ratios) are marked is_derived
  derived_series <- c("taxadesocup", "taxapartic", "nivelocup", "niveldesocup",
                       "taxacombdesosub", "taxacombdesopot", "taxacompsubutlz",
                       "taxasubocuphoras", "percdesalento", "perccontribprev")
  derived_meta <- meta[series_name %in% derived_series]
  expect_true(all(derived_meta$is_derived),
              label = "Rate/percentage series should have is_derived = TRUE")

  # 3. Verify: Population level series are NOT derived
  level_series <- c("popocup", "popdesocup", "popnaforca", "populacao")
  level_meta <- meta[series_name %in% level_series]
  expect_true(all(!level_meta$is_derived),
              label = "Population level series should have is_derived = FALSE")
})

# =============================================================================
# LANGUAGE SWITCHING
# =============================================================================

test_that("lang = 'en' switches description column to English", {
  # 1. Setup
  meta_pt <- get_sidra_series_metadata(series = "taxadesocup", lang = "pt")
  meta_en <- get_sidra_series_metadata(series = "taxadesocup", lang = "en")

  # 2. Verify: description column differs by language
  expect_equal(meta_pt$description, meta_pt$description_pt)
  expect_equal(meta_en$description, meta_en$description_en)

  # 3. Verify: English and Portuguese descriptions are different
  expect_false(identical(meta_pt$description, meta_en$description),
               label = "PT and EN descriptions should differ")

  # 4. Context: Both description_pt and description_en columns should always be present
  expect_true("description_pt" %in% names(meta_en))
  expect_true("description_en" %in% names(meta_pt))
})

test_that("lang = 'pt' is the default", {
  # 1. Setup
  meta_default <- get_sidra_series_metadata(series = "popocup")
  meta_pt <- get_sidra_series_metadata(series = "popocup", lang = "pt")

  # 2. Verify: Default description matches Portuguese
  expect_equal(meta_default$description, meta_pt$description_pt)
})

# =============================================================================
# .get_mesnotrim INTERNAL FUNCTION
# =============================================================================

test_that(".get_mesnotrim maps quarter-starting months to position 1", {
  # 1. Setup + Verify: Months 1, 4, 7, 10 are the first month of their quarter
  expect_equal(PNADCperiods:::.get_mesnotrim(1), 1)
  expect_equal(PNADCperiods:::.get_mesnotrim(4), 1)
  expect_equal(PNADCperiods:::.get_mesnotrim(7), 1)
  expect_equal(PNADCperiods:::.get_mesnotrim(10), 1)
})

test_that(".get_mesnotrim maps second months to position 2", {
  # 1. Setup + Verify: Months 2, 5, 8, 11
  expect_equal(PNADCperiods:::.get_mesnotrim(2), 2)
  expect_equal(PNADCperiods:::.get_mesnotrim(5), 2)
  expect_equal(PNADCperiods:::.get_mesnotrim(8), 2)
  expect_equal(PNADCperiods:::.get_mesnotrim(11), 2)
})

test_that(".get_mesnotrim maps quarter-ending months to position 3", {
  # 1. Setup + Verify: Months 3, 6, 9, 12
  expect_equal(PNADCperiods:::.get_mesnotrim(3), 3)
  expect_equal(PNADCperiods:::.get_mesnotrim(6), 3)
  expect_equal(PNADCperiods:::.get_mesnotrim(9), 3)
  expect_equal(PNADCperiods:::.get_mesnotrim(12), 3)
})

test_that(".get_mesnotrim is vectorized", {
  # 1. Setup: Pass all 12 months at once
  months <- 1:12
  expected <- rep(1:3, 4)
  result <- PNADCperiods:::.get_mesnotrim(months)

  # 2. Verify: Vectorized output matches individual calls
  expect_equal(result, expected)
  expect_length(result, 12)
})

# =============================================================================
# .PNADC_DATES INTERNAL CONSTANTS
# =============================================================================

test_that(".PNADC_DATES contains expected constants with correct values", {
  # 1. Setup
  dates <- PNADCperiods:::.PNADC_DATES

  # 2. Verify: Is a named list
  expect_type(dates, "list")
  expect_true(length(dates) > 0)

  # 3. Verify: Specific constant values
  expect_equal(dates$PNADC_START, 201201L,
               label = "PNADC started in January 2012")
  expect_equal(dates$VD4004_SPLIT, 201509L,
               label = "VD4004 split occurred in September 2015")
  expect_equal(dates$V4019_AVAILABLE, 201510L,
               label = "V4019 available from October 2015")
  expect_equal(dates$DEFAULT_CALIB_START, 201301L,
               label = "Default calibration starts January 2013")
  expect_equal(dates$DEFAULT_CALIB_END, 201912L,
               label = "Default calibration ends December 2019 (pre-COVID)")
  expect_equal(dates$CNPJ_CALIB_START, 201601L,
               label = "CNPJ calibration starts January 2016")
  expect_equal(dates$PRESPLIT_CALIB_END, 201412L,
               label = "Pre-split calibration ends December 2014")
})

test_that(".PNADC_DATES values are all integer scalars", {
  # 1. Setup
  dates <- PNADCperiods:::.PNADC_DATES

  # 2. Verify: Each element is a single integer
  for (nm in names(dates)) {
    expect_type(dates[[nm]], "integer")
    expect_length(dates[[nm]], 1)
  }
})

# =============================================================================
# KEY SERIES METADATA SPOT-CHECKS
# =============================================================================

test_that("taxadesocup has correct api_path and table_id", {
  # 1. Setup
  result <- get_sidra_series_metadata(series = "taxadesocup")

  # 2. Verify: Known correct values from SIDRA
  expect_equal(result$table_id, 6381L)
  expect_equal(result$variable_id, 4099L)
  expect_equal(result$api_path, "/t/6381/n1/all/v/4099/p/all/d/v4099%201")
  expect_equal(result$unit, "percent")
  expect_true(result$is_derived)
  expect_false(result$requires_deflation)
})

test_that("populacao series has correct metadata", {
  # 1. Setup
  result <- get_sidra_series_metadata(series = "populacao")

  # 2. Verify
  expect_equal(result$table_id, 6022L)
  expect_equal(result$variable_id, 606L)
  expect_equal(result$theme, "demographics")
  expect_equal(result$unit, "thousands")
  expect_false(result$is_derived)
  expect_false(result$requires_deflation)
})

test_that("rendhabrealtodos earnings series has correct metadata", {
  # 1. Setup
  result <- get_sidra_series_metadata(series = "rendhabrealtodos")

  # 2. Verify
  expect_equal(result$table_id, 6390L)
  expect_equal(result$theme, "earnings")
  expect_equal(result$unit, "currency")
  expect_false(result$is_derived)
  expect_true(result$requires_deflation)
})

# =============================================================================
# API_PATH VALIDITY FOR NON-DERIVED SERIES
# =============================================================================

test_that("all non-derived series have valid api_path structure", {
  # 1. Setup
  meta <- get_sidra_series_metadata()
  non_derived <- meta[is_derived == FALSE]

  # 2. Verify: api_path matches /t/{table_id}/... pattern
  for (i in seq_len(nrow(non_derived))) {
    row <- non_derived[i]
    expected_prefix <- paste0("/t/", row$table_id, "/")
    expect_true(
      startsWith(row$api_path, expected_prefix),
      label = paste(row$series_name, "api_path should start with", expected_prefix)
    )
  }
})

test_that("api_path contains the variable_id for all series", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: Each api_path references its variable_id in the /v/ segment
  for (i in seq_len(nrow(meta))) {
    row <- meta[i]
    var_pattern <- paste0("/v/", row$variable_id)
    expect_true(
      grepl(var_pattern, row$api_path, fixed = TRUE),
      label = paste(row$series_name, "api_path should contain", var_pattern)
    )
  }
})

# =============================================================================
# REQUIRES_DEFLATION CONSISTENCY
# =============================================================================

test_that("requires_deflation is TRUE only for real earnings series", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: Series requiring deflation are in the earnings theme
  deflated <- meta[requires_deflation == TRUE]
  expect_true(all(deflated$theme == "earnings"),
              label = "Only earnings series should require deflation")

  # 3. Verify: Non-earnings series never require deflation
  non_earnings <- meta[theme != "earnings"]
  expect_true(all(non_earnings$requires_deflation == FALSE),
              label = "Non-earnings series should not require deflation")
})

test_that("nominal earnings series do not require deflation", {
  # 1. Setup
  nominal_series <- c("rendhabnominaltodos", "rendefetnominaltodos",
                       "massahabnominaltodos", "massaefetnominaltodos")
  nominal_meta <- get_sidra_series_metadata(series = nominal_series)

  # 2. Verify: Nominal series do not require deflation
  expect_true(all(nominal_meta$requires_deflation == FALSE),
              label = "Nominal earnings series should not require deflation")
})

test_that("real earnings series require deflation", {
  # 1. Setup
  real_series <- c("rendhabrealtodos", "rendhabrealprinc",
                    "rendefetrealprinc", "massahabrealtodos", "massaefetrealtodos")
  real_meta <- get_sidra_series_metadata(series = real_series)

  # 2. Verify: Real series require deflation
  expect_true(all(real_meta$requires_deflation == TRUE),
              label = "Real earnings series should require deflation")
})

# =============================================================================
# THEME COVERAGE
# =============================================================================

test_that("all five themes are represented in the metadata", {
  # 1. Setup
  meta <- get_sidra_series_metadata()
  themes <- unique(meta$theme)

  # 2. Verify: All expected themes exist
  expected_themes <- c("labor_market", "earnings", "demographics",
                        "social_protection", "prices")
  for (th in expected_themes) {
    expect_true(th %in% themes,
                label = paste("Theme", th, "should be represented"))
  }

  # 3. Verify: No unexpected themes
  expect_true(all(themes %in% expected_themes),
              label = "No unexpected themes should exist")
})

test_that("prices theme includes IPCA and INPC series", {
  # 1. Setup
  prices <- get_sidra_series_metadata(theme = "prices")

  # 2. Verify: Known price index series
  expect_true("ipca100dez1993" %in% prices$series_name)
  expect_true("ipcavarmensal" %in% prices$series_name)
  expect_true("inpc100dez1993" %in% prices$series_name)
  expect_true("inpcvarmensal" %in% prices$series_name)

  # 3. Verify: Price index units
  ipca_idx <- prices[series_name == "ipca100dez1993"]
  expect_equal(ipca_idx$unit, "index")
  ipca_var <- prices[series_name == "ipcavarmensal"]
  expect_equal(ipca_var$unit, "percent")
})

# =============================================================================
# CLASSIFICATION COLUMNS
# =============================================================================

test_that("classification_id and classification_value are consistent", {
  # 1. Setup
  meta <- get_sidra_series_metadata()

  # 2. Verify: When classification_id is NA, classification_value is also NA
  na_class_id <- meta[is.na(classification_id)]
  expect_true(all(is.na(na_class_id$classification_value)),
              label = "classification_value should be NA when classification_id is NA")

  # 3. Verify: When classification_id is not NA, classification_value is not NA
  non_na_class_id <- meta[!is.na(classification_id)]
  expect_true(all(!is.na(non_na_class_id$classification_value)),
              label = "classification_value should not be NA when classification_id is set")
})

test_that("api_path contains classification when classification_id is set", {
  # 1. Setup
  meta <- get_sidra_series_metadata()
  classified <- meta[!is.na(classification_id)]

  # 2. Verify: api_path references the classification
  for (i in seq_len(nrow(classified))) {
    row <- classified[i]
    expect_true(
      grepl(row$classification_id, row$api_path, fixed = TRUE),
      label = paste(row$series_name, "api_path should contain", row$classification_id)
    )
    expect_true(
      grepl(row$classification_value, row$api_path, fixed = TRUE),
      label = paste(row$series_name, "api_path should contain value", row$classification_value)
    )
  }
})

---
title: "Applied Examples: Monthly vs Quarterly Labor Market Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Applied Examples: Monthly vs Quarterly Labor Market Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!--
MAINTAINER NOTE:
Code chunks marked with "# VIGNETTE CODE:" comments in
  mensalizacao_pnad/code/generate_vignette_data.R
must match the code shown here. To regenerate figures, run that script.
-->

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  echo = TRUE,
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 10,
  fig.height = 6,
  purl = FALSE
)
```

## Introduction

Brazil's Continuous National Household Sample Survey (PNADC) publishes quarterly and annual microdata for official labor market statistics. However, these **quarterly statistics are actually an average of three-month phenomena**---the SIDRA monthly series are rolling averages, not true monthly snapshots. This quarterly averaging obscures the timing and magnitude of economic shocks and blurs the detection of turning points.

The `PNADCperiods` package solves this by identifying which specific month each interview refers to, using birthday information and IBGE's interview scheduling rules. With the full PNADC history (2012-2025), the algorithm achieves a **~97% determination rate** for monthly identification.

This vignette demonstrates three cases where monthly data reveals dynamics that quarterly data hides, culminating in a powerful validation using minimum wage transitions.

> **Working with poverty or income data?** See [Monthly Poverty Analysis with Annual PNADC Data](annual-poverty-analysis.html).

---

## Prerequisites

To reproduce these analyses, you need stacked PNADC microdata spanning multiple quarters. See [Download and Prepare Data](download-and-prepare.html) for data preparation, or [Get Started](getting-started.html) for basic usage.

```{r libraries}
# Load required packages
library(PNADCperiods)
library(data.table)
library(ggplot2)
library(scales)
```

---

## Preparing the Data

This section shows how to create the aggregate time series used in the examples below.

### Step 1: Load and Mensalize Your Data

```{r load-data}
# Load your stacked PNADC data (see download-and-prepare vignette)
library(fst)
pnadc <- read_fst("path/to/your/pnadc_stacked.fst", as.data.table = TRUE)

# Build crosswalk (identify reference periods)
crosswalk <- pnadc_identify_periods(pnadc, verbose = TRUE)

# Keep a clean copy for sub-monthly calibration later (Section 4)
pnadc_clean <- copy(pnadc)

# Apply crosswalk with weight calibration
pnadc <- pnadc_apply_periods(
  pnadc,
  crosswalk,
  weight_var = "V1028",
  anchor = "quarter",
  calibrate = TRUE
)
```

### Step 2: Create Labor Market Variables

```{r create-variables}
# Filter to working-age population (14+)
pnadc <- pnadc[V2009 >= 14]

# Create labor market indicators
pnadc[, `:=`(
  # PEA: Economically Active Population (VD4001 == 1)
  pea = fifelse(VD4001 == 1, 1L, 0L),

  # Employed (VD4002 == 1)
  employed = fifelse(VD4002 == 1, 1L, 0L),

  # Unemployed (in labor force but not employed)
  unemployed = fifelse(VD4002 == 2, 1L, 0L)
)]

# Formality indicator (among employed)
# Formal: positions 1,3,5,7 OR positions 8,9 with social security contribution
pnadc[, formal := fifelse(
  VD4009 %in% c(1L, 3L, 5L, 7L), 1L,
  fifelse(VD4009 %in% c(8L, 9L) & VD4012 == 1L, 1L, 0L)
)]
pnadc[VD4002 != 1, formal := NA_integer_]
```

### Step 3: Create Quarterly Series

Quarterly aggregates use the original IBGE weight (`V1028`):

```{r create-quarterly}
quarterly_total <- pnadc[, .(
  unemployment_rate = sum(unemployed * V1028, na.rm = TRUE) /
                      sum(pea * V1028, na.rm = TRUE),
  participation_rate = sum(pea * V1028, na.rm = TRUE) /
                       sum(V1028, na.rm = TRUE),
  formalization_rate = sum(formal * V1028, na.rm = TRUE) /
                       sum(employed * V1028, na.rm = TRUE)
), by = .(Ano, Trimestre)]

# Add period column for plotting (mid-quarter date)
quarterly_total[, period := as.Date(paste0(Ano, "-", (Trimestre - 1) * 3 + 2, "-15"))]
```

### Step 4: Create Monthly Series

Monthly aggregates use `weight_monthly` from PNADCperiods:

```{r create-monthly}
# Filter to observations with determined reference month
pnadc_monthly <- pnadc[!is.na(weight_monthly)]

monthly_total <- pnadc_monthly[, .(
  unemployment_rate = sum(unemployed * weight_monthly, na.rm = TRUE) /
                      sum(pea * weight_monthly, na.rm = TRUE),
  participation_rate = sum(pea * weight_monthly, na.rm = TRUE) /
                       sum(weight_monthly, na.rm = TRUE),
  formalization_rate = sum(formal * weight_monthly, na.rm = TRUE) /
                       sum(employed * weight_monthly, na.rm = TRUE)
), by = ref_month_yyyymm]

# Add period column for plotting (mid-month date)
monthly_total[, period := as.Date(paste0(
  ref_month_yyyymm %/% 100, "-",
  ref_month_yyyymm %% 100, "-15"
))]
```

With these objects created, you can now run all the examples below.

---

## 1. COVID-19 Unemployment: The True Peak

The COVID-19 pandemic caused the sharpest labor market shock in Brazilian history. Quarterly data couldn't tell policymakers exactly when the crisis peaked or how severe it truly was---information critical for calibrating emergency responses like unemployment benefits and business support.

<details>
<summary>Show plotting code</summary>

```{r covid-unemployment}
# Filter to COVID period (Oct 2019 - Jan 2022)
covid_quarterly <- quarterly_total[period >= "2019-10-01" & period <= "2022-01-01"]
covid_monthly <- monthly_total[period >= "2019-10-01" & period <= "2022-01-01"]

# Find the unemployment peaks
peak_quarterly <- covid_quarterly[which.max(unemployment_rate)]
peak_monthly <- covid_monthly[which.max(unemployment_rate)]

# Create the comparison plot
ggplot() +
  # Quarterly as step function (emphasizes moving-average nature)
  geom_step(data = covid_quarterly,
            aes(x = period, y = unemployment_rate),
            color = "steelblue", linewidth = 1.2, direction = "mid") +
  geom_point(data = covid_quarterly,
             aes(x = period, y = unemployment_rate),
             color = "steelblue", size = 3) +

  # Monthly as line
  geom_line(data = covid_monthly,
            aes(x = period, y = unemployment_rate),
            color = "darkred", linewidth = 0.9) +
  geom_point(data = covid_monthly,
             aes(x = period, y = unemployment_rate),
             color = "darkred", size = 1.5) +

  # Annotate monthly peak
  annotate("segment",
           x = peak_monthly$period, xend = peak_monthly$period,
           y = peak_monthly$unemployment_rate + 0.005,
           yend = peak_monthly$unemployment_rate + 0.02,
           arrow = arrow(length = unit(0.2, "cm")), color = "darkred") +
  annotate("text",
           x = peak_monthly$period,
           y = peak_monthly$unemployment_rate + 0.022,
           label = paste0("Monthly peak: ", format(peak_monthly$period, "%b %Y"), "\n",
                         sprintf("%.1f%%", peak_monthly$unemployment_rate * 100)),
           size = 3.2, color = "darkred", hjust = 0.5) +

  # Annotate quarterly peak
  annotate("segment",
           x = peak_quarterly$period + 30, xend = peak_quarterly$period + 30,
           y = peak_quarterly$unemployment_rate - 0.005,
           yend = peak_quarterly$unemployment_rate - 0.02,
           arrow = arrow(length = unit(0.2, "cm")), color = "steelblue") +
  annotate("text",
           x = peak_quarterly$period + 30,
           y = peak_quarterly$unemployment_rate - 0.025,
           label = paste0("Quarterly peak: ", format(peak_quarterly$period, "%b %Y"), "\n",
                         sprintf("%.1f%%", peak_quarterly$unemployment_rate * 100)),
           size = 3.2, color = "steelblue", hjust = 0.5) +

  # Scales and labels
  scale_y_continuous(labels = percent_format(accuracy = 0.1),
                     limits = c(0.10, 0.17)) +
  scale_x_date(date_breaks = "3 months", date_labels = "%b\n%Y") +
  labs(
    title = "COVID-19 Unemployment Spike: Monthly vs Quarterly",
    subtitle = "Monthly data (red) shows the true peak; quarterly (blue) averages it away",
    x = NULL, y = "Unemployment Rate",
    caption = "Source: PNADC/IBGE. Quarterly shown as step function to emphasize moving-average nature."
  ) +
  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    panel.grid.minor = element_blank()
  )
```

</details>

![COVID-19 Unemployment: Monthly vs Quarterly](figures/applied-examples/fig-covid-unemployment.png){width=100%}

The monthly series reveals that unemployment peaked roughly 2 percentage points higher than the quarterly data suggests, and the timing of the peak is more precisely identified.

---

## 2. The 2014-2017 Recession: Tracking Month-by-Month Deterioration

Brazil's 2014-2017 recession saw unemployment rise from around 6% to over 13%. Monthly data reveals this deterioration unfolded in fits and starts, with several months showing unemployment increases exceeding 1 percentage point---sudden shocks invisible in the averaged quarterly figures.

<details>
<summary>Show plotting code</summary>

```{r recession-detail}
# Filter to recession period (Jun 2014 - Jun 2017)
recession_quarterly <- quarterly_total[period >= "2014-06-01" & period <= "2017-06-01"]
recession_monthly <- monthly_total[period >= "2014-06-01" & period <= "2017-06-01"]

# Find the peak
peak_recession <- recession_monthly[which.max(unemployment_rate)]

# Calculate month-over-month changes
setorder(recession_monthly, period)
recession_monthly[, change := unemployment_rate - shift(unemployment_rate)]

# Identify months with large jumps (> 1 percentage point)
big_jumps <- recession_monthly[!is.na(change) & change > 0.01]

# Create the plot
ggplot() +
  # Quarterly
  geom_step(data = recession_quarterly,
            aes(x = period, y = unemployment_rate),
            color = "steelblue", linewidth = 1, direction = "mid", alpha = 0.7) +
  geom_point(data = recession_quarterly,
             aes(x = period, y = unemployment_rate),
             color = "steelblue", size = 2.5) +

  # Monthly
  geom_line(data = recession_monthly,
            aes(x = period, y = unemployment_rate),
            color = "darkred", linewidth = 0.8) +
  geom_point(data = recession_monthly,
             aes(x = period, y = unemployment_rate),
             color = "darkred", size = 1.2) +

  # Highlight big monthly jumps
  geom_point(data = big_jumps,
             aes(x = period, y = unemployment_rate),
             color = "darkred", size = 3, shape = 21, fill = "yellow", stroke = 1.5) +

  # Mark the peak
  annotate("point", x = peak_recession$period, y = peak_recession$unemployment_rate,
           color = "darkred", size = 4, shape = 18) +
  annotate("text",
           x = peak_recession$period + 45, y = peak_recession$unemployment_rate,
           label = paste0("Peak: ", format(peak_recession$period, "%b %Y"), "\n",
                         sprintf("%.1f%%", peak_recession$unemployment_rate * 100)),
           size = 3, hjust = 0, color = "darkred") +

  # Scales and labels
  scale_y_continuous(labels = percent_format(accuracy = 1),
                     breaks = seq(0.06, 0.14, 0.02)) +
  scale_x_date(date_breaks = "6 months", date_labels = "%b\n%Y") +
  labs(
    title = "2014-2017 Recession: Month-by-Month Unemployment Rise",
    subtitle = "Yellow circles: months with >1 p.p. unemployment increase. Monthly data (red) vs quarterly (blue).",
    x = NULL, y = "Unemployment Rate",
    caption = "Source: PNADC/IBGE. Monthly data reveals sudden jumps hidden in quarterly averages."
  ) +
  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    panel.grid.minor = element_blank()
  )
```

</details>

![2014-2017 Recession: Month-by-Month Detail](figures/applied-examples/fig-recession-detail.png){width=100%}

The yellow circles mark months where unemployment jumped by more than 1 percentage point: sudden deteriorations that are completely invisible in the smooth quarterly trend.

---

## 3. Minimum Wage Adjustments: Validating Mensalization

Perhaps the most compelling validation of the mensalization methodology comes from tracking minimum wage adjustments. PNADC collects two income measures:

- **Habitual income (VD4016)**: What the worker usually earns---refers to the **current** reference month
- **Effective income (VD4017)**: What the worker actually received---refers to the **previous** month

When the minimum wage increases (typically in January), habitual income should show the new value immediately, while effective income should still show the old value (since it refers to the previous month). This one-month lag is a prediction of our mensalization: if reference month identification is correct, the pattern is crisp; if wrong, it blurs.

We focus on formal private sector employees (VD4009 == 1) aged 18+, tracking the share earning **exactly** each historical minimum wage value.

### Preparing the Minimum Wage Data

```{r mw-data-prep}
# Historical minimum wage values (R$)
# Note: 2020 had two adjustments (Jan and Feb), 2023 had mid-year adjustment (May)
mw_history <- data.table(
  start_yyyymm = c(201201, 201301, 201401, 201501, 201601, 201701, 201801,
                   201901, 202001, 202002, 202101, 202201, 202301, 202305,
                   202401, 202501),
  mw_value = c(622, 678, 724, 788, 880, 937, 954, 998, 1039, 1045, 1100,
               1212, 1302, 1320, 1412, 1518)
)
mw_adjustment_months <- mw_history$start_yyyymm

# For each formal private sector employee, identify which MW they earn
pnadc_formal <- pnadc[VD4009 == 1 & V2009 >= 18 & !is.na(weight_monthly)]

# Match exact habitual/effective incomes to MW values
pnadc_formal[, habitual_mw := mw_history[start_yyyymm <= ref_month_yyyymm][.N, mw_value], by = ref_month_yyyymm]
pnadc_formal[, at_habitual_mw := as.integer(VD4016 == habitual_mw)]
pnadc_formal[, at_effective_mw := as.integer(VD4017 == habitual_mw)]

# Aggregate by month and MW value
monthly_mw_exact <- pnadc_formal[, .(
  n_workers = sum(weight_monthly),
  pct_habitual = sum(at_habitual_mw * weight_monthly) / sum(weight_monthly),
  pct_effective = sum(at_effective_mw * weight_monthly) / sum(weight_monthly)
), by = .(ref_month_yyyymm, mw_value = habitual_mw)]

monthly_mw_exact[, period := as.Date(paste0(ref_month_yyyymm %/% 100, "-",
                                             ref_month_yyyymm %% 100, "-15"))]
```

### The Smoking Gun: January 2013 Transition

In January 2013, the minimum wage increased from R$ 622 to R$ 678:

<details>
<summary>Show plotting code</summary>

```{r mw-2013-transition}
# Create long format for plotting (needed for all MW examples)
mw_long <- melt(monthly_mw_exact,
                id.vars = c("ref_month_yyyymm", "period", "mw_value", "n_workers"),
                measure.vars = c("pct_habitual", "pct_effective"),
                variable.name = "income_type",
                value.name = "pct_at_mw")

# Add readable labels
mw_long[, income_label := fifelse(income_type == "pct_habitual",
                                   "Habitual Income (current month)",
                                   "Effective Income (previous month)")]

# Get adjustment dates for reference lines
adj_dates <- as.Date(paste0(mw_adjustment_months %/% 100, "-",
                             mw_adjustment_months %% 100, "-15"))

# Filter to the first transition period (Oct 2012 - Apr 2013)
mw_2013 <- mw_long[period >= "2012-10-15" & period <= "2013-04-15" &
                   mw_value %in% c(622, 678)]

# Create the plot
ggplot(mw_2013,
       aes(x = period, y = pct_at_mw, color = factor(mw_value), linetype = factor(mw_value))) +
  geom_line(linewidth = 1.2) +
  geom_point(size = 2.5) +

  # Mark the adjustment date (January 2013)
  geom_vline(xintercept = as.Date("2013-01-15"),
             linetype = "dashed", linewidth = 0.8, color = "gray40") +

  # Facet by income type
  facet_wrap(~ income_label, ncol = 2) +

  # Scales and labels
  scale_y_continuous(labels = percent_format(accuracy = 0.1)) +
  scale_x_date(date_breaks = "1 month", date_labels = "%b\n%Y") +
  scale_color_manual(values = c("622" = "#b2182b", "678" = "#2166ac"),
                     labels = c("622" = "R$ 622 (old)", "678" = "R$ 678 (new)"),
                     name = "MW Value") +
  scale_linetype_manual(values = c("622" = "solid", "678" = "solid"), guide = "none") +
  labs(
    title = "MW Transition: January 2013 (R$ 622 -> R$ 678)",
    subtitle = "Habitual income shows transition in January; Effective income shows same pattern in February",
    x = NULL, y = "Share Earning Exact MW",
    caption = "Vertical line = MW adjustment. Habitual income refers to current month; effective to previous month."
  ) +
  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    legend.position = "bottom",
    panel.grid.minor = element_blank(),
    strip.text = element_text(face = "bold")
  )
```

</details>

![MW 2013 Transition Detail](figures/applied-examples/fig-mw-2013-transition.png){width=100%}

This is the smoking gun. In the left panel (habitual income), workers shift from R$ 622 to R$ 678 in **January**. In the right panel (effective income), the same shift happens in **February**---exactly the one-month lag predicted by the temporal reference of each variable.

### A Harder Test: The 2020 Double Adjustment

In an unusual sequence, Brazil adjusted the minimum wage twice in consecutive months:

- **January 2020**: R$ 998 -> R$ 1,039 (annual adjustment)
- **February 2020**: R$ 1,039 -> R$ 1,045 (correction)

R$ 1,039 was the official minimum wage for only one month. If our reference month identification were imprecise, these rapid transitions would blur together.

<details>
<summary>Show plotting code</summary>

```{r mw-2020-double}
# Filter to 2020 double adjustment period (Jul 2019 - Jul 2020)
mw_2020 <- mw_long[period >= "2019-07-15" & period <= "2020-07-15" &
                   mw_value %in% c(998, 1039, 1045)]

# Get adjustment dates in this period
adj_2020 <- adj_dates[adj_dates >= "2019-07-15" & adj_dates <= "2020-07-15"]

# Create the plot
ggplot(mw_2020,
       aes(x = period, y = pct_at_mw, color = factor(mw_value), linetype = factor(mw_value))) +
  geom_line(linewidth = 1.1) +
  geom_point(size = 2.5) +

  # Mark adjustment months
  geom_vline(xintercept = adj_2020, linetype = "dashed", linewidth = 0.8, color = "gray40") +

  # Facet by income type
  facet_wrap(~ income_label, ncol = 2) +

  # Scales and labels
  scale_y_continuous(labels = percent_format(accuracy = 0.1)) +
  scale_x_date(date_breaks = "1 month", date_labels = "%b\n%Y") +
  scale_color_manual(values = c("998" = "#b2182b", "1039" = "#7b3294", "1045" = "#2166ac"),
                     labels = c("998" = "R$ 998 (2019)", "1039" = "R$ 1,039 (Jan 2020)",
                               "1045" = "R$ 1,045 (Feb 2020+)"),
                     name = "MW Value") +
  scale_linetype_manual(values = c("998" = "solid", "1039" = "solid", "1045" = "solid"),
                        guide = "none") +
  labs(
    title = "The 2020 Double Adjustment: A Demanding Test",
    subtitle = "R$ 998 -> R$ 1,039 (Jan) -> R$ 1,045 (Feb). Three values, two transitions, one-month lag preserved.",
    x = NULL, y = "Share Earning Exact MW",
    caption = "Vertical lines = MW adjustments. The short-lived R$ 1,039 value (one month only) creates a sharp test."
  ) +
  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    legend.position = "bottom",
    panel.grid.minor = element_blank(),
    strip.text = element_text(face = "bold")
  )
```

</details>

![MW 2020 Double Adjustment](figures/applied-examples/fig-mw-2020-double.png){width=100%}

The pattern remains crisp. In habitual income, January shows R$ 1,039 replacing R$ 998, and February shows R$ 1,045 replacing R$ 1,039. In effective income, each transition appears exactly one month later.

### The Full Picture: 2012-2025

<details>
<summary>Show plotting code</summary>

```{r mw-big-picture}
# Create color palette (blue to red gradient)
mw_vals <- sort(unique(monthly_mw_exact$mw_value))
n_mw <- length(mw_vals)
palette_fn <- colorRampPalette(c("#2166ac", "#67a9cf", "#d1e5f0",
                                  "#fddbc7", "#ef8a62", "#b2182b"))
mw_colors <- palette_fn(n_mw)
names(mw_colors) <- as.character(mw_vals)

# Filter to MW values with meaningful presence (>0.1% at some point)
mw_long[, max_pct := max(pct_at_mw), by = mw_value]
mw_long_filtered <- mw_long[max_pct > 0.001]

# Create the plot
ggplot(mw_long_filtered,
       aes(x = period, y = pct_at_mw, color = factor(mw_value), group = mw_value)) +
  geom_line(linewidth = 0.6, alpha = 0.8) +

  # Mark MW adjustment months
  geom_vline(xintercept = adj_dates, linetype = "dashed", alpha = 0.3, color = "gray40") +

  # Facet by income type
  facet_wrap(~ income_label, ncol = 1) +

  # Scales and labels
  scale_y_continuous(labels = percent_format(accuracy = 0.1)) +
  scale_x_date(date_breaks = "1 year", date_labels = "%Y") +
  scale_color_manual(values = mw_colors, name = "MW Value (R$)") +
  labs(
    title = "Share of Formal Workers Earning Each Exact MW Value",
    subtitle = "Each line = one MW value. Vertical lines = MW adjustments. Pattern shifted 1 month between panels.",
    x = NULL, y = "Share of Workers",
    caption = "Source: PNADC/IBGE. Formal private sector employees age 18+. All calculations use weight_monthly."
  ) +
  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    legend.position = "right",
    legend.key.size = unit(0.4, "cm"),
    panel.grid.minor = element_blank(),
    strip.text = element_text(face = "bold", size = 11)
  ) +
  guides(color = guide_legend(ncol = 2))
```

</details>

![MW Big Picture: All Values Over Time](figures/applied-examples/fig-mw-big-picture.png){width=100%}

Each vertical line marks a MW adjustment. The pattern in the top panel (habitual income) is shifted one month earlier than the bottom panel (effective income)---consistently across thirteen years and sixteen minimum wage values. This provides strong validation that the mensalization algorithm correctly identifies reference months.

---

## Summary: What Monthly Data Reveals

| Analysis | Quarterly Limitation | Monthly Insight |
|----------|---------------------|-----------------|
| **COVID unemployment** | Peak understated | True peak ~2 p.p. higher |
| **Recession tracking** | Appears as smooth climb | Multiple months with >1 p.p. jumps |
| **Minimum wage** | Timing impossible | 1-month lag validates methodology |

Monthly data from `PNADCperiods` enables **precise timing** of economic turning points, **true magnitudes** of shocks, and **validation** through known temporal patterns.

---

## 4. Beyond Monthly: Fortnightly and Weekly Analysis

`PNADCperiods` also identifies **fortnightly** and **weekly** reference periods. These have lower determination rates because, unlike months (which aggregate constraints across quarterly visits), fortnights and weeks can only use information from within a single quarter.

### Determination Rates and Experimental Strategies

| Granularity | Strict Rate | Experimental Rate | Improvement |
|-------------|-------------|-------------------|-------------|
| Monthly | **~97%** | -- | -- |
| Fortnightly | **~8%** | **~12%** | ~1.5x |
| Weekly | **~3%** | **~6%** | ~2x |

The `pnadc_experimental_periods()` function applies two strategies to boost sub-monthly rates: **probabilistic assignment** (when most of the interview window falls in one period) and **UPA aggregation** (when most observations in a UPA agree on the same period).

### Sub-Monthly Calibration Workflow

Sub-monthly analysis requires a separate call to `pnadc_apply_periods()` with the appropriate `calibration_unit`. This is because each calibration unit produces its own weight column (`weight_fortnight` or `weight_weekly`), calibrated to account for the non-random selection into sub-monthly determination.

**Important:** Use a clean copy of your data (before any prior `pnadc_apply_periods()` call) to avoid column conflicts from duplicate merges.

```{r submonthly-workflow}
# Step 1: Apply experimental strategies to the crosswalk
crosswalk_exp <- pnadc_experimental_periods(
  crosswalk,
  strategy = "both",
  confidence_threshold = 0.85,
  upa_proportion_threshold = 0.80,
  verbose = TRUE
)

# Step 2: Apply to CLEAN data with calibration_unit = "fortnight"
# (pnadc_apply_periods handles the crosswalk merge internally)
pnadc_fortnight <- pnadc_apply_periods(
  pnadc_clean,
  crosswalk_exp,
  weight_var = "V1028",
  anchor = "quarter",
  calibrate = TRUE,
  calibration_unit = "fortnight"
)

# Step 3: For weekly analysis, call again with calibration_unit = "week"
pnadc_weekly <- pnadc_apply_periods(
  pnadc_clean,
  crosswalk_exp,
  weight_var = "V1028",
  anchor = "quarter",
  calibrate = TRUE,
  calibration_unit = "week",
  smooth = FALSE  # No smoothing for weekly (too sparse)
)
```

### Example: COVID-19 Lockdown at Fortnightly Resolution

Brazilian lockdowns began around March 16, 2020---almost perfectly aligned with the fortnight boundary. Monthly data for March averages pre- and post-lockdown. Fortnightly data can separate them.

```{r fortnight-aggregate}
# Filter to COVID period with calibrated fortnight weights
pnadc_fn_covid <- pnadc_fortnight[
  !is.na(weight_fortnight) & Ano == 2020 & Trimestre %in% 1:2 & V2009 >= 14
]

# Compute participation rate by fortnight
fortnight_covid <- pnadc_fn_covid[, .(
  participation_rate = sum((VD4001 == 1) * weight_fortnight, na.rm = TRUE) /
                       sum(weight_fortnight, na.rm = TRUE),
  n_obs = .N
), by = .(ref_fortnight_yyyyff)]

# Add period Date column
fortnight_covid[, period := as.Date(paste0(
  ref_fortnight_yyyyff %/% 100, "-",
  ((ref_fortnight_yyyyff %% 100) - 1L) %/% 2L + 1L, "-",
  fifelse(ref_fortnight_yyyyff %% 2L == 1L, "1", "16")
))]
```

<details>
<summary>Show plotting code</summary>

```{r covid-fortnight}
# Monthly comparison data
covid_monthly_part <- monthly_total[
  period >= "2020-01-01" & period <= "2020-06-30",
  .(period, participation_rate)
]

ggplot() +
  # Monthly as reference
  geom_line(data = covid_monthly_part,
            aes(x = period, y = participation_rate),
            color = "darkred", linewidth = 1, alpha = 0.7) +
  geom_point(data = covid_monthly_part,
             aes(x = period, y = participation_rate),
             color = "darkred", size = 3) +

  # Fortnightly
  geom_point(data = fortnight_covid[n_obs >= 300],
             aes(x = period, y = participation_rate, size = n_obs),
             color = "#ff7f0e", alpha = 0.8) +
  scale_size_continuous(range = c(2, 5), guide = "none") +

  # Mark lockdown transition
  geom_vline(xintercept = as.Date("2020-03-16"),
             linetype = "dotted", color = "gray40") +

  scale_y_continuous(labels = percent_format(accuracy = 0.1)) +
  scale_x_date(date_breaks = "1 month", date_labels = "%b\n%Y") +
  labs(
    title = "COVID-19 Labor Force Participation: Monthly vs Fortnightly",
    subtitle = "Fortnightly points (orange) show sharper discontinuity at lockdown onset (experimental strategies)",
    x = NULL, y = "Labor Force Participation Rate",
    caption = "Uses pnadc_experimental_periods(strategy='both', conf=0.85, upa=0.80). Only fortnights with n >= 300 shown."
  ) +
  theme_minimal(base_size = 11)
```

</details>

![COVID Lockdown: Monthly vs Fortnightly](figures/applied-examples/fig-covid-fortnight-comparison.png){width=100%}

The fortnightly data shows a sharper discontinuity around March 16---the participation rate in the second fortnight of March dropped more steeply than the monthly average suggests.

### Example: Carnival Week Labor Market Effects

Carnival is Brazil's largest cultural event, concentrated in specific states (Bahia, Rio de Janeiro, Pernambuco). We **pool multiple years** (2019, 2022-2025, excluding COVID years) and compare Carnival states to other states, using **average hours worked** (VD4035):

```{r carnival-aggregate}
# Filter to Q1 of Carnival years with calibrated weekly weights
carnival_years <- c(2019L, 2022L, 2023L, 2024L, 2025L)
pnadc_wk <- pnadc_weekly[
  !is.na(weight_weekly) & Trimestre == 1L & Ano %in% carnival_years &
  V2009 >= 14 & VD4002 == 1 & !is.na(VD4035)
]

# Define Carnival weeks (ISO week of Carnival Tuesday)
carnival_weeks <- data.table(
  Ano = carnival_years,
  carnival_iso_week = c(10L, 9L, 8L, 7L, 10L)
)

# Extract ISO week and merge
pnadc_wk[, iso_week := ref_week_yyyyww %% 100L]
pnadc_wk <- merge(pnadc_wk, carnival_weeks, by = "Ano", all.x = TRUE)
pnadc_wk[, is_carnival_week := (iso_week == carnival_iso_week)]

# Flag Carnival-intensive states (BA=29, RJ=33, PE=26)
pnadc_wk[, is_carnival_state := UF %in% c(29L, 33L, 26L)]

# Aggregate: average hours by Carnival week x region (pooled across years)
weekly_carnival <- pnadc_wk[, .(
  avg_hours_worked = sum(VD4035 * weight_weekly, na.rm = TRUE) /
                     sum(weight_weekly, na.rm = TRUE),
  n_obs = .N
), by = .(is_carnival_week, is_carnival_state)]
```

<details>
<summary>Show plotting code</summary>

```{r carnival-weekly}
ggplot(weekly_carnival,
       aes(x = fifelse(is_carnival_week, "Carnival Week", "Other Weeks"),
           y = avg_hours_worked,
           fill = fifelse(is_carnival_state, "Carnival States (BA/RJ/PE)", "Other States"))) +
  geom_col(position = position_dodge(width = 0.8), width = 0.7) +
  scale_y_continuous(limits = c(0, 45)) +
  scale_fill_manual(values = c("Carnival States (BA/RJ/PE)" = "#ff7f0e",
                               "Other States" = "#1f77b4")) +
  labs(
    title = "Average Hours Worked by Week Type and Region",
    subtitle = "Pooled data from Q1 2019, 2022-2025 (experimental strategies)",
    x = NULL, y = "Average Hours Worked (per week)",
    fill = "Region",
    caption = "Uses pnadc_experimental_periods(strategy='both', conf=0.85, upa=0.80). Pooled across 5 years."
  ) +
  theme_minimal(base_size = 11)
```

</details>

![Carnival Week Effects](figures/applied-examples/fig-carnival-weekly-did.png){width=100%}

Workers in Carnival-intensive states reduce their average hours worked during Carnival week compared to other weeks, though the small weekly sample sizes mean estimates have considerable uncertainty.

### Practical Guidelines

| Recommended Granularity | Best For | Sample Strategy |
|------------------------|----------|-----------------|
| Monthly | Most analyses (~97% rate) | Standard strict algorithm |
| Fortnightly | Transition timing (~12% exp.) | Experimental strategies |
| Weekly | Within-week patterns (~6% exp.) | Experimental + pool multiple years |

---

## See Also

- [Get Started](getting-started.html) - Basic mensalization workflow
- [How It Works](how-it-works.html) - Algorithm details and technical explanation
- [Annual Poverty Analysis](annual-poverty-analysis.html) - Monthly poverty analysis with annual income data
- [Download and Prepare Data](download-and-prepare.html) - Data download workflow

---

## References

- HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62. Brasilia, DF: IPEA, 2020. <https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453>
- HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
- HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco, aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
- NERI, Marcelo; HECKSHER, Marcos. "A Montanha-Russa da Pobreza". FGV Social - Sumario Executivo. Rio de Janeiro: FGV, Junho/2022. <https://www.cps.fgv.br/cps/bd/docs/MontanhaRussaDaPobreza_Neri_Hecksher_FGV_Social.pdf>
- NERI, Marcelo; HECKSHER, Marcos. "A montanha-russa da pobreza mensal e um programa social alternativo". *Revista NECAT*, v. 11, n. 21, 2022.
- IBGE. Pesquisa Nacional por Amostra de Domicilios Continua (PNADC). <https://www.ibge.gov.br/estatisticas/sociais/trabalho/>
- Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version v0.1.0. <https://github.com/antrologos/PNADCperiods>

---
title: "From Rolling Quarters to Monthly Estimates: SIDRA Mensalization Guide"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{From Rolling Quarters to Monthly Estimates: SIDRA Mensalization Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!--
MAINTAINER NOTE:
This vignette uses eval=FALSE for all code chunks.
Pre-computed figures are generated by:
  code/precompute_sidra_mensalization.R

To regenerate figures and verify code:
  source("code/precompute_sidra_mensalization.R")

Plot code in the vignette is wrapped in <details> blocks and matches
the generation script exactly. Look for VIGNETTE CODE markers in the script.
-->

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  purl = FALSE
)
```

## Overview

Brazil's Continuous National Household Sample Survey (PNADC) publishes labor
market indicators as **rolling (moving) quarters** — 3-month moving averages
where each published "quarter" shares 2 months with its neighbors. This
smoothing hides short-term dynamics: turning points are delayed, seasonal
patterns are distorted, and international comparison becomes difficult.

The PNADCperiods package includes a SIDRA mensalization module that recovers
**exact monthly estimates** from rolling quarter data. This vignette explains
how to use it.

### Why Rolling Quarters Are Problematic

Each published "quarter" is actually a 3-month moving average:

- "2019-Q1" = average of Jan, Feb, Mar 2019
- "2019-Q2" = average of Feb, Mar, Apr 2019
- "2019-Q3" = average of Mar, Apr, May 2019

![Rolling quarters overlap: each 'quarter' shares 2 months with its neighbors](figures/sidra-mensalization/fig1_rolling_schematic.png){width=100%}

When unemployment jumps sharply in a single month, the rolling quarter spreads
that spike across multiple overlapping periods. The mensalization algorithm
inverts this averaging process to recover the true monthly values.

---

## Quick Start

```{r quickstart, eval=FALSE}
library(PNADCperiods)

# Step 1: Fetch rolling quarter data from SIDRA API
rolling_quarters <- fetch_sidra_rolling_quarters()

# Step 2: Convert to monthly estimates
monthly <- mensalize_sidra_series(rolling_quarters)

# Step 3: Use your monthly data!
head(monthly[, .(anomesexato, m_popocup, m_taxadesocup)])
```

That's it! You now have monthly estimates starting from January 2012.

1. **`fetch_sidra_rolling_quarters()`** downloaded 86+ economic indicators
   from IBGE's SIDRA API

2. **`mensalize_sidra_series()`** applied the mensalization formula using
   pre-computed starting points (bundled with the package)

3. The result is a `data.table` with one row per month and `m_*` columns
   for each mensalized series

---

## Understanding the Output

The mensalized output contains:

- `anomesexato`: Month identifier (YYYYMM format, e.g., 201903 = March 2019)
- `m_*` columns: Mensalized (monthly) estimates for each series
- Price indices: `ipca100dez1993`, `inpc100dez1993` (passed through for deflation)

**Key series include:**

| Column | Description | Unit |
|--------|-------------|------|
| `m_populacao` | Total population | Thousands |
| `m_pop14mais` | Population 14+ years | Thousands |
| `m_popocup` | Employed population | Thousands |
| `m_popdesocup` | Unemployed population | Thousands |
| `m_taxadesocup` | Unemployment rate | Percent |
| `m_taxapartic` | Labor force participation rate | Percent |
| `m_massahabnominaltodos` | Total nominal wage bill | Millions R$ |

Rate series (like `m_taxadesocup`) are **derived** from mensalized level
series when `compute_derived = TRUE` (the default). They are computed as
ratios of the mensalized levels, not directly mensalized from the rolling
quarter rates.

### Discovering Available Series

Use `get_sidra_series_metadata()` to explore all 86+ available series:

```{r metadata, eval=FALSE}
meta <- get_sidra_series_metadata()

# View series organized by theme
meta[, .N, by = .(theme, theme_category)]

# Filter to specific theme categories
meta[theme_category == "employment_type", .(series_name, description)]
```

The metadata uses a hierarchical taxonomy: `theme` (top level, e.g.,
"labor_market"), `theme_category` (e.g., "employment_type"), and optionally
`subcategory` (e.g., "levels", "rates").

---

## Data Flow

The mensalization process follows a three-step pipeline:

![Data flow from SIDRA to monthly estimates](figures/sidra-mensalization/fig2_data_flow.png){width=100%}

### Step 1: Fetching Rolling Quarter Data

`fetch_sidra_rolling_quarters()` downloads data from five SIDRA tables:

| Table | Content |
|-------|---------|
| 4093 | Population and labor force |
| 6390 | Income (nominal and real) |
| 6392 | Real income by occupation |
| 6399 | Employment by sector |
| 6906 | Underutilization indicators |

```{r fetch-inspect, eval=FALSE}
rq <- fetch_sidra_rolling_quarters(verbose = TRUE)

# Inspect structure
dim(rq)
names(rq)[1:20]
```

Key columns: `anomesfinaltrimmovel` (end month of rolling quarter, YYYYMM),
`mesnotrim` (month position 1/2/3), plus one column per series.

### Step 2: The Mensalization Transform

```{r mensalize-inspect, eval=FALSE}
monthly <- mensalize_sidra_series(rq, verbose = TRUE)

# Compare dimensions
cat("Rolling quarters:", nrow(rq), "rows\n")
cat("Monthly data:", nrow(monthly), "rows\n")
```

The row count is approximately the same (one per month), but the meaning
changes from "rolling quarter ending in month X" to "exact estimate for
month X".

### Step 3: Using Monthly Estimates

<details>
<summary>Show plotting code</summary>

```{r plot-unemployment, eval=FALSE}
# --- VIGNETTE CODE: plot-unemployment ---
library(ggplot2)

monthly[, date := as.Date(paste0(substr(anomesexato, 1, 4), "-",
                                  substr(anomesexato, 5, 6), "-01"))]

ggplot(monthly, aes(x = date, y = m_taxadesocup)) +
  geom_line(color = "#1976D2", linewidth = 0.8) +
  labs(title = "Monthly Unemployment Rate",
       x = NULL, y = "Unemployment Rate (%)")
```

</details>

### Population Data for Weighting

For analyses requiring monthly population estimates separately:

```{r pop-data, eval=FALSE}
pop <- fetch_monthly_population()
head(pop)
```

Returns a data.table with `ref_month_yyyymm` and `m_populacao` columns.

---

## Working with Series

### Fetching by Theme

Instead of fetching all 86+ series, filter by theme or theme category:

```{r by-theme, eval=FALSE}
# Only employment type series
employment <- fetch_sidra_rolling_quarters(theme_category = "employment_type")

# Only wage mass series
wages <- fetch_sidra_rolling_quarters(theme_category = "wage_mass")

# Only labor market theme (includes participation, unemployment, employment types, etc.)
labor <- fetch_sidra_rolling_quarters(theme = "labor_market")
```

### Fetching Specific Series

For maximum efficiency, request only the series you need:

```{r specific-series, eval=FALSE}
# Only unemployment-related series
unemp <- fetch_sidra_rolling_quarters(
  series = c("popdesocup", "taxadesocup", "popnaforca")
)
```

### Excluding Derived Series

Some series are rates computed from other series. To fetch only "base" series:

```{r exclude-derived, eval=FALSE}
# Exclude computed rates (only population and income levels)
base_only <- fetch_sidra_rolling_quarters(exclude_derived = TRUE)
```

### Selecting Output Columns

After mensalization, select columns as needed:

```{r select-columns, eval=FALSE}
monthly <- mensalize_sidra_series(rq)

# Select specific series
labor_market <- monthly[, .(
  anomesexato,
  employed = m_popocup,
  unemployed = m_popdesocup,
  unemp_rate = m_taxadesocup,
  participation = m_taxapartic
)]
```

---

## The Mensalization Methodology

*This section can be skipped by users who just need results.*

### The Core Concept

Rolling quarters are 3-month moving averages. If we denote the true monthly
value for month $t$ as $y_t$, then the rolling quarter value $x_t$ is:

$$x_t = \frac{y_{t-2} + y_{t-1} + y_t}{3}$$

The mensalization algorithm inverts this relationship to recover $y_t$ from
the sequence of $x_t$ values.

### The Mensalization Formula

**Step 1: Compute first differences**

$$d3_t = x_t - x_{t-1}$$

**Step 2: Identify month position (mesnotrim)**

Each month has a position within its quarter:
- Position 1: Jan, Apr, Jul, Oct
- Position 2: Feb, May, Aug, Nov
- Position 3: Mar, Jun, Sep, Dec

**Step 3: Cumulative sum by position**

For each position separately, compute the cumulative sum of first differences,
starting from a calibrated "starting point" $y_0$:

$$y_t = y_0 + \sum_{s \in \text{same position}, s \leq t} d3_s$$

![Mensalization process: rolling quarters (blue) vs monthly estimates (red)](figures/sidra-mensalization/fig3_mensalization_process.png){width=100%}

### The Role of Starting Points ($y_0$)

The starting point $y_0$ is crucial. It determines the **level** of all
subsequent monthly estimates. The package includes pre-computed starting points
for 53 series, calibrated during the stable 2013-2019 period.

Starting points are computed by:

1. Processing PNADC microdata to get "true" monthly aggregates ($z$ values)
2. Comparing these to rolling quarters
3. Finding the $y_0$ that makes $y_0 + \text{cumsum}(d3)$ match the microdata

### Assumptions and Limitations

- Monthly values within each position evolve smoothly
- The calibration period (2013-2019) reflects "normal" conditions
- Cannot recover intra-month variation
- Starting points are calibrated to national totals (not regional breakdowns)

---

## Practical Considerations

### API Caching

The package caches SIDRA API responses in memory during your R session:

```{r cache, eval=FALSE}
# First call: fetches from API (~10 seconds)
rq1 <- fetch_sidra_rolling_quarters()

# Second call with use_cache = TRUE: uses cached data (instant)
rq2 <- fetch_sidra_rolling_quarters(use_cache = TRUE)

# Clear all cached data (force fresh fetch on next call)
clear_sidra_cache()
```

The cache persists until you call `clear_sidra_cache()` or restart R.

### Common Errors

| Error | Cause | Solution |
|-------|-------|----------|
| "Series not found" | Misspelled series name | Check `get_sidra_series_metadata()` |
| "API timeout" | SIDRA server slow | Retry; use `use_cache = TRUE` |
| "No starting points" | Custom series | See Custom Starting Points below |

```{r error-handling, eval=FALSE}
# Check if series exists
meta <- get_sidra_series_metadata()
"taxadesocup" %in% meta$series_name  # TRUE
```

### Data Quality Notes

**COVID-19 disruptions (2020):** IBGE suspended in-person interviews during
the pandemic. Some indicators show unusual patterns in 2020-Q2.

**CNPJ series availability:** Series based on CNPJ registration
(empregadorcomcnpj, contapropriacomcnpj, etc.) are only available from
October 2015, when V4019 was introduced.

---

## Custom Starting Points

*For users with calibrated PNADC microdata.*

Use the bundled starting points (default) unless:

1. **Your series isn't bundled** — Custom variable definitions
2. **Different calibration period** — Non-standard reference period
3. **Regional breakdown** — State or metro-area mensalization

### Option A: All-in-One Function

```{r custom-y0-allinone, eval=FALSE}
# Load your stacked PNADC microdata (with pnadc_apply_periods weights)
stacked <- readRDS("my_calibrated_pnadc.rds")

# Compute starting points
custom_y0 <- compute_starting_points_from_microdata(
  data = stacked,
  calibration_start = 201301L,
  calibration_end = 201912L,
  verbose = TRUE
)

# Use custom starting points
monthly <- mensalize_sidra_series(rq, starting_points = custom_y0)
```

### Option B: Step-by-Step

```{r custom-y0-stepbystep, eval=FALSE}
# Step 1: Build crosswalk and calibrate
crosswalk <- pnadc_identify_periods(stacked)
calibrated <- pnadc_apply_periods(
  stacked, crosswalk,
  weight_var = "V1028",
  anchor = "quarter",
  calibration_unit = "month"
)

# Step 2: Compute z_ aggregates (monthly totals from microdata)
z_agg <- compute_z_aggregates(calibrated)

# Step 3: Fetch rolling quarters for comparison
rq <- fetch_sidra_rolling_quarters()

# Step 4: Compute starting points
y0 <- compute_series_starting_points(
  monthly_estimates = z_agg,
  rolling_quarters = rq,
  calibration_start = 201301L,
  calibration_end = 201912L
)

# Step 5: Use custom starting points
result <- mensalize_sidra_series(rq, starting_points = y0)
```

CNPJ-based series automatically use a later calibration period (2016-2019)
when `use_series_specific_periods = TRUE` (the default in
`compute_series_starting_points()`).

### Validating Custom Starting Points

```{r validate-y0, eval=FALSE}
bundled <- pnadc_series_starting_points

# Merge and compare
comp <- merge(custom_y0, bundled,
              by = c("series_name", "mesnotrim"),
              suffixes = c("_custom", "_bundled"))

comp[, rel_diff := abs(y0_custom - y0_bundled) / abs(y0_bundled) * 100]
comp[rel_diff > 1]  # Flag series with >1% difference
```

---

## Case Study: COVID-19 Unemployment

How quickly did unemployment rise when COVID-19 hit Brazil? Rolling quarter
data obscures these dynamics. Monthly estimates reveal the exact timing.

<details>
<summary>Show analysis code</summary>

```{r covid-analysis, eval=FALSE}
# --- VIGNETTE CODE: covid-analysis ---
# Fetch all series and mensalize
rq <- fetch_sidra_rolling_quarters()
monthly <- mensalize_sidra_series(rq)

# Filter to COVID period
covid_period <- monthly[anomesexato >= 201901 & anomesexato <= 202212]

# Create date column
covid_period[, date := as.Date(paste0(
  substr(anomesexato, 1, 4), "-",
  substr(anomesexato, 5, 6), "-01"
))]

# Find peak
peak_month <- covid_period[which.max(m_taxadesocup)]
cat("Peak unemployment:", peak_month$m_taxadesocup, "% in",
    format(peak_month$date, "%B %Y"), "\n")
```

</details>

![Monthly vs rolling quarter unemployment rate (2019-2023)](figures/sidra-mensalization/fig4_monthly_vs_quarterly.png){width=100%}

![COVID-19 impact on Brazilian unemployment](figures/sidra-mensalization/fig5_covid_case_study.png){width=100%}

**Key findings from monthly estimates:**

1. **Exact peak timing**: Monthly data pinpoints the peak month, while
   rolling quarters show only a gradual rise

2. **Speed of impact**: The monthly series reveals a sharp spike that
   rolling quarters smooth over 3+ months

3. **Recovery dynamics**: Monthly estimates show pauses and reversals
   in recovery that are hidden in quarterly averages

---

## Series Naming Conventions

| Pattern | Meaning | Example |
|---------|---------|---------|
| `m_` | Mensalized monthly estimate | `m_popocup` |
| `pop*` | Population count | `populacao`, `pop14mais` |
| `*comcart` | With formal contract | `empregprivcomcart` |
| `*semcart` | Without formal contract | `empregprivsemcart` |
| `*comcnpj` | With CNPJ registration | `empregadorcomcnpj` |
| `taxa*` | Rate (percent) | `taxadesocup` |
| `nivel*` | Level/ratio (percent) | `nivelocup` |
| `rend*` | Income (rendimento) | `rendhabnominaltodos` |
| `massa*` | Wage bill (massa salarial) | `massahabnominaltodos` |
| `*hab*` | Usually received (habitual) | `rendhabnominaltodos` |
| `*efet*` | Actually received (efetivo) | `rendefetnominaltodos` |

For the complete catalog, use `get_sidra_series_metadata()`:

```{r programmatic-access, eval=FALSE}
meta <- get_sidra_series_metadata()

# Filter by theme category
meta[theme_category == "employment_type", .(series_name, description)]

# Filter by theme and pattern
meta[theme == "labor_market" & grepl("taxa|nivel", series_name),
     .(series_name, description)]
```

---

## Function Reference

| Function | Purpose |
|----------|---------|
| `fetch_sidra_rolling_quarters()` | Download rolling quarter data from SIDRA API |
| `fetch_monthly_population()` | Get monthly population estimates |
| `mensalize_sidra_series()` | Convert rolling quarters to monthly estimates |
| `get_sidra_series_metadata()` | Explore available series and metadata |
| `clear_sidra_cache()` | Clear cached API data |
| `compute_z_aggregates()` | Compute monthly aggregates from calibrated microdata |
| `compute_series_starting_points()` | Compute $y_0$ values from aggregates |
| `compute_starting_points_from_microdata()` | All-in-one $y_0$ computation |

**Bundled data:** `pnadc_series_starting_points` — pre-computed $y_0$ for
53 series x 3 month positions (calibration period: 2013-2019).

---

## References

- HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62. Brasilia, DF: IPEA, 2020. <https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453>
- HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
- HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco, aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
- Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version v0.1.0. <https://github.com/antrologos/PNADCperiods>

---

## Further Reading

- `vignette("getting-started")` — Setting up PNADC microdata analysis
- `vignette("how-it-works")` — The period identification algorithm
- `vignette("applied-examples")` — Applied research examples
- **IBGE SIDRA API**: https://sidra.ibge.gov.br/
- **Package repository**: https://github.com/antrologos/PNADCperiods

---
title: "Complex Survey Design with Monthly Weights"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Complex Survey Design with Monthly Weights}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!--
MAINTAINER NOTE:
The code chunks in this vignette must match the code in:
  mensalizacao_pnad/code/generate_complex_survey_figures.R

Look for comments like "# --- VIGNETTE CODE: chunk-name ---" in that script.
To regenerate figures, run that script. Changes to code here
must be mirrored in the generation script (and vice versa).
-->

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  echo = TRUE,
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 10,
  fig.height = 6,
  purl = FALSE
)
```

## Introduction

When you compute a statistic from survey data---say, the unemployment rate is 7.2%---that number alone doesn't tell you much. Is 7.2% meaningfully different from last month's 7.0%? Without **standard errors** and **confidence intervals**, you can't answer this question.

This matters especially when tracking changes over time. Monthly data from `PNADCperiods` reveals dynamics hidden in quarterly averages, but those sharp month-to-month movements could be real economic changes or just sampling variability. Standard errors let you distinguish signal from noise.

The challenge is that PNADC uses a **complex sample design**---stratification (`Estrato`), clustering (`UPA`), and unequal probabilities---which means standard formulas underestimate variance. This vignette shows two strategies for proper variance estimation:

1. **Linearization**: Using design variables directly---simple and fast
2. **Replication weights**: Adjusting IBGE's 200 bootstrap weights---more intensive but exact

We demonstrate both using the **gender gap in labor force participation**, which widened dramatically during COVID-19.

---

## Why Standard Errors Matter

Before diving into methodology, consider the figure below. It shows labor force participation for men and women during COVID-19. The **shaded bands** are 95% confidence intervals from proper complex survey analysis.

<details>
<summary>Show plotting code</summary>

```{r plot-why-se-matter, fig.width=10, fig.height=5.5}
# Subset to COVID period for focused visualization
p3_data <- results_by_sex[period >= "2019-01-01" & period <= "2022-06-01"]

p3 <- ggplot(p3_data, aes(x = period, y = participation, color = sex)) +

  # Just the lines (no CI)
  geom_line(linewidth = 0.9) +

  # Add confidence bands with low alpha
  geom_ribbon(aes(ymin = ci_l, ymax = ci_u, fill = sex),
              alpha = 0.25, color = NA) +

  # Scales and labels
  scale_y_continuous(
    labels = percent_format(accuracy = 1),
    limits = c(0.45, 0.80)
  ) +
  scale_x_date(date_breaks = "6 months", date_labels = "%b\n%Y") +
  scale_color_manual(values = c("Men" = "#0074D9", "Women" = "#FF4136")) +
  scale_fill_manual(values = c("Men" = "#0074D9", "Women" = "#FF4136")) +

  # Add annotations for interpretation
  annotate("segment",
           x = as.Date("2020-06-15"), xend = as.Date("2020-06-15"),
           y = 0.52, yend = 0.62,
           arrow = arrow(ends = "both", length = unit(0.1, "cm")),
           color = "gray30") +
  annotate("text",
           x = as.Date("2020-06-15"), y = 0.50,
           label = "Is this drop\nstatistically\nsignificant?",
           size = 2.8, color = "gray30") +

  labs(
    title = "Why Confidence Intervals Matter: COVID-19 Labor Market Shock",
    subtitle = "Confidence bands (shaded) show the precision of estimates. Overlapping bands suggest uncertain differences.",
    x = NULL,
    y = "Labor Force Participation Rate",
    color = "Sex",
    fill = "Sex",
    caption = "Source: PNADC/IBGE. Confidence bands show +/- 1.96 SE from complex survey design."
  ) +

  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    legend.position = "bottom",
    panel.grid.minor = element_blank()
  )
p3
```

</details>

![Why Confidence Intervals Matter: COVID-19 Labor Market Shock](figures/complex-survey-design/fig-why-se-matter.png){width=100%}

Without confidence intervals, we might over-interpret month-to-month fluctuations as real economic changes. With them, we can assess which movements are statistically meaningful.

---

## Prerequisites

```{r packages}
# Core packages
library(PNADCperiods)
library(data.table)
library(fst)

# Survey analysis
library(survey)

# Visualization
library(ggplot2)
library(scales)
```

---

## Loading and Preparing Data

```{r load-data}
# Load stacked PNADC quarterly data (all years available)
# Requires: identification vars, calibration vars, Estrato, plus analysis vars
files <- list.files("D:/Dropbox/Bancos_Dados/PNADC/Trimestral/Dados/",
                    pattern = "^pnadc_\\d{4}-\\dq\\.fst$", full.names = TRUE)

cols_needed <- c(
  # Mensalization keys
  "Ano", "Trimestre", "UPA", "V1008", "V1014",
  # Birthday variables (for identification)
  "V2008", "V20081", "V20082", "V2009",
  # Calibration variables
  "V1028", "UF", "posest", "posest_sxi",
  # Survey design variables
  "Estrato",
  # Analysis variables
  "VD4001", "VD4002", "V2007"
)

pnadc <- rbindlist(lapply(files, function(f) {
  fst::read_fst(f, as.data.table = TRUE, columns = cols_needed)
}), fill = TRUE)

# Step 1: Build crosswalk (identify reference periods)
crosswalk <- pnadc_identify_periods(pnadc, verbose = TRUE)

# Step 2: Apply crosswalk and calibrate weights
pnadc <- pnadc_apply_periods(pnadc, crosswalk,
                              weight_var = "V1028",
                              anchor = "quarter",
                              calibrate = TRUE,
                              calibration_unit = "month",
                              verbose = TRUE)

# Filter to determined observations with monthly weights
pnadc_monthly <- pnadc[!is.na(weight_monthly)]

# Handle singleton strata globally
options(survey.lonely.psu = "adjust")
```

---

## Strategy 1: Linearization with Strata and PSU

This approach uses **Taylor series linearization** to estimate variances. PNADC uses a two-stage stratified cluster sample: within each stratum (`Estrato`), Primary Sampling Units (`UPA`) are selected with probability proportional to size, then households within selected UPAs are interviewed.

### Setting Up a Survey Design

```{r design-linearization}
# Create survey design object for a specific month (e.g., January 2020)
pnadc_jan2020 <- pnadc_monthly[ref_month_yyyymm == 202001 & V2009 >= 14]

design_jan2020 <- svydesign(
  ids = ~UPA,                    # PSU/cluster identifier
  strata = ~Estrato,             # Stratification variable
  weights = ~weight_monthly,     # Monthly-adjusted weight
  data = pnadc_jan2020,
  nest = TRUE                    # UPAs nested within strata
)

# Labor force participation rate
svymean(~I(VD4001 == 1), design_jan2020, na.rm = TRUE)
confint(svymean(~I(VD4001 == 1), design_jan2020, na.rm = TRUE))
```

### Monthly Time Series with Confidence Intervals

To create a full monthly series, loop over months with a function that handles edge cases:

```{r compute-gender-series}
# Prepare analysis data
pnadc_monthly[, V2009 := as.integer(V2009)]
pnadc_analysis <- pnadc_monthly[V2009 >= 14]
pnadc_analysis[, VD4001 := as.integer(VD4001)]
pnadc_analysis[, in_labor_force := fifelse(VD4001 == 1, 1L, 0L)]
pnadc_analysis[, V2007 := as.integer(V2007)]
pnadc_analysis[, sex := fifelse(V2007 == 1, "Men", "Women")]

# Function to compute participation by sex for one month
compute_participation_by_sex <- function(month_yyyymm, data) {

  month_data <- data[ref_month_yyyymm == month_yyyymm]

  if (nrow(month_data) < 100) return(NULL)

  tryCatch({
    design <- svydesign(
      ids = ~UPA,
      strata = ~Estrato,
      weights = ~weight_monthly,
      data = month_data,
      nest = TRUE
    )

    result <- svyby(
      ~in_labor_force,
      by = ~sex,
      design = design,
      FUN = svymean,
      na.rm = TRUE,
      vartype = c("se", "ci")
    )

    setDT(result)
    result[, ref_month_yyyymm := month_yyyymm]
    result
  }, error = function(e) {
    cat("Error processing month", month_yyyymm, ":", e$message, "\n")
    NULL
  })
}

# Compute for all months
months <- sort(unique(pnadc_analysis$ref_month_yyyymm))
results_by_sex <- rbindlist(
  lapply(months, compute_participation_by_sex, data = pnadc_analysis)
)

# Rename estimate column for clarity
setnames(results_by_sex, "in_labor_force", "participation")

# Add date column for plotting
results_by_sex[, period := as.Date(paste0(
  ref_month_yyyymm %/% 100, "-",
  ref_month_yyyymm %% 100, "-15"
))]
```

The `tryCatch` wrapper handles months where the survey design might fail (e.g., sparse data in early pandemic months). The `svyby()` function returns columns `participation`, `se`, `ci_l`, and `ci_u`.

---

## Strategy 2: Replication Weights

IBGE provides 200 **bootstrap replication weights** (`V1028001` to `V1028200`) with each quarterly release. This approach is more computationally intensive (201 computations per statistic) but handles complex statistics like quantiles, Gini coefficients, and regression coefficients without special variance formulas.

The key insight: apply the **same calibration ratio** to each replication weight:

$$
\text{V1028}_k^{\text{monthly}} = \text{V1028}_k \times \frac{\text{weight\_monthly}}{\text{V1028}}
$$

```{r create-rep-weights}
# Calculate the adjustment ratio
pnadc_monthly[, weight_ratio := weight_monthly / V1028]

# Get names of replication weight columns
rep_weight_cols <- paste0("V1028", sprintf("%03d", 1:200))

# Create adjusted replication weight columns
for (col in rep_weight_cols) {
  new_col <- paste0(col, "_monthly")
  pnadc_monthly[, (new_col) := get(col) * weight_ratio]
}
```

```{r design-replication}
# Create replicate weights design (e.g., for January 2020)
pnadc_jan2020_rep <- pnadc_monthly[ref_month_yyyymm == 202001 & V2009 >= 14]
rep_weight_cols_monthly <- paste0("V1028", sprintf("%03d", 1:200), "_monthly")

design_rep <- svrepdesign(
  data = pnadc_jan2020_rep,
  weights = ~weight_monthly,
  repweights = rep_weight_cols_monthly,
  type = "bootstrap",
  combined.weights = TRUE
)

# Use the same svymean/svyby functions as with linearization
svymean(~I(VD4001 == 1), design_rep, na.rm = TRUE)
```

### Comparing Strategies

| Aspect | Linearization | Replication |
|--------|--------------|-------------|
| **Setup** | Simpler (strata + PSU) | More complex (200 weight columns) |
| **Speed** | Fast | Slower (200x calculations) |
| **Complex statistics** | May need special formulas | Handles automatically |
| **Best for** | Routine analysis | Quantiles, Gini, publications |

**Recommendation**: Use linearization for exploratory analysis and most routine work. Use replication when computing complex statistics or preparing results for publication.

---

## Practical Example: Gender Gap in Labor Force Participation

Brazil has one of the largest gender gaps in labor force participation among major economies---about 20 percentage points. COVID-19 provides a natural experiment: school closures and increased caregiving demands fell disproportionately on women. Monthly data lets us track whether the gap widened, and confidence intervals tell us if the change was statistically significant.

### Computing the Gender Gap

The gap is a derived statistic: the difference between two survey estimates. We derive it from the by-sex results already computed above:

```{r compute-gap}
# Reshape to wide format to compute gap
results_wide <- dcast(results_by_sex, ref_month_yyyymm + period ~ sex,
                      value.var = c("participation", "se"))

# Compute gap and its SE
# SE of difference: SE(gap) = sqrt(SE_men^2 + SE_women^2)
results_wide[, `:=`(
  gap = participation_Men - participation_Women,
  gap_se = sqrt(se_Men^2 + se_Women^2)
)]

results_wide[, `:=`(
  gap_ci_lower = gap - 1.96 * gap_se,
  gap_ci_upper = gap + 1.96 * gap_se
)]
```

### Visualizing the Results

<details>
<summary>Show plotting code</summary>

```{r plot-gender-gap, fig.width=10, fig.height=6}
# Reshape back to long for plotting
results_long <- results_by_sex[!is.na(participation)]

# Ensure proper CI columns exist
if (!"ci_l" %in% names(results_long)) {
  results_long[, ci_l := participation - 1.96 * se]
  results_long[, ci_u := participation + 1.96 * se]
}

p1 <- ggplot(results_long, aes(x = period, y = participation, color = sex, fill = sex)) +

  # Confidence bands
  geom_ribbon(aes(ymin = ci_l, ymax = ci_u), alpha = 0.2, color = NA) +

  # Point estimates
  geom_line(linewidth = 0.8) +

  # Highlight COVID period
  annotate("rect",
           xmin = as.Date("2020-03-01"), xmax = as.Date("2021-12-31"),
           ymin = -Inf, ymax = Inf,
           fill = "gray80", alpha = 0.3) +
  annotate("text",
           x = as.Date("2021-01-01"), y = 0.85,
           label = "COVID-19", fontface = "italic", size = 3, color = "gray40") +

  # Scales and labels
  scale_y_continuous(
    labels = percent_format(accuracy = 1),
    limits = c(0.45, 0.85),
    breaks = seq(0.45, 0.85, 0.05)
  ) +
  scale_x_date(date_breaks = "1 year", date_labels = "%Y") +
  scale_color_manual(values = c("Men" = "#0074D9", "Women" = "#FF4136")) +
  scale_fill_manual(values = c("Men" = "#0074D9", "Women" = "#FF4136")) +

  labs(
    title = "Labor Force Participation by Sex: Monthly Series with 95% Confidence Intervals",
    subtitle = "Brazil, 2012-2025. Shaded regions show 95% confidence bands from complex survey design.",
    x = NULL,
    y = "Labor Force Participation Rate",
    color = "Sex",
    fill = "Sex",
    caption = "Source: PNADC/IBGE. Monthly weights from PNADCperiods. Variance estimated via Taylor linearization."
  ) +

  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    legend.position = "bottom",
    panel.grid.minor = element_blank()
  )
p1
```

</details>

![Labor Force Participation by Sex with 95% Confidence Intervals](figures/complex-survey-design/fig-gender-participation-ci.png){width=100%}

The confidence bands for men and women never overlap, confirming that the gender gap is statistically significant in every month. Women's participation dropped more sharply during COVID-19 (from ~53% to below 47%) than men's (from ~73% to ~66%).

<details>
<summary>Show plotting code</summary>

```{r plot-gap, fig.width=10, fig.height=5}
# Pre-COVID average for reference line
pre_covid_avg <- results_wide[period < "2020-03-01", mean(gap, na.rm = TRUE)]

p2 <- ggplot(results_wide[!is.na(gap)], aes(x = period, y = gap)) +

  # Confidence band
  geom_ribbon(aes(ymin = gap_ci_lower, ymax = gap_ci_upper),
              fill = "#7b3294", alpha = 0.3) +

  # Point estimate
  geom_line(color = "#7b3294", linewidth = 0.9) +

  # Reference line at pre-COVID average
  geom_hline(
    yintercept = pre_covid_avg,
    linetype = "dashed", color = "gray50"
  ) +
  annotate("text", x = min(results_wide$period, na.rm = TRUE) + 180,
           y = pre_covid_avg + 0.008,
           label = paste0("Pre-COVID avg: ", sprintf("%.1f", pre_covid_avg * 100), " p.p."),
           size = 3, color = "gray40", hjust = 0) +

  # Highlight COVID period
  annotate("rect",
           xmin = as.Date("2020-03-01"), xmax = as.Date("2021-12-31"),
           ymin = -Inf, ymax = Inf,
           fill = "gray80", alpha = 0.3) +

  # Scales and labels
  scale_y_continuous(
    labels = percent_format(accuracy = 1),
    breaks = seq(0.15, 0.30, 0.025)
  ) +
  scale_x_date(date_breaks = "1 year", date_labels = "%Y") +

  labs(
    title = "Gender Gap in Labor Force Participation (Men - Women)",
    subtitle = "Monthly series with 95% CI. Dashed line = pre-COVID average. Gap widened during pandemic.",
    x = NULL,
    y = "Participation Gap (percentage points)",
    caption = "Source: PNADC/IBGE. Monthly weights from PNADCperiods."
  ) +

  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    panel.grid.minor = element_blank()
  )
p2
```

</details>

![Gender Gap in Labor Force Participation](figures/complex-survey-design/fig-gender-gap-difference.png){width=100%}

The gap widened from ~20 to ~23 percentage points during COVID-19, and the confidence bands confirm this increase was statistically significant. The gap returned to pre-pandemic levels by 2022.

---

## Practical Tips

### Singleton Strata

Some months may have strata with only one PSU. The `options(survey.lonely.psu = "adjust")` setting (shown in the data preparation section above) provides a conservative adjustment. Other options include `"remove"` and `"average"`.

### Subpopulation Analysis

When analyzing subgroups, **always subset the design object**, not the raw data:

```{r subpop}
# CORRECT: Subset the design
design_women <- subset(design_jan2020, V2007 == 2)
svymean(~I(VD4001 == 1), design_women, na.rm = TRUE)

# INCORRECT: Don't do this - loses design information
# bad_design <- svydesign(..., data = pnadc[V2007 == 2])
```

Subsetting the data before creating the design object loses information about the full sample structure, leading to incorrect variance estimates.

---

## Summary

This vignette demonstrated two approaches for computing standard errors with monthly PNADC weights:

1. **Linearization**: Use `Estrato` as strata, `UPA` as PSU, and `weight_monthly` as weight in `svydesign()`. Fast, simple, works for most statistics.
2. **Replication**: Multiply each bootstrap weight by `weight_monthly / V1028`. More intensive but exact for complex statistics.

The gender gap example demonstrated that confidence intervals transform a claim like "the gap looks bigger" into "the gap increased by 2-3 percentage points, and we're 95% confident it's real."

---

## References

- HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62. Brasilia, DF: IPEA, 2020. <https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453>
- HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
- HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco, aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
- Thomas Lumley (2010). *Complex Surveys: A Guide to Analysis Using R*. Wiley.
- IBGE. *Notas Metodologicas da PNAD Continua*.
- Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version v0.1.0. <https://github.com/antrologos/PNADCperiods>

---

## Further Reading

- [Get Started](getting-started.html) - Basic mensalization workflow
- [Applied Examples](applied-examples.html) - Monthly labor market analysis with figures

---
title: "Download and Prepare PNADC Data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Download and Prepare PNADC Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  echo = TRUE,
  message = FALSE,
  warning = FALSE,
  purl = FALSE
)
```

## Introduction

This vignette provides a complete workflow for downloading Brazil's quarterly PNADC (Pesquisa Nacional por Amostra de Domicilios Continua) microdata and preparing it for mensalization. The workflow covers three steps:

1. **Downloading** quarterly PNADC microdata from IBGE using the `PNADcIBGE` package
2. **Stacking** multiple quarters into a single dataset (critical for high determination rates)
3. **Applying mensalization** using the `PNADCperiods` package

If you already have PNADC data and want to learn the package API first, see [Get Started](getting-started.html). For algorithm details, see [How PNADCperiods Works](how-it-works.html).

---

## Prerequisites

### Required Packages

```{r packages}
# Install packages if needed
install.packages(c("PNADCperiods", "PNADcIBGE", "fst"))

# Load packages
library(PNADcIBGE)
library(data.table)
library(fst)
library(PNADCperiods)
```

### System Requirements

- **Disk space**: ~5 GB for 2020-2024 data, ~15 GB for full history (2012-present)
- **RAM**: At least 8 GB recommended; 16 GB for comfortable processing
- **Time**: 2-3 hours for downloading (depends on internet speed), ~5 minutes for processing
- **Internet**: Required for downloading data and for SIDRA API access (weight calibration)

---

## Understanding PNADC Data

PNADC is Brazil's primary household survey for labor market statistics, conducted by IBGE. The survey uses a rotating panel design where each household is interviewed five times over 15 months. Each quarterly release contains approximately 500,000 observations.

**Why stack multiple quarters?** The mensalization algorithm identifies reference months by tracking households across their panel interviews. With a single quarter, the determination rate is only ~70%. By stacking multiple quarters, the algorithm leverages the rotating panel structure to achieve **over 97% determination**.

| Quarters Stacked | Month % | Fortnight % | Week % |
|------------------|---------|-------------|--------|
| 1 (single quarter) | ~70% | ~7% | ~2% |
| 8 (2 years) | ~94% | ~9% | ~3% |
| 20 (5 years) | ~95% | ~8% | ~3% |
| 55+ (full history) | **~97%** | **~9%** | **~3%** |

For most applications, we recommend stacking at least 2 years (8 quarters) of data.

---

## Step 1: Set Up Your Environment

```{r setup-dir}
# Set your data directory (adjust path as needed)
data_dir <- "path/to/your/pnadc_data/"
dir.create(data_dir, recursive = TRUE, showWarnings = FALSE)
```

## Step 2: Define Which Quarters to Download

Create a grid of year-quarter combinations. This example uses 2020-2024, which provides a good balance between data size and determination rate:

```{r editions}
# Define quarters to download (2020-2024 example)
editions <- expand.grid(
  year = 2020:2024,
  quarter = 1:4
)
# If downloading recent years, filter out quarters not yet available:
# editions <- editions[!(editions$year == 2025 & editions$quarter > 3), ]
```

## Step 3: Download the Data

The download loop fetches each quarter from IBGE and saves it in FST format for fast loading:

```{r download-loop}
for (i in 1:nrow(editions)) {
  year_i <- editions$year[i]
  quarter_i <- editions$quarter[i]

  filename <- paste0("pnadc_", year_i, "-", quarter_i, "q.fst")
  cat("Downloading:", year_i, "Q", quarter_i, "\n")

  # Download from IBGE
  pnadc_quarter <- get_pnadc(
    year = year_i,
    quarter = quarter_i,
    labels = FALSE,    # IMPORTANT: Use numeric codes, not labels
    deflator = FALSE,
    design = FALSE,
    savedir = data_dir
  )

  # Save as FST format (fast serialization)
  write_fst(pnadc_quarter, file.path(data_dir, filename))

  # Clean up temporary files created by PNADcIBGE
  temp_files <- list.files(data_dir,
                           pattern = "\\.(zip|sas|txt)$",
                           full.names = TRUE)
  file.remove(temp_files)

  rm(pnadc_quarter)
  gc()
}
```

> **Important**: Always use `labels = FALSE` when downloading. The mensalization algorithm requires numeric codes for the birthday variables (V2008, V20081, V20082). Using labeled factors will cause errors.

---

## Step 4: Stack the Quarterly Files

Stack all quarterly files into a single dataset. To save memory, only load the columns needed for mensalization:

```{r stack-data}
# Columns needed for mensalization
cols_needed <- c(
  # Time and identifiers
  "Ano", "Trimestre", "UPA", "V1008", "V1014",
  # Birthday variables (for reference period algorithm)
  "V2008", "V20081", "V20082", "V2009",
  # Weight and stratification (for weight calibration)
  "V1028", "UF", "posest", "posest_sxi"
)

# Stack all quarters
files <- list.files(data_dir, pattern = "pnadc_.*\\.fst$", full.names = TRUE)

pnadc_stacked <- rbindlist(lapply(files, function(f) {
  cat("Loading:", basename(f), "\n")
  read_fst(f, columns = cols_needed, as.data.table = TRUE)
}))

cat("Total observations:", format(nrow(pnadc_stacked), big.mark = ","), "\n")
```

---

## Step 5: Apply Mensalization

Build the crosswalk (identify reference periods) and calibrate weights:

```{r mensalize}
# Step 1: Build crosswalk (identify reference periods)
crosswalk <- pnadc_identify_periods(pnadc_stacked, verbose = TRUE)

# Check determination rates
crosswalk[, .(
  month_rate = mean(determined_month),
  fortnight_rate = mean(determined_fortnight),
  week_rate = mean(determined_week)
)]

# Step 2: Apply crosswalk and calibrate weights
result <- pnadc_apply_periods(
  data = pnadc_stacked,
  crosswalk = crosswalk,
  weight_var = "V1028",
  anchor = "quarter",
  calibrate = TRUE,
  calibration_unit = "month",
  verbose = TRUE
)
```

The verbose output shows progress and determination rates for each phase (month, fortnight, week). With 20 quarters stacked (2020-2024), expect ~95% month determination.

---

## Step 6: Explore the Results

The result contains all original columns plus reference period indicators and calibrated weights:

```{r explore}
# Key new columns
names(result)[grep("ref_|determined_|weight_", names(result))]

# Distribution of reference months within quarters
result[, .N, by = ref_month_in_quarter][order(ref_month_in_quarter)]
```

Key output columns:

| Column | Description |
|--------|-------------|
| `ref_month_in_quarter` | Position within quarter (1, 2, or 3; NA if indeterminate) |
| `ref_month_yyyymm` | Reference month as YYYYMM integer (e.g., 202301) |
| `determined_month` | Logical flag (TRUE if month was determined) |
| `weight_monthly` | Calibrated monthly weight (if `calibrate = TRUE`) |

The distribution is approximately equal across months 1, 2, and 3 (each around 31-32%), with the remaining observations having `NA` for indeterminate cases.

---

## Step 7: Save and Use the Results

Save the mensalized data for future use:

```{r save}
write_fst(result, file.path(data_dir, "pnadc_mensalized.fst"))
```

To compute monthly estimates, filter to determined observations and aggregate by `ref_month_yyyymm`:

```{r monthly-analysis}
# Monthly unemployment rate
monthly_unemployment <- result[determined_month == TRUE, .(
  unemployment_rate = sum((VD4002 == 2) * weight_monthly, na.rm = TRUE) /
                      sum((VD4001 == 1) * weight_monthly, na.rm = TRUE)
), by = ref_month_yyyymm]

# Monthly population
monthly_pop <- result[determined_month == TRUE, .(
  population = sum(weight_monthly, na.rm = TRUE)
), by = ref_month_yyyymm]
```

For more analysis examples, see [Applied Examples](applied-examples.html).

---

## Memory and Performance Tips

1. **Selective column loading**: Only load the columns you need with `read_fst(..., columns = ...)`. This dramatically reduces memory usage.

2. **Process in batches**: For very large analyses, process one year at a time and combine results.

3. **Use FST format**: FST is much faster than CSV or RDS for large datasets. A typical quarter loads in seconds rather than minutes.

4. **Clean up regularly**: Use `rm()` and `gc()` to free memory after processing each quarter.

### File Size Reference

| Period | Quarters | Observations | FST Size |
|--------|----------|--------------|----------|
| 2020-2024 | 20 | ~8.9M | ~5 GB |
| 2012-2025 | 55 | ~29M | ~15 GB |

---

## Extending to Full History

For the best determination rate and longitudinal analysis, download all available quarters:

```{r full-history}
# Download all available data (2012-present)
editions_full <- expand.grid(
  year = 2012:2025,
  quarter = 1:4
)
editions_full <- editions_full[!(editions_full$year == 2025 &
                                   editions_full$quarter > 3), ]

# Use the same download and stacking workflow as above
```

The full history provides approximately 29 million observations and achieves the highest possible determination rate (~97% month).

---

## Troubleshooting

1. **"Column not found" errors**: Ensure you used `labels = FALSE` when downloading. The algorithm requires numeric codes.

2. **Download failures**: IBGE servers can be slow or unavailable. The `PNADcIBGE` package will retry automatically, but you may need to restart interrupted downloads.

3. **Memory errors**: Try processing fewer quarters at a time, or use a machine with more RAM.

4. **SIDRA API errors**: Weight calibration requires internet access to the SIDRA API. If it fails, try again later or use `calibrate = FALSE` for reference period identification without weight calibration.

---

## Next Steps

- Follow the usage patterns in [Get Started](getting-started.html) with your real data
- See analysis examples in [Applied Examples](applied-examples.html)
- Learn about the algorithm in [How PNADCperiods Works](how-it-works.html)

> **Working with annual PNADC data?** Annual data (visit-specific microdata with comprehensive income variables) requires a different workflow. See [Monthly Poverty Analysis with Annual PNADC Data](annual-poverty-analysis.html) for details on using `pnadc_apply_periods()` with `anchor = "year"`.

---

## References

- HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62. Brasilia, DF: IPEA, 2020. <https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453>
- HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
- HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco, aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
- Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version v0.1.0. <https://github.com/antrologos/PNADCperiods>

---
title: "Monthly Poverty Analysis with Annual PNADC Data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Monthly Poverty Analysis with Annual PNADC Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!--
MAINTAINER NOTE:
The code chunks in this vignette must match the code in:
  mensalizacao_pnad/code/generate_annual_poverty_figures.R

Look for comments like "# VIGNETTE CODE: chunk-name" in that script.
To regenerate figures, run that script.
-->

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  echo = TRUE,
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 10,
  fig.height = 6,
  purl = FALSE
)
```

## Introduction

When exactly did poverty spike during COVID-19? Official annual statistics tell us that 2020 was a difficult year---but they can't tell us whether the crisis peaked in April or June, whether the Auxilio Emergencial reduced poverty immediately or with a delay, or what the month-by-month recovery path looked like. **Monthly data can.**

This vignette combines the mensalization algorithm with **annual PNADC data** to produce monthly poverty statistics. The annual PNADC releases contain comprehensive household income measures (`VD5008`) that aren't available in the quarterly releases. By applying a mensalization crosswalk (built from quarterly data) to annual income data, we get monthly temporal precision with comprehensive income measurement.

![Data Workflow: Monthly Poverty Analysis with Annual PNADC](figures/annual-poverty-analysis/fig-workflow-diagram.png){width=100%}

IBGE's PNADC uses a rotating panel design where the same households appear in both quarterly and annual data. The workflow is:

1. Build a **crosswalk** from quarterly data using `pnadc_identify_periods()`
2. **Apply** the crosswalk to annual income data with `pnadc_apply_periods()` (which handles the merge internally and calibrates weights)
3. Analyze detailed income and poverty measures at monthly frequency

---

## Prerequisites

```{r prerequisites}
library(PNADCperiods)
library(data.table)
library(fst)
library(readxl)      # Read IBGE deflator Excel files
library(deflateBR)   # INPC deflator
library(ggplot2)
library(scales)
```

You also need:

- **Quarterly PNADC data** (2015-2024) in `.fst` format for creating the mensalization crosswalk
- **Annual PNADC data** (2015-2024) in `.fst` format with income supplement variables
- **Deflator file** from IBGE documentation (`deflator_pnadc_2024.xls`)

---

## Complete Workflow

### Step 1: Create Mensalization Crosswalk

Load stacked quarterly PNADC data and run the mensalization algorithm. See the [Download and Prepare Data](download-and-prepare.html) vignette for details on obtaining and formatting PNADC microdata.

```{r create-crosswalk}
# Define paths
pnad_quarterly_dir <- "path/to/quarterly/data"

# List quarterly files (2015-2024)
quarterly_files <- list.files(
  path = pnad_quarterly_dir,
  pattern = "pnadc_20(1[5-9]|2[0-4])-[1-4]q\\.fst$",
  full.names = TRUE
)

# Variables needed for mensalization
quarterly_vars <- c(
  "Ano", "Trimestre", "UPA", "V1008", "V1014",
  "V2008", "V20081", "V20082", "V2009",
  "V1028", "UF", "posest", "posest_sxi", "Estrato"
)

# Load and stack quarterly data
quarterly_data <- rbindlist(
  lapply(quarterly_files, function(f) {
    read_fst(f, as.data.table = TRUE, columns = quarterly_vars)
  }),
  fill = TRUE
)

# Build the crosswalk (identifies reference periods)
crosswalk <- pnadc_identify_periods(
  quarterly_data,
  verbose = TRUE
)

# Check determination rate (expect ~96% with 40 quarters of data)
crosswalk[, mean(determined_month, na.rm = TRUE)]
```

### Step 2: Load Annual PNADC Data

Annual PNADC files follow a specific naming convention. Note that 2020-2021 use visit 5 (due to COVID-related field disruptions), while other years use visit 1:

```{r load-annual-data}
pnad_annual_dir <- "path/to/annual/data"

# Define which visit to use for each year
visit_selection <- data.table(
  ano = 2015:2024,
  visita = c(1, 1, 1, 1, 1, 5, 5, 1, 1, 1)  # 2020-2021 use visit 5
)
```

> **Why Visit 5 for 2020-2021?**
>
> During COVID-19, IBGE suspended in-person data collection. Visit 1 interviews
> for 2020-2021 have significant quality issues or are unavailable entirely.
> Visit 5 interviews were conducted later under improved conditions and are
> the standard choice for COVID-era income and poverty analysis.

```{r load-annual-continued}
# Build file paths
annual_files <- visit_selection[, .(
  file = file.path(pnad_annual_dir, sprintf("pnadc_%d_visita%d.fst", ano, visita))
), by = ano]

# Variables to load
annual_vars <- c(
  # Join keys
  "ano", "trimestre", "upa", "v1008", "v1014",
  # Demographics
  "v2005", "v2007", "v2009", "v2010", "uf", "estrato",
  # Weights and calibration
  "v1032", "posest", "posest_sxi",
  # Household per capita income (IBGE pre-calculated)
  "vd5008"
)

# Load and stack annual data
annual_data <- rbindlist(
  lapply(annual_files[file.exists(file), file], function(f) {
    dt <- read_fst(f, as.data.table = TRUE)
    setnames(dt, tolower(names(dt)))
    cols_present <- intersect(annual_vars, names(dt))
    dt[, ..cols_present]
  }),
  fill = TRUE
)
```

### Step 2b: Standardize Column Names

The annual data has lowercase column names, but `pnadc_apply_periods()` requires
specific casing for the join keys. Standardize before applying the crosswalk:

```{r standardize-columns}
# pnadc_apply_periods() expects uppercase join keys
key_mappings <- c(
  "ano" = "Ano", "trimestre" = "Trimestre",
  "upa" = "UPA", "v1008" = "V1008", "v1014" = "V1014",
  "v1032" = "V1032", "uf" = "UF", "v2009" = "V2009"
)

for (old_name in names(key_mappings)) {
  if (old_name %in% names(annual_data)) {
    setnames(annual_data, old_name, key_mappings[[old_name]])
  }
}
```

> **Note:** The calibration columns `posest` and `posest_sxi` stay lowercase---the
> package expects them in that case. Only the join keys (`Ano`, `Trimestre`, `UPA`,
> `V1008`, `V1014`) and weight column (`V1032`) need uppercase.

### Step 3: Apply Crosswalk and Calibrate Weights

Apply the crosswalk to annual data and calibrate weights using `pnadc_apply_periods()`.
The function handles the merge internally using the five join keys (`Ano`, `Trimestre`,
`UPA`, `V1008`, `V1014`):

```{r apply-crosswalk}
d <- pnadc_apply_periods(
  annual_data,
  crosswalk,
  weight_var = "V1032",
  anchor = "year",
  calibrate = TRUE,
  calibration_unit = "month",
  smooth = TRUE,
  verbose = TRUE
)

# Check match rate (expect ~97% with year anchor)
mean(!is.na(d$ref_month_in_quarter))
```

> **Why `anchor = "year"`?** Annual PNADC data contains only one visit per
> household (e.g., visit 1 or visit 5), not all rotation groups like quarterly
> data. The `"year"` anchor calibrates the annual weight `V1032` to monthly SIDRA
> population totals while preserving yearly totals.

### Step 4: Construct Per Capita Income

Use IBGE's pre-calculated household per capita income variable:

```{r construct-income}
# Filter to household members only
d <- d[v2005 <= 14 | v2005 == 16]

# Use IBGE's pre-calculated per capita household income
d[, hhinc_pc_nominal := fifelse(is.na(vd5008), 0, vd5008)]
```

### Step 5: Apply Deflation

Convert nominal income to real values using IBGE deflators:

```{r apply-deflation}
# Load deflator data (from IBGE documentation)
deflator <- readxl::read_excel("path/to/deflator_pnadc_2024.xls")
setDT(deflator)
deflator <- deflator[, .(Ano = ano, Trimestre = trim, UF = uf, CO2, CO2e, CO3)]

# Merge deflators with data
setkeyv(deflator, c("Ano", "Trimestre", "UF"))
setkeyv(d, c("Ano", "Trimestre", "UF"))
d <- deflator[d]

# INPC adjustment factor to reference date (December 2025)
inpc_factor <- deflateBR::inpc(1,
                               nominal_dates = as.Date("2024-07-01"),
                               real_date = "12/2025")

# Apply deflation
d[, hhinc_pc := hhinc_pc_nominal * CO2 * inpc_factor]
```

### Step 6: Define Poverty Line

Calculate the World Bank PPP-based poverty threshold:

```{r define-poverty-lines}
# World Bank poverty line: USD 8.30 PPP per day (upper-middle income threshold)
poverty_line_830_ppp_daily <- 8.30

# 2021 PPP conversion factor (World Bank)
# https://data.worldbank.org/indicator/PA.NUS.PRVT.PP?year=2021
usd_to_brl_ppp <- 2.45
days_to_month <- 365/12

# Monthly value in 2021 BRL
poverty_line_830_brl_monthly_2021 <- poverty_line_830_ppp_daily *
                                     usd_to_brl_ppp * days_to_month

# Deflate to December 2025 reference
poverty_line_830_brl_monthly_2025 <- deflateBR::inpc(
  poverty_line_830_brl_monthly_2021,
  nominal_dates = as.Date("2021-07-01"),
  real_date = "12/2025"
)

d[, poverty_line := poverty_line_830_brl_monthly_2025]
```

> **Why USD 8.30/day?** This is the World Bank's upper-middle income poverty
> threshold, appropriate for Brazil. We use the 2021 PPP conversion factor
> (2.45 BRL per USD) because 2021 is the World Bank's reference year for
> the current poverty lines.

---

## Analysis Examples

### Helper Functions

Before computing poverty measures, we define the FGT family of poverty indices:

```{r helper-functions}
# FGT poverty measure family (Foster-Greer-Thorbecke)
# alpha = 0: Headcount ratio (share below line)
# alpha = 1: Poverty gap (average shortfall)
# alpha = 2: Squared poverty gap (sensitive to inequality among poor)
fgt <- function(x, z, w = NULL, alpha = 0) {
  if (is.null(w)) w <- rep(1, length(x))
  if (length(z) == 1) z <- rep(z, length(x))

  idx <- complete.cases(x, z, w)
  x <- x[idx]; z <- z[idx]; w <- w[idx]

  g <- pmax(0, (z - x) / z)
  fgt_val <- ifelse(x < z, g^alpha, 0)

  sum(w * fgt_val) / sum(w)
}
```

### Example 1: Monthly FGT Poverty Measures

Calculate monthly poverty rates using the FGT family:

```{r example-fgt-family}
# Filter to determined observations
d_monthly <- d[!is.na(ref_month_yyyymm)]

# Use calibrated monthly weight (from pnadc_apply_periods())
d_monthly[, peso := weight_monthly]

# Compute monthly poverty statistics
monthly_poverty <- d_monthly[, .(
  # FGT-0 (Headcount ratio)
  poverty_rate = fgt(hhinc_pc, poverty_line, peso, alpha = 0),

  # FGT-1 (Poverty gap)
  poverty_gap = fgt(hhinc_pc, poverty_line, peso, alpha = 1),

  # Mean income
  mean_income = weighted.mean(hhinc_pc, peso, na.rm = TRUE),

  # Sample size
  n_obs = .N

), by = ref_month_yyyymm]

# Add date for plotting
monthly_poverty[, period := as.Date(paste0(
  ref_month_yyyymm %/% 100, "-",
  ref_month_yyyymm %% 100, "-15"
))]
```

<details>
<summary>Show plotting code</summary>

```{r fgt-plot}
# Prepare data for plotting
fgt_data <- melt(
  monthly_poverty[, .(period,
                      `PPP 8.30/day` = poverty_rate)],
  id.vars = "period",
  variable.name = "poverty_line",
  value.name = "rate"
)

fgt_gap_data <- melt(
  monthly_poverty[, .(period,
                      `PPP 8.30/day` = poverty_gap)],
  id.vars = "period",
  variable.name = "poverty_line",
  value.name = "gap"
)

# Panel A: Headcount ratio (FGT-0)
p1 <- ggplot(fgt_data, aes(x = period, y = rate, color = poverty_line)) +
  geom_line(linewidth = 0.8) +
  geom_point(size = 1) +
  scale_y_continuous(labels = percent_format(accuracy = 1),
                     limits = c(0, NA)) +
  scale_x_date(date_breaks = "1 year", date_labels = "%Y") +
  scale_color_manual(values = c("PPP 8.30/day" = "#b2182b")) +
  labs(title = "A. Poverty Headcount (FGT-0)",
       subtitle = "Share of population below poverty line",
       x = NULL, y = "Poverty Rate",
       color = "Poverty Line") +
  theme_minimal(base_size = 11) +
  theme(legend.position = "bottom",
        panel.grid.minor = element_blank(),
        plot.title = element_text(face = "bold"))

# Panel B: Poverty gap (FGT-1)
p2 <- ggplot(fgt_gap_data, aes(x = period, y = gap, color = poverty_line)) +
  geom_line(linewidth = 0.8) +
  geom_point(size = 1) +
  scale_y_continuous(labels = percent_format(accuracy = 0.1),
                     limits = c(0, NA)) +
  scale_x_date(date_breaks = "1 year", date_labels = "%Y") +
  scale_color_manual(values = c("PPP 8.30/day" = "#ef8a62")) +
  labs(title = "B. Relative Poverty Gap (FGT-1)",
       subtitle = "Average shortfall as share of poverty line",
       x = NULL, y = "Relative Poverty Gap",
       color = "Poverty Line") +
  theme_minimal(base_size = 11) +
  theme(legend.position = "bottom",
        panel.grid.minor = element_blank(),
        plot.title = element_text(face = "bold"))

# Combine panels
library(patchwork)
fig_fgt <- p1 / p2 +
  plot_annotation(
    title = "Monthly Poverty Measures: Brazil, 2015-2024",
    caption = "Source: PNADC/IBGE. Annual data with monthly reference periods from PNADCperiods.",
    theme = theme(
      plot.title = element_text(face = "bold", size = 14),
      plot.subtitle = element_text(size = 11),
      plot.caption = element_text(size = 9, hjust = 0)
    )
  )
fig_fgt
```

</details>

![Monthly Poverty Measures: Brazil, 2015-2024](figures/annual-poverty-analysis/fig-fgt-monthly-series.png){width=100%}

The figure reveals several key dynamics:

1. **COVID-19 spike (March-April 2020)**: The poverty rate shows a sharp increase in early 2020.

2. **Auxilio Emergencial effect (May-December 2020)**: Emergency cash transfers dramatically reduced poverty below pre-pandemic levels.

3. **Post-Auxilio adjustment (2021)**: As emergency aid was reduced, poverty rates partially rebounded.

For proper inference with confidence intervals, use complex survey design with monthly weights---see the [Complex Survey Design](complex-survey-design.html) vignette.

---

## Summary

| Insight | Annual Data | Monthly Data |
|---------|-------------|--------------|
| **COVID poverty spike** | Averaged across year | Visible March-April 2020 |
| **Auxilio timing** | Effect blurred | Clear May 2020 onset |
| **Recovery dynamics** | Single 2021 estimate | Monthly trajectory |
| **Seasonal patterns** | Invisible | December income spikes |

**Limitations**: ~3% sample loss from undetermined reference months; annual PNADC is released with 18+ month delay; monthly estimates have wider confidence intervals than annual.

---

## Further Reading

- [Get Started](getting-started.html) - Basic mensalization workflow
- [How It Works](how-it-works.html) - Algorithm details
- [Complex Survey Design](complex-survey-design.html) - Variance estimation
- [Applied Examples](applied-examples.html) - Unemployment and minimum wage examples

## References

- HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62. Brasilia, DF: IPEA, 2020. <https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453>
- HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
- HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco, aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
- NERI, Marcelo; HECKSHER, Marcos. "A Montanha-Russa da Pobreza". FGV Social - Sumario Executivo. Rio de Janeiro: FGV, Junho/2022. <https://www.cps.fgv.br/cps/bd/docs/MontanhaRussaDaPobreza_Neri_Hecksher_FGV_Social.pdf>
- NERI, Marcelo; HECKSHER, Marcos. "A montanha-russa da pobreza mensal e um programa social alternativo". *Revista NECAT*, v. 11, n. 21, 2022.
- IBGE. Pesquisa Nacional por Amostra de Domicilios Continua (PNADC). <https://www.ibge.gov.br/estatisticas/sociais/trabalho/>
- World Bank. Poverty and Shared Prosperity Reports. Various years.
- Foster, J., Greer, J., & Thorbecke, E. (1984). A class of decomposable poverty measures. *Econometrica*, 52(3), 761-766.
- Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version v0.1.0. <https://github.com/antrologos/PNADCperiods>

---
title: "Get Started with PNADCperiods"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get Started with PNADCperiods}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  purl = FALSE
)
```

## Overview

The `PNADCperiods` package converts Brazil's quarterly PNADC (Pesquisa Nacional por Amostra de Domicilios Continua) survey data into sub-quarterly time series. PNADC quarterly statistics are actually moving averages of three months, which obscures the true timing of economic shocks, the actual magnitude of changes, and turning points in trends.

The package offers **two complementary approaches**:

1. **Microdata mensalization** — Identify which specific month, fortnight, or week each survey observation refers to, then calibrate weights for sub-quarterly analysis. Use this for custom variables, subgroup analysis, or individual-level regressions.

2. **SIDRA mensalization** — Convert IBGE's published rolling quarter aggregate series (86+ indicators) into exact monthly estimates. No microdata needed — just 3 lines of code.

For a detailed explanation of the algorithm, see [How PNADCperiods Works](how-it-works.html).

---

## Installation

```{r install}
# From CRAN
install.packages("PNADCperiods")

# Development version from GitHub
# remotes::install_github("antrologos/PNADCperiods")
```

```{r libraries}
library(PNADCperiods)
library(data.table)
```

**Dependencies:** `data.table`, `checkmate`, and `sidrar` (for weight calibration and SIDRA API access).

---

## Microdata Mensalization

Use the microdata workflow when you need custom variable definitions, subgroup analysis, individual-level regressions, or indicators not available via SIDRA. This requires PNADC microdata files — see [Download and Prepare Data](download-and-prepare.html) for how to obtain them.

### Required Columns

The algorithm needs these columns from your PNADC data:

| Column | Description |
|--------|-------------|
| `Ano` | Survey year |
| `Trimestre` | Quarter (1-4) |
| `UPA` | Primary Sampling Unit |
| `V1008` | Household identifier |
| `V1014` | Panel identifier (rotation group 1-8) |
| `V2008` | Birth day (1-31, or 99 for unknown) |
| `V20081` | Birth month (1-12, or 99 for unknown) |
| `V20082` | Birth year (or 9999 for unknown) |
| `V2009` | Age |

For weight calibration, you also need: `V1028` (or `V1032` for annual data), `UF`, `posest`, and `posest_sxi`.

### Step 1: Build the Crosswalk

```{r load-data}
# Load your stacked quarterly PNADC data
pnadc <- fread("pnadc_stacked.csv")
```

```{r build-crosswalk}
# Identify reference periods (month, fortnight, week)
crosswalk <- pnadc_identify_periods(pnadc, verbose = TRUE)
```

**Stack multiple quarters for best results.** The algorithm exploits PNADC's rotating panel design — each household (UPA + V1014) is interviewed across 5 consecutive quarters at the same relative month position. Cross-quarter aggregation achieves ~97% month determination from the full 2012-2025 history, vs ~70% from a single quarter.

```{r check-rates}
# Check determination rates
crosswalk[, .(
  month_rate = mean(determined_month),
  fortnight_rate = mean(determined_fortnight),
  week_rate = mean(determined_week)
)]
```

See `?pnadc_identify_periods` for full documentation of all crosswalk output columns.

### Step 2: Apply Crosswalk and Calibrate Weights

```{r apply-crosswalk}
result <- pnadc_apply_periods(
  pnadc_2023q1,
  crosswalk,
  weight_var = "V1028",
  anchor = "quarter",
  calibrate = TRUE,
  calibration_unit = "month"
)
```

**Key parameters:**

| Parameter | Values | Default |
|-----------|--------|---------|
| `weight_var` | `"V1028"` (quarterly) or `"V1032"` (annual) | Required |
| `anchor` | `"quarter"` or `"year"` | Required |
| `calibrate` | `TRUE` / `FALSE` | `TRUE` |
| `calibration_unit` | `"month"`, `"fortnight"`, `"week"` | `"month"` |
| `smooth` | `TRUE` / `FALSE` | `FALSE` |

Weight calibration adjusts survey weights to match known population benchmarks from SIDRA, ensuring monthly totals are consistent. The result includes all original columns plus reference period indicators and calibrated weights (e.g., `weight_monthly`).

### Step 3: Compute Monthly Estimates

```{r monthly-analysis}
# Monthly unemployment rate
monthly_unemployment <- result[determined_month == TRUE, .(
  unemployment_rate = sum((VD4002 == 2) * weight_monthly, na.rm = TRUE) /
                      sum((VD4001 == 1) * weight_monthly, na.rm = TRUE)
), by = ref_month_yyyymm]

# Monthly population
monthly_pop <- result[, .(
  population = sum(weight_monthly, na.rm = TRUE)
), by = ref_month_yyyymm]
```

Use `determined_month == TRUE` (or equivalently, `!is.na(weight_monthly)`) to filter to observations with determined reference months.

For complete analysis examples with plots, see [Applied Examples](applied-examples.html).

### Build Once, Apply Many

The crosswalk only needs identification columns, so you can build it once and reuse it:

```{r build-apply-pattern}
# Build crosswalk once from stacked data
crosswalk <- pnadc_identify_periods(pnadc_stacked)
saveRDS(crosswalk, "crosswalk.rds")

# Apply to any quarterly or annual dataset
crosswalk <- readRDS("crosswalk.rds")
result_q1 <- pnadc_apply_periods(pnadc_2023q1, crosswalk,
                                  weight_var = "V1028", anchor = "quarter")
result_annual <- pnadc_apply_periods(pnadc_annual_2023, crosswalk,
                                      weight_var = "V1032", anchor = "year")
```

---

## Monthly Series from SIDRA — No Microdata Needed

If you need aggregate monthly labor market statistics (unemployment rate, employment levels, income), you can get them directly from IBGE's published data without any microdata. The SIDRA module converts the rolling quarter series published by IBGE into exact monthly estimates.

### Quick Start

```{r sidra-quickstart}
# Step 1: Fetch rolling quarter data from SIDRA API
rolling_quarters <- fetch_sidra_rolling_quarters()

# Step 2: Convert to exact monthly estimates
monthly <- mensalize_sidra_series(rolling_quarters)

# Step 3: Use your monthly data
head(monthly[, .(anomesexato, m_popocup, m_taxadesocup)])
```

`fetch_sidra_rolling_quarters()` downloads 70+ economic indicators from IBGE's SIDRA API (Tables 4093, 6390, 6392, 6399, 6906). `mensalize_sidra_series()` applies the mensalization formula using pre-computed starting points bundled with the package, converting rolling quarter averages into exact monthly values.

The output contains `anomesexato` (month in YYYYMM format) and `m_*` columns with mensalized monthly estimates, starting from March 2012.

### Discovering Available Series

```{r sidra-metadata}
# Browse all 86+ available series
meta <- get_sidra_series_metadata()
meta[, .(series_name, description, unit)]
```

**Most commonly used series:**

| Column | Description | Unit |
|--------|-------------|------|
| `m_taxadesocup` | Unemployment rate | Percent |
| `m_popocup` | Employed population | Thousands |
| `m_taxapartic` | Labor force participation rate | Percent |
| `m_massahabnominaltodos` | Total nominal wage bill | Millions R$ |
| `m_rendhabnominaltodos` | Average nominal usual income | R$ |
| `m_taxacompsubutlz` | Composite underutilization rate | Percent |

### Visualizing Monthly vs Rolling Quarter Data

Since we have both the original rolling quarter data and the mensalized monthly estimates, comparing them is straightforward:

![Monthly estimates reveal dynamics hidden by rolling quarter averaging](figures/getting-started/fig_monthly_vs_quarterly.png){width=100%}

<details>
<summary>Show plotting code</summary>

```{r sidra-plot}
library(ggplot2)

# Prepare comparison data (merge rolling quarter and monthly estimates)
plot_data <- merge(
  rolling_quarters[, .(anomesexato = anomesfinaltrimmovel, rolling = taxadesocup)],
  monthly[, .(anomesexato, monthly = m_taxadesocup)],
  by = "anomesexato"
)[anomesexato >= 201901 & anomesexato <= 202312]

plot_data[, date := as.Date(paste0(substr(anomesexato, 1, 4), "-",
                                    substr(anomesexato, 5, 6), "-01"))]

# Reshape to long format
plot_long <- melt(plot_data, id.vars = c("anomesexato", "date"),
                  variable.name = "type", value.name = "rate")
plot_long[, type := factor(type,
                           levels = c("rolling", "monthly"),
                           labels = c("Rolling Quarter", "Monthly"))]

# Plot
ggplot(plot_long, aes(x = date, y = rate, color = type)) +
  geom_line(linewidth = 0.8) +
  annotate("rect", xmin = as.Date("2020-03-01"), xmax = as.Date("2020-12-31"),
           ymin = -Inf, ymax = Inf, fill = "red", alpha = 0.08) +
  scale_color_manual(values = c("Rolling Quarter" = "#888888",
                                "Monthly" = "#E53935"),
                     name = NULL) +
  scale_x_date(date_breaks = "6 months", date_labels = "%b\n%Y") +
  scale_y_continuous(labels = function(x) paste0(x, "%")) +
  labs(
    title = "Unemployment Rate: Monthly vs Rolling Quarter (2019-2023)",
    subtitle = "Monthly estimates capture the true timing and magnitude of economic shocks",
    x = NULL,
    y = "Unemployment Rate (%)"
  ) +
  theme_minimal(base_size = 11) +
  theme(
    plot.title = element_text(face = "bold"),
    legend.position = "top",
    panel.grid.minor = element_blank()
  )
```

</details>

For the full SIDRA guide — including custom starting points, methodology, and a COVID case study — see the [SIDRA Mensalization Guide](sidra-mensalization.html).

---

## Improving Determination Rates

PNADC uses a rotating panel where each household (UPA + V1014) is interviewed in 5 consecutive quarters, always at the same relative month position. This means birthday constraints from **any** quarter can determine the month for **all** quarters — which is why stacking more data dramatically improves rates:

| Data Stacked | Month Rate | Fortnight Rate | Week Rate |
|--------------|------------|----------------|-----------|
| 1 quarter | ~70% | ~7% | ~2% |
| 8 quarters (2 years) | ~94% | ~9% | ~3% |
| 55 quarters (full 2012-2025) | **97.0%** | **9.2%** | **3.3%** |

Fortnight and week rates remain low regardless of stacking because their constraints cannot aggregate across quarters — only month benefits from the panel design.

For analyses requiring higher determination, experimental strategies make informed probabilistic assignments:

```{r experimental}
# Build crosswalk with date bounds for experimental strategies
crosswalk <- pnadc_identify_periods(pnadc, store_date_bounds = TRUE)

# Apply experimental strategies
crosswalk_exp <- pnadc_experimental_periods(
  crosswalk,
  strategy = "both",
  confidence_threshold = 0.9
)
```

With `strategy = "both"` and `confidence_threshold = 0.9`, month determination improves to ~97.3% and fortnight to ~13.5%. See [How PNADCperiods Works](how-it-works.html) for details on experimental strategies.

---

## Annual Data

Annual PNADC data uses different weights and achieves higher determination rates (~98% month) due to more complete panel coverage:

```{r annual}
result_annual <- pnadc_apply_periods(
  pnadc_annual,
  crosswalk,
  weight_var = "V1032",
  anchor = "year",
  calibrate = TRUE,
  calibration_unit = "month"
)
```

For poverty and income analysis using annual data, see [Monthly Poverty Analysis with Annual PNADC Data](annual-poverty-analysis.html).

---

## Function Overview

| Function | Purpose |
|----------|---------|
| **Microdata Workflow** | |
| `pnadc_identify_periods()` | Build period crosswalk from stacked microdata |
| `pnadc_apply_periods()` | Apply crosswalk + calibrate weights |
| `pnadc_experimental_periods()` | Experimental strategies for higher determination |
| `validate_pnadc()` | Check required columns before processing |
| **SIDRA Mensalization** | |
| `fetch_sidra_rolling_quarters()` | Download rolling quarter series from SIDRA API |
| `mensalize_sidra_series()` | Convert rolling quarters to exact monthly estimates |
| `get_sidra_series_metadata()` | Browse 86+ available series with metadata |
| `fetch_monthly_population()` | Fetch monthly population totals |
| `clear_sidra_cache()` | Clear cached API responses |
| `compute_starting_points_from_microdata()` | Compute custom starting points for SIDRA mensalization |

---

## Next Steps

- **Download data**: [Download and Prepare Data](download-and-prepare.html) — complete workflow to download and stack quarterly microdata from IBGE
- **Analysis examples**: [Applied Examples](applied-examples.html) — monthly vs quarterly labor market analysis, COVID unemployment, minimum wage validation
- **SIDRA deep dive**: [SIDRA Mensalization Guide](sidra-mensalization.html) — full guide to rolling quarter mensalization, custom starting points, and methodology
- **Annual/poverty data**: [Monthly Poverty Analysis](annual-poverty-analysis.html) — poverty analysis using annual PNADC income data
- **Survey design**: [Complex Survey Design](complex-survey-design.html) — working with survey design objects and variance estimation
- **Algorithm details**: [How PNADCperiods Works](how-it-works.html) — complete methodology, experimental strategies, and weight calibration
- **Function reference**: Use `?pnadc_identify_periods` or visit the [package website](https://antrologos.github.io/PNADCperiods/reference/) for full documentation

---

## References

- HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62. Brasilia, DF: IPEA, 2020. <https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453>
- HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.
- HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco, aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.
- Barbosa, Rogerio J; Hecksher, Marcos. (2026). PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data. R package version v0.1.0. <https://github.com/antrologos/PNADCperiods>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sidra-series-metadata.R
\name{.get_mesnotrim}
\alias{.get_mesnotrim}
\title{Get Month Position in Quarter (mesnotrim)}
\usage{
.get_mesnotrim(month)
}
\arguments{
\item{month}{Integer. Month number (1-12).}
}
\value{
Integer. Position in quarter (1, 2, or 3).
}
\description{
Given a month (1-12), returns its position in the rolling quarter (1, 2, or 3).
\itemize{
\item January, April, July, October = position 1
\item February, May, August, November = position 2
\item March, June, September, December = position 3
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mensalize-sidra-series.R
\name{mensalize-sidra-series}
\alias{mensalize-sidra-series}
\title{Mensalize SIDRA Rolling Quarter Series}
\description{
Functions to convert IBGE's rolling quarterly (trimestre movel) series into
exact monthly estimates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-validation.R
\name{utils-validation}
\alias{utils-validation}
\title{Input Validation Utilities}
\description{
Internal helper functions for validating PNADC input data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetch-sidra-series.R
\name{clear_sidra_cache}
\alias{clear_sidra_cache}
\title{Clear All SIDRA Caches}
\usage{
clear_sidra_cache()
}
\value{
Invisibly returns TRUE if any cache was cleared, FALSE if all empty.
}
\description{
Clears all cached SIDRA data (both rolling quarter series and population).
Use this if you need to force a fresh download, for example after IBGE
updates their data.
}
\examples{
clear_sidra_cache()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetch-sidra-population.R
\name{fetch_monthly_population}
\alias{fetch_monthly_population}
\title{Fetch Monthly Population from SIDRA}
\usage{
fetch_monthly_population(
  start_yyyymm = NULL,
  end_yyyymm = NULL,
  verbose = TRUE,
  use_cache = FALSE,
  cache_max_age_hours = 24
)
}
\arguments{
\item{start_yyyymm}{Integer. First month to include (YYYYMM format).
If NULL, returns all available months.}

\item{end_yyyymm}{Integer. Last month to include (YYYYMM format).
If NULL, returns all available months.}

\item{verbose}{Logical. Print progress messages? Default TRUE.}

\item{use_cache}{Logical. If TRUE, uses cached data if available and not
expired. Default FALSE (always fetch fresh data for consistency).
Set to TRUE for faster repeated calls during development.}

\item{cache_max_age_hours}{Numeric. Maximum cache age in hours before
automatic expiration when use_cache=TRUE. Default 24 hours.}
}
\value{
A data.table with columns:
\itemize{
\item \code{ref_month_yyyymm}: Integer in YYYYMM format
\item \code{m_populacao}: Monthly population in thousands
}
}
\description{
Downloads population estimates from IBGE SIDRA API (table 6022) and
transforms from moving-quarter to exact monthly values.
}
\details{
SIDRA table 6022 provides moving-quarter population estimates. Each value
represents the 3-month average centered on the middle month. For example,
the value for code 201203 (quarter ending March 2012) represents the
population for February 2012.

This function:
\enumerate{
\item Fetches raw moving-quarter data from SIDRA
\item Transforms to exact monthly values by aligning with middle months
\item Extrapolates boundary months (first and last) using quadratic regression
}

The extrapolation uses quadratic regression on population differences to
estimate the first month (Jan 2012) and the most recent month.
}
\section{Dependencies}{

This function requires the \code{sidrar} package for API access.
Install with: \code{install.packages("sidrar")}
}

\examples{
\donttest{
pop <- fetch_monthly_population()

pop <- fetch_monthly_population(201301, 201912)
}

}
\seealso{
\code{\link{pnadc_apply_periods}} which uses this function when
\code{calibrate = TRUE}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetch-sidra-population.R
\name{fetch-sidra-population}
\alias{fetch-sidra-population}
\title{Fetch Monthly Population from SIDRA}
\description{
Functions to download and transform IBGE's population estimates
from SIDRA API for use in monthly weight calibration.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-starting-points.R
\docType{data}
\name{pnadc_series_starting_points}
\alias{pnadc_series_starting_points}
\title{Starting Points for SIDRA Series Mensalization}
\format{
A data.table with 159 rows and 3 columns:
\describe{
\item{series_name}{Character. Name of the SIDRA series (53 series)}
\item{mesnotrim}{Integer. Month position in quarter (1, 2, or 3)}
\item{y0}{Numeric. Starting point value for this series and position}
}
}
\source{
Computed using \code{data-raw/regenerate_starting_points_from_microdata.R}
}
\usage{
data(pnadc_series_starting_points)
}
\description{
Pre-computed starting point values (y0) for mensalizing IBGE's rolling
quarterly series into exact monthly estimates.
}
\details{
These starting points were computed from PNADC microdata using the full
R package pipeline, ensuring consistency with
\code{\link{compute_starting_points_from_microdata}}:
\enumerate{
\item Weight calibration via \code{\link{pnadc_apply_periods}}: all months
scaled to SIDRA monthly population totals
\item z_ aggregates computed via \code{\link{compute_z_aggregates}} using
calibrated \code{weight_monthly}
\item Starting points computed via \code{\link{compute_series_starting_points}}
with CNPJ-aware calibration periods
}

The calibration period (2013-2019) was chosen because:
\itemize{
\item It includes stable pre-pandemic data
\item IBGE methodology was consistent during this period
\item Sufficient observations for reliable estimates
}

CNPJ series (empregadorcomcnpj, empregadorsemcnpj, contapropriacomcnpj,
contapropriasemcnpj) use calibration period 2016-2019 with cumulative sum
starting from October 2015 due to V4019 variable availability.
}
\section{Methodology Consistency}{

The bundled starting points are generated using the same pipeline as
\code{\link{compute_starting_points_from_microdata}}, ensuring that users
who compute custom starting points will get consistent results.
}

\section{When to Use Custom Starting Points}{

The bundled starting points are suitable for most users. Consider computing
custom starting points with \code{\link{compute_starting_points_from_microdata}} if:
\itemize{
\item IBGE makes major methodological changes to the PNADC
\item You need series not included in the bundled data
\item You want to use a different calibration period
\item You are working with updated or different microdata
}
}

\examples{
data(pnadc_series_starting_points)
head(pnadc_series_starting_points)
unique(pnadc_series_starting_points$series_name)

}
\seealso{
\code{\link{mensalize_sidra_series}} which uses this data by default
\code{\link{compute_series_starting_points}} for custom calibration
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-dates.R
\name{utils-dates}
\alias{utils-dates}
\title{Date Utility Functions}
\description{
Internal helper functions for date calculations.

This file includes:
\itemize{
\item Day of week calculation (dow)
\item ISO 8601 week utilities (Monday-Sunday weeks)
\item IBGE first Saturday calculation for reference weeks
\item Month position calculation for quarter mapping
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pnadc-experimental-periods.R
\name{pnadc_experimental_periods}
\alias{pnadc_experimental_periods}
\title{Experimental Period Identification Strategies}
\usage{
pnadc_experimental_periods(
  crosswalk,
  strategy = c("probabilistic", "upa_aggregation", "both"),
  confidence_threshold = 0.9,
  upa_proportion_threshold = 0.5,
  verbose = TRUE
)
}
\arguments{
\item{crosswalk}{A crosswalk data.table from \code{pnadc_identify_periods()}}

\item{strategy}{Character specifying which strategy to apply.
Options: "probabilistic", "upa_aggregation", "both"}

\item{confidence_threshold}{Numeric (0-1). Minimum confidence required to
assign a probabilistic period. Used by probabilistic and combined strategies.
Default 0.9.}

\item{upa_proportion_threshold}{Numeric (0-1). Minimum proportion of UPA
observations (within quarter) that must have strict identification with consensus
for extending to unidentified observations. Default 0.5.}

\item{verbose}{Logical. If TRUE, print progress information.}
}
\value{
A modified crosswalk with additional columns. Output is directly compatible
with \code{pnadc_apply_periods()}:
\itemize{
\item \code{ref_month_in_quarter}, \code{ref_month_in_year}, \code{ref_month_yyyymm}:
Month position (combined strict + experimental, strict takes priority)
\item \code{ref_fortnight_in_month}, \code{ref_fortnight_in_quarter}, \code{ref_fortnight_yyyyff}:
Fortnight position (combined strict + experimental)
\item \code{ref_week_in_month}, \code{ref_week_in_quarter}, \code{ref_week_yyyyww}:
Week position (combined strict + experimental)
\item \code{determined_month}, \code{determined_fortnight}, \code{determined_week}:
TRUE if period is assigned (strictly or experimentally)
\item \code{determined_probable_month}, \code{determined_probable_fortnight},
\code{determined_probable_week}: TRUE if period was assigned by probabilistic strategy
\item \code{probabilistic_assignment}: TRUE if any period was assigned experimentally
(vs strictly deterministic)
\item \code{week_1_start}, \code{week_1_end}, ..., \code{week_4_start}, \code{week_4_end}:
IBGE week boundaries for the assigned month
}
}
\description{
Three experimental strategies are available, all properly nested by period:
\itemize{
\item \strong{probabilistic}: For narrow ranges (2 possible periods),
classifies based on where most of the date interval falls. Assigns only
when confidence exceeds threshold.
\item \strong{upa_aggregation}: Extends strictly identified periods to other
observations in the same UPA-V1014 within the quarter, if a sufficient
proportion already have strict identification.
\item \strong{both}: Sequentially applies probabilistic strategy first, then
UPA aggregation on top. Guarantees identification rate >= max of individual strategies.
}
}
\details{
Provides experimental strategies for improving period identification rates
beyond the standard deterministic algorithm. All strategies respect the nested
identification hierarchy: weeks require fortnights, fortnights require months.
\subsection{Nesting Enforcement}{

All strategies enforce proper nesting:
\itemize{
\item Fortnights can only be assigned if month is identified (strictly OR experimentally)
\item Weeks can only be assigned if fortnight is identified (strictly OR experimentally)
}
}

\subsection{Probabilistic Strategy}{

For each period type (processed in order: months, then fortnights, then weeks):
\enumerate{
\item Check that the required parent period is identified
\item If bounds are narrowed to exactly 2 sequential periods, calculate
which period contains most of the date interval
\item Calculate confidence based on the proportion of interval in the likely period (0-1)
\item Only assign if confidence >= \code{confidence_threshold}
}

For months: aggregates at UPA-V1014 level across all quarters (like strict algorithm)
For fortnights and weeks: works at household level within quarter
}

\subsection{UPA Aggregation Strategy}{

Extends strictly identified periods based on consensus within geographic groups:
\itemize{
\item \strong{Months}: Uses UPA level within quarter
\item \strong{Fortnights/Weeks}: Uses UPA level within quarter (all households in same UPA
are interviewed in same fortnight/week within a quarter)
}
\enumerate{
\item Calculate proportion of observations with strictly identified period
\item If proportion >= \code{upa_proportion_threshold} AND consensus exists, extend
\item Apply in nested order: months first, then fortnights, then weeks
}
}

\subsection{Combined Strategy ("both")}{

Sequentially applies both strategies to maximize identification:
\enumerate{
\item First, apply the probabilistic strategy (captures observations with
narrow date ranges and high confidence)
\item Then, apply UPA aggregation (extends based on strict consensus
within UPA/UPA-V1014 groups)
}

This guarantees that "both" identifies at least as many observations as
either individual strategy alone. The strategies operate independently
(UPA aggregation considers only strict identifications), so the result
is the union of both strategies.
}

\subsection{Integration with Weight Calibration}{

The output can be passed directly to \code{pnadc_apply_periods()} for weight calibration.
The derived columns combine strict and experimental assignments, with strict taking priority. Use the
\code{probabilistic_assignment} flag to filter if you only want strict determinations.
}
}
\note{
These strategies produce "experimental" assignments, not strict determinations.
The standard \code{pnadc_identify_periods()} function should be used for
rigorous analysis. Experimental outputs are useful for:
\itemize{
\item Sensitivity analysis
\item Robustness checks
\item Research into identification algorithm improvements
}
}
\examples{
\dontrun{
crosswalk <- pnadc_identify_periods(pnadc_data)

crosswalk_exp <- pnadc_experimental_periods(
  crosswalk,
  strategy = "probabilistic",
  confidence_threshold = 0.9
)

crosswalk_exp[, .(
  strict = sum(!is.na(ref_month_in_quarter) & !probabilistic_assignment),
  experimental = sum(probabilistic_assignment, na.rm = TRUE),
  total = sum(determined_month)
)]

result <- pnadc_apply_periods(pnadc_data, crosswalk_exp,
                              weight_var = "V1028", anchor = "quarter")

strict_only <- crosswalk_exp[
  probabilistic_assignment == FALSE | is.na(probabilistic_assignment)
]
}

}
\seealso{
\code{\link{pnadc_identify_periods}} to build the crosswalk that this function modifies.
\code{\link{pnadc_apply_periods}} to apply period crosswalk and calibrate weights.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mensalize-sidra-series.R
\name{compute_series_starting_points}
\alias{compute_series_starting_points}
\title{Compute Starting Points from Microdata}
\usage{
compute_series_starting_points(
  monthly_estimates,
  rolling_quarters,
  calibration_start = NULL,
  calibration_end = NULL,
  scale_factor = 1000,
  use_series_specific_periods = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{monthly_estimates}{data.table with columns:
\itemize{
\item \code{anomesexato}: YYYYMM exact month
\item \code{z_*}: Monthly estimates from calibrated microdata (one column per series)
}}

\item{rolling_quarters}{data.table from \code{fetch_sidra_rolling_quarters}}

\item{calibration_start}{Integer. Start of calibration period (YYYYMM).
Default NULL uses .PNADC_DATES$DEFAULT_CALIB_START (201301).
Note: CNPJ series automatically use CNPJ_CALIB_START (201601) regardless.}

\item{calibration_end}{Integer. End of calibration period (YYYYMM).
Default NULL uses .PNADC_DATES$DEFAULT_CALIB_END (201912).}

\item{scale_factor}{Numeric. Scale factor for z_ values (usually 1000). Default 1000.}

\item{use_series_specific_periods}{Logical. If TRUE (default), use series-specific
calibration periods for CNPJ series (201601-201912) and cumsum starting dates
(201510). Set to FALSE to use uniform calibration for all series.}

\item{verbose}{Logical. Print progress? Default TRUE.}
}
\value{
data.table with columns:
\describe{
\item{series_name}{Character. Series name}
\item{mesnotrim}{Integer. Month position (1, 2, or 3)}
\item{y0}{Numeric. Starting point value}
}
}
\description{
For advanced users who want to compute custom starting points using their
own calibrated microdata estimates.
}
\details{
The starting points (y0) are computed by:
\enumerate{
\item Calculating cumulative variations from SIDRA rolling quarters
\item Computing backprojection: e0 = z / scale_factor - cum
\item Averaging e0 by mesnotrim over the calibration period
}
}
\section{Series-Specific Handling}{

When \code{use_series_specific_periods = TRUE}, the following series receive
special handling for series-specific data availability:
\describe{
\item{CNPJ series}{empregadorcomcnpj, empregadorsemcnpj, contapropriacomcnpj,
contapropriasemcnpj use calibration period 201601-201912 and cumsum starts
from 201510 (when V4019 became available)}
}
}

\examples{
\dontrun{
rq <- fetch_sidra_rolling_quarters()
z_agg <- compute_z_aggregates(calibrated_data)
y0 <- compute_series_starting_points(z_agg, rq)
monthly <- mensalize_sidra_series(rq, starting_points = y0)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetch-sidra-series.R
\name{fetch_sidra_rolling_quarters}
\alias{fetch_sidra_rolling_quarters}
\title{Fetch Rolling Quarter Series from SIDRA}
\usage{
fetch_sidra_rolling_quarters(
  series = "all",
  theme = NULL,
  theme_category = NULL,
  subcategory = NULL,
  exclude_derived = FALSE,
  use_cache = FALSE,
  verbose = TRUE,
  retry_failed = TRUE,
  max_retries = 3
)
}
\arguments{
\item{series}{Character vector of series names to fetch, or "all" (default)
for all available series. Use \code{get_sidra_series_metadata()$series_name}
to see available names.}

\item{theme}{Character vector of themes to filter by. Valid options:
"labor_market", "earnings", "demographics", "social_protection", "prices".
Use NULL for no filter.}

\item{theme_category}{Character vector of theme categories to filter by.
Use NULL for no filter.}

\item{subcategory}{Character vector of subcategories to filter by.
Use NULL for no filter.}

\item{exclude_derived}{Logical. If TRUE, exclude series marked as derived
(is_derived = TRUE in metadata). Default FALSE for backward compatibility.
Derived series (rates) are computed from other series during mensalization,
so excluding them saves API calls when fetching for mensalization.}

\item{use_cache}{Logical. Use cached data if available? Default FALSE.
When TRUE, shows the date when data was cached (may be outdated).
Use \code{\link{clear_sidra_cache}} to force fresh download.}

\item{verbose}{Logical. Print progress messages? Default TRUE.}

\item{retry_failed}{Logical. Retry failed series downloads? Default TRUE.}

\item{max_retries}{Integer. Maximum retry attempts per series. Default 3.}
}
\value{
A data.table with columns:
\describe{
\item{anomesfinaltrimmovel}{Integer. YYYYMM of rolling quarter end month}
\item{mesnotrim}{Integer. Month position in quarter (1, 2, or 3)}
\item{<series_name>}{Numeric. One column per requested series}
}
}
\description{
Downloads PNADC labor market indicators from IBGE's SIDRA API. These series
are published as rolling quarterly averages (trimestre movel), with 12
observations per year.
}
\details{
Rolling quarters are labeled by their ending month:
\itemize{
\item 201201 = Nov 2011 - Jan 2012 (mesnotrim = 1)
\item 201202 = Dec 2011 - Feb 2012 (mesnotrim = 2)
\item 201203 = Jan - Mar 2012 (mesnotrim = 3)
\item 201204 = Feb - Apr 2012 (mesnotrim = 1)
\item etc.
}

The \code{mesnotrim} column indicates the month's position within its rolling
quarter, which is essential for the mensalization algorithm.
}
\section{Rate Limiting}{

SIDRA API may have rate limits. The function includes automatic retry logic
with exponential backoff for failed requests.
}

\examples{
\donttest{
rq <- fetch_sidra_rolling_quarters(
  series = c("taxadesocup", "popocup", "popdesocup")
)
head(rq)
}

\donttest{
rq_labor <- fetch_sidra_rolling_quarters(theme = "labor_market")
}

}
\seealso{
\code{\link{get_sidra_series_metadata}} for available series names and metadata
\code{\link{mensalize_sidra_series}} to convert to exact months
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sidra-series-metadata.R
\name{sidra-series-metadata}
\alias{sidra-series-metadata}
\title{SIDRA Series Metadata for PNADC Mensalization}
\description{
This file contains metadata definitions for all 86 SIDRA series used in the
mensalization process. It maps series names to their SIDRA API endpoints,
table IDs, variable codes, and hierarchical categorization.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mensalize-sidra-series.R
\name{mensalize_sidra_series}
\alias{mensalize_sidra_series}
\title{Convert Rolling Quarters to Exact Monthly Series}
\usage{
mensalize_sidra_series(
  rolling_quarters,
  starting_points = NULL,
  series = "all",
  compute_derived = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{rolling_quarters}{data.table from \code{\link{fetch_sidra_rolling_quarters}}
containing rolling quarter series with columns \code{anomesfinaltrimmovel},
\code{mesnotrim}, and series value columns.}

\item{starting_points}{Optional data.table with precomputed starting points
(y0 values). If NULL (default), uses bundled \code{pnadc_series_starting_points}.
See Details for format.}

\item{series}{Character vector of series names to mensalize, or "all" (default)
for all series in the input data (except price indices).}

\item{compute_derived}{Logical. Compute derived series (rates, aggregates)?
Default TRUE.}

\item{verbose}{Logical. Print progress messages? Default TRUE.}
}
\value{
A data.table with columns:
\describe{
\item{anomesexato}{Integer. YYYYMM exact month}
\item{m_*}{Numeric. Mensalized value for each series (one column per series)}
}
}
\description{
Transforms SIDRA rolling quarterly averages into exact monthly values using
the mathematical relationship between consecutive rolling quarters.
}
\details{
The algorithm exploits the mathematical property of rolling quarterly averages:

\deqn{RQ_t - RQ_{t-1} = (Month_t - Month_{t-3}) / 3}

This means exact 3-month variations can be extracted from consecutive rolling
quarters. By accumulating these variations separately for each month-position
(1, 2, or 3), we build cumulative variation series. The only unknown is the
starting level for Jan, Feb, and Mar 2012.

\strong{Starting points} are estimated by:
\enumerate{
\item Computing monthly estimates from calibrated microdata (z_ variables)
\item Calculating cumulative variations from SIDRA (cum_ variables)
\item Backprojecting: e0 = z_ - cum_ over calibration period (2013-2019)
\item Averaging e0 by month position to get y0_ for each position
}

\strong{Final adjustment} ensures the average of 3 consecutive mensalized values
equals the original rolling quarter value.
}
\section{Starting Points Format}{

If providing custom starting points, the data.table must have columns:
\itemize{
\item \code{series_name}: Character. Series name matching rolling_quarters columns
\item \code{mesnotrim}: Integer (1, 2, or 3). Month position in quarter
\item \code{y0}: Numeric. Starting point value
}
}

\section{Mathematical Foundation}{

The mensalization algorithm proceeds in steps:
\enumerate{
\item Calculate d3 = 3 * (RQ_t - RQ_t-1)
\item Separate d3 by month position: d3m1, d3m2, d3m3
\item Cumulate separately: cum1, cum2, cum3
\item Apply starting points: y = y0 + cum
\item Final adjustment for rolling quarter consistency
}
}

\examples{
\donttest{
rq <- fetch_sidra_rolling_quarters(
  series = c("taxadesocup", "popocup", "popdesocup")
)
monthly <- mensalize_sidra_series(rq)
head(monthly)
}

}
\seealso{
\code{\link{fetch_sidra_rolling_quarters}} to obtain input data
\code{\link{compute_series_starting_points}} for custom calibration
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pnadc-apply-periods.R
\name{pnadc_apply_periods}
\alias{pnadc_apply_periods}
\title{Apply Reference Period Crosswalk to PNADC Data}
\usage{
pnadc_apply_periods(
  data,
  crosswalk,
  weight_var,
  anchor,
  calibrate = TRUE,
  calibration_unit = c("month", "fortnight", "week"),
  calibration_min_cell_size = 1,
  target_totals = NULL,
  smooth = FALSE,
  keep_all = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{data}{A data.frame or data.table with PNADC microdata. Must contain
join keys \code{Ano}, \code{Trimestre}, \code{UPA}, \code{V1008}, and
\code{V1014} to merge with the crosswalk.}

\item{crosswalk}{A data.table crosswalk from \code{\link{pnadc_identify_periods}}.}

\item{weight_var}{Character. Name of the survey weight column. Must be specified:
\itemize{
\item \code{"V1028"} for quarterly PNADC data
\item \code{"V1032"} for annual PNADC data (visit-specific or annual releases organized by quarters)
}}

\item{anchor}{Character. How to anchor the weight redistribution. Must be specified:
\itemize{
\item \code{"quarter"} for quarterly data or annual releases organized by quarters (preserves quarterly totals)
\item \code{"year"} for annual visit-specific data (preserves yearly totals)
}}

\item{calibrate}{Logical. If TRUE (default), calibrate weights to external
population totals. If FALSE, only merge the crosswalk without calibration.}

\item{calibration_unit}{Character. Temporal unit for weight calibration.
One of \code{"month"} (default), \code{"fortnight"}, or \code{"week"}.}

\item{calibration_min_cell_size}{Integer. Minimum sample size required in a cell
for it to be used in hierarchical raking. Cells smaller than this threshold
are collapsed to coarser levels. Default: 1 (use all cells).}

\item{target_totals}{Optional data.table with population targets. If NULL
(default), fetches monthly population from SIDRA and derives targets for
fortnight/week. Each time period (month, fortnight, or week) is calibrated
to the FULL Brazilian population from SIDRA.

If providing custom targets, the population column (\code{m_populacao} for
months, \code{f_populacao} for fortnights, \code{w_populacao} for weeks)
must be in \strong{thousands}. The function multiplies by 1000 internally.}

\item{smooth}{Logical. If TRUE, smooth calibrated weights to
remove quarterly artifacts. Smoothing is adapted per time period:
monthly (3-period window), fortnight (7-period window), weekly (no smoothing).
Default: FALSE.}

\item{keep_all}{Logical. If TRUE (default), keep all observations including
those with undetermined reference periods. If FALSE, drop undetermined rows.}

\item{verbose}{Logical. If TRUE (default), print progress messages.}
}
\value{
A data.table with the input data plus crosswalk columns:
\describe{
\item{ref_month_in_quarter, ref_month_in_year}{Month position (1-3 in quarter, 1-12 in year)}
\item{ref_fortnight_in_month, ref_fortnight_in_quarter}{Fortnight position (1-2 in month, 1-6 in quarter)}
\item{ref_week_in_month, ref_week_in_quarter}{Week position (1-4 in month, 1-12 in quarter)}
\item{ref_month_yyyymm, ref_fortnight_yyyyff, ref_week_yyyyww}{Integer period codes}
\item{determined_month, determined_fortnight, determined_week}{Logical determination flags}
\item{weight_monthly, weight_fortnight, or weight_weekly}{Calibrated weights (if calibrate=TRUE)}
}
}
\description{
This function takes a crosswalk from \code{\link{pnadc_identify_periods}} and
applies it to any PNADC dataset (quarterly or annual). It can optionally
calibrate the survey weights to match external population totals at the
chosen temporal granularity (month, fortnight, or week).
}
\details{
Merges a reference period crosswalk with PNADC microdata and optionally
calibrates survey weights for sub-quarterly analysis.
\subsection{Weight Calibration}{

When \code{calibrate = TRUE}, the function performs hierarchical rake weighting:
\enumerate{
\item Groups observations by nested demographic/geographic cells
\item Iteratively adjusts weights so sub-period totals match anchor-period totals
\item Calibrates final weights against external population totals (FULL Brazilian population)
\item Optionally smooths weights to remove quarterly artifacts
}
}

\subsection{Population Targets}{

All time periods (months, fortnights, and weeks) are calibrated to the FULL
Brazilian population from SIDRA. This means:
\itemize{
\item Monthly weights sum to the Brazilian population for that month
\item Fortnight weights sum to the Brazilian population for the containing month
\item Weekly weights sum to the Brazilian population for the containing month
}
}

\subsection{Hierarchical Raking Levels}{

The number of hierarchical cell levels is automatically adjusted based on the
calibration unit to avoid sparse cell issues:
\itemize{
\item \code{"month"}: 4 levels (age, region, state, post-stratum) - full hierarchy
\item \code{"fortnight"}: 2 levels (age, region) - simplified for lower sample size
\item \code{"week"}: 1 level (age groups only) - minimal hierarchy for sparse data
}
}

\subsection{Anchor Period}{

The \code{anchor} parameter determines how weights are redistributed:
\itemize{
\item \code{"quarter"}: Quarterly totals are preserved and redistributed to months/fortnights/weeks
\item \code{"year"}: Yearly totals are preserved and redistributed to months/fortnights/weeks
}

Use \code{anchor = "quarter"} with quarterly V1028 weights, and
\code{anchor = "year"} with annual V1032 weights.
}
}
\examples{
\dontrun{
crosswalk <- pnadc_identify_periods(pnadc_stacked)

result <- pnadc_apply_periods(
  pnadc_2023,
  crosswalk,
  weight_var = "V1028",
  anchor = "quarter"
)

result <- pnadc_apply_periods(
  pnadc_annual,
  crosswalk,
  weight_var = "V1032",
  anchor = "year"
)

result <- pnadc_apply_periods(
  pnadc_2023,
  crosswalk,
  weight_var = "V1028",
  anchor = "quarter",
  calibration_unit = "week"
)

result <- pnadc_apply_periods(
  pnadc_2023,
  crosswalk,
  weight_var = "V1028",
  anchor = "quarter",
  calibrate = FALSE
)
}

}
\seealso{
\code{\link{pnadc_identify_periods}} to build the crosswalk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mensalize-sidra-series.R
\name{compute_z_aggregates}
\alias{compute_z_aggregates}
\title{Compute z_ Aggregates from Monthly Microdata}
\usage{
compute_z_aggregates(calibrated_data, verbose = TRUE)
}
\arguments{
\item{calibrated_data}{PNADC microdata output from \code{pnadc_apply_periods()}
with \code{calibrate = TRUE}. Must include \code{weight_monthly},
\code{ref_month_yyyymm}, and \code{ref_month_in_quarter}.}

\item{verbose}{Print progress messages.}
}
\value{
data.table with columns:
\describe{
\item{anomesexato}{Integer YYYYMM month}
\item{z_*}{Numeric weighted aggregates for each series (one column per series)}
}
}
\description{
Computes monthly z_ aggregates from PNADC microdata using calibrated monthly
weights, with options for different population scaling approaches.
}
\details{
This function creates z_ indicator variables and aggregates them using the
calibrated \code{weight_monthly} from \code{pnadc_apply_periods()}.

The \code{pnadc_apply_periods()} function implements the calibration
methodology as follows:
All months are scaled uniformly to SIDRA monthly population totals.

This function simply aggregates the indicators using the already-calibrated weights.
}
\examples{
\dontrun{
crosswalk <- pnadc_identify_periods(stacked_data)

calibrated <- pnadc_apply_periods(stacked_data, crosswalk,
                                   weight_var = "V1028",
                                   anchor = "quarter",
                                   calibration_unit = "month")

z_agg <- compute_z_aggregates(calibrated)

rq <- fetch_sidra_rolling_quarters()
y0 <- compute_series_starting_points(z_agg, rq)
}

}
\seealso{
\code{\link{pnadc_apply_periods}} for the calibration step
\code{\link{compute_series_starting_points}} for the y0 computation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PNADCperiods-package.R
\docType{package}
\name{PNADCperiods-package}
\alias{PNADCperiods}
\alias{PNADCperiods-package}
\title{PNADCperiods: Identify Reference Periods in Brazil's PNADC Survey Data}
\description{
The PNADCperiods package provides tools to identify the exact reference
period (month, fortnight, or week) of Brazil's quarterly official household
survey, PNADC (Pesquisa Nacional por Amostra de Domicilios Continua - IBGE),
allowing for analyzing the survey data at sub-quarterly temporal granularity.
}
\details{
The package offers four main capabilities:
\enumerate{
\item \strong{Reference period identification}: Determines which temporal
period (month, fortnight, or week) within each quarter each survey
observation refers to, using IBGE's "Parada Técnica" rules and
respondent birthdates. The identification is \strong{nested by construction}:
fortnights require months, and weeks require fortnights.
\item \strong{Period-specific weight calibration}: Adjusts survey weights
for sub-quarterly estimates using adaptive hierarchical rake weighting
(4/2/1 cell levels for month/fortnight/week respectively)
\item \strong{Experimental strategies}: Probabilistic assignment and UPA
aggregation to boost fortnight/week determination rates for sensitivity
analysis
\item \strong{SIDRA series mensalization}: Converts IBGE's rolling quarterly
(trimestre móvel) aggregate series to exact monthly estimates
}

\strong{Determination rates (strict, full series 2012-2025):}
\itemize{
\item Monthly: ~97\\%, ~98\\% with annual anchor
\item Fortnight: ~9\\%
\item Week: ~3\\%
}

With smaller datasets, rates may differ (e.g., 8 quarters: ~94\\% monthly).
Experimental strategies (probabilistic + UPA aggregation) further improve
these determination rates.

The package is highly optimized for large datasets (~450,000 rows/sec).
Uses pre-computed lookup tables for 20x faster date creation.

\strong{Note:} Strict fortnight and week determination rates are inherently low
because they cannot leverage cross-quarter aggregation like months can. Only
birthday constraints within a single quarter are available to narrow the
interview window.

The main functions are:
\itemize{
\item \code{\link{pnadc_identify_periods}}: Builds a crosswalk containing
month, fortnight, and week reference periods with IBGE calendar-based dates
\item \code{\link{pnadc_apply_periods}}: Applies the crosswalk to any
PNADC dataset and optionally calibrates weights
\item \code{\link{pnadc_experimental_periods}}: Applies experimental strategies
(probabilistic, UPA aggregation) for improved fortnight/week determination
\item \code{\link{get_sidra_series_metadata}}: Lists 86+ available PNADC
rolling quarter series from SIDRA
\item \code{\link{fetch_sidra_rolling_quarters}}: Downloads rolling quarterly
data from IBGE SIDRA API
\item \code{\link{mensalize_sidra_series}}: Converts rolling quarters to exact
monthly estimates
}
}
\references{
HECKSHER, Marcos. "Valor Impreciso por Mes Exato: Microdados e Indicadores
Mensais Baseados na Pnad Continua". IPEA - Nota Tecnica Disoc, n. 62.
Brasilia, DF: IPEA, 2020.
\url{https://portalantigo.ipea.gov.br/portal/index.php?option=com_content&view=article&id=35453}

HECKSHER, M. "Cinco meses de perdas de empregos e simulacao de um incentivo
a contratacoes". IPEA - Nota Tecnica Disoc, n. 87. Brasilia, DF: IPEA, 2020.

HECKSHER, Marcos. "Mercado de trabalho: A queda da segunda quinzena de marco,
aprofundada em abril". IPEA - Carta de Conjuntura, v. 47, p. 1-6, 2020.

IBGE. Manual Basico da Entrevista PNADC (methodology on "Parada Tecnica").
}
\seealso{
Useful links:
\itemize{
  \item \url{https://antrologos.github.io/PNADCperiods/}
  \item \url{https://github.com/antrologos/PNADCperiods}
  \item Report bugs at \url{https://github.com/antrologos/PNADCperiods/issues}
}

}
\author{
Rogerio Barbosa \email{rogerio.barbosa@iesp.uerj.br} (R package, dashboard, and website; Ceres-IESP/UERJ)

Marcos Hecksher \email{mdhecksher@gmail.com} (mensalization methodology; Ipea)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mensalize-sidra-series.R
\name{compute_starting_points_from_microdata}
\alias{compute_starting_points_from_microdata}
\title{Compute Starting Points from Raw PNADC Microdata}
\usage{
compute_starting_points_from_microdata(
  data,
  calibration_start = NULL,
  calibration_end = NULL,
  verbose = TRUE
)
}
\arguments{
\item{data}{Stacked PNADC microdata (multiple quarters). Must contain variables
for period identification (see \code{\link{pnadc_identify_periods}}).}

\item{calibration_start}{Integer. Start of calibration period (YYYYMM).
Default NULL uses .PNADC_DATES$DEFAULT_CALIB_START (201301).}

\item{calibration_end}{Integer. End of calibration period (YYYYMM).
Default NULL uses .PNADC_DATES$DEFAULT_CALIB_END (201912).}

\item{verbose}{Print progress messages.}
}
\value{
data.table with columns:
\describe{
\item{series_name}{Character. Series name}
\item{mesnotrim}{Integer. Month position (1, 2, or 3)}
\item{y0}{Numeric. Starting point value}
}
}
\description{
Complete workflow to compute y0 starting points from raw PNADC microdata.
This is a convenience wrapper that combines period identification, weight
calibration, z_ aggregation, and starting point computation.
}
\details{
This function performs the complete workflow:
\enumerate{
\item Build crosswalk via \code{pnadc_identify_periods()}
\item Calibrate weights via \code{pnadc_apply_periods()}
\item Compute z_ aggregates via \code{compute_z_aggregates()}
\item Fetch SIDRA rolling quarters
\item Compute starting points via \code{compute_series_starting_points()}
}
}
\section{Weight Calibration}{

All months are scaled uniformly to SIDRA monthly population totals.
}

\examples{
\dontrun{
stacked <- fst::read_fst("pnadc_stacked.fst", as.data.table = TRUE)

y0 <- compute_starting_points_from_microdata(stacked)

bundled <- pnadc_series_starting_points
comparison <- merge(y0, bundled, by = c("series_name", "mesnotrim"))
}

}
\seealso{
\code{\link{pnadc_apply_periods}} for the weight calibration step
\code{\link{compute_z_aggregates}} for the z_ aggregation step
\code{\link{compute_series_starting_points}} for the y0 computation
\code{\link{pnadc_identify_periods}} for period identification
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-validation.R
\name{validate_pnadc}
\alias{validate_pnadc}
\title{Validate PNADC Input Data}
\usage{
validate_pnadc(data, check_weights = FALSE, stop_on_error = TRUE)
}
\arguments{
\item{data}{A data.frame or data.table with PNADC microdata}

\item{check_weights}{Logical. If TRUE, also check for weight-related variables.}

\item{stop_on_error}{Logical. If TRUE, stops with an error. If FALSE, returns
a validation report list.}
}
\value{
If \code{stop_on_error = TRUE}, returns invisibly if valid or stops with error.
If \code{stop_on_error = FALSE}, returns a list with:
\itemize{
\item \code{valid}: Logical indicating if data passed all validations
\item \code{issues}: Named list of validation issues found (empty if none)
\item \code{n_rows}: Number of rows in input data
\item \code{n_cols}: Number of columns in input data
\item \code{join_keys_available}: Character vector of available join key columns
}
}
\description{
Checks that input data has required columns for the specified processing.
}
\details{
The function performs the following validations:
\itemize{
\item Checks for required columns for reference period identification:
\code{Ano}, \code{Trimestre}, \code{UPA}, \code{V1008}, \code{V1014},
\code{V2008}, \code{V20081}, \code{V20082}, \code{V2009}
\item Validates year range (2012-2100 for PNADC coverage)
\item Validates quarter values (must be 1-4)
\item Validates birth day values (must be 1-31 or 99 for unknown)
\item Validates birth month values (must be 1-12 or 99 for unknown)
\item Warns about unusual ages (outside 0-130 range)
\item If \code{check_weights = TRUE}, also validates weight-related columns:
\code{V1028}, \code{UF}, \code{posest}, \code{posest_sxi}
}
}
\examples{
# Minimal valid data (all 9 required columns)
sample_data <- data.frame(
  Ano = 2023L, Trimestre = 1L, UPA = 110000001L,
  V1008 = 1L, V1014 = 1L,
  V2008 = 15L, V20081 = 3L, V20082 = 1990L, V2009 = 33L
)
validate_pnadc(sample_data)

# Data with missing columns returns issues (non-stop mode)
incomplete_data <- data.frame(Ano = 2023L, Trimestre = 1L)
result <- validate_pnadc(incomplete_data, stop_on_error = FALSE)
result$valid    # FALSE
result$issues   # lists missing columns

}
\seealso{
\code{\link{pnadc_identify_periods}} which calls this function
internally to validate input data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sidra-series-metadata.R
\name{get_sidra_series_metadata}
\alias{get_sidra_series_metadata}
\title{Get SIDRA Series Metadata}
\usage{
get_sidra_series_metadata(
  series = "all",
  theme = NULL,
  theme_category = NULL,
  subcategory = NULL,
  lang = "pt"
)
}
\arguments{
\item{series}{Character vector of series names to retrieve, or "all" (default)
for all series.}

\item{theme}{Character vector of themes to filter by. Valid themes:
"labor_market", "earnings", "demographics", "social_protection", "prices".
Use NULL (default) for no filtering.}

\item{theme_category}{Character vector of theme categories to filter by.
Use NULL (default) for no filtering.}

\item{subcategory}{Character vector of subcategories to filter by.
Use NULL (default) for no filtering.}

\item{lang}{Character. Language for descriptions: "pt" (Portuguese, default)
or "en" (English). When "en", the \code{description} column contains English text.}
}
\value{
A data.table with columns:
\describe{
\item{series_name}{Character. Internal name used in the package}
\item{api_path}{Character. SIDRA API path for get_sidra()}
\item{table_id}{Integer. SIDRA table number}
\item{variable_id}{Integer. SIDRA variable code}
\item{theme}{Character. Top-level theme}
\item{theme_category}{Character. Middle-level category within theme}
\item{subcategory}{Character. Optional subcategory for filtering}
\item{description_pt}{Character. Portuguese description}
\item{description_en}{Character. English description}
\item{description}{Character. Description in the requested language}
\item{unit}{Character. Unit of measurement}
\item{unit_label_pt}{Character. Unit label in Portuguese}
\item{unit_label_en}{Character. Unit label in English}
\item{is_derived}{Logical. TRUE if computed from other series}
\item{requires_deflation}{Logical. TRUE if needs IPCA deflation}
}
}
\description{
Returns a data.table with metadata for all PNADC rolling quarter series
available from IBGE's SIDRA API.
}
\examples{
meta <- get_sidra_series_metadata()
head(meta)

labor <- get_sidra_series_metadata(theme = "labor_market")

unemp <- get_sidra_series_metadata(theme = "labor_market",
                                    theme_category = "unemployment")

meta_en <- get_sidra_series_metadata(series = c("taxadesocup", "popocup"),
                                     lang = "en")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetch-sidra-series.R
\name{fetch-sidra-series}
\alias{fetch-sidra-series}
\title{Fetch SIDRA Rolling Quarter Series}
\description{
Functions to download PNADC labor market indicators from IBGE's SIDRA API.
These series are published as rolling quarterly averages (trimestre movel).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pnadc-identify-periods.R
\name{pnadc_identify_periods}
\alias{pnadc_identify_periods}
\title{Identify Reference Periods in PNADC Data}
\usage{
pnadc_identify_periods(data, verbose = TRUE, store_date_bounds = FALSE)
}
\arguments{
\item{data}{A data.frame or data.table with PNADC microdata. Required columns:
\itemize{
\item \code{Ano}: Survey year
\item \code{Trimestre}: Quarter (1-4)
\item \code{UPA}: Primary Sampling Unit
\item \code{V1008}: Household id/sequence within UPA
\item \code{V1014}: Panel identifier
\item \code{V2008}: Birth day (1-31)
\item \code{V20081}: Birth month (1-12)
\item \code{V20082}: Birth year
\item \code{V2009}: Age
}
Optional but recommended:
\itemize{
\item \code{V2003}: Person sequence within household
}}

\item{verbose}{Logical. If TRUE (default), display progress information.}

\item{store_date_bounds}{Logical. If TRUE, stores date bounds and exception
flags in the crosswalk for optimization when calling
\code{pnadc_experimental_periods()}. This enables 10-20x speedup for the
probabilistic strategy by avoiding redundant computation. Default FALSE.}
}
\value{
A data.table crosswalk with columns:
\describe{
\item{Ano, Trimestre, UPA, V1008, V1014}{Join keys (year, quarter, UPA, household, panel)}
\item{ref_month_in_quarter}{Integer. Month position in quarter (1, 2, 3) or NA}
\item{ref_month_in_year}{Integer. Month position in year (1-12) or NA}
\item{ref_fortnight_in_month}{Integer. Fortnight position in month (1 or 2) or NA}
\item{ref_fortnight_in_quarter}{Integer. Fortnight position in quarter (1-6) or NA}
\item{ref_week_in_month}{Integer. Week position in month (1-4) or NA}
\item{ref_week_in_quarter}{Integer. Week position in quarter (1-12) or NA}
\item{date_min}{Date. Lower bound of the interview reference date for the individual. Only returned if store_date_bounds = TRUE}
\item{date_max}{Date. Upper bound of the interview reference date for the individual. Only returned if store_date_bounds = TRUE}
\item{week_1_start}{Date. Sunday of the IBGE first reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_1_end}{Date. Saturday of the IBGE first reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_2_start}{Date. Sunday of the IBGE second reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_2_end}{Date. Saturday of the IBGE second reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_3_start}{Date. Sunday of the IBGE third reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_3_end}{Date. Saturday of the IBGE third reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_4_start}{Date. Sunday of the IBGE fourth reference week of the month. Only returned if store_date_bounds = TRUE}
\item{week_4_end}{Date. Saturday of the IBGE fourth reference week of the month. Only returned if store_date_bounds = TRUE}
\item{month_max_upa}{Integer. Maximum month position across UPA-V1014 group (for debugging). Only returned if store_date_bounds = TRUE}
\item{month_min_upa}{Integer. Minimum month position across UPA-V1014 group (for debugging). Only returned if store_date_bounds = TRUE}
\item{fortnight_max_hh}{Integer. Maximum fortnight position within household (for debugging). Only returned if store_date_bounds = TRUE}
\item{fortnight_min_hh}{Integer. Minimum fortnight position within household (for debugging). Only returned if store_date_bounds = TRUE}
\item{week_min_hh}{Integer. Minimum week position within household (for debugging). Only returned if store_date_bounds = TRUE}
\item{week_max_hh}{Integer. Maximum week position within household (for debugging). Only returned if store_date_bounds = TRUE}
\item{ref_month_yyyymm}{Integer. Identified reference month in the format YYYYMM, where MM follows the IBGE calendar. 1 <= MM  <= 12}
\item{ref_fortnight_yyyyff}{Integer. Identified reference fortnight in the format YYYYFF, where FF follows the IBGE calendar. 1 <= FF  <= 24}
\item{ref_week_yyyyww}{Integer. Identified reference Week in the format YYYYWW, where WW follows the IBGE calendar. 1 <= WW  <= 48}
\item{determined_month}{Logical. Flags if the month was determined.}
\item{determined_fortnight}{Logical. Flags if the fortnight was determined.}
\item{determined_week}{Logical. Flags if the week was determined.}
}
}
\description{
PNADC is a quarterly survey, but each interview actually refers to a specific
temporal period within the quarter. This function identifies which month,
fortnight (quinzena), and week each observation belongs to, enabling
sub-quarterly time series analysis.

The algorithm uses a \strong{nested identification approach}:
\itemize{
\item \strong{Phase 1}: Identify MONTHS for all observations using:
\itemize{
\item IBGE's reference week timing rules (first reference week -- ending in a Saturday -- with sufficient days)
\item Respondent birthdates to constrain possible interview dates
\item UPA-panel level aggregation across ALL quarters (panel design)
\item Dynamic exception detection (identifies quarters needing relaxed rules)
}
\item \strong{Phase 2}: Identify FORTNIGHTS for month-determined observations:
\itemize{
\item Search space constrained to 2 fortnights within determined month
\item Household-level aggregation within each quarter
}
\item \strong{Phase 3}: Identify WEEKS for fortnight-determined observations:
\itemize{
\item Search space constrained to ~2 weeks within determined fortnight
\item Household-level aggregation within each quarter
}
}
}
\details{
Builds a crosswalk containing reference periods (month, fortnight,
and week) for PNADC survey data based on IBGE's interview timing rules.
\subsection{Temporal Granularity}{

The crosswalk contains three levels of temporal granularity:
\itemize{
\item \strong{Month}: 3 per quarter, ~97\\% determination rate with full series (aggregates across quarters)
\item \strong{Fortnight (quinzena)}: 6 per quarter, ~9\\% determination rate (within-quarter only)
\item \strong{Week}: 12 per quarter, ~3\\% determination rate (within-quarter only)
}
}

\subsection{Cross-Quarter Aggregation (Important!)}{

\strong{For optimal month determination rates, input data should be stacked across
multiple quarters} (ideally 4+ years). The algorithm leverages PNADC's
rotating panel design where the same UPA-V1014 is interviewed in the same
relative position across quarterly visits.
}

\subsection{Fortnight Definition}{

Fortnights are numbered 1-6 per quarter (2 per month), based on the IBGE
reference week calendar (not calendar days). Each IBGE "month" consists of
exactly 4 reference weeks (28 days), starting on a Sunday:
\itemize{
\item Fortnight 1 in month: IBGE weeks 1-2 (days 1-14 of the IBGE month)
\item Fortnight 2 in month: IBGE weeks 3-4 (days 15-28 of the IBGE month)
}
}
}
\note{
\subsection{Nested Identification Hierarchy}{

The algorithm enforces strict nesting by construction:
\itemize{
\item Fortnights can ONLY be identified for observations with determined months
\item Weeks can ONLY be identified for observations with determined fortnights
}

This guarantees: \code{determined_week => determined_fortnight => determined_month}
}

\subsection{Aggregation Levels}{

The crosswalk aggregates at different levels:
\itemize{
\item \strong{Months}: UPA-V1014 level across ALL quarters
(PNADC panel design ensures same month position)
\item \strong{Fortnights}: Household level within quarter only
\item \strong{Weeks}: Household level within quarter only
}
}
}
\examples{
\dontrun{
crosswalk <- pnadc_identify_periods(pnadc_stacked)

crosswalk[, .(
  month_rate = mean(determined_month),
  fortnight_rate = mean(determined_fortnight),
  week_rate = mean(determined_week)
)]

crosswalk[determined_fortnight, all(determined_month)]
crosswalk[determined_week, all(determined_fortnight)]

result <- pnadc_apply_periods(pnadc_2023, crosswalk,
                              weight_var = "V1028",
                              anchor = "quarter")
}

}
\seealso{
\code{\link{pnadc_apply_periods}} to apply the crosswalk and
calibrate weights
}
